@@ -20,9 +20,9 @@
 #include "object/waypoint.h"
 
 struct ship_weapon;
-struct ship_subsys;
-struct object;
-struct ship_info;
+class ship_subsys;
+class object;
+class ship_info;
 
 #define	AI_DEFAULT_CLASS 3  // default AI class for new ships (Fred)
 
@@ -32,7 +32,7 @@ typedef struct ai_flag_name {
 	int flag_list;
 } ai_flag_name;
 
-#define MAX_AI_FLAG_NAMES			1
+#define MAX_AI_FLAG_NAMES			2
 extern ai_flag_name Ai_flag_names[];
 
 #define	AIF_FORMATION_WING					(1 << 0)	//	Fly in formation as part of wing.
@@ -64,6 +64,8 @@ extern ai_flag_name Ai_flag_names[];
 #define	AIF_UNLOAD_PRIMARIES				(1 << 24)	//	Fire primaries as fast as possible!
 #define AIF_TRYING_UNSUCCESSFULLY_TO_WARP	(1 << 25)	// Trying to warp, but can't warp at the moment
 
+#define AIF_FREE_AFTERBURNER_USE			(1 << 26)	// Use afterburners while following waypoints or flying towards objects
+
 #define	AIF_AVOID_SHOCKWAVE		(AIF_AVOID_SHOCKWAVE_SHIP | AIF_AVOID_SHOCKWAVE_WEAPON)
 #define	AIF_FORMATION			(AIF_FORMATION_WING | AIF_FORMATION_OBJECT)
 
@@ -107,6 +109,7 @@ extern ai_flag_name Ai_flag_names[];
 #define AIGF_PURGE					(1<<5)	// purge this goal next time we process
 #define AIGF_GOALS_PURGED			(1<<6)	// this goal has already caused other goals to get purged
 #define AIGF_DEPART_SOUND_PLAYED	(1<<7)	// Goober5000 - replacement for AL's hack ;)
+#define AIGF_TARGET_OWN_TEAM		(1<<8)	// Goober5000 - removes standard precautions against AI ships taking traitorous actions
 
 #define AIGF_DOCK_INDEXES_VALID		(AIGF_DOCKER_INDEX_VALID|AIGF_DOCKEE_INDEX_VALID)
 
@@ -366,10 +369,10 @@ typedef struct ai_info {
 	int		ai_class;				//	Class.  Might be override of default.
 
 	//	Probably become obsolete, to be replaced by path_start, path_cur, etc.
-	waypoint_list				*wp_list;		// waypoint list being followed
-	SCP_list<waypoint>::iterator wp_index;	// waypoint index in list
+	waypoint_list	*wp_list;		// waypoint list being followed
+	size_t			 wp_index;		// waypoint index in list
 	int		wp_flags;				//	waypoint flags, see WPF_xxxx
-	int		waypoint_speed_cap;	// -1 no cap, otherwise cap - changed to int by Goober5000
+	int		waypoint_speed_cap;		// -1 no cap, otherwise cap - changed to int by Goober5000
 
 	//	Path following information
 	int		path_start;				//	Index into global array, start of path.
@@ -587,7 +590,7 @@ extern char** Ai_class_names;
 extern int Num_ai_classes;
 extern int Ai_firing_enabled;
 
-extern char	*Skill_level_names(int skill_level, int translate = 1);
+extern const char *Skill_level_names(int skill_level, int translate = 1);
 extern int Ai_goal_signature;
 
 // need access to following data in AiBig.cpp
@@ -622,7 +625,6 @@ extern void ai_ignore_object(object *ignorer, object *ignored, int ignore_new);
 extern void ai_ignore_wing(object *ignorer, int wingnum, int priority);
 extern void ai_dock_with_object(object *docker, int docker_index, object *dockee, int dockee_index, int dock_type);
 extern void ai_stay_still(object *still_objp, vec3d *view_pos);
-extern void ai_set_default_behavior(object *obj, int classnum);
 extern void ai_do_default_behavior(object *obj);
 extern void ai_start_waypoints(object *objp, waypoint_list *wp_list, int wp_flags);
 extern void ai_ship_hit(object *objp_ship, object *hit_objp, vec3d *hitpos, int shield_quadrant, vec3d *hit_normal);
