@@ -67,7 +67,7 @@ int ai_path_type_match(char *p)
 	return -1;
 }
 
-void parse_ai_profiles_tbl(char *filename)
+void parse_ai_profiles_tbl(const char *filename)
 {
 	int i, rval;
 	char profile_name[NAME_LENGTH];
@@ -75,13 +75,9 @@ void parse_ai_profiles_tbl(char *filename)
 	char *saved_Mp = NULL;
 	char buf[NAME_LENGTH];
 
-	// open localization
-	lcl_ext_open();
-
 	if ((rval = setjmp(parse_abort)) != 0)
 	{
 		mprintf(("TABLES: Unable to parse '%s'!  Error code = %i.\n", (filename) ? filename : "<default ai_profiles.tbl>", rval));
-		lcl_ext_close();
 		return;
 	}
 
@@ -467,6 +463,8 @@ void parse_ai_profiles_tbl(char *filename)
 
 			set_flag(profile, "$fix ai path order bug:", AIPF2_FIX_AI_PATH_ORDER_BUG, AIP_FLAG2);
 
+			set_flag(profile, "$strict turret-tagged-only targeting:", AIPF2_STRICT_TURRET_TAGGED_ONLY_TARGETING, AIP_FLAG2);
+
 			// ----------
 
 			// compatibility
@@ -512,11 +510,8 @@ void parse_ai_profiles_tbl(char *filename)
 	required_string("#End");
 
 	// add tbl/tbm to multiplayer validation list
-	extern void fs2netd_add_table_validation(char *tblname);
+	extern void fs2netd_add_table_validation(const char *tblname);
 	fs2netd_add_table_validation(filename);
-
-	// close localization
-	lcl_ext_close();
 }
 
 void ai_profiles_init()
