@@ -171,6 +171,8 @@ done_1:
 	}
 	
 	mc_info mc;
+	mc_info_init(&mc);
+
 	mc.model_instance_num = Ships[objp->instance].model_instance_num;
 	mc.model_num = sip->model_num;
 	mc.orient = &objp->orient;
@@ -367,9 +369,13 @@ int ai_big_maybe_follow_subsys_path(int do_dot_check)
 	object	*target_objp;
 
 	aip = &Ai_info[Ships[Pl_objp->instance].ai_index];
+    
+	if ( aip->target_objnum < 0 )
+		return 0;
+    
 	target_objp = &Objects[aip->target_objnum];
 
-	if ( (aip->targeted_subsys != NULL) && (aip->target_objnum >= 0) && (aip->targeted_subsys->system_info->path_num >= 0) ) {
+	if ( (aip->targeted_subsys != NULL) && (aip->targeted_subsys->system_info->path_num >= 0) ) {
 		int			subsys_path_num, subsys_in_sight, checked_sight;
 		float			dist;
 
@@ -597,11 +603,15 @@ void ai_big_chase_attack(ai_info *aip, ship_info *sip, vec3d *enemy_pos, float d
 
 		// see if Pl_objp needs to reposition to get a good shot at subsystem which is being attacked
 		if ( ai_big_maybe_follow_subsys_path() ) {
+			if ((Pl_objp->phys_info.flags & PF_AFTERBURNER_ON) && !(aip->ai_flags & AIF_KAMIKAZE)) {
+				afterburners_stop(Pl_objp);
+			}
 			return;
 		}
 
-		vec3d	*rel_pos, vec_to_enemy;
-		float		weapon_travel_dist;
+		vec3d	*rel_pos;
+		vec3d	vec_to_enemy;
+		float	weapon_travel_dist;
 
 		start_bank = Ships[aip->shipnum].weapons.current_primary_bank;
 
@@ -626,11 +636,25 @@ void ai_big_chase_attack(ai_info *aip, ship_info *sip, vec3d *enemy_pos, float d
 		weapon_travel_dist = ai_get_weapon_dist(&Ships[Pl_objp->instance].weapons);
 
 		if ( aip->targeted_subsys != NULL ) {
-			if (dist_to_enemy > (weapon_travel_dist-20))
+			if (dist_to_enemy > (weapon_travel_dist-20)) {
 				accelerate_ship(aip, 1.0f);
-			else {
+				
+				ship	*shipp = &Ships[Pl_objp->instance];
+				if ((aip->ai_flags & AIF_FREE_AFTERBURNER_USE) && !(shipp->flags2 & SF2_AFTERBURNER_LOCKED) && (dot_to_enemy > 0.75f)) {
+					if (ai_maybe_fire_afterburner(Pl_objp, aip)) {
+						afterburners_start(Pl_objp);
+						aip->afterburner_stop_time = Missiontime + 3*F1_0;
+					}
+				} else if ((Pl_objp->phys_info.flags & PF_AFTERBURNER_ON) && !(aip->ai_flags & AIF_KAMIKAZE)) {
+					afterburners_stop(Pl_objp);
+				}
+			} else {
 				// AL 12-31-97: Move at least as quickly as your target is moving...
 				accelerate_ship(aip, MAX(1.0f - dot_to_enemy, Objects[aip->target_objnum].phys_info.fspeed/sip->max_speed));
+
+				if ((Pl_objp->phys_info.flags & PF_AFTERBURNER_ON) && !(aip->ai_flags & AIF_KAMIKAZE)) {
+					afterburners_stop(Pl_objp);
+				}
 			}
 
 		} else {
@@ -651,10 +675,22 @@ void ai_big_chase_attack(ai_info *aip, ship_info *sip, vec3d *enemy_pos, float d
 			if (dist_to_enemy < ATTACK_STOP_DISTANCE) {
 //				accelerate_ship(aip, accel * 0.5f);
 				accelerate_ship(aip, -1.0f);
+				if ((Pl_objp->phys_info.flags & PF_AFTERBURNER_ON) && !(aip->ai_flags & AIF_KAMIKAZE)) {
+					afterburners_stop(Pl_objp);
+				}
 			} else {
 				accelerate_ship(aip, accel);
-			}
 
+				ship	*shipp = &Ships[Pl_objp->instance];
+				if ((aip->ai_flags & AIF_FREE_AFTERBURNER_USE) && !(shipp->flags2 & SF2_AFTERBURNER_LOCKED) && (accel > 0.95f)) {
+					if (ai_maybe_fire_afterburner(Pl_objp, aip)) {
+						afterburners_start(Pl_objp);
+						aip->afterburner_stop_time = Missiontime + 3*F1_0;
+					}
+				} else if ((Pl_objp->phys_info.flags & PF_AFTERBURNER_ON) && !(aip->ai_flags & AIF_KAMIKAZE)) {
+					afterburners_stop(Pl_objp);
+				}
+			}
 		}
 	}
 }
@@ -711,86 +747,75 @@ void ai_big_maybe_fire_weapons(float dist_to_enemy, float dot_to_enemy, vec3d *f
 			}
 
 			if (tswp->num_secondary_banks > 0) {
-
-				int	priority1, priority2;
-
-				priority1 = -1;
-				priority2 = -1;
-
-				//	Maybe favor selecting a bomb.
-				//	Note, if you're firing a bomb, if it's aspect seeking, the firing conditions can be looser.
-				if (Ship_info[Ships[En_objp->instance].ship_info_index].flags & (SIF_BIG_SHIP | SIF_HUGE_SHIP))
-					if (En_objp->phys_info.speed * dist_to_enemy < 5000.0f)		//	Don't select a bomb if enemy moving fast relative to distance
-						priority1 = WIF_BOMB;
-
 				if (!(En_objp->flags & OF_PROTECTED) || (aip->goals[0].ai_mode & (AI_GOAL_DISABLE_SHIP | AI_GOAL_DISARM_SHIP))) {
-					//ai_select_secondary_weapon(Pl_objp, tswp, priority1, priority2);	//	Note, need to select to get weapon speed and lifetime
-					if(aip->goals[0].ai_mode & (AI_GOAL_DISABLE_SHIP | AI_GOAL_DISARM_SHIP)) {
-						priority1 = WIF_PUNCTURE;
-					}
 					ai_choose_secondary_weapon(Pl_objp, aip, En_objp);
 					int current_bank = tswp->current_secondary_bank;
-					weapon_info	*swip = &Weapon_info[tswp->secondary_bank_weapons[current_bank]];
+					if (current_bank > -1) {
+						weapon_info	*swip = &Weapon_info[tswp->secondary_bank_weapons[current_bank]];
 
-					if(!(En_objp->flags & OF_PROTECTED) || ((aip->goals[0].ai_mode & (AI_GOAL_DISABLE_SHIP | AI_GOAL_DISARM_SHIP)) && swip->wi_flags & WIF_PUNCTURE )) { //override lockdown on protected ships when using anti subsystem weapons - Valathil
-						//	If ship is protected and very low on hits, don't fire missiles.
-						if ((current_bank > -1) &&  (!(En_objp->flags & OF_PROTECTED) || (En_objp->hull_strength > 10*swip->damage))) {
-							if (aip->ai_flags & AIF_UNLOAD_SECONDARIES) {
-								if (timestamp_until(swp->next_secondary_fire_stamp[current_bank]) > swip->fire_wait*1000.0f) {
-									swp->next_secondary_fire_stamp[current_bank] = timestamp((int) (swip->fire_wait*1000.0f));
-								}
-							}
-
-							if (timestamp_elapsed(swp->next_secondary_fire_stamp[current_bank])) {
-								float firing_range;
-								if (swip->wi_flags2 & WIF2_LOCAL_SSM)
-									firing_range=swip->lssm_lock_range;
-								else
-									firing_range = MIN((swip->max_speed * swip->lifetime), swip->weapon_range);
-								// reduce firing range of secondaries in nebula
-								extern int Nebula_sec_range;
-								if ((The_mission.flags & MISSION_FLAG_FULLNEB) && Nebula_sec_range) {
-									firing_range *= 0.8f;
+						if(!(En_objp->flags & OF_PROTECTED) || ((aip->goals[0].ai_mode & (AI_GOAL_DISABLE_SHIP | AI_GOAL_DISARM_SHIP)) && swip->wi_flags & WIF_PUNCTURE )) { //override lockdown on protected ships when using anti subsystem weapons - Valathil
+							//	If ship is protected and very low on hits, don't fire missiles.
+							if (!(En_objp->flags & OF_PROTECTED) || (En_objp->hull_strength > 10*swip->damage)) {
+								if (aip->ai_flags & AIF_UNLOAD_SECONDARIES) {
+									if (timestamp_until(swp->next_secondary_fire_stamp[current_bank]) > swip->fire_wait*1000.0f) {
+										swp->next_secondary_fire_stamp[current_bank] = timestamp((int) (swip->fire_wait*1000.0f));
+									}
 								}
 
-								float t = 0.25f;	//	default delay in seconds until next fire.
+								if (timestamp_elapsed(swp->next_secondary_fire_stamp[current_bank])) {
+									float firing_range;
+									if (swip->wi_flags2 & WIF2_LOCAL_SSM)
+										firing_range=swip->lssm_lock_range;
+									else
+										firing_range = MIN((swip->max_speed * swip->lifetime), swip->weapon_range);
+									// reduce firing range of secondaries in nebula
+									extern int Nebula_sec_range;
+									if ((The_mission.flags & MISSION_FLAG_FULLNEB) && Nebula_sec_range) {
+										firing_range *= 0.8f;
+									}
 
-								if (dist_to_enemy < firing_range*1.0f) {
+									float t = 0.25f;	//	default delay in seconds until next fire.
+
+									if (dist_to_enemy < firing_range*1.0f) {
 
 
-									//vm_vec_scale_add(&future_enemy_pos, enemy_pos, enemy_vel, dist_to_enemy/swip->max_speed);
-									//if (vm_vec_dist_quick(&future_enemy_pos, firing_pos) < firing_range * 0.8f) {
-										if (ai_fire_secondary_weapon(Pl_objp)) {
-											if ((aip->ai_flags & AIF_UNLOAD_SECONDARIES) || (swip->burst_flags & WBF_FAST_FIRING)) {
-												if (swip->burst_shots > swp->burst_counter[current_bank]) {
-													t = swip->burst_delay;
-													swp->burst_counter[current_bank]++;
-												} else {
-													t = swip->fire_wait;
-													if ((swip->burst_shots > 0) && (swip->burst_flags & WBF_RANDOM_LENGTH)) {
-														swp->burst_counter[current_bank] = myrand() % swip->burst_shots;
+										//vm_vec_scale_add(&future_enemy_pos, enemy_pos, enemy_vel, dist_to_enemy/swip->max_speed);
+										//if (vm_vec_dist_quick(&future_enemy_pos, firing_pos) < firing_range * 0.8f) {
+											if (ai_fire_secondary_weapon(Pl_objp)) {
+
+												int current_bank_adjusted = MAX_SHIP_PRIMARY_BANKS + current_bank;
+
+												if ((aip->ai_flags & AIF_UNLOAD_SECONDARIES) || (swip->burst_flags & WBF_FAST_FIRING)) {
+													if (swip->burst_shots > swp->burst_counter[current_bank_adjusted]) {
+														t = swip->burst_delay;
+														swp->burst_counter[current_bank_adjusted]++;
 													} else {
- 														swp->burst_counter[current_bank] = 0;
+														t = swip->fire_wait;
+														if ((swip->burst_shots > 0) && (swip->burst_flags & WBF_RANDOM_LENGTH)) {
+															swp->burst_counter[current_bank_adjusted] = myrand() % swip->burst_shots;
+														} else {
+															swp->burst_counter[current_bank_adjusted] = 0;
+														}
 													}
-												}
-											} else {
-												if (swip->burst_shots > swp->burst_counter[current_bank]) {
-													t = set_secondary_fire_delay(aip, temp_shipp, swip, true);
-													swp->burst_counter[current_bank]++;
 												} else {
-													t = set_secondary_fire_delay(aip, temp_shipp, swip, false);
-													if ((swip->burst_shots > 0) && (swip->burst_flags & WBF_RANDOM_LENGTH)) {
-														swp->burst_counter[current_bank] = myrand() % swip->burst_shots;
+													if (swip->burst_shots > swp->burst_counter[current_bank_adjusted]) {
+														t = set_secondary_fire_delay(aip, temp_shipp, swip, true);
+														swp->burst_counter[current_bank_adjusted]++;
 													} else {
-														swp->burst_counter[current_bank] = 0;
+														t = set_secondary_fire_delay(aip, temp_shipp, swip, false);
+														if ((swip->burst_shots > 0) && (swip->burst_flags & WBF_RANDOM_LENGTH)) {
+															swp->burst_counter[current_bank_adjusted] = myrand() % swip->burst_shots;
+														} else {
+															swp->burst_counter[current_bank_adjusted] = 0;
+														}
 													}
 												}
+												swp->next_secondary_fire_stamp[current_bank] = timestamp((int) (t*1000.0f));
 											}
-											swp->next_secondary_fire_stamp[current_bank] = timestamp((int) (t*1000.0f));
-										}
-									//}
+										//}
+									}
+									swp->next_secondary_fire_stamp[current_bank] = timestamp((int) (t*1000.0f));
 								}
-								swp->next_secondary_fire_stamp[current_bank] = timestamp((int) (t*1000.0f));
 							}
 						}
 					}
@@ -861,6 +886,9 @@ void ai_big_chase()
 			aip->submode_parm0 = Missiontime;	// use parm0 as time strafe mode entered (i.e. MODE start time)
 			aip->submode = AIS_STRAFE_AVOID;
 			aip->submode_start_time = Missiontime;
+			if (Pl_objp->phys_info.flags & PF_AFTERBURNER_ON) {
+				afterburners_stop(Pl_objp);
+			}
 			return;
 		}
 	}
@@ -901,8 +929,12 @@ void ai_big_chase()
 	case SM_ATTACK:
 	case SM_SUPER_ATTACK:
 		if (vm_vec_dist_quick(&Pl_objp->pos, &predicted_enemy_pos) > 100.0f + En_objp->radius * 2.0f) {
-			if (maybe_avoid_big_ship(Pl_objp, En_objp, aip, &predicted_enemy_pos, 10.0f))
+			if (maybe_avoid_big_ship(Pl_objp, En_objp, aip, &predicted_enemy_pos, 10.0f)) {
+				if ((Pl_objp->phys_info.flags & PF_AFTERBURNER_ON) && !(aip->ai_flags & AIF_KAMIKAZE)) {
+					afterburners_stop(Pl_objp);
+				}
 				return;
+			}
 		}
 
 		if (aip->target_time < 2.0f)
@@ -918,6 +950,13 @@ void ai_big_chase()
 	//
 	//	Set turn and acceleration based on submode.
 	//
+
+	if (((aip->submode != SM_ATTACK) && (aip->submode != SM_SUPER_ATTACK) && (aip->submode != SM_ATTACK_FOREVER) && (aip->submode != SM_EVADE_WEAPON)) || (aip->mode == AIM_STRAFE)){
+		if ((Pl_objp->phys_info.flags & PF_AFTERBURNER_ON) && !(aip->ai_flags & AIF_KAMIKAZE)) {
+			afterburners_stop(Pl_objp);
+		}
+	}
+
 	switch (aip->submode) {
 	case SM_CONTINUOUS_TURN:
 		ai_big_chase_ct();
