@@ -93,7 +93,7 @@ int Num_ai_goals = sizeof(Ai_goal_names) / sizeof(ai_goal_list);
 // HUD what a ship's current orders are.  If the AI goal doesn't correspond to something that
 // ought to be printable, then NULL is used.
 // JAS: Converted to a function in order to externalize the strings
-char *Ai_goal_text(int goal)
+const char *Ai_goal_text(int goal)
 {
 	switch(goal)	{
 	case 1:
@@ -979,6 +979,16 @@ void ai_add_goal_sub_sexp( int sexp, int type, ai_goal *aigp, char *actor_name )
 		aigp->priority = MAX_GOAL_PRIORITY;
 	}
 
+	// Goober5000 - we now have an extra optional chase argument to allow chasing our own team
+	if ( op == OP_AI_CHASE || op == OP_AI_CHASE_WING || op == OP_AI_DISABLE_SHIP || op == OP_AI_DISARM_SHIP ) {
+		if ((CDDDR(node) != -1) && is_sexp_true(CDDDR(node)))
+			aigp->flags |= AIGF_TARGET_OWN_TEAM;
+	}
+	if ( op == OP_AI_DESTROY_SUBSYS ) {
+		if ((CDDDDR(node) != -1) && is_sexp_true(CDDDDR(node)))
+			aigp->flags |= AIGF_TARGET_OWN_TEAM;
+	}
+
 	// Goober5000 - since none of the goals act on the actor,
 	// don't assign the goal if the actor's goal target is itself
 	if (aigp->target_name != NULL && !strcmp(aigp->target_name, actor_name))
@@ -1374,13 +1384,17 @@ int ai_mission_goal_achievable( int objnum, ai_goal *aigp )
 		if (!(shipp->flags2 & SF2_NO_SUBSPACE_DRIVE))
 			return AI_GOAL_ACHIEVABLE;
 
-		// if no subspace drive, only valid if there's somewhere to depart to
+		// if no subspace drive, only valid if our mothership is present
 
-		// locate a capital ship on the same team:
-		if (ship_get_ship_with_dock_bay(shipp->team) >= 0)
-			return AI_GOAL_ACHIEVABLE;
-		else
-			return AI_GOAL_NOT_KNOWN;
+		// check that we have a mothership and that we can depart to it
+		if (shipp->departure_location == DEPART_AT_DOCK_BAY)
+		{
+			int anchor_shipnum = ship_name_lookup(Parse_names[shipp->departure_anchor]);
+			if (anchor_shipnum >= 0 && ship_useful_for_departure(anchor_shipnum, shipp->departure_path_mask))
+				return AI_GOAL_ACHIEVABLE;
+		}
+
+		return AI_GOAL_NOT_KNOWN;
 	}
 
 
@@ -2340,7 +2354,7 @@ void ai_update_goal_references(ai_goal *goals, int type, const char *old_name, c
 	}
 }
 
-int query_referenced_in_ai_goals(ai_goal *goals, int type, char *name)
+int query_referenced_in_ai_goals(ai_goal *goals, int type, const char *name)
 {
 	int i, mode, flag;
 
@@ -2405,7 +2419,7 @@ int query_referenced_in_ai_goals(ai_goal *goals, int type, char *name)
 	return 0;
 }
 
-char *ai_add_dock_name(char *str)
+char *ai_add_dock_name(const char *str)
 {
 	char *ptr;
 	int i;
