@@ -96,6 +96,7 @@ extern int Cmdline_ship_choice_3d;
 extern int Cmdline_weapon_choice_3d;
 extern int Cmdline_warp_flash;
 extern int Cmdline_autopilot_interruptable;
+extern int Cmdline_stretch_menu;
 
 // Audio related
 extern int Cmdline_query_speech;
@@ -120,6 +121,7 @@ extern int Cmdline_nomovies;	// WMC Toggles movie playing support
 extern int Cmdline_no_set_gamma;
 extern int Cmdline_novbo;
 extern int Cmdline_no_fbo;
+extern int Cmdline_no_pbo;
 extern int Cmdline_noglsl;
 extern int Cmdline_mipmap;
 extern int Cmdline_ati_color_swap;
@@ -128,6 +130,7 @@ extern int Cmdline_no_glsl_model_rendering;
 extern int Cmdline_no_di_mouse;
 extern int Cmdline_drawelements;
 extern char* Cmdline_keyboard_layout;
+extern bool Cmdline_gl_finish;
 
 // Developer/Testing related
 extern char *Cmdline_start_mission;
@@ -149,5 +152,8 @@ extern int Cmdline_verify_vps;
 extern int Cmdline_no_grab;
 #endif
 extern int Cmdline_reparse_mainhall;
+extern bool Cmdline_frame_profile;
+extern bool Cmdline_profile_write_file;
+extern bool Cmdline_no_unfocus_pause;
 
 #endif
