@@ -101,6 +101,7 @@ void debris_init()
 	for (i=0; i<MAX_DEBRIS_PIECES; i++ )	{
 		Debris[i].flags = 0;
 		Debris[i].sound_delay = 0;
+		Debris[i].objnum = -1;
 	}
 		
 	Num_hull_pieces = 0;
@@ -237,6 +238,7 @@ void debris_delete( object * obj )
 	}
 
 	db->flags = 0;
+	db->objnum = -1;
 	Num_debris_pieces--;
 }
 
@@ -524,11 +526,11 @@ object *debris_create(object *source_obj, int model_num, int submodel_num, vec3d
 		// Create Debris piece n!
 		if ( hull_flag ) {
 			if (rand() < RAND_MAX/6)	// Make some pieces blow up shortly after explosion.
-				db->lifeleft = 2.0f * ((float) myrand()/(float) RAND_MAX) + 0.5f;
+				db->lifeleft = 2.0f * (myrand() * RAND_MAX_1f) + 0.5f;
 			else
 				db->lifeleft = -1.0f;		// large hull pieces stay around forever
 		} else {
-			db->lifeleft = (i2fl(myrand())/i2fl(RAND_MAX))*2.0f+0.1f;
+			db->lifeleft = (myrand() * RAND_MAX_1f) * 2.0f + 0.1f;
 		}
 	}
 
@@ -805,6 +807,7 @@ void debris_hit(object *debris_obj, object *other_obj, vec3d *hitpos, float dama
 int debris_check_collision(object *pdebris, object *other_obj, vec3d *hitpos, collision_info_struct *debris_hit_info)
 {
 	mc_info	mc;
+	mc_info_init(&mc);
 	int		num;
 
 	Assert( pdebris->type == OBJ_DEBRIS );
@@ -834,7 +837,7 @@ int debris_check_collision(object *pdebris, object *other_obj, vec3d *hitpos, co
 
 		weapon *wp = &Weapons[other_obj->instance];
 		wp->collisionOccured = true;
-		wp->collisionInfo = mc_info(mc);
+		memcpy(&wp->collisionInfo, &mc, sizeof(mc_info));
 
 		return mc.num_hits;
 	}
