@@ -157,18 +157,14 @@ static void fireball_set_default_color(int idx)
  *
  * NOTE: we can't be too trusting here so a tbm will only modify the LOD count, not add an entry
  */
-void parse_fireball_tbl(char *filename)
+void parse_fireball_tbl(const char *filename)
 {
 	int rval;
 	lod_checker lod_check;
 	color fb_color;
 
-	// open localization
-	lcl_ext_open();
-
 	if ((rval = setjmp(parse_abort)) != 0) {
 		mprintf(("TABLES: Unable to parse '%s'!  Error code = %i.\n", filename, rval));
-		lcl_ext_close();
 		return;
 	}
 
@@ -241,9 +237,6 @@ void parse_fireball_tbl(char *filename)
 	}
 
 	required_string("#End");
-
-	// close localization
-	lcl_ext_close();
 }
 
 void fireball_parse_tbl()
@@ -381,7 +374,9 @@ void fireball_render(object * obj)
     
 	MONITOR_INC( NumFireballsRend, 1 );	
 	
-	num = obj->instance;
+	memset(&p, 0, sizeof(p));
+    
+    num = obj->instance;
 	fb = &Fireballs[num];
 
 	if ( Fireballs[num].current_bitmap < 0 )
