@@ -65,7 +65,9 @@ BEGIN_MESSAGE_MAP(player_start_editor, CDialog)
 	//{{AFX_MSG_MAP(player_start_editor)
 	ON_WM_INITMENU()
 	ON_LBN_SELCHANGE(IDC_SHIP_LIST, OnSelchangeShipList)	
+	ON_CLBN_CHKCHANGE(IDC_SHIP_LIST, OnSelchangeShipList)
 	ON_LBN_SELCHANGE(IDC_WEAPON_LIST, OnSelchangeWeaponList)	
+	ON_CLBN_CHKCHANGE(IDC_SHIP_LIST, OnSelchangeWeaponList)
 	ON_EN_UPDATE(IDC_SHIP_POOL, OnUpdateShipPool)
 	ON_EN_UPDATE(IDC_WEAPON_POOL, OnUpdateWeaponPool)
 	ON_LBN_SELCHANGE(IDC_SHIP_VARIABLES_LIST, OnSelchangeShipVariablesList)
