@@ -963,6 +963,7 @@
 #define IDC_MAX_RESPAWN_DELAY_SPIN      1487
 #define IDC_NAV_NEEDSLINK               1488
 #define IDC_OBJ_TEAM                    1489
+#define IDC_SCRAMBLE_MESSAGES           1489
 #define IDC_CAMPAIGN_TYPE               1491
 #define IDC_SUPPORT_ALLOWED             1492
 #define IDC_SUBSPACE                    1493
@@ -1056,7 +1057,11 @@
 #define IDC_USE_AUTOPILOT_CINEMATICS    1547
 #define IDC_2D_MISSION                  1548
 #define IDC_LABEL1                      1549
+#define IDC_2D_MISSION2                 1549
+#define IDC_ALWAYS_SHOW_GOALS           1549
 #define IDC_LABEL2                      1550
+#define IDC_ALWAYS_SHOW_GOALS2          1550
+#define IDC_END_TO_MAINHALL             1550
 #define IDC_NEW_TEXTURE                 1551
 #define IDC_TEXTURES                    1552
 #define IDC_OLD_TEXTURE_LIST            1553
@@ -1109,8 +1114,10 @@
 #define IDC_FLIP_ICON                   1578
 #define IDC_DISABLE_BUILTIN_SHIP        1579
 #define IDC_OTHER                       1579
+#define IDC_USE_WING_ICON               1579
 #define IDC_DISABLE_BUILTIN_COMMAND_MSGS 1580
 #define IDC_LBL_OTHER_SUFFIX            1580
+#define IDC_USE_CARGO_ICON              1580
 #define IDC_PRIMARIES_LOCKED            1581
 #define IDC_EXPORT                      1581
 #define IDC_SECONDARIES_LOCKED          1582
@@ -1147,6 +1154,8 @@
 #define IDC_FONT_FILE                   1619
 #define IDC_FICTION_MUSIC               1620
 #define IDC_SOUND_ENVIRONMENT_LABEL     1621
+#define IDC_FONT_FILE2                  1621
+#define IDC_VOICE_FILE                  1621
 #define IDC_SOUND_ENVIRONMENT_VOLUME_LABEL 1622
 #define IDC_SOUND_ENVIRONENT_DAMPING_LABEL 1623
 #define IDC_SOUND_ENVIRONMENT_DECAY_LABEL 1624
@@ -1164,12 +1173,13 @@
 #define IDC_NEB_FAR_MULTIPLIER          1634
 #define IDC_MISSION_LOG_TRUE            1635
 #define IDC_MISSION_LOG_FALSE           1636
-#define IDC_MISSION_LOG_ALWAYS_TRUE     1637
+#define IDC_MISSION_LOG_STATE_CHANGE    1637
 #define IDC_MISSION_LOG_ALWAYS_FALSE    1638
 #define IDC_MISSION_LOG_1ST_REPEAT      1639
 #define IDC_MISSION_LOG_LAST_REPEAT     1640
 #define IDC_MISSION_LOG_1ST_TRIGGER     1641
 #define IDC_MISSION_LOG_LAST_TRIGGER    1642
+#define IDC_NO_COLLIDE                  1643
 #define ID_FILE_MISSIONNOTES            32771
 #define ID_DUPLICATE                    32774
 #define ID_VIEW_ROTATE                  32775
@@ -1434,7 +1444,6 @@
 #define ID_FORMAT_FS1_RETAIL            33072
 #define ID_EDITORS_SET_GLOBAL_SHIP_FLAGS 33073
 #define ID_FILE_IMPORT_FSM              33074
-#define ID_FILE_IMPORT_WEAPONS          33075
 #define ID_VIEW_LIGHTING                33079
 #define ID_EDITORS_VOICE                33080
 #define ID_SHOW_IFF_0                   33081
@@ -1463,7 +1472,7 @@
 #define _APS_3D_CONTROLS                     1
 #define _APS_NEXT_RESOURCE_VALUE        320
 #define _APS_NEXT_COMMAND_VALUE         33098
-#define _APS_NEXT_CONTROL_VALUE         1643
+#define _APS_NEXT_CONTROL_VALUE         1644
 #define _APS_NEXT_SYMED_VALUE           105
 #endif
 #endif
