@@ -32,6 +32,7 @@
 #include "cmdline/cmdline.h"
 #include "cfile/cfilesystem.h"
 #include "network/multimsgs.h"
+#include "mod_table/mod_table.h"
 
 #ifndef WIN32
 #include <cstdio>
@@ -51,7 +52,7 @@
 
 
 extern int Multi_debrief_stats_accept_code;
-extern void HUD_printf(char *format, ...);
+extern void HUD_printf(const char *format, ...);
 extern int game_hacked_data();
 
 
@@ -122,8 +123,19 @@ void fs2netd_options_config_init()
 	}
 
 	if ( !strlen(Multi_options_g.tracker_port) ) {
-		ml_printf("NOTICE: Port for game/user trackers not specified, using default instead (%s).", FS2NETD_DEFAULT_PORT);
-		strncpy( Multi_options_g.tracker_port, FS2NETD_DEFAULT_PORT, STD_NAME_LEN );
+		if ( FS2NetD_port >= 1024 && FS2NetD_port <= USHRT_MAX ) {
+			ml_printf("NOTICE: User override for game/user tracker port not specified, using game_settings.tbl override (%i).", FS2NetD_port);
+			int result;
+			result = sprintf(Multi_options_g.tracker_port, "%i", FS2NetD_port);
+			Assertion( result > 0, "Copying port %i to tracker_port failed\n", FS2NetD_port );
+		}
+		else {
+			if ( FS2NetD_port != 0 ) {
+				ml_printf("ERROR: game_settings.tbl override for game/user tracker port '%i' must be between %i and %i.", FS2NetD_port, 1024, USHRT_MAX);
+			}
+			ml_printf("NOTICE: Port for game/user trackers not specified, using default instead (%s).", FS2NETD_DEFAULT_PORT);
+			strncpy( Multi_options_g.tracker_port, FS2NETD_DEFAULT_PORT, STD_NAME_LEN );
+		}
 	} else {
 		long port_tmp = strtol(Multi_options_g.tracker_port, (char**)NULL, 10);
 
@@ -725,7 +737,7 @@ static void fs2netd_handle_messages()
 
 			case PCKT_SLIST_REPLY: {
 				int numServers = 0;
-				int svr_flags;
+				int svr_flags __attribute__((__unused__)); // gcc [-Wunused-but-set-variable] doesn't like MACROs
 				ushort svr_port;
 				char svr_ip[16];
 				active_game ag;
@@ -1539,7 +1551,7 @@ int fs2netd_update_valid_tables()
 	return hacked;
 }
 
-void fs2netd_add_table_validation(char *tblname)
+void fs2netd_add_table_validation(const char *tblname)
 {
 	uint chksum = 0;
 
@@ -1560,7 +1572,7 @@ void fs2netd_add_table_validation(char *tblname)
 
 	crc_valid_status tbl_crc;
 
-	strncpy(tbl_crc.name, tblname, NAME_LENGTH);
+	strcpy_s(tbl_crc.name, tblname);
 	tbl_crc.crc32 = chksum;
 	tbl_crc.valid = 0;
 
@@ -1583,9 +1595,6 @@ int fs2netd_get_pilot_info(const char *callsign, player *out_plr, bool first_cal
 		new_plr.reset();
 		strncpy( new_plr.callsign, callsign, CALLSIGN_LEN );
 
-		// initialize the stats to default values
-		init_scoring_element( &new_plr.stats );
-
 		out_plr->reset();
 
 		Local_timeout = timer_get_seconds() + 30;
@@ -1612,7 +1621,7 @@ int fs2netd_get_pilot_info(const char *callsign, player *out_plr, bool first_cal
 	}
 
 	if (rc == 0) {
-		memcpy( out_plr, &new_plr, sizeof(player) );
+		out_plr->assign(&new_plr);
 		In_process = false;
 		Local_timeout = -1;
 	}
@@ -1637,7 +1646,7 @@ void fs2netd_close()
 	FS2NetD_ban_list.clear();
 }
 
-void fs2netd_update_game_count(char *chan_name)
+void fs2netd_update_game_count(const char *chan_name)
 {
 	if ( !Logged_in ) {
 		return;
@@ -1653,7 +1662,6 @@ void fs2netd_update_game_count(char *chan_name)
 void fs2netd_spew_table_checksums(char *outfile)
 {
 	char full_name[MAX_PATH_LEN];
-	int count;
 	FILE *out = NULL;
 	char description[512] = { 0 };
 	char filename[65] = { 0 };
@@ -1676,7 +1684,7 @@ void fs2netd_spew_table_checksums(char *outfile)
 	p = Cmdline_spew_table_crcs;
 
 	while (*p && (offset < sizeof(description))) {
-		if (*p == '"') {
+		if (*p == '"' && offset < sizeof(description)-1) {
 			description[offset++] = '"';
 			description[offset++] = '"';
 		} else {
@@ -1689,15 +1697,13 @@ void fs2netd_spew_table_checksums(char *outfile)
 	// header
 	fprintf(out, "filename,CRC32,description\r\n");
 
-	count = (int)Table_valid_status.size();
-
 	// do all the checksums
 	for (SCP_vector<crc_valid_status>::iterator tvs = Table_valid_status.begin(); tvs != Table_valid_status.end(); ++tvs) {
 		offset = 0;
 		p = tvs->name;
 
 		while (*p && (offset < sizeof(filename))) {
-			if (*p == '"') {
+			if (*p == '"' && offset < sizeof(filename)-1) {
 				filename[offset++] = '"';
 				filename[offset++] = '"';
 			} else {
@@ -1707,7 +1713,11 @@ void fs2netd_spew_table_checksums(char *outfile)
 			p++;
 		}
 
-		filename[offset] = '\0';
+		if (offset < sizeof(filename)) {
+			filename[offset] = '\0';
+		} else {
+			filename[sizeof(filename)-1] = '\0';
+		}
 
 		fprintf(out, "\"%s\",%u,\"%s\"\r\n", filename, tvs->crc32, description);
 	}
