@@ -131,7 +131,6 @@ int FS2NetD_SendPlayerData(const char *player_name, player *pl, bool do_send)
 		}
 
 		Assert( (num_type_kills >= 0) && (num_type_kills < USHRT_MAX) );
-
 		PXO_ADD_USHORT( (ushort)num_type_kills );
 
 		for (i = 0; i < MAX_SHIP_CLASSES; i++) {
@@ -141,10 +140,11 @@ int FS2NetD_SendPlayerData(const char *player_name, player *pl, bool do_send)
 			}
 		}
 
-		PXO_ADD_USHORT( (ushort)MAX_MEDALS );
+		Assert( (Num_medals >= 0) && (Num_medals < USHRT_MAX) );
+		PXO_ADD_USHORT( (ushort)Num_medals );
 
-		for (i = 0; i < MAX_MEDALS; i++) {
-			PXO_ADD_INT( pl->stats.medals[i] );
+		for (i = 0; i < Num_medals; i++) {
+			PXO_ADD_INT( pl->stats.medal_counts[i] );
 		}
 
 		DONE_PACKET();
@@ -249,7 +249,7 @@ int FS2NetD_GetPlayerData(const char *player_name, player *pl, bool can_create,
 		}
 
 		// make sure that we get the entire packet
-		while ( (rc_total < (uint)buffer_size) && (rc_total <= sizeof(buffer)) && (timer_get_fixed_seconds() <= end_time) ) {
+		while ( (rc_total < (uint)buffer_size) && (rc_total < sizeof(buffer)) && (timer_get_fixed_seconds() <= end_time) ) {
 			if ( FS2NetD_DataReady() ) {
 				rc = FS2NetD_GetData(buffer+rc_total, sizeof(buffer) - rc_total);
 
@@ -303,8 +303,8 @@ int FS2NetD_GetPlayerData(const char *player_name, player *pl, bool can_create,
 
 		PXO_GET_USHORT( num_medals );
 
-		for (idx = 0; (idx < MAX_MEDALS) && (idx < num_medals); idx++) {
-			PXO_GET_INT( pl->stats.medals[idx] );
+		for (idx = 0; (idx < Num_medals) && (idx < num_medals); idx++) {
+			PXO_GET_INT( pl->stats.medal_counts[idx] );
 		}
 
 		return (int)reply_type;
@@ -361,7 +361,7 @@ int FS2NetD_GetBanList(SCP_vector<SCP_string> &mask_list, bool do_send)
 		}
 
 		// make sure that we get the entire packet
-		while ( (rc_total < (uint)buffer_size) && (rc_total <= sizeof(buffer)) && (timer_get_fixed_seconds() <= end_time) ) {
+		while ( (rc_total < (uint)buffer_size) && (rc_total < sizeof(buffer)) && (timer_get_fixed_seconds() <= end_time) ) {
 			if ( FS2NetD_DataReady() ) {
 				rc = FS2NetD_GetData(buffer+rc_total, sizeof(buffer) - rc_total);
 
@@ -435,7 +435,7 @@ int FS2NetD_GetMissionsList(SCP_vector<file_record> &m_list, bool do_send)
 		}
 
 		// make sure that we get the entire packet
-		while ( (rc_total < (uint)buffer_size) && (rc_total <= sizeof(buffer)) && (timer_get_fixed_seconds() <= end_time) ) {
+		while ( (rc_total < (uint)buffer_size) && (rc_total < sizeof(buffer)) && (timer_get_fixed_seconds() <= end_time) ) {
 			if ( FS2NetD_DataReady() ) {
 				rc = FS2NetD_GetData(buffer+rc_total, sizeof(buffer) - rc_total);
 
@@ -735,7 +735,7 @@ int FS2NetD_ValidateTableList(bool do_send)
 		}
 
 		// make sure that we get the entire packet
-		while ( (rc_total < (uint)buffer_size) && (rc_total <= sizeof(buffer)) && (timer_get_fixed_seconds() <= end_time) ) {
+		while ( (rc_total < (uint)buffer_size) && (rc_total < sizeof(buffer)) && (timer_get_fixed_seconds() <= end_time) ) {
 			if ( FS2NetD_DataReady() ) {
 				rc = FS2NetD_GetData(buffer+rc_total, sizeof(buffer) - rc_total);
 
@@ -773,7 +773,7 @@ int FS2NetD_ValidateTableList(bool do_send)
 	return 0;
 }
 
-void FS2NetD_GameCountUpdate(char *chan_name)
+void FS2NetD_GameCountUpdate(const char *chan_name)
 {
 	int buffer_size;
 	char buffer[MAX_PATH+BASE_PACKET_SIZE+10];
