@@ -282,22 +282,14 @@ void event_music_init()
 	event_music_reset_choices();
 
 	// Goober5000
+	// set all the filenames to "none" so we're compatible with the extra NRMLs in FS1 music
+	memset(Soundtracks, 0, MAX_SOUNDTRACKS * sizeof(SOUNDTRACK_INFO));
 	for (i = 0; i < MAX_SOUNDTRACKS; i++)
-	{
-		memset(&Soundtracks[i], 0, sizeof(SOUNDTRACK_INFO));
-
-		// set all the filenames to "none" so we're compatible with the extra NRMLs in FS1 music
 		for (j = 0; j < MAX_PATTERNS; j++)
-		{
 			strcpy_s(Soundtracks[i].pattern_fnames[j], NOX("none.wav"));
-		}
-	}
 
 	// Goober5000
-	for (i = 0; i < MAX_SPOOLED_MUSIC; i++)
-	{
-		memset(&Spooled_music[i], 0, sizeof(menu_music));
-	}
+	memset(Spooled_music, 0, MAX_SPOOLED_MUSIC * sizeof(menu_music));
 
 	//Do teh parsing
 	event_music_parse_musictbl("music.tbl");
@@ -602,7 +594,7 @@ void event_music_level_init(int force_soundtrack)
 	{
 		if (!strnicmp(strack->pattern_fnames[i], NOX("none.wav"), 4))
 		{
-			Patterns[i].handle = -1;	
+			Patterns[i].handle = -1;
 			continue;
 		}
 
@@ -796,6 +788,9 @@ int event_music_enemy_arrival()
 	else {
 		next_pattern = SONG_EARV_1;
 	}
+    
+	if ( Current_pattern < 0 )
+		return 0;
 
 	if ( Current_pattern == next_pattern )
 		return 0;	// already playing
@@ -903,7 +898,7 @@ int event_music_friendly_arrival()
 	// Goober5000 - to avoid array out-of-bounds
 	//Assert(Current_pattern >= 0 && Current_pattern < MAX_PATTERNS);
 
-	if(Current_pattern < 0 || Current_pattern > MAX_PATTERNS)
+	if(Current_pattern < 0 || Current_pattern >= MAX_PATTERNS)
 		return 0;
 
 	if ( Patterns[Current_pattern].next_pattern != Patterns[Current_pattern].default_next_pattern )
@@ -1133,7 +1128,7 @@ int event_music_player_respawn_as_observer()
 bool parse_soundtrack_line(int strack_idx, int pattern_idx)
 {
 	char fname[MAX_FILENAME_LEN];
-	char line_buf[128];
+	char line_buf[MAX_PATH_LEN];
 	char *token;
 	int count = 0;
 
@@ -1196,7 +1191,7 @@ void parse_soundtrack()
 	if(strack_idx < 0 && (nocreate || Num_soundtracks >= MAX_SOUNDTRACKS))
 	{
 		if(Num_soundtracks >= MAX_SOUNDTRACKS) {
-			Warning(LOCATION, "Maximum number of soundtracks reached after '%s'; max is '%d'", Soundtracks[Num_soundtracks].name, MAX_SOUNDTRACKS);
+			Warning(LOCATION, "Maximum number of soundtracks reached after '%s'; max is '%d'", Soundtracks[MAX_SOUNDTRACKS-1].name, MAX_SOUNDTRACKS);
 		}
 
 		//Track doesn't exist and has nocreate, so don't create it
@@ -1379,20 +1374,16 @@ void parse_menumusic()
 // event_music_parse_musictbl() will parse the music.tbl file, and set up the Mission_songs[]
 // array
 //
-void event_music_parse_musictbl(char *filename)
+void event_music_parse_musictbl(const char *filename)
 {
 	int rval;
 
 	if ((rval = setjmp(parse_abort)) != 0) {
 		mprintf(("TABLES: Unable to parse '%s'!  Error code = %i.\n", filename, rval));
-		lcl_ext_close();
 
 	} else {
-		// open localization
-		lcl_ext_open();
-
 		read_file_text(filename, CF_TYPE_TABLES);
-		reset_parse();		
+		reset_parse();
 
 		while ( skip_to_start_of_string_either("#Soundtrack Start", "#Menu Music Start", NULL ) )
 		{
@@ -1409,9 +1400,6 @@ void event_music_parse_musictbl(char *filename)
 				}
 			}
 		}
-
-		// close localization
-		lcl_ext_close();
 	}
 }
 
