@@ -22,6 +22,7 @@
 #define	FILESPEC_LENGTH			64
 #define	MESSAGE_LENGTH			512
 #define TRAINING_MESSAGE_LENGTH	512
+#define CONDITION_LENGTH		64
 
 // from missionparse.h
 #define MISSION_DESC_LENGTH		512
@@ -78,10 +79,6 @@
 
 #define MAX_POLYGON_MODELS  300
 
-#define MAX_MEDALS			18
-#define NUM_MEDALS_FS1		16
-extern int Num_medals;
-
 // object.h
 #define MAX_OBJECTS			3500		
 
