@@ -31,6 +31,8 @@
 #include "parse/scripting.h"
 #include "gamesequence/gamesequence.h"	//WMC - for scripting hooks in gr_flip()
 #include "io/keycontrol.h" // m!m
+#include "debugconsole/console.h"
+#include "debugconsole/console.h"
 
 
 #if defined(SCP_UNIX) && !defined(__APPLE__)
@@ -42,6 +44,8 @@
 // Includes for different rendering systems
 #include "graphics/gropengl.h"
 
+const char *Resolution_prefixes[GR_NUM_RESOLUTIONS] = { "", "2_" };
+
 screen gr_screen;
 
 color_gun Gr_red, Gr_green, Gr_blue, Gr_alpha;
@@ -86,75 +90,178 @@ float Max_draw_distance = Default_max_draw_distance;
 static int GL_cursor_nframes = 0;
 
 // Pre-computed screen resize vars
+static float Gr_full_resize_X = 1.0f, Gr_full_resize_Y = 1.0f;
 static float Gr_resize_X = 1.0f, Gr_resize_Y = 1.0f;
-static float Gr_unsize_X = 1.0f, Gr_unsize_Y = 1.0f;
+static float Gr_menu_offset_X = 0.0f, Gr_menu_offset_Y = 0.0f;
+static float Gr_menu_zoomed_offset_X = 0.0f, Gr_menu_zoomed_offset_Y = 0.0f;
 
+float Gr_save_full_resize_X = 1.0f, Gr_save_full_resize_Y = 1.0f;
 float Gr_save_resize_X = 1.0f, Gr_save_resize_Y = 1.0f;
-float Gr_save_unsize_X = 1.0f, Gr_save_unsize_Y = 1.0f;
+float Gr_save_menu_offset_X = 0.0f, Gr_save_menu_offset_Y = 0.0f;
+float Gr_save_menu_zoomed_offset_X = 0.0f, Gr_save_menu_zoomed_offset_Y = 0.0f;
 
 bool Save_custom_screen_size;
 
-void gr_set_screen_scale(int w, int h)
+void gr_set_screen_scale(int w, int h, int zoom_w, int zoom_h, int max_w, int max_h, bool force_stretch)
 {
-	Gr_resize_X = (float)gr_screen.max_w / (float)w;
-	Gr_resize_Y = (float)gr_screen.max_h / (float)h;
+	bool do_zoom = zoom_w > 0 && zoom_h > 0 && (zoom_w != w || zoom_h != h);
 
-	Gr_unsize_X = (float)w / (float)gr_screen.max_w;
-	Gr_unsize_Y = (float)h / (float)gr_screen.max_h;
+	Gr_full_resize_X = (float)max_w / (float)w;
+	Gr_full_resize_Y = (float)max_h / (float)h;
 
-	Save_custom_screen_size = gr_screen.custom_size;
+	if (do_zoom) {
+		float aspect_quotient = ((float)max_w / (float)max_h) / ((float)zoom_w / (float)zoom_h);
 
-	gr_screen.custom_size = true;
-}
+		Gr_resize_X = (float)max_w / (float)zoom_w / ((aspect_quotient > 1.0f) ? aspect_quotient : 1.0f);
+		Gr_resize_Y = (float)max_h / (float)zoom_h * ((aspect_quotient < 1.0f) ? aspect_quotient : 1.0f);
 
-void gr_set_screen_scale(int w, int h, int max_w, int max_h)
-{
-	Gr_resize_X = (float)max_w / (float)w;
-	Gr_resize_Y = (float)max_h / (float)h;
+		Gr_menu_offset_X = (max_w - w * Gr_resize_X) / 2.0f;
+		Gr_menu_offset_Y = (max_h - h * Gr_resize_Y) / 2.0f;
 
-	Gr_unsize_X = (float)w / (float)max_w;
-	Gr_unsize_Y = (float)h / (float)max_h;
+		Gr_menu_zoomed_offset_X = (Gr_menu_offset_X >= 0.0f) ? Gr_menu_offset_X : 0.0f;
+		Gr_menu_zoomed_offset_Y = (Gr_menu_offset_Y >= 0.0f) ? Gr_menu_offset_Y : 0.0f;
 
-	Save_custom_screen_size = gr_screen.custom_size;
+		if (force_stretch || Cmdline_stretch_menu) {
+			if (Gr_menu_offset_X > 0.0f) {
+				Gr_resize_X = Gr_full_resize_X;
+				Gr_menu_offset_X = Gr_menu_zoomed_offset_X = 0.0f;
+			}
+			if (Gr_menu_offset_Y > 0.0f) {
+				Gr_resize_Y = Gr_full_resize_Y;
+				Gr_menu_offset_Y = Gr_menu_zoomed_offset_Y = 0.0f;
+			}
+		}
+	} else {
+		if (force_stretch || Cmdline_stretch_menu) {
+			Gr_resize_X = Gr_full_resize_X;
+			Gr_resize_Y = Gr_full_resize_Y;
 
-	gr_screen.custom_size = true;
+			Gr_menu_offset_X = Gr_menu_zoomed_offset_X = 0.0f;
+			Gr_menu_offset_Y = Gr_menu_zoomed_offset_Y = 0.0f;
+		} else {
+			float aspect_quotient = ((float)max_w / (float)max_h) / ((float)w / (float)h);
+
+			Gr_resize_X = Gr_full_resize_X / ((aspect_quotient > 1.0f) ? aspect_quotient : 1.0f);
+			Gr_resize_Y = Gr_full_resize_Y * ((aspect_quotient < 1.0f) ? aspect_quotient : 1.0f);
+
+			Gr_menu_offset_X = Gr_menu_zoomed_offset_X = (aspect_quotient > 1.0f) ? ((max_w - w * Gr_resize_X) / 2.0f) : 0.0f;
+			Gr_menu_offset_Y = Gr_menu_zoomed_offset_Y = (aspect_quotient < 1.0f) ? ((max_h - h * Gr_resize_Y) / 2.0f) : 0.0f;
+		}
+	}
+
+	gr_screen.custom_size = (w != max_w || h != max_h);
+
+	if (gr_screen.rendering_to_texture == -1) {
+		gr_screen.max_w_unscaled = w;
+		gr_screen.max_h_unscaled = h;
+
+		if (do_zoom) {
+			gr_screen.max_w_unscaled_zoomed = gr_screen.max_w_unscaled + fl2i(Gr_menu_offset_X * 2.0f / Gr_resize_X);
+			gr_screen.max_h_unscaled_zoomed = gr_screen.max_h_unscaled + fl2i(Gr_menu_offset_Y * 2.0f / Gr_resize_Y);
+			if (gr_screen.max_w_unscaled_zoomed > gr_screen.max_w_unscaled) {
+				gr_screen.max_w_unscaled_zoomed = gr_screen.max_w_unscaled;
+			}
+			if (gr_screen.max_h_unscaled_zoomed > gr_screen.max_h_unscaled) {
+				gr_screen.max_h_unscaled_zoomed = gr_screen.max_h_unscaled;
+			}
+		} else {
+			gr_screen.max_w_unscaled_zoomed = gr_screen.max_w_unscaled;
+			gr_screen.max_h_unscaled_zoomed = gr_screen.max_h_unscaled;
+		}
+	}
 }
 
 void gr_reset_screen_scale()
 {
+	Gr_full_resize_X = Gr_save_full_resize_X;
+	Gr_full_resize_Y = Gr_save_full_resize_Y;
+
 	Gr_resize_X = Gr_save_resize_X;
 	Gr_resize_Y = Gr_save_resize_Y;
 
-	Gr_unsize_X = Gr_save_unsize_X;
-	Gr_unsize_Y = Gr_save_unsize_Y;
+	Gr_menu_offset_X = Gr_save_menu_offset_X;
+	Gr_menu_offset_Y = Gr_save_menu_offset_Y;
+
+	Gr_menu_zoomed_offset_X = Gr_save_menu_zoomed_offset_X;
+	Gr_menu_zoomed_offset_Y = Gr_save_menu_zoomed_offset_Y;
 
 	gr_screen.custom_size = Save_custom_screen_size;
+
+	if (gr_screen.rendering_to_texture == -1) {
+		gr_screen.max_w_unscaled = gr_screen.max_w_unscaled_zoomed = (gr_screen.res == GR_1024) ? 1024 : 640;
+		gr_screen.max_h_unscaled = gr_screen.max_h_unscaled_zoomed = (gr_screen.res == GR_1024) ?  768 : 480;
+	}
 }
 
 /**
  * This function is to be called if you wish to scale GR_1024 or GR_640 x and y positions or
  * lengths in order to keep the correctly scaled to nonstandard resolutions
  *
- * @param x X value (width to be scaled), can be NULL
- * @param y Y value (height to be scaled), can be NULL
+ * @param x X value, can be NULL
+ * @param y Y value, can be NULL
+ * @param w width, can be NULL
+ * @param h height, can be NULL
+ * @param resize_mode
  * @return always true unless error
  */
-bool gr_resize_screen_pos(int *x, int *y)
+bool gr_resize_screen_pos(int *x, int *y, int *w, int *h, int resize_mode)
 {
-	if ( !gr_screen.custom_size && (gr_screen.rendering_to_texture == -1) ) {
+	if ( resize_mode == GR_RESIZE_NONE || (!gr_screen.custom_size && (gr_screen.rendering_to_texture == -1)) ) {
 		return false;
 	}
 
 	float xy_tmp = 0.0f;
 
-	if ( x && (*x != 0) ) {
-		xy_tmp = (*x) * Gr_resize_X;
-		(*x) = fl2i(xy_tmp);
+	if ( x ) {
+		switch (resize_mode) {
+		case GR_RESIZE_FULL:
+			xy_tmp = (*x) * Gr_full_resize_X;
+			break;
+
+		case GR_RESIZE_MENU:
+			xy_tmp = (*x) * Gr_resize_X + Gr_menu_offset_X;
+			break;
+
+		case GR_RESIZE_MENU_ZOOMED:
+			xy_tmp = (*x) * Gr_resize_X + Gr_menu_zoomed_offset_X;
+			break;
+
+		case GR_RESIZE_MENU_NO_OFFSET:
+			xy_tmp = (*x) * Gr_resize_X;
+			break;
+		}
+		(*x) = fl2ir(xy_tmp);
 	}
 
-	if ( y && (*y != 0) ) {
-		xy_tmp = (*y) * Gr_resize_Y;
-		(*y) = fl2i(xy_tmp);
+	if ( y ) {
+		switch (resize_mode) {
+		case GR_RESIZE_FULL:
+			xy_tmp = (*y) * Gr_full_resize_Y;
+			break;
+
+		case GR_RESIZE_MENU:
+			xy_tmp = (*y) * Gr_resize_Y + Gr_menu_offset_Y;
+			break;
+
+		case GR_RESIZE_MENU_ZOOMED:
+			xy_tmp = (*y) * Gr_resize_Y + Gr_menu_zoomed_offset_Y;
+			break;
+
+		case GR_RESIZE_MENU_NO_OFFSET:
+			xy_tmp = (*y) * Gr_resize_Y;
+			break;
+		}
+		(*y) = fl2ir(xy_tmp);
+	}
+
+	if ( w ) {
+		xy_tmp = (*w) * ((resize_mode == GR_RESIZE_FULL) ? Gr_full_resize_X : Gr_resize_X);
+		(*w) = fl2ir(xy_tmp);
+	}
+
+	if ( h ) {
+		xy_tmp = (*h) * ((resize_mode == GR_RESIZE_FULL) ? Gr_full_resize_Y : Gr_resize_Y);
+		(*h) = fl2ir(xy_tmp);
 	}
 
 	return true;
@@ -162,26 +269,71 @@ bool gr_resize_screen_pos(int *x, int *y)
 
 /**
  *
- * @param x X value (width to be unscaled), can be NULL
- * @param y Y value (height to be unscaled), can be NULL
+ * @param x X value, can be NULL
+ * @param y Y value, can be NULL
+ * @param w width, can be NULL
+ * @param h height, can be NULL
+ * @param resize_mode
  * @return always true unless error
  */
-bool gr_unsize_screen_pos(int *x, int *y)
+bool gr_unsize_screen_pos(int *x, int *y, int *w, int *h, int resize_mode)
 {
-	if ( !gr_screen.custom_size && (gr_screen.rendering_to_texture == -1) ) {
+	if ( resize_mode == GR_RESIZE_NONE || (!gr_screen.custom_size && (gr_screen.rendering_to_texture == -1)) ) {
 		return false;
 	}
 
 	float xy_tmp = 0.0f;
 
-	if ( x && (*x != 0) ) {
-		xy_tmp = (*x) * Gr_unsize_X;
-		(*x) = fl2i(xy_tmp);
+	if ( x ) {
+		switch (resize_mode) {
+		case GR_RESIZE_FULL:
+			xy_tmp = (*x) / Gr_full_resize_X;
+			break;
+
+		case GR_RESIZE_MENU:
+			xy_tmp = ((*x) - Gr_menu_offset_X) / Gr_resize_X;
+			break;
+
+		case GR_RESIZE_MENU_ZOOMED:
+			xy_tmp = ((*x) - Gr_menu_zoomed_offset_X) / Gr_resize_X;
+			break;
+
+		case GR_RESIZE_MENU_NO_OFFSET:
+			xy_tmp = (*x) / Gr_resize_X;
+			break;
+		}
+		(*x) = fl2ir(xy_tmp);
 	}
 
-	if ( y && (*y != 0) ) {
-		xy_tmp = (*y) * Gr_unsize_Y;
-		(*y) = fl2i(xy_tmp);
+	if ( y ) {
+		switch (resize_mode) {
+		case GR_RESIZE_FULL:
+			xy_tmp = (*y) / Gr_full_resize_Y;
+			break;
+
+		case GR_RESIZE_MENU:
+			xy_tmp = ((*y) - Gr_menu_offset_Y) / Gr_resize_Y;
+			break;
+
+		case GR_RESIZE_MENU_ZOOMED:
+			xy_tmp = ((*y) - Gr_menu_zoomed_offset_Y) / Gr_resize_Y;
+			break;
+
+		case GR_RESIZE_MENU_NO_OFFSET:
+			xy_tmp = (*y) / Gr_resize_Y;
+			break;
+		}
+		(*y) = fl2ir(xy_tmp);
+	}
+
+	if ( w ) {
+		xy_tmp = (*w) / ((resize_mode == GR_RESIZE_FULL) ? Gr_full_resize_X : Gr_resize_X);
+		(*w) = fl2ir(xy_tmp);
+	}
+
+	if ( h ) {
+		xy_tmp = (*h) / ((resize_mode == GR_RESIZE_FULL) ? Gr_full_resize_Y : Gr_resize_Y);
+		(*h) = fl2ir(xy_tmp);
 	}
 
 	return true;
@@ -191,42 +343,144 @@ bool gr_unsize_screen_pos(int *x, int *y)
  * This function is to be called if you wish to scale GR_1024 or GR_640 x and y positions or
  * lengths in order to keep the correctly scaled to nonstandard resolutions
  *
- * @param x X value (width to be scaled), can be NULL
- * @param y Y value (height to be scaled), can be NULL
+ * @param x X value, can be NULL
+ * @param y Y value, can be NULL
+ * @param w width, can be NULL
+ * @param h height, can be NULL
+ * @param resize_mode
  * @return always true unless error
  */
-bool gr_resize_screen_posf(float *x, float *y)
+bool gr_resize_screen_posf(float *x, float *y, float *w, float *h, int resize_mode)
 {
-	if ( !gr_screen.custom_size && (gr_screen.rendering_to_texture == -1) ) {
+	if ( resize_mode == GR_RESIZE_NONE || (!gr_screen.custom_size && (gr_screen.rendering_to_texture == -1)) ) {
 		return false;
 	}
 
-	if ( x && (*x != 0) )
-		(*x) *= Gr_resize_X;
+	float xy_tmp = 0.0f;
 
-	if ( y && (*y != 0) )
-		(*y) *= Gr_resize_Y;
+	if ( x ) {
+		switch (resize_mode) {
+		case GR_RESIZE_FULL:
+			xy_tmp = (*x) * Gr_full_resize_X;
+			break;
+
+		case GR_RESIZE_MENU:
+			xy_tmp = (*x) * Gr_resize_X + Gr_menu_offset_X;
+			break;
+
+		case GR_RESIZE_MENU_ZOOMED:
+			xy_tmp = (*x) * Gr_resize_X + Gr_menu_zoomed_offset_X;
+			break;
+
+		case GR_RESIZE_MENU_NO_OFFSET:
+			xy_tmp = (*x) * Gr_resize_X;
+			break;
+		}
+		(*x) = xy_tmp;
+	}
+
+	if ( y ) {
+		switch (resize_mode) {
+		case GR_RESIZE_FULL:
+			xy_tmp = (*y) * Gr_full_resize_Y;
+			break;
+
+		case GR_RESIZE_MENU:
+			xy_tmp = (*y) * Gr_resize_Y + Gr_menu_offset_Y;
+			break;
+
+		case GR_RESIZE_MENU_ZOOMED:
+			xy_tmp = (*y) * Gr_resize_Y + Gr_menu_zoomed_offset_Y;
+			break;
+
+		case GR_RESIZE_MENU_NO_OFFSET:
+			xy_tmp = (*y) * Gr_resize_Y;
+			break;
+		}
+		(*y) = xy_tmp;
+	}
+
+	if ( w ) {
+		xy_tmp = (*w) * ((resize_mode == GR_RESIZE_FULL) ? Gr_full_resize_X : Gr_resize_X);
+		(*w) = xy_tmp;
+	}
+
+	if ( h ) {
+		xy_tmp = (*h) * ((resize_mode == GR_RESIZE_FULL) ? Gr_full_resize_Y : Gr_resize_Y);
+		(*h) = xy_tmp;
+	}
 
 	return true;
 }
 
 /**
  *
- * @param x X value (width to be unscaled), can be NULL
- * @param y Y value (height to be unscaled), can be NULL
+ * @param x X value, can be NULL
+ * @param y Y value, can be NULL
+ * @param w width, can be NULL
+ * @param h height, can be NULL
+ * @param resize_mode
  * @return always true unless error
  */
-bool gr_unsize_screen_posf(float *x, float *y)
+bool gr_unsize_screen_posf(float *x, float *y, float *w, float *h, int resize_mode)
 {
-	if ( !gr_screen.custom_size && (gr_screen.rendering_to_texture == -1) ) {
+	if ( resize_mode == GR_RESIZE_NONE || (!gr_screen.custom_size && (gr_screen.rendering_to_texture == -1)) ) {
 		return false;
 	}
 
-	if ( x && (*x != 0) )
-		(*x) *= Gr_unsize_X;
+	float xy_tmp = 0.0f;
 
-	if ( y && (*y != 0) )
-		(*y) *= Gr_unsize_Y;
+	if ( x ) {
+		switch (resize_mode) {
+		case GR_RESIZE_FULL:
+			xy_tmp = (*x) / Gr_full_resize_X;
+			break;
+
+		case GR_RESIZE_MENU:
+			xy_tmp = ((*x) - Gr_menu_offset_X) / Gr_resize_X;
+			break;
+
+		case GR_RESIZE_MENU_ZOOMED:
+			xy_tmp = ((*x) - Gr_menu_zoomed_offset_X) / Gr_resize_X;
+			break;
+
+		case GR_RESIZE_MENU_NO_OFFSET:
+			xy_tmp = (*x) / Gr_resize_X;
+			break;
+		}
+		(*x) = xy_tmp;
+	}
+
+	if ( y ) {
+		switch (resize_mode) {
+		case GR_RESIZE_FULL:
+			xy_tmp = (*y) / Gr_full_resize_Y;
+			break;
+
+		case GR_RESIZE_MENU:
+			xy_tmp = ((*y) - Gr_menu_offset_Y) / Gr_resize_Y;
+			break;
+
+		case GR_RESIZE_MENU_ZOOMED:
+			xy_tmp = ((*y) - Gr_menu_zoomed_offset_Y) / Gr_resize_Y;
+			break;
+
+		case GR_RESIZE_MENU_NO_OFFSET:
+			xy_tmp = (*y) / Gr_resize_Y;
+			break;
+		}
+		(*y) = xy_tmp;
+	}
+
+	if ( w ) {
+		xy_tmp = (*w) / ((resize_mode == GR_RESIZE_FULL) ? Gr_full_resize_X : Gr_resize_X);
+		(*w) = xy_tmp;
+	}
+
+	if ( h ) {
+		xy_tmp = (*h) / ((resize_mode == GR_RESIZE_FULL) ? Gr_full_resize_Y : Gr_resize_Y);
+		(*h) = xy_tmp;
+	}
 
 	return true;
 }
@@ -264,18 +518,15 @@ DCF(clear_color, "set clear color r, g, b")
 {
 	ubyte r, g, b;
 
-	dc_get_arg(ARG_UBYTE);
-	r = Dc_arg_ubyte;
-	dc_get_arg(ARG_UBYTE);
-	g = Dc_arg_ubyte;
-	dc_get_arg(ARG_UBYTE);
-	b = Dc_arg_ubyte;
+	dc_stuff_ubyte(&r);
+	dc_stuff_ubyte(&g);
+	dc_stuff_ubyte(&b);
 
 	// set the color
 	gr_set_clear_color(r, g, b);
 }
 
-void gr_set_palette_internal( char *name, ubyte * palette, int restrict_font_to_128 )
+void gr_set_palette_internal( const char *name, ubyte * palette, int restrict_font_to_128 )
 {
 	if ( palette == NULL ) {
 		// Create a default palette
@@ -319,7 +570,7 @@ void gr_set_palette_internal( char *name, ubyte * palette, int restrict_font_to_
 }
 
 
-void gr_set_palette( char *name, ubyte * palette, int restrict_font_to_128 )
+void gr_set_palette( const char *name, ubyte * palette, int restrict_font_to_128 )
 {
 	char *p;
 	palette_flush();
@@ -338,8 +589,8 @@ void gr_screen_resize(int width, int height)
 		return;
 	}
 
-	gr_screen.save_max_w = gr_screen.max_w = gr_screen.max_w_unscaled = width;
-	gr_screen.save_max_h = gr_screen.max_h = gr_screen.max_h_unscaled = height;
+	gr_screen.save_max_w = gr_screen.max_w = gr_screen.max_w_unscaled = gr_screen.max_w_unscaled_zoomed = width;
+	gr_screen.save_max_h = gr_screen.max_h = gr_screen.max_h_unscaled = gr_screen.max_h_unscaled_zoomed = height;
 
 	gr_screen.offset_x = gr_screen.offset_x_unscaled = 0;
 	gr_screen.offset_y = gr_screen.offset_y_unscaled = 0;
@@ -354,12 +605,15 @@ void gr_screen_resize(int width, int height)
 
 	if (gr_screen.custom_size) {
 		gr_unsize_screen_pos( &gr_screen.max_w_unscaled, &gr_screen.max_h_unscaled );
+		gr_unsize_screen_pos( &gr_screen.max_w_unscaled_zoomed, &gr_screen.max_h_unscaled_zoomed );
 		gr_unsize_screen_pos( &gr_screen.clip_right_unscaled, &gr_screen.clip_bottom_unscaled );
 		gr_unsize_screen_pos( &gr_screen.clip_width_unscaled, &gr_screen.clip_height_unscaled );
 	}
 
 	gr_screen.save_max_w_unscaled = gr_screen.max_w_unscaled;
 	gr_screen.save_max_h_unscaled = gr_screen.max_h_unscaled;
+	gr_screen.save_max_w_unscaled_zoomed = gr_screen.max_w_unscaled_zoomed;
+	gr_screen.save_max_h_unscaled_zoomed = gr_screen.max_h_unscaled_zoomed;
 
 	if (gr_screen.mode == GR_OPENGL) {
 		extern void opengl_setup_viewport();
@@ -380,7 +634,7 @@ static bool gr_init_sub(int mode, int width, int height, int depth)
 		gr_screen.custom_size = true;
 	}
 
-	if ( (width >= 1024) && (height >= 600) ) {
+	if ( (width >= GR_1024_THRESHOLD_WIDTH) && (height >= GR_1024_THRESHOLD_HEIGHT) ) {
 		res = GR_1024;
 	} else {
 		res = GR_640;
@@ -392,12 +646,30 @@ static bool gr_init_sub(int mode, int width, int height, int depth)
 		mode = GR_OPENGL;
 	}
 
-	Gr_save_resize_X = Gr_resize_X = (float)width / ((res == GR_1024) ? 1024.0f : 640.0f);
-	Gr_save_resize_Y = Gr_resize_Y = (float)height / ((res == GR_1024) ?  768.0f : 480.0f);
+	Save_custom_screen_size = gr_screen.custom_size;
 
-	Gr_save_unsize_X = Gr_unsize_X = ((res == GR_1024) ? 1024.0f : 640.0f) / (float)width;
-	Gr_save_unsize_Y = Gr_unsize_Y = ((res == GR_1024) ?  768.0f : 480.0f) / (float)height;
+	Gr_save_full_resize_X = Gr_full_resize_X = (float)width / ((res == GR_1024) ? 1024.0f : 640.0f);
+	Gr_save_full_resize_Y = Gr_full_resize_Y = (float)height / ((res == GR_1024) ?  768.0f : 480.0f);
 
+	if (gr_screen.custom_size && !Cmdline_stretch_menu) {
+		float aspect_quotient = ((float)width / (float)height) / (4.0f / 3.0f);
+
+		Gr_save_resize_X = Gr_resize_X = Gr_full_resize_X / ((aspect_quotient > 1.0f) ? aspect_quotient : 1.0f);
+		Gr_save_resize_Y = Gr_resize_Y = Gr_full_resize_Y * ((aspect_quotient < 1.0f) ? aspect_quotient : 1.0f);
+
+		Gr_save_menu_offset_X = Gr_menu_offset_X = (aspect_quotient > 1.0f) ? ((width - width / aspect_quotient) / 2.0f) : 0.0f;
+		Gr_save_menu_offset_Y = Gr_menu_offset_Y = (aspect_quotient < 1.0f) ? ((height - height * aspect_quotient) / 2.0f) : 0.0f;
+	} else {
+		Gr_save_resize_X = Gr_resize_X = Gr_full_resize_X;
+		Gr_save_resize_Y = Gr_resize_Y = Gr_full_resize_Y;
+
+		Gr_save_menu_offset_X = Gr_menu_offset_X = 0.0f;
+		Gr_save_menu_offset_Y = Gr_menu_offset_Y = 0.0f;
+	}
+
+	Gr_save_menu_zoomed_offset_X = Gr_menu_zoomed_offset_X = Gr_menu_offset_X;
+	Gr_save_menu_zoomed_offset_Y = Gr_menu_zoomed_offset_Y = Gr_menu_offset_Y;
+	
 
 	gr_screen.signature = Gr_signature++;
 	gr_screen.bits_per_pixel = depth;
@@ -409,8 +681,8 @@ static bool gr_init_sub(int mode, int width, int height, int depth)
 	gr_screen.res = res;
 	gr_screen.aspect = 1.0f;			// Normal PC screen
 
-	gr_screen.save_max_w = gr_screen.max_w = gr_screen.max_w_unscaled = width;
-	gr_screen.save_max_h = gr_screen.max_h = gr_screen.max_h_unscaled = height;
+	gr_screen.save_max_w = gr_screen.max_w = gr_screen.max_w_unscaled = gr_screen.max_w_unscaled_zoomed = width;
+	gr_screen.save_max_h = gr_screen.max_h = gr_screen.max_h_unscaled = gr_screen.max_h_unscaled_zoomed = height;
 
 	gr_screen.offset_x = gr_screen.offset_x_unscaled = 0;
 	gr_screen.offset_y = gr_screen.offset_y_unscaled = 0;
@@ -427,12 +699,15 @@ static bool gr_init_sub(int mode, int width, int height, int depth)
 
 	if (gr_screen.custom_size) {
 		gr_unsize_screen_pos( &gr_screen.max_w_unscaled, &gr_screen.max_h_unscaled );
+		gr_unsize_screen_pos( &gr_screen.max_w_unscaled_zoomed, &gr_screen.max_h_unscaled_zoomed );
 		gr_unsize_screen_pos( &gr_screen.clip_right_unscaled, &gr_screen.clip_bottom_unscaled );
 		gr_unsize_screen_pos( &gr_screen.clip_width_unscaled, &gr_screen.clip_height_unscaled );
 	}
 
 	gr_screen.save_max_w_unscaled = gr_screen.max_w_unscaled;
 	gr_screen.save_max_h_unscaled = gr_screen.max_h_unscaled;
+	gr_screen.save_max_w_unscaled_zoomed = gr_screen.max_w_unscaled_zoomed;
+	gr_screen.save_max_h_unscaled_zoomed = gr_screen.max_h_unscaled_zoomed;
 
 #ifdef WIN32
 	// FRED doesn't need this
@@ -463,9 +738,9 @@ static bool gr_init_sub(int mode, int width, int height, int depth)
 
 bool gr_init(int d_mode, int d_width, int d_height, int d_depth)
 {
-	int width = 1024, height = 768, depth = 16, mode = GR_OPENGL;
-	char *ptr = NULL;
-	char *Default_video_settings = "OGL -(1024x768)x16 bit";
+	int width = 1024, height = 768, depth = 32, mode = GR_OPENGL;
+	const char *ptr = NULL;
+	const char *Default_video_settings = "OGL -(1024x768)x32 bit";
 
 	if ( !Gr_inited ) {
 		atexit(gr_close);
@@ -489,7 +764,7 @@ bool gr_init(int d_mode, int d_width, int d_height, int d_depth)
 	// We cannot continue without this, quit, but try to help the user out first
 	ptr = os_config_read_string(NULL, NOX("VideocardFs2open"), NULL);
 
-	// if we don't have a config string then construct one, using OpenGL 1024x768 16-bit as the default
+	// if we don't have a config string then construct one, using OpenGL 1024x768 32-bit as the default
 	if (ptr == NULL) {
 		ptr = Default_video_settings;
 	}
@@ -696,6 +971,7 @@ void gr_init_color(color *c, int r, int g, int b)
 	c->alphacolor = -1;
 	c->is_alphacolor = 0;
 	c->magic = 0xAC01;
+	c->raw8 = 0;
 }
 
 void gr_init_alphacolor( color *clr, int r, int g, int b, int alpha, int type )
@@ -828,7 +1104,7 @@ int gr_get_cursor_bitmap()
 }
 
 // new bitmap functions
-void gr_bitmap(int _x, int _y, bool allow_scaling)
+void gr_bitmap(int _x, int _y, int resize_mode)
 {
 	int _w, _h;
 	float x, y, w, h;
@@ -846,9 +1122,8 @@ void gr_bitmap(int _x, int _y, bool allow_scaling)
 	h = i2fl(_h);
 
 	// I will tidy this up later - RT
-	if ( allow_scaling && (gr_screen.custom_size || (gr_screen.rendering_to_texture != -1)) ) {
-		gr_resize_screen_posf(&x, &y);
-		gr_resize_screen_posf(&w, &h);
+	if ( resize_mode != GR_RESIZE_NONE && (gr_screen.custom_size || (gr_screen.rendering_to_texture != -1)) ) {
+		gr_resize_screen_posf(&x, &y, &w, &h, resize_mode);
 	}
 
 	memset(verts, 0, sizeof(verts));
@@ -882,7 +1157,7 @@ void gr_bitmap(int _x, int _y, bool allow_scaling)
 	gr_zbuffer_set(saved_zbuffer_mode);
 }
 
-void gr_bitmap_uv(int _x, int _y, int _w, int _h, float _u0, float _v0, float _u1, float _v1, bool allow_scaling)
+void gr_bitmap_uv(int _x, int _y, int _w, int _h, float _u0, float _v0, float _u1, float _v1, int resize_mode)
 {
 	float x, y, w, h;
 	vertex verts[4];
@@ -897,9 +1172,8 @@ void gr_bitmap_uv(int _x, int _y, int _w, int _h, float _u0, float _v0, float _u
 	h = i2fl(_h);
 
 	// I will tidy this up later - RT
-	if ( allow_scaling && (gr_screen.custom_size || (gr_screen.rendering_to_texture != -1)) ) {
-		gr_resize_screen_posf(&x, &y);
-		gr_resize_screen_posf(&w, &h);
+	if ( resize_mode != GR_RESIZE_NONE && (gr_screen.custom_size || (gr_screen.rendering_to_texture != -1)) ) {
+		gr_resize_screen_posf(&x, &y, &w, &h, resize_mode);
 	}
 
 	memset(verts, 0, sizeof(verts));
@@ -934,16 +1208,15 @@ void gr_bitmap_uv(int _x, int _y, int _w, int _h, float _u0, float _v0, float _u
 }
 
 // NEW new bitmap functions -Bobboau
-void gr_bitmap_list(bitmap_2d_list* list, int n_bm, bool allow_scaling)
+void gr_bitmap_list(bitmap_2d_list* list, int n_bm, int resize_mode)
 {
 	for (int i = 0; i < n_bm; i++) {
 		bitmap_2d_list *l = &list[i];
 
 		bm_get_info(gr_screen.current_bitmap, &l->w, &l->h, NULL, NULL, NULL);
 
-		if ( allow_scaling && (gr_screen.custom_size || (gr_screen.rendering_to_texture != -1)) ) {
-			gr_resize_screen_pos(&l->x, &l->y);
-			gr_resize_screen_pos(&l->w, &l->h);
+		if ( resize_mode != GR_RESIZE_NONE && (gr_screen.custom_size || (gr_screen.rendering_to_texture != -1)) ) {
+			gr_resize_screen_pos(&l->x, &l->y, &l->w, &l->h, resize_mode);
 		}
 	}
 
@@ -952,7 +1225,7 @@ void gr_bitmap_list(bitmap_2d_list* list, int n_bm, bool allow_scaling)
 
 // _->NEW<-_ NEW new bitmap functions -Bobboau
 //takes a list of rectangles that have assosiated rectangles in a texture
-void gr_bitmap_list(bitmap_rect_list* list, int n_bm, bool allow_scaling)
+void gr_bitmap_list(bitmap_rect_list* list, int n_bm, int resize_mode)
 {
 	for(int i = 0; i < n_bm; i++) {
 		bitmap_2d_list *l = &list[i].screen_rect;
@@ -962,9 +1235,8 @@ void gr_bitmap_list(bitmap_rect_list* list, int n_bm, bool allow_scaling)
 			bm_get_info(gr_screen.current_bitmap, &l->w, &l->h, NULL, NULL, NULL);
 		}
 
-		if ( allow_scaling && (gr_screen.custom_size || (gr_screen.rendering_to_texture != -1)) ) {
-			gr_resize_screen_pos(&l->x, &l->y);
-			gr_resize_screen_pos(&l->w, &l->h);
+		if ( resize_mode != GR_RESIZE_NONE && (gr_screen.custom_size || (gr_screen.rendering_to_texture != -1)) ) {
+			gr_resize_screen_pos(&l->x, &l->y, &l->w, &l->h, resize_mode);
 		}
 	}
 
@@ -1000,7 +1272,7 @@ void gr_pline_helper(vec3d *out, vec3d *in1, vec3d *in2, int thickness)
  * is no more than 90 degrees away from a previous section.
  * Moreover, it is _really_ intended for use with 45 degree angles. 
  */
-void gr_pline_special(vec3d **pts, int num_pts, int thickness,bool resize)
+void gr_pline_special(SCP_vector<vec3d> *pts, int thickness,int resize_mode)
 {
 	vec3d s1, s2, e1, e2, dir;
 	vec3d last_e1, last_e2;
@@ -1009,6 +1281,8 @@ void gr_pline_special(vec3d **pts, int num_pts, int thickness,bool resize)
 	int saved_zbuffer_mode, idx;
 	int started_frame = 0;
 
+	int num_pts = pts->size();
+
 	// if we have less than 2 pts, bail
 	if(num_pts < 2) {
 		return;
@@ -1033,11 +1307,11 @@ void gr_pline_special(vec3d **pts, int num_pts, int thickness,bool resize)
 	int j;
 	for(idx=0; idx<num_pts-1; idx++) {
 		// get the start and endpoints
-		s1 = *pts[idx];											// start 1 (on the line)
-		gr_pline_helper(&s2, pts[idx], pts[idx+1], thickness);	// start 2
-		e1 = *pts[idx+1];										// end 1 (on the line)
-		vm_vec_sub(&dir, pts[idx+1], pts[idx]);
-		vm_vec_add(&e2, &s2, &dir);								// end 2
+		s1 = pts->at(idx);													// start 1 (on the line)
+		e1 = pts->at(idx+1);												// end 1 (on the line)
+		gr_pline_helper(&s2, &s1, &e1, thickness);	// start 2
+		vm_vec_sub(&dir, &e1, &s1);
+		vm_vec_add(&e2, &s2, &dir);											// end 2
 		
 		// stuff coords
 		v[0].screen.xyw.x = (float)ceil(s1.xyz.x);
@@ -1085,9 +1359,9 @@ void gr_pline_special(vec3d **pts, int num_pts, int thickness,bool resize)
 		v[3].b = gr_screen.current_color.blue;
 
 		//We could really do this better...but oh well. _WMC
-		if(resize) {
+		if(resize_mode != GR_RESIZE_NONE) {
 			for(j=0;j<4;j++) {
-				gr_resize_screen_posf(&v[j].screen.xyw.x,&v[j].screen.xyw.y);
+				gr_resize_screen_posf(&v[j].screen.xyw.x,&v[j].screen.xyw.y,NULL,NULL,resize_mode);
 			}
 		}
 
@@ -1132,9 +1406,9 @@ void gr_pline_special(vec3d **pts, int num_pts, int thickness,bool resize)
 			v[2].b = gr_screen.current_color.blue;
 
 			//Inefficiency or flexibility? you be the judge -WMC
-			if(resize) {
+			if(resize_mode != GR_RESIZE_NONE) {
 				for(j=0;j<3;j++) {
-					gr_resize_screen_posf(&v[j].screen.xyw.x,&v[j].screen.xyw.y);
+					gr_resize_screen_posf(&v[j].screen.xyw.x,&v[j].screen.xyw.y,NULL,NULL,resize_mode);
 				}
 			}
 
@@ -1427,19 +1701,19 @@ poly_list& poly_list::operator = (poly_list &other_list)
 	return *this;
 }
 
-void gr_shield_icon(coord2d coords[6], bool resize)
+void gr_shield_icon(coord2d coords[6], int resize_mode)
 {
 	if (gr_screen.mode == GR_STUB) {
 		return;
 	}
 
-	if (resize) {
-		gr_resize_screen_pos(&coords[0].x, &coords[0].y);
-		gr_resize_screen_pos(&coords[1].x, &coords[1].y);
-		gr_resize_screen_pos(&coords[2].x, &coords[2].y);
-		gr_resize_screen_pos(&coords[3].x, &coords[3].y);
-		gr_resize_screen_pos(&coords[4].x, &coords[4].y);
-		gr_resize_screen_pos(&coords[5].x, &coords[5].y);
+	if (resize_mode != GR_RESIZE_NONE) {
+		gr_resize_screen_pos(&coords[0].x, &coords[0].y, NULL, NULL, resize_mode);
+		gr_resize_screen_pos(&coords[1].x, &coords[1].y, NULL, NULL, resize_mode);
+		gr_resize_screen_pos(&coords[2].x, &coords[2].y, NULL, NULL, resize_mode);
+		gr_resize_screen_pos(&coords[3].x, &coords[3].y, NULL, NULL, resize_mode);
+		gr_resize_screen_pos(&coords[4].x, &coords[4].y, NULL, NULL, resize_mode);
+		gr_resize_screen_pos(&coords[5].x, &coords[5].y, NULL, NULL, resize_mode);
 	}
 
 	g3_draw_2d_shield_icon(coords,
@@ -1449,15 +1723,14 @@ void gr_shield_icon(coord2d coords[6], bool resize)
 		gr_screen.current_color.alpha);
 }
 
-void gr_rect(int x, int y, int w, int h, bool resize)
+void gr_rect(int x, int y, int w, int h, int resize_mode)
 {
 	if (gr_screen.mode == GR_STUB) {
 		return;
 	}
 
-	if (resize) {
-		gr_resize_screen_pos(&x, &y);
-		gr_resize_screen_pos(&w, &h);
+	if (resize_mode != GR_RESIZE_NONE) {
+		gr_resize_screen_pos(&x, &y, &w, &h, resize_mode);
 	}
 
 	g3_draw_2d_rect(x, y, w, h,
@@ -1467,7 +1740,7 @@ void gr_rect(int x, int y, int w, int h, bool resize)
 		gr_screen.current_color.alpha);
 }
 
-void gr_shade(int x, int y, int w, int h, bool resize)
+void gr_shade(int x, int y, int w, int h, int resize_mode)
 {
 	int r, g, b, a;
 
@@ -1475,9 +1748,8 @@ void gr_shade(int x, int y, int w, int h, bool resize)
 		return;
 	}
 
-	if (resize) {
-		gr_resize_screen_pos(&x, &y);
-		gr_resize_screen_pos(&w, &h);
+	if (resize_mode != GR_RESIZE_NONE) {
+		gr_resize_screen_pos(&x, &y, &w, &h, resize_mode);
 	}
 
 	r = (int)gr_screen.current_shader.r;
