@@ -100,7 +100,9 @@ typedef struct tsb_t {
  * This should be basicly just like it is in the VB
  * a list of triangles and their associated normals
  */
-struct poly_list {
+class poly_list
+{
+public:
 	poly_list(): n_verts(0), vert(NULL), norm(NULL), tsb(NULL), currently_allocated(0) {}
 	~poly_list();
 	poly_list& operator = (poly_list&);
@@ -121,8 +123,13 @@ private:
 };
 
 
-struct colored_vector {
-	colored_vector(): pad(1.0f) {}
+class colored_vector
+{
+public:
+	colored_vector()
+		: pad(1.0f)
+	{}
+
 	vec3d vec;
 	float pad;	//needed so I can just memcpy it in d3d
 	ubyte color[4];
@@ -134,7 +141,9 @@ struct line_list {
 	vertex *vert;
 };
 
-struct buffer_data {
+class buffer_data
+{
+public:
 	int flags;
 
 	int texture;		// this is the texture the vertex buffer will use
@@ -168,18 +177,71 @@ struct buffer_data {
 			i_last = j;
 	}
 
+	// Constructor
 	buffer_data(int n_vrts) :
 		flags(0), texture(-1), n_verts(n_vrts), index_offset(0),
 		i_first(1), i_last(0)
 	{
 		index = new(std::nothrow) uint[n_verts];
 	}
+    
+	// Copy-constructor
+	buffer_data(const buffer_data& other)
+	{
+		index = new(std::nothrow) uint[other.n_verts];
+		for (size_t i=0; i < (size_t) other.n_verts; i++)
+		{
+			index[i] = other.index[i];
+		}
+        
+		flags   = other.flags;
+		texture = other.texture;
+		n_verts = other.n_verts;
+
+		i_first = other.i_first;
+		i_last  = other.i_last;
+        
+		index_offset = other.index_offset;
+	}
+    
+	// Copy-assignment operator
+	buffer_data& operator=(const buffer_data& rhs)
+	{
+		if (this != &rhs)
+		{
+			delete [] index;
+            
+			index = new(std::nothrow) uint[rhs.n_verts];
+			for (size_t i=0; i < (size_t) rhs.n_verts; i++)
+			{
+				index[i] = rhs.index[i];
+			}
+            
+			flags   = rhs.flags;
+			texture = rhs.texture;
+			n_verts = rhs.n_verts;
+            
+			i_first = rhs.i_first;
+			i_last  = rhs.i_last;
+            
+			index_offset = rhs.index_offset;
+		}
+		return *this;
+	}
+    
+	// Destructor
+	~buffer_data()
+	{
+		release();
+	}
 
 private:
 	uint *index;
 };
 
-struct vertex_buffer {
+class vertex_buffer
+{
+public:
 	int flags;
 
 	uint stride;
@@ -234,9 +296,11 @@ struct light;
 typedef struct screen {
 	uint	signature;			// changes when mode or palette or width or height changes
 	int	max_w, max_h;		// Width and height
-	int max_w_unscaled, max_h_unscaled;		// Width and height, should be 1024x768 or 640x480 in non-standard resolutions
+	int max_w_unscaled, max_h_unscaled;
+	int max_w_unscaled_zoomed, max_h_unscaled_zoomed;
 	int	save_max_w, save_max_h;		// Width and height
 	int save_max_w_unscaled, save_max_h_unscaled;
+	int save_max_w_unscaled_zoomed, save_max_h_unscaled_zoomed;
 	int	res;					// GR_640 or GR_1024
 	int	mode;					// What mode gr_init was called with.
 	float	aspect, clip_aspect;				// Aspect ratio, aspect of clip_width/clip_height
@@ -244,9 +308,9 @@ typedef struct screen {
 	int	bits_per_pixel;	// How many bits per pixel it is. (7,8,15,16,24,32)
 	int	bytes_per_pixel;	// How many bytes per pixel (1,2,3,4)
 	int	offset_x, offset_y;		// The offsets into the screen
-	int offset_x_unscaled, offset_y_unscaled;	// Offsets into the screen, in 1024x768 or 640x480 dimensions
+	int offset_x_unscaled, offset_y_unscaled;	// Offsets into the screen, in unscaled dimensions
 	int	clip_width, clip_height;
-	int clip_width_unscaled, clip_height_unscaled;	// Height and width of clip aread, in 1024x768 or 640x480 dimensions
+	int clip_width_unscaled, clip_height_unscaled;	// Height and width of clip aread, in unscaled dimensions
 	// center of clip area
 	float	clip_center_x, clip_center_y;
 
@@ -256,7 +320,7 @@ typedef struct screen {
 	// actually always 0, but it's nice to have the code work with
 	// arbitrary clipping regions.
 	int		clip_left, clip_right, clip_top, clip_bottom;
-	// same as above except in 1024x768 or 640x480 dimensions
+	// same as above except in unscaled dimensions
 	int		clip_left_unscaled, clip_right_unscaled, clip_top_unscaled, clip_bottom_unscaled;
 
 	int		current_alphablend_mode;		// See GR_ALPHABLEND defines above
@@ -297,7 +361,7 @@ typedef struct screen {
 	void (*gf_flash_alpha)(int r, int g, int b, int a);
 
 	// sets the clipping region
-	void (*gf_set_clip)(int x, int y, int w, int h, bool resize);
+	void (*gf_set_clip)(int x, int y, int w, int h, int resize_mode);
 
 	// resets the clipping region to entire screen
 	void (*gf_reset_clip)();
@@ -305,22 +369,24 @@ typedef struct screen {
 	// clears entire clipping region to current color
 	void (*gf_clear)();
 
-	// void (*gf_bitmap)(int x, int y, bool resize);
-	void (*gf_bitmap_ex)(int x, int y, int w, int h, int sx, int sy, bool resize);
+	// void (*gf_bitmap)(int x, int y, int resize_mode);
+	void (*gf_bitmap_ex)(int x, int y, int w, int h, int sx, int sy, int resize_mode);
 
-	void (*gf_aabitmap)(int x, int y, bool resize, bool mirror);
-	void (*gf_aabitmap_ex)(int x, int y, int w, int h, int sx, int sy, bool resize, bool mirror);
+	void (*gf_aabitmap)(int x, int y, int resize_mode, bool mirror);
+	void (*gf_aabitmap_ex)(int x, int y, int w, int h, int sx, int sy, int resize_mode, bool mirror);
 
-	void (*gf_string)(int x, int y, const char * text,bool resize);
+	void (*gf_string)(int x, int y, const char * text,int resize_mode);
 
 	// Draw a gradient line... x1,y1 is bright, x2,y2 is transparent.
-	void (*gf_gradient)(int x1, int y1, int x2, int y2, bool resize);
+	void (*gf_gradient)(int x1, int y1, int x2, int y2, int resize_mode);
  
-	void (*gf_circle)(int x, int y, int r, bool resize);
-	void (*gf_curve)(int x, int y, int r, int direction);
+	void (*gf_circle)(int x, int y, int r, int resize_mode);
+	void (*gf_unfilled_circle)(int x, int y, int r, int resize_mode);
+	void (*gf_arc)(int x, int y, float r, float angle_start, float angle_end, bool fill, int resize_mode);
+	void (*gf_curve)(int x, int y, int r, int direction, int resize_mode);
 
 	// Integer line. Used to draw a fast but pixely line.  
-	void (*gf_line)(int x1, int y1, int x2, int y2, bool resize);
+	void (*gf_line)(int x1, int y1, int x2, int y2, int resize_mode);
 
 	// Draws an antialiased line is the current color is an 
 	// alphacolor, otherwise just draws a fast line.  This
@@ -329,7 +395,7 @@ typedef struct screen {
 	// not this if you have two 3d points.
 	void (*gf_aaline)(vertex *v1, vertex *v2);
 
-	void (*gf_pixel)( int x, int y, bool resize );
+	void (*gf_pixel)( int x, int y, int resize_mode );
 
 	// Scales current bitmap between va and vb with clipping
 	void (*gf_scaler)(vertex *va, vertex *vb, bool bw_bitmap );
@@ -346,7 +412,7 @@ typedef struct screen {
 	void (*gf_render_effect)(int nv, vertex *verts, float *radius_list, uint flags);
 
 	// dumps the current screen to a file
-	void (*gf_print_screen)(char * filename);
+	void (*gf_print_screen)(const char * filename);
 
 	// Call once before rendering anything.
 	void (*gf_start_frame)();
@@ -411,7 +477,7 @@ typedef struct screen {
 	int (*gf_set_color_buffer)(int mode);
 
 	// cross fade
-	void (*gf_cross_fade)(int bmap1, int bmap2, int x1, int y1, int x2, int y2, float pct);
+	void (*gf_cross_fade)(int bmap1, int bmap2, int x1, int y1, int x2, int y2, float pct, int resize_mode);
 
 	// set a texture into cache. for sectioned bitmaps, pass in sx and sy to set that particular section of the bitmap
 	int (*gf_tcache_set)(int bitmap_id, int bitmap_type, float *u_scale, float *v_scale, int stage);	
@@ -425,10 +491,10 @@ typedef struct screen {
 	// Here be the bitmap functions
 	void (*gf_bm_free_data)(int n, bool release);
 	void (*gf_bm_create)(int n);
-	int (*gf_bm_load)(ubyte type, int n, char *filename, CFILE *img_cfp, int *w, int *h, int *bpp, ubyte *c_type, int *mm_lvl, int *size);
+	int (*gf_bm_load)(ubyte type, int n, const char *filename, CFILE *img_cfp, int *w, int *h, int *bpp, ubyte *c_type, int *mm_lvl, int *size);
 	void (*gf_bm_init)(int n);
 	void (*gf_bm_page_in_start)();
-	int (*gf_bm_lock)(char *filename, int handle, int bitmapnum, ubyte bpp, ubyte flags, bool nodebug);
+	int (*gf_bm_lock)(const char *filename, int handle, int bitmapnum, ubyte bpp, ubyte flags, bool nodebug);
 
 	int (*gf_bm_make_render_target)(int n, int *width, int *height, ubyte *bpp, int *mm_lvl, int flags );
 	int (*gf_bm_set_render_target)(int n, int face);
@@ -535,6 +601,11 @@ typedef struct screen {
 #define GR_640							0		// 640 x 480
 #define GR_1024						1		// 1024 x 768
 
+#define GR_1024_THRESHOLD_WIDTH		1024
+#define GR_1024_THRESHOLD_HEIGHT	600
+
+extern const char *Resolution_prefixes[GR_NUM_RESOLUTIONS];
+
 extern bool gr_init(int d_mode = GR_DEFAULT, int d_width = GR_DEFAULT, int d_height = GR_DEFAULT, int d_depth = GR_DEFAULT);
 extern void gr_screen_resize(int width, int height);
 
@@ -555,38 +626,42 @@ extern screen gr_screen;
 #define GR_STENCIL_READ		1
 #define GR_STENCIL_WRITE	2
 
-void gr_set_screen_scale(int x, int y);
-void gr_set_screen_scale(int x, int y, int max_x, int max_y);
+#define GR_RESIZE_NONE				0
+#define GR_RESIZE_FULL				1
+#define GR_RESIZE_MENU				2
+#define GR_RESIZE_MENU_ZOOMED		3
+#define GR_RESIZE_MENU_NO_OFFSET	4
+
+void gr_set_screen_scale(int x, int y, int zoom_x = -1, int zoom_y = -1, int max_x = gr_screen.max_w, int max_y = gr_screen.max_h, bool force_stretch = false);
 void gr_reset_screen_scale();
-bool gr_unsize_screen_pos(int *x, int *y);
-bool gr_resize_screen_pos(int *x, int *y);
-bool gr_unsize_screen_posf(float *x, float *y);
-bool gr_resize_screen_posf(float *x, float *y);
+bool gr_unsize_screen_pos(int *x, int *y, int *w = NULL, int *h = NULL, int resize_mode = GR_RESIZE_FULL);
+bool gr_resize_screen_pos(int *x, int *y, int *w = NULL, int *h = NULL, int resize_mode = GR_RESIZE_FULL);
+bool gr_unsize_screen_posf(float *x, float *y, float *w = NULL, float *h = NULL, int resize_mode = GR_RESIZE_FULL);
+bool gr_resize_screen_posf(float *x, float *y, float *w = NULL, float *h = NULL, int resize_mode = GR_RESIZE_FULL);
 
 // Does formatted printing.  This calls gr_string after formatting,
 // so if you don't need to format the string, then call gr_string
 // directly.
-extern void _cdecl gr_printf( int x, int y, char * format, ... );
-// same as above but doesn't resize for non-standard resolutions
-extern void _cdecl gr_printf_no_resize( int x, int y, char * format, ... );
+extern void _cdecl gr_printf( int x, int y, const char * format, ... );
+// same as gr_printf but positions text correctly in menus
+extern void _cdecl gr_printf_menu( int x, int y, const char * format, ... );
+// same as gr_printf_menu but accounts for menu zooming
+extern void _cdecl gr_printf_menu_zoomed( int x, int y, const char * format, ... );
+// same as gr_printf but doesn't resize for non-standard resolutions
+extern void _cdecl gr_printf_no_resize( int x, int y, const char * format, ... );
 
 // Returns the size of the string in pixels in w and h
 extern void gr_get_string_size( int *w, int *h, const char * text, int len = 9999 );
 
-__inline void gr_get_string_size( int *w, int *h, char * text, int len = 9999 )
-{
-	gr_get_string_size(w, h, const_cast<const char*>(text), len);
-}
-
 // Returns the height of the current font
 extern int gr_get_font_height();
 
-extern void gr_set_palette(char *name, ubyte *palette, int restrict_to_128 = 0);
+extern void gr_set_palette(const char *name, ubyte *palette, int restrict_to_128 = 0);
 
 // These two functions use a Windows mono font.  Only for use
 // in the editor, please.
-void gr_get_string_size_win(int *w, int *h, char *text);
-void gr_string_win(int x, int y, char *s );
+void gr_get_string_size_win(int *w, int *h, const char *text);
+void gr_string_win(int x, int y, const char *s );
 
 // set the mouse pointer to a specific bitmap, used for animating cursors
 #define GR_CURSOR_LOCK		1
@@ -610,60 +685,66 @@ extern void gr_activate(int active);
 void gr_flip();
 
 //#define gr_set_clip			GR_CALL(gr_screen.gf_set_clip)
-__inline void gr_set_clip(int x, int y, int w, int h, bool resize=true)
+__inline void gr_set_clip(int x, int y, int w, int h, int resize_mode=GR_RESIZE_FULL)
 {
-	(*gr_screen.gf_set_clip)(x,y,w,h,resize);
+	(*gr_screen.gf_set_clip)(x,y,w,h,resize_mode);
 }
 #define gr_reset_clip		GR_CALL(gr_screen.gf_reset_clip)
 
 void gr_set_bitmap(int bitmap_num, int alphablend = GR_ALPHABLEND_NONE, int bitbltmode = GR_BITBLT_MODE_NORMAL, float alpha = 1.0f);
 
 #define gr_clear				GR_CALL(gr_screen.gf_clear)
-__inline void gr_aabitmap(int x, int y, bool resize = true, bool mirror = false)
+__inline void gr_aabitmap(int x, int y, int resize_mode = GR_RESIZE_FULL, bool mirror = false)
 {
-	(*gr_screen.gf_aabitmap)(x,y,resize,mirror);
+	(*gr_screen.gf_aabitmap)(x,y,resize_mode,mirror);
 }
 
-__inline void gr_aabitmap_ex(int x, int y, int w, int h, int sx, int sy, bool resize = true, bool mirror = false)
+__inline void gr_aabitmap_ex(int x, int y, int w, int h, int sx, int sy, int resize_mode = GR_RESIZE_FULL, bool mirror = false)
 {
-	(*gr_screen.gf_aabitmap_ex)(x,y,w,h,sx,sy,resize,mirror);
+	(*gr_screen.gf_aabitmap_ex)(x,y,w,h,sx,sy,resize_mode,mirror);
 }
 
-__inline void gr_bitmap_ex(int x, int y, int w, int h, int sx, int sy, bool resize = true)
+__inline void gr_bitmap_ex(int x, int y, int w, int h, int sx, int sy, int resize_mode = GR_RESIZE_FULL)
 {
-	(*gr_screen.gf_bitmap_ex)(x, y, w, h, sx, sy, resize);
+	(*gr_screen.gf_bitmap_ex)(x, y, w, h, sx, sy, resize_mode);
 }
 
-void gr_shield_icon(coord2d coords[6], const bool resize = true);
-void gr_rect(int x, int y, int w, int h, bool resize = true);
-void gr_shade(int x, int y, int w, int h, bool resize = true);
+void gr_shield_icon(coord2d coords[6], const int resize_mode = GR_RESIZE_FULL);
+void gr_rect(int x, int y, int w, int h, int resize_mode = GR_RESIZE_FULL);
+void gr_shade(int x, int y, int w, int h, int resize_mode = GR_RESIZE_FULL);
 
-__inline void gr_string(int x, int y, const char* string, bool resize = true)
+__inline void gr_string(int x, int y, const char* string, int resize_mode = GR_RESIZE_FULL)
 {
-	(*gr_screen.gf_string)(x,y,string,resize);
+	(*gr_screen.gf_string)(x,y,string,resize_mode);
 }
 
-__inline void gr_string(int x, int y, char* string, bool resize = true)
+__inline void gr_circle(int xc, int yc, int d, int resize_mode = GR_RESIZE_FULL)
 {
-	(*gr_screen.gf_string)(x,y,const_cast<char*>(string),resize);
+	(*gr_screen.gf_circle)(xc,yc,d,resize_mode);
 }
 
-__inline void gr_circle(int xc, int yc, int d, bool resize = true)
+__inline void gr_unfilled_circle(int xc, int yc, int d, int resize_mode = GR_RESIZE_FULL)
 {
-	(*gr_screen.gf_circle)(xc,yc,d,resize);
+	(*gr_screen.gf_unfilled_circle)(xc,yc,d,resize_mode);
 }
+
+__inline void gr_arc(int xc, int yc, float r, float angle_start, float angle_end, bool fill, int resize_mode = GR_RESIZE_FULL)
+{
+	(*gr_screen.gf_arc)(xc,yc,r,angle_start,angle_end,fill,resize_mode);
+}
+
 #define gr_curve				GR_CALL(gr_screen.gf_curve)
 
-__inline void gr_line(int x1, int y1, int x2, int y2, bool resize = true)
+__inline void gr_line(int x1, int y1, int x2, int y2, int resize_mode = GR_RESIZE_FULL)
 {
-	(*gr_screen.gf_line)(x1, y1, x2, y2, resize);
+	(*gr_screen.gf_line)(x1, y1, x2, y2, resize_mode);
 }
 
 #define gr_aaline				GR_CALL(gr_screen.gf_aaline)
 
-__inline void gr_pixel(int x, int y, bool resize = true)
+__inline void gr_pixel(int x, int y, int resize_mode = GR_RESIZE_FULL)
 {
-	(*gr_screen.gf_pixel)(x, y, resize);
+	(*gr_screen.gf_pixel)(x, y, resize_mode);
 }
 #define gr_scaler				GR_CALL(gr_screen.gf_scaler)
 #define gr_aascaler			GR_CALL(gr_screen.gf_aascaler)
@@ -671,9 +752,9 @@ __inline void gr_pixel(int x, int y, bool resize = true)
 #define gr_render			GR_CALL(gr_screen.gf_render)
 #define gr_render_effect	GR_CALL(gr_screen.gf_render_effect)
 
-__inline void gr_gradient(int x1, int y1, int x2, int y2, bool resize = true)
+__inline void gr_gradient(int x1, int y1, int x2, int y2, int resize_mode = GR_RESIZE_FULL)
 {
-	(*gr_screen.gf_gradient)(x1, y1, x2, y2, resize);
+	(*gr_screen.gf_gradient)(x1, y1, x2, y2, resize_mode);
 }
 
 #define gr_fade_in			GR_CALL(gr_screen.gf_fade_in)
@@ -730,7 +811,7 @@ __inline int gr_tcache_set(int bitmap_id, int bitmap_type, float *u_scale, float
 #define gr_bm_free_data				GR_CALL(*gr_screen.gf_bm_free_data)
 #define gr_bm_create				GR_CALL(*gr_screen.gf_bm_create)
 #define gr_bm_init					GR_CALL(*gr_screen.gf_bm_init)
-__inline int gr_bm_load(ubyte type, int n, char *filename, CFILE *img_cfp = NULL, int *w = 0, int *h = 0, int *bpp = 0, ubyte *c_type = 0, int *mm_lvl = 0, int *size = 0)
+__inline int gr_bm_load(ubyte type, int n, const char *filename, CFILE *img_cfp = NULL, int *w = 0, int *h = 0, int *bpp = 0, ubyte *c_type = 0, int *mm_lvl = 0, int *size = 0)
 {
 	return (*gr_screen.gf_bm_load)(type, n, filename, img_cfp, w, h, bpp, c_type, mm_lvl, size);
 }
@@ -836,10 +917,10 @@ void gr_create_shader(shader *shade, ubyte r, ubyte g, ubyte b, ubyte c);
 void gr_set_shader(shader *shade);
 
 // new bitmap functions
-void gr_bitmap(int x, int y, bool resize = true);
-void gr_bitmap_uv(int _x, int _y, int _w, int _h, float _u0, float _v0, float _u1, float _v1, bool resize = true);
-void gr_bitmap_list(bitmap_2d_list* list, int n_bm, bool allow_scaling);
-void gr_bitmap_list(bitmap_rect_list* list, int n_bm, bool allow_scaling);
+void gr_bitmap(int x, int y, int resize_mode = GR_RESIZE_FULL);
+void gr_bitmap_uv(int _x, int _y, int _w, int _h, float _u0, float _v0, float _u1, float _v1, int resize_mode = GR_RESIZE_FULL);
+void gr_bitmap_list(bitmap_2d_list* list, int n_bm, int resize_mode);
+void gr_bitmap_list(bitmap_rect_list* list, int n_bm, int resize_mode);
 
 // texture update functions
 ubyte* gr_opengl_get_texture_update_pointer(int bitmap_handle);
@@ -848,7 +929,7 @@ void gr_opengl_update_texture(int bitmap_handle, int bpp, ubyte* data, int width
 // special function for drawing polylines. this function is specifically intended for
 // polylines where each section is no more than 90 degrees away from a previous section.
 // Moreover, it is _really_ intended for use with 45 degree angles. 
-void gr_pline_special(vec3d **pts, int num_pts, int thickness,bool resize=true);
+void gr_pline_special(SCP_vector<vec3d> *pts, int thickness,int resize_mode=GR_RESIZE_FULL);
 
 #define VB_FLAG_POSITION	(1<<0)	
 #define VB_FLAG_RHW			(1<<1)	//incompatable with the next normal
