@@ -22,6 +22,7 @@
 #include "io/timer.h"
 #include "ddsutils/ddsutils.h"
 #include "model/model.h"
+#include "debugconsole/console.h"
 #include "debugconsole/timerbar.h"
 #include "graphics/gropenglbmpman.h"
 #include "graphics/gropengllight.h"
@@ -97,7 +98,7 @@ static int GL_mouse_saved_y2 = 0;
 
 void opengl_save_mouse_area(int x, int y, int w, int h);
 
-extern char *Osreg_title;
+extern const char *Osreg_title;
 
 extern GLfloat GL_anisotropy;
 
@@ -346,13 +347,15 @@ void gr_opengl_flip()
 
 		if (Gr_cursor != -1 && bm_is_valid(Gr_cursor)) {
 			gr_set_bitmap(Gr_cursor);
-			gr_bitmap( mx, my, false);
+			gr_bitmap( mx, my, GR_RESIZE_NONE);
 		}
 	}
 
 #ifdef _WIN32
 	SwapBuffers(GL_device_context);
 #else
+	if (Cmdline_gl_finish)
+		glFinish ();
 	SDL_GL_SwapBuffers();
 #endif
 
@@ -367,7 +370,7 @@ void gr_opengl_flip()
 #endif
 }
 
-void gr_opengl_set_clip(int x, int y, int w, int h, bool resize)
+void gr_opengl_set_clip(int x, int y, int w, int h, int resize_mode)
 {
 	// check for sanity of parameters
 	if (x < 0) {
@@ -378,11 +381,15 @@ void gr_opengl_set_clip(int x, int y, int w, int h, bool resize)
 		y = 0;
 	}
 
-	int to_resize = (resize && (gr_screen.custom_size || (gr_screen.rendering_to_texture != -1)));
+	int to_resize = (resize_mode != GR_RESIZE_NONE && (gr_screen.custom_size || (gr_screen.rendering_to_texture != -1)));
 
 	int max_w = ((to_resize) ? gr_screen.max_w_unscaled : gr_screen.max_w);
 	int max_h = ((to_resize) ? gr_screen.max_h_unscaled : gr_screen.max_h);
 
+	if ((gr_screen.rendering_to_texture != -1) && to_resize) {
+		gr_unsize_screen_pos(&max_w, &max_h);
+	}
+
 	if (x >= max_w) {
 		x = max_w - 1;
 	}
@@ -417,8 +424,7 @@ void gr_opengl_set_clip(int x, int y, int w, int h, bool resize)
 	gr_screen.clip_height_unscaled = h;
 
 	if (to_resize) {
-		gr_resize_screen_pos(&x, &y);
-		gr_resize_screen_pos(&w, &h);
+		gr_resize_screen_pos(&x, &y, &w, &h, resize_mode);
 	} else {
 		gr_unsize_screen_pos( &gr_screen.offset_x_unscaled, &gr_screen.offset_y_unscaled );
 		gr_unsize_screen_pos( &gr_screen.clip_right_unscaled, &gr_screen.clip_bottom_unscaled );
@@ -479,7 +485,7 @@ void gr_opengl_set_palette(ubyte *new_palette, int is_alphacolor)
 {
 }
 
-void gr_opengl_print_screen(char *filename)
+void gr_opengl_print_screen(const char *filename)
 {
 	char tmp[MAX_PATH_LEN];
 	ubyte tga_hdr[18];
@@ -650,27 +656,25 @@ void gr_opengl_fog_set(int fog_mode, int r, int g, int b, float fog_near, float
 		return;
 	}
 	
-  	if (gr_screen.current_fog_mode != fog_mode) {
-	  	if (OGL_fogmode == 3) {
-			glFogf(GL_FOG_DISTANCE_MODE_NV, GL_EYE_RADIAL_NV);
-			glFogf(GL_FOG_COORDINATE_SOURCE, GL_FRAGMENT_DEPTH);
-		}
-		// Um.. this is not the correct way to fog in software, probably doesn't matter though
-		else if ( (OGL_fogmode == 2) && Cmdline_nohtl ) {
-			glFogf(GL_FOG_COORDINATE_SOURCE_EXT, GL_FOG_COORDINATE_EXT);
-			fog_near *= fog_near;		// it's faster this way
-			fog_far *= fog_far;		
-		} else {
-			glFogf(GL_FOG_COORDINATE_SOURCE, GL_FRAGMENT_DEPTH);
-		}
-
-		GL_state.Fog(GL_TRUE); 
-		glFogf(GL_FOG_MODE, GL_LINEAR);
-		glFogf(GL_FOG_START, fog_near);
-		glFogf(GL_FOG_END, fog_far);
-
-		gr_screen.current_fog_mode = fog_mode;
+  	if (OGL_fogmode == 3) {
+		glFogf(GL_FOG_DISTANCE_MODE_NV, GL_EYE_RADIAL_NV);
+		glFogf(GL_FOG_COORDINATE_SOURCE, GL_FRAGMENT_DEPTH);
 	}
+	// Um.. this is not the correct way to fog in software, probably doesn't matter though
+	else if ( (OGL_fogmode == 2) && Cmdline_nohtl ) {
+		glFogf(GL_FOG_COORDINATE_SOURCE_EXT, GL_FOG_COORDINATE_EXT);
+		fog_near *= fog_near;		// it's faster this way
+		fog_far *= fog_far;		
+	} else {
+		glFogf(GL_FOG_COORDINATE_SOURCE, GL_FRAGMENT_DEPTH);
+	}
+
+	GL_state.Fog(GL_TRUE); 
+	glFogf(GL_FOG_MODE, GL_LINEAR);
+	glFogf(GL_FOG_START, fog_near);
+	glFogf(GL_FOG_END, fog_far);
+
+	gr_screen.current_fog_mode = fog_mode;
 	
 	if ( (gr_screen.current_fog_color.red != r) ||
 			(gr_screen.current_fog_color.green != g) ||
@@ -1130,7 +1134,7 @@ void gr_opengl_restore_screen(int bmp_id)
 		return;
 
 	gr_set_bitmap(GL_saved_screen_id);
-	gr_bitmap(0, 0, false);	// don't scale here since we already have real screen size
+	gr_bitmap(0, 0, GR_RESIZE_NONE);	// don't scale here since we already have real screen size
 }
 
 void gr_opengl_free_screen(int bmp_id)
@@ -1773,6 +1777,8 @@ void opengl_setup_function_pointers()
 //	gr_screen.gf_shade				= gr_opengl_shade;
 	gr_screen.gf_string				= gr_opengl_string;
 	gr_screen.gf_circle				= gr_opengl_circle;
+	gr_screen.gf_unfilled_circle	= gr_opengl_unfilled_circle;
+	gr_screen.gf_arc				= gr_opengl_arc;
 	gr_screen.gf_curve				= gr_opengl_curve;
 
 	gr_screen.gf_line				= gr_opengl_line;
@@ -2076,56 +2082,61 @@ bool gr_opengl_init()
 
 DCF(ogl_minimize, "Minimizes opengl")
 {
+	bool minimize_ogl = false;
+
 	if ( gr_screen.mode != GR_OPENGL ) {
 		dc_printf("Command only available in OpenGL mode.\n");
 		return;
 	}
 
-	if (Dc_command) {
-		dc_get_arg(ARG_TRUE);
-
-		if ( Dc_arg_type & ARG_TRUE ) {
-			opengl_minimize();
-		}
+	if (dc_optional_string_either("help", "--help")) {
+		dc_printf("[bool] If true is passed, then the OpenGL window will minimize.\n");
+		return;
 	}
+	dc_stuff_boolean(&minimize_ogl);
 
-	if (Dc_help)
-		dc_printf("If set to true then the OpenGL window will minimize.\n");
+	if (minimize_ogl) {
+		opengl_minimize();
+	}
 }
 
 DCF(ogl_anisotropy, "toggles anisotropic filtering")
 {
+	bool process = true;
+	int value;
+
 	if ( gr_screen.mode != GR_OPENGL ) {
 		dc_printf("Can only set anisotropic filter in OpenGL mode.\n");
 		return;
 	}
 
-	if ( Dc_command && !Is_Extension_Enabled(OGL_EXT_TEXTURE_FILTER_ANISOTROPIC) ) {
+	if (dc_optional_string_either("help", "--help")) {
+		dc_printf("Sets OpenGL anisotropic filtering level.\n");
+		dc_printf("GL_anisotropy [int]  Valid values are 0 to %i. 0 turns off anisotropic filtering.\n", (int)opengl_get_max_anisotropy());
+		process = false;
+	}
+
+	if (dc_optional_string_either("status", "--status") || dc_optional_string_either("?", "--?")) {
+		dc_printf("Current anisotropic filter value is %i\n", (int)GL_anisotropy);
+		process = false;
+	}
+
+	if (!process) {
+		return;
+	}
+
+	if ( !Is_Extension_Enabled(OGL_EXT_TEXTURE_FILTER_ANISOTROPIC) ) {
 		dc_printf("Error: Anisotropic filter is not settable!\n");
 		return;
 	}
 
-	if ( Dc_command ) {
-		dc_get_arg(ARG_INT | ARG_NONE);
-
-		if ( Dc_arg_type & ARG_NONE ) {
+	if (!dc_maybe_stuff_int(&value)) {
+		// No arg passed, set to default
 			GL_anisotropy = 1.0f;
 		//	opengl_set_anisotropy();
 			dc_printf("Anisotropic filter value reset to default level.\n");
-		}
-
-		if ( Dc_arg_type & ARG_INT ) {
-			GL_anisotropy = (GLfloat)Dc_arg_float;
+	} else {
+		GL_anisotropy = (GLfloat)value;
 		//	opengl_set_anisotropy( (float)Dc_arg_float );
-		}
-	}
-
-	if ( Dc_status ) {
-		dc_printf("Current anisotropic filter value is %i\n", (int)GL_anisotropy);
-	}
-
-	if (Dc_help) {
-		dc_printf("Sets OpenGL anisotropic filtering level.\n");
-		dc_printf("Valid values are 1 to %i, or 0 to turn off.\n", (int)opengl_get_max_anisotropy());
 	}
 }
