@@ -54,12 +54,12 @@ int Scene_texture_height;
 GLfloat Scene_texture_u_scale = 1.0f;
 GLfloat Scene_texture_v_scale = 1.0f;
 
-void gr_opengl_pixel(int x, int y, bool resize)
+void gr_opengl_pixel(int x, int y, int resize_mode)
 {
-	gr_line(x, y, x, y, resize);
+	gr_line(x, y, x, y, resize_mode);
 }
 
-void opengl_aabitmap_ex_internal(int x, int y, int w, int h, int sx, int sy, bool resize, bool mirror)
+void opengl_aabitmap_ex_internal(int x, int y, int w, int h, int sx, int sy, int resize_mode, bool mirror)
 {
 	if ( (w < 1) || (h < 1) ) {
 		return;
@@ -86,7 +86,7 @@ void opengl_aabitmap_ex_internal(int x, int y, int w, int h, int sx, int sy, boo
 	float x1, x2, y1, y2;
 	int bw, bh, do_resize;
 
-	if ( resize && (gr_screen.custom_size || (gr_screen.rendering_to_texture != -1)) ) {
+	if ( resize_mode != GR_RESIZE_NONE && (gr_screen.custom_size || (gr_screen.rendering_to_texture != -1)) ) {
 		do_resize = 1;
 	} else {
 		do_resize = 0;
@@ -106,8 +106,8 @@ void opengl_aabitmap_ex_internal(int x, int y, int w, int h, int sx, int sy, boo
 	y2 = y1 + i2fl(h);
 
 	if (do_resize) {
-		gr_resize_screen_posf(&x1, &y1);
-		gr_resize_screen_posf(&x2, &y2);
+		gr_resize_screen_posf(&x1, &y1, NULL, NULL, resize_mode);
+		gr_resize_screen_posf(&x2, &y2, NULL, NULL, resize_mode);
 	}
 
 	GL_state.Color(gr_screen.current_color.red, gr_screen.current_color.green, gr_screen.current_color.blue, gr_screen.current_color.alpha);
@@ -127,7 +127,7 @@ void opengl_aabitmap_ex_internal(int x, int y, int w, int h, int sx, int sy, boo
 	GL_CHECK_FOR_ERRORS("end of aabitmap_ex_internal()");
 }
 
-void gr_opengl_aabitmap_ex(int x, int y, int w, int h, int sx, int sy, bool resize, bool mirror)
+void gr_opengl_aabitmap_ex(int x, int y, int w, int h, int sx, int sy, int resize_mode, bool mirror)
 {
 	int reclip;
 #ifndef NDEBUG
@@ -143,7 +143,7 @@ void gr_opengl_aabitmap_ex(int x, int y, int w, int h, int sx, int sy, bool resi
 
 	bm_get_info(gr_screen.current_bitmap, &bw, &bh);
 
-	if ( resize && (gr_screen.custom_size || (gr_screen.rendering_to_texture != -1)) ) {
+	if ( resize_mode != GR_RESIZE_NONE && (gr_screen.custom_size || (gr_screen.rendering_to_texture != -1)) ) {
 		do_resize = 1;
 	} else {
 		do_resize = 0;
@@ -242,16 +242,16 @@ void gr_opengl_aabitmap_ex(int x, int y, int w, int h, int sx, int sy, bool resi
 #endif
 
 	// We now have dx1,dy1 and dx2,dy2 and sx, sy all set validly within clip regions.
-	opengl_aabitmap_ex_internal(dx1, dy1, (dx2 - dx1 + 1), (dy2 - dy1 + 1), sx, sy, resize, mirror);
+	opengl_aabitmap_ex_internal(dx1, dy1, (dx2 - dx1 + 1), (dy2 - dy1 + 1), sx, sy, resize_mode, mirror);
 }
 
-void gr_opengl_aabitmap(int x, int y, bool resize, bool mirror)
+void gr_opengl_aabitmap(int x, int y, int resize_mode, bool mirror)
 {
 	int w, h, do_resize;
 
 	bm_get_info(gr_screen.current_bitmap, &w, &h);
 
-	if ( resize && (gr_screen.custom_size || (gr_screen.rendering_to_texture != -1)) ) {
+	if ( resize_mode != GR_RESIZE_NONE && (gr_screen.custom_size || (gr_screen.rendering_to_texture != -1)) ) {
 		do_resize = 1;
 	} else {
 		do_resize = 0;
@@ -303,18 +303,21 @@ void gr_opengl_aabitmap(int x, int y, bool resize, bool mirror)
 	}
 
 	// Draw bitmap bm[sx,sy] into (dx1,dy1)-(dx2,dy2)
-	opengl_aabitmap_ex_internal(dx1, dy1, (dx2 - dx1 + 1), (dy2 - dy1 + 1), sx, sy, resize, mirror);
+	opengl_aabitmap_ex_internal(dx1, dy1, (dx2 - dx1 + 1), (dy2 - dy1 + 1), sx, sy, resize_mode, mirror);
 }
 
+#define MAX_VERTS_PER_DRAW 120
 struct v4 { GLfloat x,y,u,v; };
+static v4 GL_string_render_buff[MAX_VERTS_PER_DRAW];
 
-void gr_opengl_string(int sx, int sy, const char *s, bool resize)
+void gr_opengl_string(float sx, float sy, const char *s, int resize_mode)
 {
 	int width, spacing, letter;
-	int x, y, do_resize;
+	float x, y;
+	bool do_resize;
 	float bw, bh;
 	float u0, u1, v0, v1;
-	int x1, x2, y1, y2;
+	float x1, x2, y1, y2;
 	float u_scale, v_scale;
 
 	if ( !Current_font || (*s == 0) ) {
@@ -333,9 +336,7 @@ void gr_opengl_string(int sx, int sy, const char *s, bool resize)
 		return;
 	}
 
-	// conversion from quads to triangles requires six vertices per quad
-	struct v4 *glVert = (struct v4*) vm_malloc(sizeof(struct v4) * strlen(s) * 6);
-	int curChar = 0;
+	int buffer_offset = 0;
 
 	int ibw, ibh;
 
@@ -352,10 +353,10 @@ void gr_opengl_string(int sx, int sy, const char *s, bool resize)
 	}
 
 //	if ( (gr_screen.custom_size && resize) || (gr_screen.rendering_to_texture != -1) ) {
-	if ( resize && (gr_screen.custom_size || (gr_screen.rendering_to_texture != -1)) ) {
-		do_resize = 1;
+	if ( resize_mode != GR_RESIZE_NONE && (gr_screen.custom_size || (gr_screen.rendering_to_texture != -1)) ) {
+		do_resize = true;
 	} else {
-		do_resize = 0;
+		do_resize = false;
 	}
 
 	int clip_left = ((do_resize) ? gr_screen.clip_left_unscaled : gr_screen.clip_left);
@@ -366,9 +367,9 @@ void gr_opengl_string(int sx, int sy, const char *s, bool resize)
 	x = sx;
 	y = sy;
 
-	if (sx == 0x8000) {
+	if (sx == (float)0x8000) {
 		// centered
-		x = get_centered_x(s);
+		x = (float)get_centered_x(s, !do_resize);
 	} else {
 		x = sx;
 	}
@@ -377,6 +378,15 @@ void gr_opengl_string(int sx, int sy, const char *s, bool resize)
 
 	GLboolean cull_face = GL_state.CullFace(GL_FALSE);
 
+	GL_state.Array.BindArrayBuffer(0);
+
+	GL_state.Array.EnableClientVertex();
+	GL_state.Array.VertexPointer(2, GL_FLOAT, sizeof(v4), &GL_string_render_buff[0].x);
+
+	GL_state.Array.SetActiveClientUnit(0);
+	GL_state.Array.EnableClientTexture();
+	GL_state.Array.TexPointer(2, GL_FLOAT, sizeof(v4), &GL_string_render_buff[0].u);
+
 	// pick out letter coords, draw it, goto next letter and do the same
 	while (*s)	{
 		x += spacing;
@@ -385,9 +395,9 @@ void gr_opengl_string(int sx, int sy, const char *s, bool resize)
 			s++;
 			y += Current_font->h;
 
-			if (sx == 0x8000) {
+			if (sx == (float)0x8000) {
 				// centered
-				x = get_centered_x(s);
+				x = (float)get_centered_x(s, !do_resize);
 			} else {
 				x = sx;
 			}
@@ -400,13 +410,13 @@ void gr_opengl_string(int sx, int sy, const char *s, bool resize)
 		letter = get_char_width(s[0], s[1], &width, &spacing);
 		s++;
 
-		//not in font, draw as space
+		// not in font, draw as space
 		if (letter < 0) {
 			continue;
 		}
 
-		int xd, yd, xc, yc;
-		int wc, hc;
+		float xd, yd, xc, yc;
+		float wc, hc;
 
 		// Check if this character is totally clipped
 		if ( (x + width) < clip_left ) {
@@ -462,8 +472,8 @@ void gr_opengl_string(int sx, int sy, const char *s, bool resize)
 		y2 = y1 + hc;
 
 		if (do_resize) {
-			gr_resize_screen_pos( &x1, &y1 );
-			gr_resize_screen_pos( &x2, &y2 );
+			gr_resize_screen_posf( &x1, &y1, NULL, NULL, resize_mode );
+			gr_resize_screen_posf( &x2, &y2, NULL, NULL, resize_mode );
 		}
 
 		u0 = u_scale * (i2fl(u+xd) / bw);
@@ -471,61 +481,73 @@ void gr_opengl_string(int sx, int sy, const char *s, bool resize)
 
 		u1 = u_scale * (i2fl((u+xd)+wc) / bw);
 		v1 = v_scale * (i2fl((v+yd)+hc) / bh);
+		
+		if ( buffer_offset == MAX_VERTS_PER_DRAW ) {
+			glDrawArrays(GL_TRIANGLES, 0, buffer_offset);
+			buffer_offset = 0;
+		}
 
-		glVert[curChar*6 + 0].x = (GLfloat)x1;
-		glVert[curChar*6 + 0].y = (GLfloat)y2;
-		glVert[curChar*6 + 0].u = u0;
-		glVert[curChar*6 + 0].v = v1;
+		GL_string_render_buff[buffer_offset].x = (GLfloat)x1;
+		GL_string_render_buff[buffer_offset].y = (GLfloat)y1;
+		GL_string_render_buff[buffer_offset].u = u0;
+		GL_string_render_buff[buffer_offset].v = v0;
+		buffer_offset++;
 
-		glVert[curChar*6 + 1].x = (GLfloat)x2;
-		glVert[curChar*6 + 1].y = (GLfloat)y2;
-		glVert[curChar*6 + 1].u = u1;
-		glVert[curChar*6 + 1].v = v1;
+		GL_string_render_buff[buffer_offset].x = (GLfloat)x1;
+		GL_string_render_buff[buffer_offset].y = (GLfloat)y2;
+		GL_string_render_buff[buffer_offset].u = u0;
+		GL_string_render_buff[buffer_offset].v = v1;
+		buffer_offset++;
 
-		glVert[curChar*6 + 2].x = (GLfloat)x1;
-		glVert[curChar*6 + 2].y = (GLfloat)y1;
-		glVert[curChar*6 + 2].u = u0;
-		glVert[curChar*6 + 2].v = v0;
+		GL_string_render_buff[buffer_offset].x = (GLfloat)x2;
+		GL_string_render_buff[buffer_offset].y = (GLfloat)y1;
+		GL_string_render_buff[buffer_offset].u = u1;
+		GL_string_render_buff[buffer_offset].v = v0;
+		buffer_offset++;
 
-		glVert[curChar*6 + 3] = glVert[curChar*6 + 1];
-		glVert[curChar*6 + 4] = glVert[curChar*6 + 2];
+		GL_string_render_buff[buffer_offset].x = (GLfloat)x1;
+		GL_string_render_buff[buffer_offset].y = (GLfloat)y2;
+		GL_string_render_buff[buffer_offset].u = u0;
+		GL_string_render_buff[buffer_offset].v = v1;
+		buffer_offset++;
 
-		glVert[curChar*6 + 5].x = (GLfloat)x2;
-		glVert[curChar*6 + 5].y = (GLfloat)y1;
-		glVert[curChar*6 + 5].u = u1;
-		glVert[curChar*6 + 5].v = v0;
+		GL_string_render_buff[buffer_offset].x = (GLfloat)x2;
+		GL_string_render_buff[buffer_offset].y = (GLfloat)y1;
+		GL_string_render_buff[buffer_offset].u = u1;
+		GL_string_render_buff[buffer_offset].v = v0;
+		buffer_offset++;
 
-		curChar++;
+		GL_string_render_buff[buffer_offset].x = (GLfloat)x2;
+		GL_string_render_buff[buffer_offset].y = (GLfloat)y2;
+		GL_string_render_buff[buffer_offset].u = u1;
+		GL_string_render_buff[buffer_offset].v = v1;
+		buffer_offset++;
 	}
 
-	GL_state.Array.BindArrayBuffer(0);
-
-	GL_state.Array.EnableClientVertex();
-	GL_state.Array.VertexPointer(2, GL_FLOAT, sizeof(struct v4), &glVert[0].x);
-
-	GL_state.Array.SetActiveClientUnit(0);
-	GL_state.Array.EnableClientTexture();
-	GL_state.Array.TexPointer(2, GL_FLOAT, sizeof(struct v4), &glVert[0].u);
-
-	glDrawArrays(GL_TRIANGLES, 0, curChar * 6);
+	if ( buffer_offset ) {
+		glDrawArrays(GL_TRIANGLES, 0, buffer_offset);
+	}
 
 	GL_state.Array.DisableClientVertex();
 	GL_state.Array.DisableClientTexture();
 
 	GL_state.CullFace(cull_face);
 
-	vm_free(glVert);
-
 	GL_CHECK_FOR_ERRORS("end of string()");
 }
 
-void gr_opengl_line(int x1,int y1,int x2,int y2, bool resize)
+void gr_opengl_string(int sx, int sy, const char *s, int resize_mode)
 {
-	int do_resize, clipped = 0, swapped = 0;
+	gr_opengl_string(i2fl(sx), i2fl(sy), s, resize_mode);
+}
+
+void gr_opengl_line(int x1,int y1,int x2,int y2, int resize_mode)
+{
+	int do_resize;
 	float sx1, sy1;
 	float sx2, sy2;
 
-	if ( resize && (gr_screen.custom_size || (gr_screen.rendering_to_texture != -1)) ) {
+	if ( resize_mode != GR_RESIZE_NONE && (gr_screen.custom_size || (gr_screen.rendering_to_texture != -1)) ) {
 		do_resize = 1;
 	} else {
 		do_resize = 0;
@@ -539,7 +561,7 @@ void gr_opengl_line(int x1,int y1,int x2,int y2, bool resize)
 	int offset_y = ((do_resize) ? gr_screen.offset_y_unscaled : gr_screen.offset_y);
 
 
-	INT_CLIPLINE(x1, y1, x2, y2, clip_left, clip_top, clip_right, clip_bottom, return, clipped = 1, swapped = 1);
+	INT_CLIPLINE(x1, y1, x2, y2, clip_left, clip_top, clip_right, clip_bottom, return, ;, ;);
 
 	sx1 = i2fl(x1 + offset_x);
 	sy1 = i2fl(y1 + offset_y);
@@ -548,8 +570,8 @@ void gr_opengl_line(int x1,int y1,int x2,int y2, bool resize)
 
 
 	if (do_resize) {
-		gr_resize_screen_posf(&sx1, &sy1);
-		gr_resize_screen_posf(&sx2, &sy2);
+		gr_resize_screen_posf(&sx1, &sy1, NULL, NULL, resize_mode);
+		gr_resize_screen_posf(&sx2, &sy2, NULL, NULL, resize_mode);
 	}
 
 	GL_state.SetTextureSource(TEXTURE_SOURCE_NONE);
@@ -656,7 +678,6 @@ void gr_opengl_aaline(vertex *v1, vertex *v2)
 //	glHint( GL_LINE_SMOOTH_HINT, GL_FASTEST );
 //	glLineWidth( 1.0 );
 
-	int clipped = 0, swapped = 0;
 	float x1 = v1->screen.xyw.x;
 	float y1 = v1->screen.xyw.y;
 	float x2 = v2->screen.xyw.x;
@@ -665,7 +686,7 @@ void gr_opengl_aaline(vertex *v1, vertex *v2)
 	float sx2, sy2;
 
 
-	FL_CLIPLINE(x1, y1, x2, y2, (float)gr_screen.clip_left, (float)gr_screen.clip_top, (float)gr_screen.clip_right, (float)gr_screen.clip_bottom, return, clipped = 1, swapped = 1);
+	FL_CLIPLINE(x1, y1, x2, y2, (float)gr_screen.clip_left, (float)gr_screen.clip_top, (float)gr_screen.clip_right, (float)gr_screen.clip_bottom, return, ;, ;);
 
 	sx1 = x1 + (float)gr_screen.offset_x;
 	sy1 = y1 + (float)gr_screen.offset_y;
@@ -733,21 +754,21 @@ void gr_opengl_aaline(vertex *v1, vertex *v2)
 //	glDisable( GL_LINE_SMOOTH );
 }
 
-void gr_opengl_gradient(int x1, int y1, int x2, int y2, bool resize)
+void gr_opengl_gradient(int x1, int y1, int x2, int y2, int resize_mode)
 {
-	int clipped = 0, swapped = 0;
+	int swapped = 0;
 
 	if ( !gr_screen.current_color.is_alphacolor ) {
-		gr_opengl_line(x1, y1, x2, y2, resize);
+		gr_opengl_line(x1, y1, x2, y2, resize_mode);
 		return;
 	}
 
-	if ( resize && (gr_screen.custom_size || (gr_screen.rendering_to_texture != -1)) ) {
-		gr_resize_screen_pos(&x1, &y1);
-		gr_resize_screen_pos(&x2, &y2);
+	if ( resize_mode != GR_RESIZE_NONE && (gr_screen.custom_size || (gr_screen.rendering_to_texture != -1)) ) {
+		gr_resize_screen_pos(&x1, &y1, NULL, NULL, resize_mode);
+		gr_resize_screen_pos(&x2, &y2, NULL, NULL, resize_mode);
 	}
 
-	INT_CLIPLINE(x1, y1, x2, y2, gr_screen.clip_left, gr_screen.clip_top, gr_screen.clip_right, gr_screen.clip_bottom, return, clipped = 1, swapped = 1);
+	INT_CLIPLINE(x1, y1, x2, y2, gr_screen.clip_left, gr_screen.clip_top, gr_screen.clip_right, gr_screen.clip_bottom, return, ;, swapped = 1);
 
 	GL_state.SetTextureSource(TEXTURE_SOURCE_NONE);
 	GL_state.SetAlphaBlendMode(ALPHA_BLEND_ALPHA_BLEND_ALPHA);
@@ -800,67 +821,229 @@ void gr_opengl_gradient(int x1, int y1, int x2, int y2, bool resize)
 
 }
 
-void gr_opengl_circle(int xc, int yc, int d, bool resize)
+void gr_opengl_circle(int xc, int yc, int d, int resize_mode)
 {
-	int p, x, y, r;
-
-	if ( resize && (gr_screen.custom_size || (gr_screen.rendering_to_texture != -1)) ) {
-		gr_resize_screen_pos(&xc, &yc);
-	}
-
-	r = d / 2;
-	p = 3 - d;
-	x = 0;
-	y = r;
-
-	// Big clip
-	if ( (xc+r) < gr_screen.clip_left ) {
-		return;
-	}
-
-	if ( (xc-r) > gr_screen.clip_right ) {
-		return;
-	}
-
-	if ( (yc+r) < gr_screen.clip_top ) {
-		return;
-	}
-
-	if ( (yc-r) > gr_screen.clip_bottom ) {
-		return;
-	}
-
-	while (x < y) {
-		// Draw the first octant
-		gr_opengl_line(xc-y, yc-x, xc+y, yc-x, false);
-		if (x > 0) // Don't draw the center horizontal line twice
-			gr_opengl_line(xc-y, yc+x, xc+y, yc+x, false);
-
-		if (p < 0) {
-			p += (x << 2) + 6;
-		} else {
-			// Draw the second octant
-			gr_opengl_line(xc-x, yc-y, xc+x, yc-y, false);
-			gr_opengl_line(xc-x, yc+y, xc+x, yc+y, false);
-
-			p += ((x - y) << 2) + 10;
-			y--;
-		}
-
-		x++;
-	}
-
-	if (x == y) {
-		gr_opengl_line(xc-x, yc-y, xc+x, yc-y, false);
-		gr_opengl_line(xc-x, yc+y, xc+x, yc+y, false);
-	}
+	gr_opengl_arc(xc, yc, d / 2.0f, 0.0f, 360.0f, true, resize_mode);
 }
 
-void gr_opengl_curve(int xc, int yc, int r, int direction)
+void gr_opengl_unfilled_circle(int xc, int yc, int d, int resize_mode)
+{
+	int r = d / 2;
+	int segments = 4 + (int)(r); // seems like a good approximation
+	float theta = 2 * PI / float(segments - 1); 
+	float c = cosf(theta);
+	float s = sinf(theta);
+	float t;
+
+	float x1 = 1.0f;
+	float y1 = 0.0f; 
+	float x2 = x1;
+	float y2 = y1;
+
+	float linewidth;
+	glGetFloatv(GL_LINE_WIDTH, &linewidth);
+
+	float halflinewidth = linewidth / 2.0f;
+	float inner_rad = r - halflinewidth;
+	float outer_rad = r + halflinewidth;
+
+	int do_resize = 0;
+
+	if ( resize_mode != GR_RESIZE_NONE && (gr_screen.custom_size || (gr_screen.rendering_to_texture != -1)) ) {
+		gr_resize_screen_pos(&xc, &yc, NULL, NULL, resize_mode);
+		do_resize = 1;
+	}
+
+	// Big clip
+	if ( (xc+outer_rad) < gr_screen.clip_left ) {
+		return;
+	}
+
+	if ( (xc-outer_rad) > gr_screen.clip_right ) {
+		return;
+	}
+
+	if ( (yc+outer_rad) < gr_screen.clip_top ) {
+		return;
+	}
+
+	if ( (yc-outer_rad) > gr_screen.clip_bottom ) {
+		return;
+	}
+
+	int offset_x = ((do_resize) ? gr_screen.offset_x_unscaled : gr_screen.offset_x);
+	int offset_y = ((do_resize) ? gr_screen.offset_y_unscaled : gr_screen.offset_y);
+
+	GL_state.SetTextureSource(TEXTURE_SOURCE_NONE);
+	GL_state.SetAlphaBlendMode(ALPHA_BLEND_ALPHA_BLEND_ALPHA);
+	GL_state.SetZbufferType(ZBUFFER_TYPE_NONE);
+
+	GLfloat *circle = new GLfloat[segments * 4];
+
+	for (int i=0; i < segments * 4; i+=4) {
+		circle[i] = i2fl(xc + (x2 * outer_rad) + offset_x);
+		circle[i+1] = i2fl(yc + (y2 * outer_rad) + offset_y);
+
+		circle[i+2] = i2fl(xc + (x2 * inner_rad) + offset_x);
+		circle[i+3] = i2fl(yc + (y2 * inner_rad) + offset_y);
+
+		t = x2;
+		x2 = c * x1 - s * y1;
+		y2 = s * t + c * y1;
+
+		x1 = x2;
+		y1 = y2;
+	}
+
+	gr_opengl_set_2d_matrix();
+
+	GL_state.Color(gr_screen.current_color.red, gr_screen.current_color.green, gr_screen.current_color.blue, gr_screen.current_color.alpha);
+
+	GL_state.Array.EnableClientVertex();
+	GL_state.Array.VertexPointer(2, GL_FLOAT, 0, circle);
+
+	glDrawArrays(GL_QUAD_STRIP, 0, segments * 2);
+
+	GL_state.Array.DisableClientVertex();
+
+	GL_CHECK_FOR_ERRORS("end of opengl_unfilled_circle()");
+
+	gr_opengl_end_2d_matrix();
+
+	delete [] circle;
+}
+
+void gr_opengl_arc(int xc, int yc, float r, float angle_start, float angle_end, bool fill, int resize_mode)
+{
+	// Ensure that angle_start < angle_end
+	if (angle_end < angle_start) {
+		float temp = angle_start;
+		angle_start = angle_end;
+		angle_end = temp;
+	}
+
+	float arc_length_ratio;
+	arc_length_ratio = MIN(angle_end - angle_start, 360.0f) / 360.0f;
+
+	int segments = 4 + (int)(r * arc_length_ratio); // seems like a good approximation
+	float theta = 2 * PI / float(segments - 1) * arc_length_ratio; 
+	float c = cosf(theta);
+	float s = sinf(theta);
+	float t;
+
+	float x1 = cosf(ANG_TO_RAD(angle_start));
+	float y1 = sinf(ANG_TO_RAD(angle_start));
+	float x2 = x1;
+	float y2 = y1;
+
+	float halflinewidth = 0.0f;
+	float inner_rad = 0.0f; // only used if fill==false
+	float outer_rad = r;
+
+	if (!fill) {
+		float linewidth;
+		glGetFloatv(GL_LINE_WIDTH, &linewidth);
+
+		halflinewidth = linewidth / 2.0f;
+		inner_rad = r - halflinewidth;
+		outer_rad = r + halflinewidth;
+	}
+
+	int do_resize = 0;
+
+	if ( resize_mode != GR_RESIZE_NONE && (gr_screen.custom_size || (gr_screen.rendering_to_texture != -1)) ) {
+		gr_resize_screen_pos(&xc, &yc, NULL, NULL, resize_mode);
+		do_resize = 1;
+	}
+
+	// Big clip
+	if ( (xc+outer_rad) < gr_screen.clip_left ) {
+		return;
+	}
+
+	if ( (xc-outer_rad) > gr_screen.clip_right ) {
+		return;
+	}
+
+	if ( (yc+outer_rad) < gr_screen.clip_top ) {
+		return;
+	}
+
+	if ( (yc-outer_rad) > gr_screen.clip_bottom ) {
+		return;
+	}
+
+	int offset_x = ((do_resize) ? gr_screen.offset_x_unscaled : gr_screen.offset_x);
+	int offset_y = ((do_resize) ? gr_screen.offset_y_unscaled : gr_screen.offset_y);
+
+	GL_state.SetTextureSource(TEXTURE_SOURCE_NONE);
+	GL_state.SetAlphaBlendMode(ALPHA_BLEND_ALPHA_BLEND_ALPHA);
+	GL_state.SetZbufferType(ZBUFFER_TYPE_NONE);
+
+	GLfloat *arc;
+
+	gr_opengl_set_2d_matrix();
+	GL_state.Color(gr_screen.current_color.red, gr_screen.current_color.green, gr_screen.current_color.blue, gr_screen.current_color.alpha);
+	GL_state.Array.EnableClientVertex();
+
+	if (fill) {
+		arc = new GLfloat[segments * 2 + 2];
+
+		arc[0] = i2fl(xc);
+		arc[1] = i2fl(yc);
+
+		for (int i=2; i < segments * 2 + 2; i+=2) {
+			arc[i] = i2fl(xc + (x2 * outer_rad) + offset_x);
+			arc[i+1] = i2fl(yc + (y2 * outer_rad) + offset_y);
+
+			t = x2;
+			x2 = c * x1 - s * y1;
+			y2 = s * t + c * y1;
+
+			x1 = x2;
+			y1 = y2;
+		}
+
+		GL_state.Array.VertexPointer(2, GL_FLOAT, 0, arc);
+		glDrawArrays(GL_TRIANGLE_FAN, 0, segments + 1);
+	} else {
+		arc = new GLfloat[segments * 4];
+
+		for (int i=0; i < segments * 4; i+=4) {
+			arc[i] = i2fl(xc + (x2 * outer_rad) + offset_x);
+			arc[i+1] = i2fl(yc + (y2 * outer_rad) + offset_y);
+
+			arc[i+2] = i2fl(xc + (x2 * inner_rad) + offset_x);
+			arc[i+3] = i2fl(yc + (y2 * inner_rad) + offset_y);
+
+			t = x2;
+			x2 = c * x1 - s * y1;
+			y2 = s * t + c * y1;
+
+			x1 = x2;
+			y1 = y2;
+		}
+
+		GL_state.Array.VertexPointer(2, GL_FLOAT, 0, arc);
+		glDrawArrays(GL_QUAD_STRIP, 0, segments * 2);
+	}
+
+	GL_state.Array.DisableClientVertex();
+
+	GL_CHECK_FOR_ERRORS("end of opengl_arc()");
+
+	gr_opengl_end_2d_matrix();
+
+	delete [] arc;
+}
+
+void gr_opengl_curve(int xc, int yc, int r, int direction, int resize_mode)
 {
 	int a, b, p;
 
-	gr_resize_screen_pos(&xc, &yc);
+	if (resize_mode != GR_RESIZE_NONE) {
+		gr_resize_screen_pos(&xc, &yc, NULL, NULL, resize_mode);
+	}
 
 	if ( (xc + r) < gr_screen.clip_left ) {
 		return;
@@ -881,13 +1064,13 @@ void gr_opengl_curve(int xc, int yc, int r, int direction)
 
 			while (a < b) {
 				// Draw the first octant
-				gr_opengl_line(xc - b + 1, yc - a, xc - b, yc - a, false);
+				gr_opengl_line(xc - b + 1, yc - a, xc - b, yc - a, GR_RESIZE_NONE);
 
 				if (p < 0) {
 					p += (a << 2) + 6;
 				} else {
 					// Draw the second octant
-					gr_opengl_line(xc - a + 1, yc - b, xc - a, yc - b, false);
+					gr_opengl_line(xc - a + 1, yc - b, xc - a, yc - b, GR_RESIZE_NONE);
 					p += ((a - b) << 2) + 10;
 					b--;
 				}
@@ -903,13 +1086,13 @@ void gr_opengl_curve(int xc, int yc, int r, int direction)
 
 			while (a < b) {
 				// Draw the first octant
-				gr_opengl_line(xc + b - 1, yc - a, xc + b, yc - a, false);
+				gr_opengl_line(xc + b - 1, yc - a, xc + b, yc - a, GR_RESIZE_NONE);
 
 				if (p < 0) {
 					p += (a << 2) + 6;
 				} else {
 					// Draw the second octant
-					gr_opengl_line(xc + a - 1, yc - b, xc + a, yc - b, false);
+					gr_opengl_line(xc + a - 1, yc - b, xc + a, yc - b, GR_RESIZE_NONE);
 					p += ((a - b) << 2) + 10;
 					b--;
 				}
@@ -925,13 +1108,13 @@ void gr_opengl_curve(int xc, int yc, int r, int direction)
 
 			while (a < b) {
 				// Draw the first octant
-				gr_opengl_line(xc - b + 1, yc + a, xc - b, yc + a, false);
+				gr_opengl_line(xc - b + 1, yc + a, xc - b, yc + a, GR_RESIZE_NONE);
 
 				if (p < 0) {
 					p += (a << 2) + 6;
 				} else {
 					// Draw the second octant
-					gr_opengl_line(xc - a + 1, yc + b, xc - a, yc + b, false);
+					gr_opengl_line(xc - a + 1, yc + b, xc - a, yc + b, GR_RESIZE_NONE);
 					p += ((a - b) << 2) + 10;
 					b--;
 				}
@@ -945,13 +1128,13 @@ void gr_opengl_curve(int xc, int yc, int r, int direction)
 		case 3: {
 			while (a < b) {
 				// Draw the first octant
-				gr_opengl_line(xc + b - 1, yc + a, xc + b, yc + a, false);
+				gr_opengl_line(xc + b - 1, yc + a, xc + b, yc + a, GR_RESIZE_NONE);
 
 				if (p < 0) {
 					p += (a << 2) + 6;
 				} else {
 					// Draw the second octant
-					gr_opengl_line(xc + a - 1, yc + b, xc + a, yc + b, false);
+					gr_opengl_line(xc + a - 1, yc + b, xc + a, yc + b, GR_RESIZE_NONE);
 					p += ((a - b) << 2) + 10;
 					b--;
 				}
@@ -1744,20 +1927,19 @@ void gr_opengl_scaler(vertex *va, vertex *vb, bool bw_bitmap = false)
 
 
 // cross fade
-void gr_opengl_cross_fade(int bmap1, int bmap2, int x1, int y1, int x2, int y2, float pct)
+void gr_opengl_cross_fade(int bmap1, int bmap2, int x1, int y1, int x2, int y2, float pct, int resize_mode)
 {
    	gr_set_bitmap(bmap1, GR_ALPHABLEND_FILTER, GR_BITBLT_MODE_NORMAL, 1.0f - pct);
-	gr_bitmap(x1, y1);
+	gr_bitmap(x1, y1, resize_mode);
 
   	gr_set_bitmap(bmap2, GR_ALPHABLEND_FILTER, GR_BITBLT_MODE_NORMAL, pct);
-	gr_bitmap(x2, y2);
+	gr_bitmap(x2, y2, resize_mode);
 }
 
-void gr_opengl_shade(int x, int y, int w, int h, bool resize)
+void gr_opengl_shade(int x, int y, int w, int h, int resize_mode)
 {
-	if (resize) {
-		gr_resize_screen_pos(&x, &y);
-		gr_resize_screen_pos(&w, &h);
+	if (resize_mode != GR_RESIZE_NONE) {
+		gr_resize_screen_pos(&x, &y, &w, &h, resize_mode);
 	}
 
 	int x1 = (gr_screen.offset_x + x);
@@ -1846,7 +2028,7 @@ void gr_opengl_fade_out(int instantaneous)
 	// Empty - DDOI
 }
 
-void opengl_bitmap_ex_internal(int x, int y, int w, int h, int sx, int sy, bool resize)
+void opengl_bitmap_ex_internal(int x, int y, int w, int h, int sx, int sy, int resize_mode)
 {
 	if ( (w < 1) || (h < 1) ) {
 		return;
@@ -1865,7 +2047,7 @@ void opengl_bitmap_ex_internal(int x, int y, int w, int h, int sx, int sy, bool
 		return;
 	}
 
-	if ( resize && (gr_screen.custom_size || (gr_screen.rendering_to_texture != -1)) ) {
+	if ( resize_mode != GR_RESIZE_NONE && (gr_screen.custom_size || (gr_screen.rendering_to_texture != -1)) ) {
 		do_resize = 1;
 	} else {
 		do_resize = 0;
@@ -1885,8 +2067,8 @@ void opengl_bitmap_ex_internal(int x, int y, int w, int h, int sx, int sy, bool
 	y2 = y1 + i2fl(h);
 
 	if (do_resize) {
-		gr_resize_screen_posf(&x1, &y1);
-		gr_resize_screen_posf(&x2, &y2);
+		gr_resize_screen_posf(&x1, &y1, NULL, NULL, resize_mode);
+		gr_resize_screen_posf(&x2, &y2, NULL, NULL, resize_mode);
 	}
 
 	GL_state.Color(255, 255, 255, (GLubyte)(gr_screen.current_alpha * 255));
@@ -1896,7 +2078,7 @@ void opengl_bitmap_ex_internal(int x, int y, int w, int h, int sx, int sy, bool
 
 
 //these are penguins bitmap functions
-void gr_opengl_bitmap_ex(int x, int y, int w, int h, int sx, int sy, bool resize)
+void gr_opengl_bitmap_ex(int x, int y, int w, int h, int sx, int sy, int resize_mode)
 {
 	int reclip;
 #ifndef NDEBUG
@@ -1912,7 +2094,7 @@ void gr_opengl_bitmap_ex(int x, int y, int w, int h, int sx, int sy, bool resize
 
 	bm_get_info(gr_screen.current_bitmap, &bw, &bh);
 
-	if ( resize && (gr_screen.custom_size || (gr_screen.rendering_to_texture != -1)) ) {
+	if ( resize_mode != GR_RESIZE_NONE && (gr_screen.custom_size || (gr_screen.rendering_to_texture != -1)) ) {
 		do_resize = 1;
 	} else {
 		do_resize = 0;
@@ -1948,7 +2130,7 @@ void gr_opengl_bitmap_ex(int x, int y, int w, int h, int sx, int sy, bool resize
 		}
 
 		if ( dy1 < clip_top ) {
-			sy += clip_top-dx1;
+			sy += clip_top-dy1;
 			dy1 = clip_top;
 		}
 
@@ -2010,16 +2192,16 @@ void gr_opengl_bitmap_ex(int x, int y, int w, int h, int sx, int sy, bool resize
 #endif
 
 	// We now have dx1,dy1 and dx2,dy2 and sx, sy all set validly within clip regions.
-	opengl_bitmap_ex_internal(dx1, dy1, (dx2 - dx1 + 1), (dy2 - dy1 + 1), sx, sy, resize);
+	opengl_bitmap_ex_internal(dx1, dy1, (dx2 - dx1 + 1), (dy2 - dy1 + 1), sx, sy, resize_mode);
 }
 
-/*void gr_opengl_bitmap(int x, int y, bool resize)
+/*void gr_opengl_bitmap(int x, int y, int resize_mode)
 {
 	int w, h, do_resize;
 
 	bm_get_info( gr_screen.current_bitmap, &w, &h, NULL );
 
-	if ( resize && (gr_screen.custom_size || (gr_screen.rendering_to_texture != -1)) ) {
+	if ( resize_mode != GR_RESIZE_NONE && (gr_screen.custom_size || (gr_screen.rendering_to_texture != -1)) ) {
 		do_resize = 1;
 	} else {
 		do_resize = 0;
@@ -2047,7 +2229,7 @@ void gr_opengl_bitmap_ex(int x, int y, int w, int h, int sx, int sy, bool resize
 	if ( sy >= h ) return;
 
 	// Draw bitmap bm[sx,sy] into (dx1,dy1)-(dx2,dy2)
-	gr_opengl_bitmap_ex(dx1, dy1, dx2-dx1+1, dy2-dy1+1, sx, sy, resize);
+	gr_opengl_bitmap_ex(dx1, dy1, dx2-dx1+1, dy2-dy1+1, sx, sy, resize_mode);
 }*/
 
 void opengl_render_timer_bar(int colour, float x, float y, float w, float h)
@@ -2138,6 +2320,7 @@ void opengl_setup_scene_textures()
 	if ( !Use_GLSL || Cmdline_no_fbo || !Is_Extension_Enabled(OGL_EXT_FRAMEBUFFER_OBJECT) ) {
 		Cmdline_postprocess = 0;
 		Cmdline_softparticles = 0;
+		Cmdline_fb_explosions = 0;
 
 		Scene_color_texture = 0;
 		Scene_effect_texture = 0;
@@ -2373,11 +2556,9 @@ void opengl_scene_texture_shutdown()
 		Scene_framebuffer = 0;
 	}
 
-	if ( Distortion_texture ) {
-		glDeleteTextures(2, Distortion_texture);
-		Distortion_texture[0] = 0;
-		Distortion_texture[1] = 0;
-	}
+	glDeleteTextures(2, Distortion_texture);
+	Distortion_texture[0] = 0;
+	Distortion_texture[1] = 0;
 
 	if ( Distortion_framebuffer ) {
 		vglDeleteFramebuffersEXT(1, &Distortion_framebuffer);
@@ -2481,7 +2662,7 @@ void gr_opengl_scene_texture_end()
 			GL_state.Array.EnableClientTexture();
 			GL_state.Array.TexPointer(2, GL_FLOAT, 0, uvcoords);
 		
-			glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
+			glDrawArrays(GL_QUADS, 0, 4);
 			
 			GL_state.Array.DisableClientVertex();
 			GL_state.Array.DisableClientTexture();
@@ -2586,9 +2767,9 @@ void gr_opengl_update_distortion()
 	GL_state.Texture.Disable();
 
 	SCP_vector<ubyte> colours;
-	SCP_vector<GLfloat> vertex;
+	SCP_vector<GLfloat> distortion_vertex;
 	colours.reserve(33 * 4);
-	vertex.reserve(33 * 2);
+	distortion_vertex.reserve(33 * 2);
 	for(int i = 0; i < 33; i++)
 	{
 		colours.push_back((ubyte) rand()%256);
@@ -2596,12 +2777,12 @@ void gr_opengl_update_distortion()
 		colours.push_back(255);
 		colours.push_back(255);
 
-		vertex.push_back(0.04f);
-		vertex.push_back((float)gr_screen.max_h*0.03125f*i);
+		distortion_vertex.push_back(0.04f);
+		distortion_vertex.push_back((float)gr_screen.max_h*0.03125f*i);
 	}
 
 	GL_state.Array.EnableClientVertex();
-	GL_state.Array.VertexPointer(2, GL_FLOAT, 0, &vertex.front());
+	GL_state.Array.VertexPointer(2, GL_FLOAT, 0, &distortion_vertex.front());
 
 	GL_state.Array.EnableClientColor();
 	GL_state.Array.ColorPointer(4, GL_UNSIGNED_BYTE, 0, &colours.front());
