@@ -13,18 +13,20 @@
 
 #include "graphics/gropenglstate.h"
 
-void gr_opengl_aabitmap_ex(int x, int y, int w, int h, int sx, int sy, bool resize, bool mirror);
-void gr_opengl_aabitmap(int x, int y, bool resize, bool mirror);
-void gr_opengl_string(int sx, int sy, const char *s, bool resize = true);
-void gr_opengl_line(int x1,int y1,int x2,int y2, bool resize);
+void gr_opengl_aabitmap_ex(int x, int y, int w, int h, int sx, int sy, int resize_mode, bool mirror);
+void gr_opengl_aabitmap(int x, int y, int resize_mode, bool mirror);
+void gr_opengl_string(int sx, int sy, const char *s, int resize_mode = GR_RESIZE_FULL);
+void gr_opengl_line(int x1,int y1,int x2,int y2, int resize_mode);
 void gr_opengl_aaline(vertex *v1, vertex *v2);
-void gr_opengl_pixel(int x, int y, bool resize);
-void gr_opengl_gradient(int x1, int y1, int x2, int y2, bool resize);
-void gr_opengl_circle(int xc, int yc, int d, bool resize);
-void gr_opengl_curve(int xc, int yc, int r, int direction);
+void gr_opengl_pixel(int x, int y, int resize_mode);
+void gr_opengl_gradient(int x1, int y1, int x2, int y2, int resize_mode);
+void gr_opengl_circle(int xc, int yc, int d, int resize_mode);
+void gr_opengl_unfilled_circle(int xc, int yc, int d, int resize_mode);
+void gr_opengl_arc(int xc, int yc, float r, float angle_start, float angle_end, bool fill, int resize_mode);
+void gr_opengl_curve(int xc, int yc, int r, int direction, int resize_mode);
 void gr_opengl_scaler(vertex *va, vertex *vb, bool bw_bitmap );
-void gr_opengl_cross_fade(int bmap1, int bmap2, int x1, int y1, int x2, int y2, float pct);
-void gr_opengl_shade(int x, int y, int w, int h, bool resize);
+void gr_opengl_cross_fade(int bmap1, int bmap2, int x1, int y1, int x2, int y2, float pct, int resize_mode);
+void gr_opengl_shade(int x, int y, int w, int h, int resize_mode);
 void gr_opengl_flash(int r, int g, int b);
 void gr_opengl_flash_alpha(int r, int g, int b, int a);
 void gr_opengl_fade_in(int instantaneous);
@@ -32,7 +34,7 @@ void gr_opengl_fade_out(int instantaneous);
 void gr_opengl_tmapper(int nverts, vertex **verts, uint flags);
 void gr_opengl_render(int nverts, vertex *verts, uint flags);
 void gr_opengl_render_effect(int nverts, vertex *verts, float *radius_list, uint flags);
-void gr_opengl_bitmap_ex(int x, int y, int w, int h, int sx, int sy, bool resize);
+void gr_opengl_bitmap_ex(int x, int y, int w, int h, int sx, int sy, int resize_mode);
 void gr_opengl_update_distortion();
 
 void opengl_render_timer_bar(int colour, float x, float y, float w, float h);
