@@ -192,6 +192,10 @@ int gr_opengl_create_stream_buffer()
 {
 	opengl_vertex_buffer buffer;
 
+	if ( !Use_VBOs ) {
+		return -1;
+	}
+
 	vglGenBuffersARB(1, &buffer.vbo);
 
 	GL_vertex_buffers.push_back(buffer);
@@ -484,7 +488,7 @@ void mix_two_team_colors(team_color* dest, team_color* a, team_color* b, float m
 }
 
 void gr_opengl_set_team_color(const SCP_string &team, const SCP_string &secondaryteam, fix timestamp, int fadetime) {
-	if (secondaryteam == "<none>") {
+	if (!stricmp(secondaryteam.c_str(), "none")) {
 		if (Team_Colors.find(team) != Team_Colors.end()) {
 			Current_team_color = &Team_Colors[team];
 			Using_Team_Color = true;
