@@ -398,14 +398,20 @@ disabled_views(VM_EXTERNAL | VM_DEAD_VIEW | VM_WARP_CHASE | VM_PADLOCK_ANY), cus
 	}
 }
 
+HudGauge::~HudGauge() {};
+
 void HudGauge::initPosition(int x, int y)
 {
-	Assert(x >= 0 && y >= 0);
-
 	position[0] = x;
 	position[1] = y;
 }
 
+void HudGauge::getPosition(int *x, int *y)
+{
+	*x = position[0];
+	*y = position[1];
+}
+
 void HudGauge::initBaseResolution(int w, int h)
 {
 	Assert(w >= 640 && h >= 480);
@@ -419,14 +425,14 @@ void HudGauge::initSlew(bool slew)
 	reticle_follow = slew;
 }
 
-void HudGauge::initFont(int font)
+void HudGauge::initFont(int input_font_num)
 {
-	if ( font >= 0 && font < Num_fonts) {
-		font_num = font;
+	if ( input_font_num >= 0 && input_font_num < Num_fonts) {
+		font_num = input_font_num;
 	}
 }
 
-char* HudGauge::getCustomGaugeName()
+const char* HudGauge::getCustomGaugeName()
 {
 	return custom_name;
 }
@@ -468,7 +474,7 @@ void HudGauge::updateCustomGaugeText(const char* txt)
 	custom_text = txt;
 }
 
-void HudGauge::updateCustomGaugeText(SCP_string& txt)
+void HudGauge::updateCustomGaugeText(const SCP_string& txt)
 {
 	if(!custom_gauge) {
 		return;
@@ -580,9 +586,9 @@ void HudGauge::updateActive(bool show)
 	active = show;
 }
 
-void HudGauge::initRenderStatus(bool render)
+void HudGauge::initRenderStatus(bool do_render)
 {
-	off_by_default = !render;
+	off_by_default = !do_render;
 }
 
 bool HudGauge::isOffbyDefault()
@@ -691,7 +697,7 @@ void HudGauge::render(float frametime)
 		char *text = new char[custom_text.size()+1];
 		strcpy(text, custom_text.c_str());
 
-		hud_num_make_mono(text);
+		hud_num_make_mono(text, font_num);
 		renderString(position[0] + textoffset_x, position[1] + textoffset_y, text);
 
 		delete[] text;
@@ -702,12 +708,12 @@ void HudGauge::render(float frametime)
 	}
 }
 
-void HudGauge::renderString(int x, int y, char *str)
+void HudGauge::renderString(int x, int y, const char *str)
 {
 	int nx = 0, ny = 0;
 
 	if ( gr_screen.rendering_to_texture != -1 ) {
-		gr_set_screen_scale(canvas_w, canvas_h, target_w, target_h);
+		gr_set_screen_scale(canvas_w, canvas_h, -1, -1, target_w, target_h, true);
 	} else {
 		if ( reticle_follow ) {
 			nx = HUD_nose_x;
@@ -725,12 +731,12 @@ void HudGauge::renderString(int x, int y, char *str)
 	gr_reset_screen_scale();
 }
 
-void HudGauge::renderString(int x, int y, int gauge_id, char *str)
+void HudGauge::renderString(int x, int y, int gauge_id, const char *str)
 {
 	int nx = 0, ny = 0;
 
 	if ( gr_screen.rendering_to_texture != -1 ) {
-		gr_set_screen_scale(canvas_w, canvas_h, target_w, target_h);
+		gr_set_screen_scale(canvas_w, canvas_h, -1, -1, target_w, target_h, true);
 	} else {
 		if ( reticle_follow ) {
 			nx = HUD_nose_x;
@@ -753,7 +759,7 @@ void HudGauge::renderString(int x, int y, int gauge_id, char *str)
 	gr_reset_screen_scale();
 }
 
-void HudGauge::renderStringAlignCenter(int x, int y, int area_width, char *s)
+void HudGauge::renderStringAlignCenter(int x, int y, int area_width, const char *s)
 {
 	int w, h;
 
@@ -761,28 +767,30 @@ void HudGauge::renderStringAlignCenter(int x, int y, int area_width, char *s)
 	renderString(x + ((area_width - w) / 2), y, s);
 }
 
-void HudGauge::renderPrintf(int x, int y, char* format, ...)
+void HudGauge::renderPrintf(int x, int y, const char* format, ...)
 {
 	char tmp[256] = "";
-	va_list args;	
+	va_list args;
 	
 	// format the text
 	va_start(args, format);
-	vsprintf(tmp, format, args);
+	vsnprintf(tmp, sizeof(tmp)-1, format, args);
 	va_end(args);
+	tmp[sizeof(tmp)-1] = '\0';
 
 	renderString(x, y, tmp);
 }
 
-void HudGauge::renderPrintf(int x, int y, int gauge_id, char* format, ...)
+void HudGauge::renderPrintf(int x, int y, int gauge_id, const char* format, ...)
 {
 	char tmp[256] = "";
-	va_list args;	
+	va_list args;
 	
 	// format the text
 	va_start(args, format);
-	vsprintf(tmp, format, args);
+	vsnprintf(tmp, sizeof(tmp)-1, format, args);
 	va_end(args);
+	tmp[sizeof(tmp)-1] = '\0';
 
 	renderString(x, y, gauge_id, tmp);
 }
@@ -798,7 +806,7 @@ void HudGauge::renderBitmapColor(int frame, int x, int y)
 	emp_hud_jitter(&x, &y);
 
 	if ( gr_screen.rendering_to_texture != -1 ) {
-		gr_set_screen_scale(canvas_w, canvas_h, target_w, target_h);
+		gr_set_screen_scale(canvas_w, canvas_h, -1, -1, target_w, target_h, true);
 	} else {
 		if ( reticle_follow ) {
 			nx = HUD_nose_x;
@@ -828,7 +836,7 @@ void HudGauge::renderBitmap(int x, int y)
 	emp_hud_jitter(&x, &y);
 
 	if ( gr_screen.rendering_to_texture != -1 ) {
-		gr_set_screen_scale(canvas_w, canvas_h, target_w, target_h);
+		gr_set_screen_scale(canvas_w, canvas_h, -1, -1, target_w, target_h, true);
 	} else {
 		if ( reticle_follow ) {
 			nx = HUD_nose_x;
@@ -866,7 +874,7 @@ void HudGauge::renderBitmapEx(int frame, int x, int y, int w, int h, int sx, int
 	gr_set_bitmap(frame);
 
 	if( gr_screen.rendering_to_texture != -1 ) {
-		gr_set_screen_scale(canvas_w, canvas_h, target_w, target_h);
+		gr_set_screen_scale(canvas_w, canvas_h, -1, -1, target_w, target_h, true);
 	} else {
 		if ( reticle_follow ) {
 			nx = HUD_nose_x;
@@ -890,7 +898,7 @@ void HudGauge::renderLine(int x1, int y1, int x2, int y2)
 	int nx = 0, ny = 0;
 
 	if ( gr_screen.rendering_to_texture != -1 ) {
-		gr_set_screen_scale(canvas_w, canvas_h, target_w, target_h);
+		gr_set_screen_scale(canvas_w, canvas_h, -1, -1, target_w, target_h, true);
 	} else {
 		if ( reticle_follow ) {
 			nx = HUD_nose_x;
@@ -913,7 +921,7 @@ void HudGauge::renderGradientLine(int x1, int y1, int x2, int y2)
 	int nx = 0, ny = 0;
 
 	if ( gr_screen.rendering_to_texture != -1 ) {
-		gr_set_screen_scale(canvas_w, canvas_h, target_w, target_h);
+		gr_set_screen_scale(canvas_w, canvas_h, -1, -1, target_w, target_h, true);
 	} else {
 		if(reticle_follow) {
 			nx = HUD_nose_x;
@@ -936,7 +944,7 @@ void HudGauge::renderRect(int x, int y, int w, int h)
 	int nx = 0, ny = 0;
 
 	if ( gr_screen.rendering_to_texture != -1 ) {
-		gr_set_screen_scale(canvas_w, canvas_h, target_w, target_h);
+		gr_set_screen_scale(canvas_w, canvas_h, -1, -1, target_w, target_h, true);
 	} else {
 		if ( reticle_follow ) {
 			nx = HUD_nose_x;
@@ -959,7 +967,7 @@ void HudGauge::renderCircle(int x, int y, int diameter)
 	int nx = 0, ny = 0;
 
 	if ( gr_screen.rendering_to_texture != -1 ) {
-		gr_set_screen_scale(canvas_w, canvas_h, target_w, target_h);
+		gr_set_screen_scale(canvas_w, canvas_h, -1, -1, target_w, target_h, true);
 	} else {
 		if ( reticle_follow ) {
 			nx = HUD_nose_x;
@@ -983,7 +991,7 @@ void HudGauge::setClip(int x, int y, int w, int h)
 	int hy = fl2i(HUD_offset_y);
 
 	if ( gr_screen.rendering_to_texture != -1 ) {
-		gr_set_screen_scale(canvas_w, canvas_h, target_w, target_h);
+		gr_set_screen_scale(canvas_w, canvas_h, -1, -1, target_w, target_h, true);
 
 		hx = display_offset_x;
 		hy = display_offset_y;
@@ -1005,10 +1013,9 @@ void HudGauge::setClip(int x, int y, int w, int h)
 		gr_resize_screen_pos(&hx, &hy);
 
 		gr_set_screen_scale(base_w, base_h);
-		gr_resize_screen_pos(&x, &y);
-		gr_resize_screen_pos(&w, &h);
+		gr_resize_screen_pos(&x, &y, &w, &h);
 
-		gr_set_clip(hx+x, hy+y, w, h, false);
+		gr_set_clip(hx+x, hy+y, w, h, GR_RESIZE_NONE);
 	}
 
 	gr_reset_screen_scale();
@@ -1020,7 +1027,7 @@ void HudGauge::resetClip()
 	int w, h;
 
 	if ( gr_screen.rendering_to_texture != -1 ) {
-		gr_set_screen_scale(canvas_w, canvas_h, target_w, target_h);
+		gr_set_screen_scale(canvas_w, canvas_h, -1, -1, target_w, target_h, true);
 		
 		hx = display_offset_x;
 		hy = display_offset_y;
@@ -1042,7 +1049,7 @@ void HudGauge::resetClip()
 		h = gr_screen.max_h;
 
 		// clip the screen based on the actual resolution
-		gr_set_clip(hx, hy, w, h, false);
+		gr_set_clip(hx, hy, w, h, GR_RESIZE_NONE);
 	}
 
 	gr_reset_screen_scale();
@@ -1135,7 +1142,7 @@ bool HudGauge::canRender()
 	return true;
 }
 
-void HudGauge::initCockpitTarget(char* display_name, int _target_x, int _target_y, int _target_w, int _target_h, int _canvas_w, int _canvas_h)
+void HudGauge::initCockpitTarget(const char* display_name, int _target_x, int _target_y, int _target_w, int _target_h, int _canvas_w, int _canvas_h)
 {
 	if ( strlen(display_name) <= 0 ) {
 		return;
@@ -1172,7 +1179,7 @@ bool HudGauge::setupRenderCanvas(int render_target)
 	return false;
 }
 
-void HudGauge::setCockpitTarget(cockpit_display *display)
+void HudGauge::setCockpitTarget(const cockpit_display *display)
 {
 	if ( display == NULL ) {
 		return;
@@ -1304,7 +1311,7 @@ void hud_close()
 	num_gauges = default_hud_gauges.size();
 
 	for(j = 0; j < num_gauges; j++) {
-		vm_free(default_hud_gauges[j]);
+		delete default_hud_gauges[j];
 		default_hud_gauges[j] = NULL;
 	}
 	default_hud_gauges.clear();
@@ -1578,7 +1585,7 @@ void hud_update_frame(float frametime)
 	}
 
 	// Make sure that the player isn't targeting a 3rd stage local ssm
-	if (Objects[Player_ai->target_objnum].type == OBJ_WEAPON)
+	if (Player_ai->target_objnum >= 0 && Objects[Player_ai->target_objnum].type == OBJ_WEAPON)
 	{
 		if (Weapons[Objects[Player_ai->target_objnum].instance].lssm_stage==3)
 		{
@@ -1689,7 +1696,7 @@ void HudGaugeMissionTime::initValueOffsets(int x, int y)
 	time_val_offsets[1] = y;
 }
 
-void HudGaugeMissionTime::initBitmaps(char *fname)
+void HudGaugeMissionTime::initBitmaps(const char *fname)
 {
 	time_gauge.first_frame = bm_load_animation(fname, &time_gauge.num_frames);
 
@@ -1746,7 +1753,11 @@ void hud_maybe_display_supernova()
 	}
 
 	gr_set_color_fast(&Color_bright_red);
-	gr_printf(Supernova_coords[gr_screen.res][0], Supernova_coords[gr_screen.res][1], "Supernova Warning: %.2f s", time_left);
+	if (Lcl_pl) {
+		gr_printf(Supernova_coords[gr_screen.res][0], Supernova_coords[gr_screen.res][1], "Wybuch supernowej: %.2f s", time_left);
+	} else {
+		gr_printf(Supernova_coords[gr_screen.res][0], Supernova_coords[gr_screen.res][1], "Supernova Warning: %.2f s", time_left);
+	}
 }
 
 /**
@@ -1991,7 +2002,7 @@ void HudGaugeDamage::initLineHeight(int h)
 	line_h = h;
 }
 
-void HudGaugeDamage::initBitmaps(char *fname_top, char *fname_middle, char *fname_bottom)
+void HudGaugeDamage::initBitmaps(const char *fname_top, const char *fname_middle, const char *fname_bottom)
 {
 	damage_top.first_frame = bm_load_animation(fname_top, &damage_top.num_frames);
 	if ( damage_top.first_frame == -1 ) {
@@ -2073,7 +2084,7 @@ void HudGaugeDamage::render(float frametime)
 			screen_integrity = 1;
 		}
 		sprintf(buf, XSTR( "%d%%", 219), screen_integrity);
-		hud_num_make_mono(buf);
+		hud_num_make_mono(buf, font_num);
 		gr_get_string_size(&w, &h, buf);
 		if ( screen_integrity < 30 ) {
 			gr_set_color_fast(&Color_red);
@@ -2187,7 +2198,7 @@ void HudGaugeDamage::render(float frametime)
 		}
 
 		sprintf(buf, XSTR( "%d%%", 219), best_str);
-		hud_num_make_mono(buf);
+		hud_num_make_mono(buf, font_num);
 		gr_get_string_size(&w, &h, buf);
 		renderString(position[0] + subsys_integ_val_offset_x - w, sy, buf);
 		sy += line_h;
@@ -2205,7 +2216,7 @@ void HudGaugeDamage::render(float frametime)
 /** 
  * @brief Initialise the members of the ::hud_anim struct to default values
  */
-void hud_anim_init(hud_anim *ha, int sx, int sy, char *filename)
+void hud_anim_init(hud_anim *ha, int sx, int sy, const char *filename)
 {
 	ha->first_frame		= -1;
 	ha->num_frames		= 0;
@@ -2264,7 +2275,7 @@ int hud_anim_load(hud_anim *ha)
  * @param resize		Resize for non-standard resolutions
  * @param mirror		Mirror along y-axis so icon points left instead of right
  */
-int hud_anim_render(hud_anim *ha, float frametime, int draw_alpha, int loop, int hold_last, int reverse, bool resize, bool mirror)
+int hud_anim_render(hud_anim *ha, float frametime, int draw_alpha, int loop, int hold_last, int reverse, int resize_mode, bool mirror)
 {
 	int framenum;
 
@@ -2299,9 +2310,9 @@ int hud_anim_render(hud_anim *ha, float frametime, int draw_alpha, int loop, int
 	if(emp_should_blit_gauge()){
 		gr_set_bitmap(ha->first_frame + framenum);
 		if ( draw_alpha ){
-			gr_aabitmap(ha->sx, ha->sy, resize, mirror);
+			gr_aabitmap(ha->sx, ha->sy, resize_mode, mirror);
 		} else {
-			gr_bitmap(ha->sx, ha->sy, resize);
+			gr_bitmap(ha->sx, ha->sy, resize_mode);
 		}
 	}
 
@@ -2311,12 +2322,18 @@ int hud_anim_render(hud_anim *ha, float frametime, int draw_alpha, int loop, int
 /**
  * @brief Convert a number string to use mono-spaced 1 character
  */
-void hud_num_make_mono(char *num_str)
+void hud_num_make_mono(char *num_str, int font_num)
 {
-	int len, i, sc;
-	len = strlen(num_str);
+	int len, i;
+	ubyte sc;
 
-	sc = Lcl_special_chars;
+	sc = lcl_get_font_index(font_num);
+	if (sc == 0) {
+		// specified font has no mono-spaced 1, make do with non-mono-spaced 1
+		return;
+	}
+
+	len = strlen(num_str);
 	for ( i = 0; i < len; i++ ) {
 		if ( num_str[i] == '1' ) {
 			num_str[i] = (char)(sc + 1);
@@ -2334,7 +2351,7 @@ void hud_init_text_flash_gauge()
 	Hud_text_flash_interval = 0;
 }
 
-void hud_start_text_flash(char *txt, int t, int interval)
+void hud_start_text_flash(const char *txt, int t, int interval)
 {
 	// bogus
 	if(txt == NULL){
@@ -2383,11 +2400,8 @@ int HudGaugeTextWarnings::maybeTextFlash()
 
 void HudGaugeTextWarnings::render(float frametime)
 {
-	if ( timestamp_elapsed(Hud_text_flash_timer) || !Hud_text_flash) {
-		return;
-	}
-
-	if(strlen(Hud_text_flash) <= 0) {
+	// note: Hud_text_flash globally allocated, address can't be NULL
+	if ( timestamp_elapsed(Hud_text_flash_timer) || Hud_text_flash[0] == '\0' ) {
 		return;
 	}
 
@@ -2414,7 +2428,7 @@ HudGauge(HUD_OBJECT_KILLS, HUD_KILLS_GAUGE, false, false, (VM_EXTERNAL | VM_DEAD
 {
 }
 
-void HudGaugeKills::initBitmaps(char *fname)
+void HudGaugeKills::initBitmaps(const char *fname)
 {
 	Kills_gauge.first_frame = bm_load_animation(fname, &Kills_gauge.num_frames);
 	if ( Kills_gauge.first_frame == -1 ) {
@@ -2475,7 +2489,7 @@ HudGauge(HUD_OBJECT_LAG, HUD_LAG_GAUGE, false, false, (VM_EXTERNAL | VM_DEAD_VIE
 
 }
 
-void HudGaugeLag::initBitmaps(char *fname)
+void HudGaugeLag::initBitmaps(const char *fname)
 {
 	Netlag_icon.first_frame = bm_load_animation(fname, &Netlag_icon.num_frames);
 
@@ -2799,7 +2813,7 @@ void HudGaugeSupport::initTextDockValueOffsetX(int x)
 	text_dock_val_offset_x = x;
 }
 
-void HudGaugeSupport::initBitmaps(char *fname)
+void HudGaugeSupport::initBitmaps(const char *fname)
 {
 	background.first_frame = bm_load_animation(fname, &background.num_frames);
 	if ( background.first_frame == -1 ) {
@@ -3323,7 +3337,7 @@ void HudGaugeObjectiveNotify::initRedAlertValueOffsetY(int y)
 	Red_text_val_offset_y = y;
 }
 
-void HudGaugeObjectiveNotify::initBitmaps(char *fname)
+void HudGaugeObjectiveNotify::initBitmaps(const char *fname)
 {
 	Objective_display_gauge.first_frame = bm_load_animation(fname, &Objective_display_gauge.num_frames);
 	if ( Objective_display_gauge.first_frame == -1 ) {
@@ -3582,14 +3596,13 @@ void HUD_set_offsets(object *viewer_obj, int wiggedy_wack, matrix *eye_orient)
 	if ( (viewer_obj == Player_obj) && wiggedy_wack ){		
 		vec3d tmp;
 		vertex pt;
-		ubyte flags;		
 
 		HUD_offset_x = 0.0f;
 		HUD_offset_y = 0.0f;
 
 		vm_vec_scale_add( &tmp, &Eye_position, &eye_orient->vec.fvec, 100.0f );
 		
-		flags = g3_rotate_vertex(&pt,&tmp);
+		(void) g3_rotate_vertex(&pt,&tmp);
 
 		g3_project_vertex(&pt);
 
@@ -3764,9 +3777,9 @@ void hud_page_in()
 	}
 }
 
-HudGauge* hud_get_gauge(char* name)
+HudGauge* hud_get_gauge(const char* name)
 {
-	char* gauge_name;
+	const char* gauge_name;
 	size_t j;
 
 	// go through all gauges and return the gauge that matches
@@ -3907,7 +3920,11 @@ void HudGaugeSupernova::render(float frametime)
 	}
 
 	gr_set_color_fast(&Color_bright_red);
-	renderPrintf(position[0], position[1], "Supernova Warning: %.2f s", time_left);
+	if (Lcl_pl) {
+		renderPrintf(position[0], position[1], "Wybuch supernowej: %.2f s", time_left);
+	} else {
+		renderPrintf(position[0], position[1], "Supernova Warning: %.2f s", time_left);
+	}
 }
 
 HudGaugeFlightPath::HudGaugeFlightPath():
@@ -3915,7 +3932,7 @@ HudGauge(HUD_OBJECT_FLIGHT_PATH, HUD_CENTER_RETICLE, false, false, VM_EXTERNAL |
 {
 }
 
-void HudGaugeFlightPath::initBitmap(char *fname)
+void HudGaugeFlightPath::initBitmap(const char *fname)
 {
 	Marker.first_frame = bm_load_animation(fname, &Marker.num_frames);
 
