@@ -26,6 +26,7 @@
 #include "mission/missionparse.h"
 #include "iff_defs/iff_defs.h"
 #include "network/multi.h"
+#include "debugconsole/console.h"
 
 
 vec3d lock_world_pos;
@@ -64,9 +65,7 @@ int Lock_gauge_half_h[GR_NUM_RESOLUTIONS] = {
 	25
 };
 
-// hud_frames Lock_gauge;
 int Lock_gauge_loaded = 0;
-hud_anim Lock_gauge;
 int Lock_gauge_draw = 0;
 int Lock_gauge_draw_stamp = -1;
 #define LOCK_GAUGE_BLINK_RATE			5			// blinks/sec
@@ -79,7 +78,6 @@ int Lockspin_half_h[NUM_HUD_RETICLE_STYLES][GR_NUM_RESOLUTIONS] = {
 	{ 16, 26 },
 	{ 32, 52 }
 };
-hud_anim	Lock_anim;
 
 char Lock_fname[NUM_HUD_RETICLE_STYLES][GR_NUM_RESOLUTIONS][MAX_FILENAME_LEN] =
 {
@@ -300,7 +298,9 @@ void hud_lock_reset(float lock_time_scale)
 
 	swp = &Player_ship->weapons;
     
-	if (swp->current_secondary_bank > 0) {
+	if ((swp->current_secondary_bank >= 0) && (swp->secondary_bank_weapons[swp->current_secondary_bank] >= 0)) {
+		Assert(swp->current_secondary_bank < MAX_SHIP_SECONDARY_BANKS);
+		Assert(swp->secondary_bank_weapons[swp->current_secondary_bank] < MAX_WEAPON_TYPES);
 		wip = &Weapon_info[swp->secondary_bank_weapons[swp->current_secondary_bank]];
 		Player->lock_time_to_target = i2fl(wip->min_lock_time*lock_time_scale);
 	} else {
@@ -321,7 +321,7 @@ void hud_lock_reset(float lock_time_scale)
 	Lock_gauge_draw = 0;
 
 	// reset the lock anim time elapsed
-	Lock_anim.time_elapsed = 0.0f;
+//	Lock_anim.time_elapsed = 0.0f;
 }
 
 // Determine if the locking code has a point to track
@@ -334,7 +334,7 @@ int hud_lock_has_homing_point()
 }
 
 int Nebula_sec_range = 0;
-DCF_BOOL(nebula_sec_range, Nebula_sec_range)
+DCF_BOOL(nebula_sec_range, Nebula_sec_range);
 
 int hud_lock_world_pos_in_range(vec3d *target_world_pos, vec3d *vec_to_target)
 {
