@@ -15,7 +15,7 @@
 #include "hud/hudtargetbox.h"
 
 
-extern void hud_target_show_dist_on_bracket(int x, int y, float distance);
+extern void hud_target_show_dist_on_bracket(int x, int y, float distance, int font_num);
 extern void draw_brackets_square_quick(int x1, int y1, int x2, int y2, int thick);
 
 /**
@@ -35,6 +35,8 @@ void hud_draw_navigation()
 		vec3d *target_pos = Navs[CurrentNav].GetPosition();
 
 		color NavColor;
+        
+        memset(&target_point, 0, sizeof(target_point));
 
 		unsigned int alpha = HUD_COLOR_ALPHA_MAX * 16;
 
