@@ -166,6 +166,34 @@ void hud_targetbox_truncate_subsys_name(char *outstr)
 		} else if ( strstr(outstr, "laser") || strstr(outstr, "turret") || strstr(outstr, "missile") ) {
 			strcpy(outstr, "tourelle");
 		} 
+	} else if(Lcl_pl){	
+		if ( strstr(outstr, "communication") )	{
+			strcpy(outstr, "komunikacja");
+		} else if ( !stricmp(outstr, "weapons") ) {
+			strcpy(outstr, "uzbrojenie");
+		} else if ( strstr(outstr, "engine") || strstr(outstr, "Engine")) {
+			strcpy(outstr, "silnik");
+		} else if ( !stricmp(outstr, "sensors") ) {
+			strcpy(outstr, "sensory");
+		} else if ( strstr(outstr, "navigat") ) {
+			strcpy(outstr, "nawigacja");
+		} else if ( strstr(outstr, "fighterbay") || strstr(outstr, "Fighterbay") ) {
+			strcpy(outstr, "dok my\x9Cliw.");
+		} else if ( strstr(outstr, "missile") ) {
+			strcpy(outstr, "wie\xBF. rakiet.");
+		} else if ( strstr(outstr, "laser") || strstr(outstr, "turret") ) {
+			strcpy(outstr, "wie\xBFyczka");
+		} else if ( strstr(outstr, "Command Tower") || strstr(outstr, "Bridge") ) {
+			strcpy(outstr, "mostek");
+		} else if ( strstr(outstr, "Barracks") ) {
+			strcpy(outstr, "koszary");
+		} else if ( strstr(outstr, "Reactor") ) {
+			strcpy(outstr, "reaktor");
+		} else if ( strstr(outstr, "RadarDish") || strstr(outstr, "Radar Dish") ) {
+			strcpy(outstr, "antena radaru");
+		} else if (!stricmp(outstr, "Gas Collector")) {
+			strcpy(outstr, "zbieracz gazu");
+		} 
 	} else {
 		if (strstr(outstr, XSTR("communication", 333)))	{
 			strcpy(outstr, XSTR("comm", 334));
@@ -522,7 +550,14 @@ void HudGaugeTargetBox::renderTargetShip(object *target_objp)
 		factor = -target_sip->closeup_pos.xyz.z;
 
 		// use the player's up vector, and construct the viewers orientation matrix
-		up_vector = Player_obj->orient.vec.uvec;
+		if (Player_obj->type == OBJ_SHIP) {
+			vec3d tempv;
+			ship_get_eye(&tempv, &camera_orient, Player_obj, false, false);
+		} else {
+			camera_orient = Player_obj->orient;
+		}
+
+		up_vector = camera_orient.vec.uvec;
 		vm_vector_2_matrix(&camera_orient,&orient_vec,&up_vector,NULL);
 
 		// normalize the vector from the player to the current target, and scale by a factor to calculate
@@ -531,6 +566,10 @@ void HudGaugeTargetBox::renderTargetShip(object *target_objp)
 
 		// RT, changed scaling here
 		renderTargetSetup(&camera_eye, &camera_orient, target_sip->closeup_zoom);
+
+		// IMPORTANT NOTE! Code handling the case 'missile_view == TRUE' in rendering section of renderTargetWeapon()
+		//                 is largely copied over from renderTargetShip(). To keep the codes similar please update
+		//                 both if and when needed
 		ship_model_start( target_objp );
 
 		switch (Targetbox_wire) {
@@ -669,7 +708,14 @@ void HudGaugeTargetBox::renderTargetDebris(object *target_objp)
 		factor = 2*target_objp->radius;
 
 		// use the player's up vector, and construct the viewers orientation matrix
-		up_vector = Player_obj->orient.vec.uvec;
+		if (Player_obj->type == OBJ_SHIP) {
+			vec3d tempv;
+			ship_get_eye(&tempv, &camera_orient, Player_obj, false, false);
+		} else {
+			camera_orient = Player_obj->orient;
+		}
+
+		up_vector = camera_orient.vec.uvec;
 		vm_vector_2_matrix(&camera_orient,&orient_vec,&up_vector,NULL);
 
 		// normalize the vector from the player to the current target, and scale by a factor to calculate
@@ -749,6 +795,7 @@ void HudGaugeTargetBox::renderTargetWeapon(object *target_objp)
 	vec3d		camera_eye = ZERO_VECTOR;
 	matrix		camera_orient = IDENTITY_MATRIX;
 	vec3d		orient_vec, up_vector;
+	vec3d		projection_vec;
 	weapon_info	*target_wip = NULL;
 	weapon		*wp = NULL;
 	object		*viewer_obj, *viewed_obj;
@@ -776,15 +823,18 @@ void HudGaugeTargetBox::renderTargetWeapon(object *target_objp)
 	}
 
 	if ( Detail.targetview_model )	{
+		ship *homing_shipp = NULL;
+		ship_info *homing_sip = NULL;
 
 		viewer_obj			= Player_obj;
 		viewed_obj			= target_objp;
 		missile_view		= FALSE;
 		viewed_model_num	= target_wip->model_num;
 		hud_target_lod		= target_wip->hud_target_lod;
-		if ( is_homing && is_player_missile ) {
-			ship *homing_shipp = &Ships[wp->homing_object->instance];
-			ship_info *homing_sip = &Ship_info[homing_shipp->ship_info_index];
+		// adding a check here to make sure the homing object is a ship, technically it could be some other object just as well
+		if ( is_homing && is_player_missile && (wp->homing_object->type == OBJ_SHIP)) {
+			homing_shipp = &Ships[wp->homing_object->instance];
+			homing_sip = &Ship_info[homing_shipp->ship_info_index];
 
 			viewer_obj			= target_objp;
 			viewed_obj			= wp->homing_object;
@@ -798,42 +848,100 @@ void HudGaugeTargetBox::renderTargetWeapon(object *target_objp)
 		vm_vec_sub(&orient_vec, &viewed_obj->pos, &viewer_obj->pos);
 		vm_vec_normalize(&orient_vec);
 
+		if (missile_view == TRUE) {
+			vm_vec_sub(&projection_vec, &wp->homing_pos, &viewer_obj->pos);
+			vm_vec_normalize(&projection_vec);
+		}
+
 		if ( missile_view == FALSE )
 			factor = 2*target_objp->radius;
 		else
 			factor = vm_vec_dist_quick(&viewer_obj->pos, &viewed_obj->pos);
 
 		// use the viewer's up vector, and construct the viewers orientation matrix
-		up_vector = viewer_obj->orient.vec.uvec;
-		vm_vector_2_matrix(&camera_orient,&orient_vec,&up_vector,NULL);
+		if (viewer_obj == Player_obj && Player_obj->type == OBJ_SHIP) {
+			vec3d tempv;
+			ship_get_eye(&tempv, &camera_orient, Player_obj, false, false);
+		} else {
+			camera_orient = viewer_obj->orient;
+		}
+
+		up_vector = camera_orient.vec.uvec;
+
+		if (missile_view == FALSE)
+			vm_vector_2_matrix(&camera_orient,&orient_vec,&up_vector,NULL);
+		else
+			vm_vector_2_matrix(&camera_orient,&projection_vec,&up_vector,NULL);
 
 		// normalize the vector from the viewer to the viwed target, and scale by a factor to calculate
 		// the objects position
-		vm_vec_copy_scale(&obj_pos,&orient_vec,factor);
+		if (missile_view == FALSE) {
+			vm_vec_copy_scale(&obj_pos,&orient_vec,factor);
+		} else {
+			vm_vec_sub(&obj_pos, &viewed_obj->pos, &viewer_obj->pos);
+		}
 
 		renderTargetSetup(&camera_eye, &camera_orient, View_zoom/3);
 		model_clear_instance(viewed_model_num);
+		
+		// IMPORTANT NOTE! Code handling the rendering when 'missile_view == TRUE' is largely copied over from
+		//                 renderTargetShip(). To keep the codes similar please update both if and when needed
+		if (missile_view == FALSE) {
+			switch (Targetbox_wire) {
+				case 0:
+					flags |= MR_NO_LIGHTING;
 
-		switch (Targetbox_wire) {
-			case 0:
-				flags |= MR_NO_LIGHTING;
+					break;
+				case 1:
+					model_set_outline_color_fast(iff_get_color_by_team_and_object(target_team, Player_ship->team, 0, target_objp));
 
-				break;
-			case 1:
-				model_set_outline_color_fast(iff_get_color_by_team_and_object(target_team, Player_ship->team, 0, target_objp));
+					flags = (Cmdline_nohtl) ? MR_SHOW_OUTLINE : MR_SHOW_OUTLINE_HTL;
+					flags |= MR_NO_POLYS | MR_NO_LIGHTING;
 
-				flags = (Cmdline_nohtl) ? MR_SHOW_OUTLINE : MR_SHOW_OUTLINE_HTL;
-				flags |= MR_NO_POLYS | MR_NO_LIGHTING;
+					break;
+				case 2:
+					break;
+				case 3:
+					model_set_outline_color_fast(iff_get_color_by_team_and_object(target_team, Player_ship->team, 0, target_objp));
 
-				break;
-			case 2:
-				break;
-			case 3:
-				model_set_outline_color_fast(iff_get_color_by_team_and_object(target_team, Player_ship->team, 0, target_objp));
+					flags |= MR_NO_LIGHTING | MR_NO_TEXTURING;
 
-				flags |= MR_NO_LIGHTING | MR_NO_TEXTURING;
+					break;
+			}
+		} else {
+			ship_model_start( viewed_obj );
 
-				break;
+			switch (Targetbox_wire) {
+				case 0:
+					flags |= MR_NO_LIGHTING;
+
+					break;
+				case 1:
+					if (ship_is_tagged(viewed_obj))
+						model_set_outline_color_fast(iff_get_color(IFF_COLOR_TAGGED, 1));
+					else
+						model_set_outline_color_fast(iff_get_color_by_team_and_object(homing_shipp->team, Player_ship->team, 1, viewed_obj));
+
+					if (homing_sip->uses_team_colors) {
+						gr_set_team_color(homing_shipp->team_name, homing_shipp->secondary_team_name, homing_shipp->team_change_timestamp, homing_shipp->team_change_time);
+					}
+
+					flags = (Cmdline_nohtl) ? MR_SHOW_OUTLINE : MR_SHOW_OUTLINE_HTL;
+					flags |= MR_NO_POLYS | MR_NO_LIGHTING;
+
+					break;
+				case 2:
+					break;
+				case 3:
+					if (ship_is_tagged(viewed_obj))
+						model_set_outline_color_fast(iff_get_color(IFF_COLOR_TAGGED, 1));
+					else
+						model_set_outline_color_fast(iff_get_color_by_team_and_object(homing_shipp->team, Player_ship->team, 1, viewed_obj));
+
+					flags |= MR_NO_LIGHTING | MR_NO_TEXTURING;
+
+					break;
+			}
 		}
 
 		if (hud_target_lod >= 0) {
@@ -852,9 +960,36 @@ void HudGaugeTargetBox::renderTargetWeapon(object *target_objp)
 
 		Interp_desaturate = Desaturated;
 
-		model_render( viewed_model_num, &viewed_obj->orient, &obj_pos, flags | MR_LOCK_DETAIL | MR_AUTOCENTER | MR_IS_MISSILE | MR_NO_FOGGING, -1, -1, replacement_textures);
+		if (missile_view == TRUE) {
+			if (!Glowpoint_override)
+				Glowpoint_override = true;
+
+			// set glowmap flag here since model_render (etc) require an objnum to handle glowmaps
+			// if we did pass the objnum, we'd also have thrusters drawn in the targetbox
+			if (homing_shipp->flags2 & SF2_GLOWMAPS_DISABLED) {
+				flags |= MR_NO_GLOWMAPS;
+			}
+		}
+		
+		if (missile_view == FALSE ) {
+			model_render( viewed_model_num, &viewed_obj->orient, &obj_pos, flags | MR_LOCK_DETAIL | MR_AUTOCENTER | MR_IS_MISSILE | MR_NO_FOGGING, -1, -1, replacement_textures);
+		} else {
+			// maybe render a special hud-target-only model
+			// autocentering is bad in this one
+			if(homing_sip->model_num_hud >= 0){
+				model_render( homing_sip->model_num_hud, &viewed_obj->orient, &obj_pos, flags | MR_LOCK_DETAIL | MR_NO_FOGGING);
+			} else {
+				model_render( homing_sip->model_num, &viewed_obj->orient, &obj_pos, flags | MR_LOCK_DETAIL | MR_NO_FOGGING, -1, -1, homing_shipp->ship_replacement_textures);
+			}
+		}
 
 		Interp_desaturate = false;
+		if (missile_view == TRUE) {
+			Glowpoint_override = false;
+
+			ship_model_stop( viewed_obj );
+			gr_disable_team_color();
+		}
 
 		if ( Monitor_mask >= 0 ) {
 			gr_stencil_set(GR_STENCIL_NONE);
@@ -880,8 +1015,19 @@ void HudGaugeTargetBox::renderTargetWeapon(object *target_objp)
 	if ( is_homing ) {
 		float dist, speed;
 
-		dist = vm_vec_dist(&target_objp->pos, &wp->homing_object->pos);
 		speed = vm_vec_mag(&target_objp->phys_info.vel);
+
+		// do the extra math only if it won't lead to null vec issues
+		if(!(IS_VEC_NULL_SQ_SAFE(&target_objp->phys_info.vel))){
+			vec3d unit_vec, component_vec;
+
+			// in other words substract the magnitude of the target's velocity vectors parallel component from the speed of the weapon
+			vm_vec_copy_normalize(&unit_vec, &target_objp->phys_info.vel);
+			speed -= vm_vec_projection_parallel(&component_vec, &wp->homing_object->phys_info.vel, &unit_vec);
+		}
+
+		dist = vm_vec_dist(&target_objp->pos, &wp->homing_pos);
+		
 		if ( speed > 0 ) {
 			sprintf(outstr, NOX("impact: %.1f sec"), dist/speed);
 		} else {
@@ -920,7 +1066,14 @@ void HudGaugeTargetBox::renderTargetAsteroid(object *target_objp)
 		factor = 2*target_objp->radius;
 
 		// use the player's up vector, and construct the viewers orientation matrix
-		up_vector = Player_obj->orient.vec.uvec;
+		if (Player_obj->type == OBJ_SHIP) {
+			vec3d tempv;
+			ship_get_eye(&tempv, &camera_orient, Player_obj, false, false);
+		} else {
+			camera_orient = Player_obj->orient;
+		}
+
+		up_vector = camera_orient.vec.uvec;
 		vm_vector_2_matrix(&camera_orient,&orient_vec,&up_vector,NULL);
 
 		// normalize the vector from the player to the current target, and scale by a factor to calculate
@@ -1038,7 +1191,14 @@ void HudGaugeTargetBox::renderTargetJumpNode(object *target_objp)
 			factor = target_objp->radius*4.0f;
 
 			// use the player's up vector, and construct the viewers orientation matrix
-			up_vector = Player_obj->orient.vec.uvec;
+			if (Player_obj->type == OBJ_SHIP) {
+				vec3d tempv;
+				ship_get_eye(&tempv, &camera_orient, Player_obj, false, false);
+			} else {
+				camera_orient = Player_obj->orient;
+			}
+
+			up_vector = camera_orient.vec.uvec;
 			vm_vector_2_matrix(&camera_orient,&orient_vec,&up_vector,NULL);
 
 			// normalize the vector from the player to the current target, and scale by a factor to calculate
@@ -1078,7 +1238,7 @@ void HudGaugeTargetBox::renderTargetJumpNode(object *target_objp)
 		hy = fl2i(HUD_offset_y);
 
 		sprintf(outstr,XSTR( "d: %.0f", 340), dist);
-		hud_num_make_mono(outstr);
+		hud_num_make_mono(outstr, font_num);
 		gr_get_string_size(&w,&h,outstr);
 	
 		renderPrintf(position[0] + Dist_offsets[0]+hx, position[1] + Dist_offsets[1]+hy, EG_TBOX_DIST, outstr);
@@ -1366,8 +1526,8 @@ void HudGaugeExtraTargetData::endFlashDock()
 }
 
 //from aicode.cpp. Less include...problems...this way.
-extern bool turret_weapon_has_flags(ship_weapon *swp, int flags);
-extern bool turret_weapon_has_flags2(ship_weapon *swp, int flags);
+extern int turret_weapon_aggregate_flags(ship_weapon *swp);
+extern int turret_weapon_aggregate_flags2(ship_weapon *swp);
 extern bool turret_weapon_has_subtype(ship_weapon *swp, int subtype);
 void get_turret_subsys_name(ship_weapon *swp, char *outstr)
 {
@@ -1375,21 +1535,25 @@ void get_turret_subsys_name(ship_weapon *swp, char *outstr)
 
 	//WMC - find the first weapon, if there is one
 	if (swp->num_primary_banks || swp->num_secondary_banks) {
+		int flags = turret_weapon_aggregate_flags(swp);
+		int flags2 = turret_weapon_aggregate_flags2(swp);
+
 		// check if beam or flak using weapon flags
-		if (turret_weapon_has_flags(swp, WIF_BEAM)) {
+		if (flags & WIF_BEAM) {
 			sprintf(outstr, "%s", XSTR("Beam turret", 1567));
-		}else if (turret_weapon_has_flags(swp, WIF_FLAK)) {
+		} else if (flags & WIF_FLAK) {
 			sprintf(outstr, "%s", XSTR("Flak turret", 1566));
 		} else {
-
-			if (!turret_weapon_has_subtype(swp, WP_MISSILE) && turret_weapon_has_subtype(swp, WP_LASER)) {
+			if (turret_weapon_has_subtype(swp, WP_MISSILE)) {
+				sprintf(outstr, "%s", XSTR("Missile lnchr", 1569));
+			} else if (turret_weapon_has_subtype(swp, WP_LASER)) {
 				// ballistic too! - Goober5000
-				if (turret_weapon_has_flags2(swp, WIF2_BALLISTIC))
+				if (flags2 & WIF2_BALLISTIC)
 				{
 					sprintf(outstr, "%s", XSTR("Turret", 1487));
 				}
 				// the TVWP has some primaries flagged as bombs
-				else if (turret_weapon_has_flags(swp, WIF_BOMB))
+				else if (flags & WIF_BOMB)
 				{
 					sprintf(outstr, "%s", XSTR("Missile lnchr", 1569));
 				}
@@ -1397,12 +1561,19 @@ void get_turret_subsys_name(ship_weapon *swp, char *outstr)
 				{
 					sprintf(outstr, "%s", XSTR("Laser turret", 1568));
 				}
-			} else if (turret_weapon_has_subtype(swp, WP_MISSILE)) {
-				sprintf(outstr, "%s", XSTR("Missile lnchr", 1569));
 			} else {
-				// Illegal subtype
-				Int3();
-				sprintf(outstr, "%s", XSTR("Turret", 1487));
+				// Mantis #2226: find out if there are any weapons here at all
+				if (flags == 0 && flags2 == 0) {
+					sprintf(outstr, "%s", NOX("Unused"));
+				} else {
+					// Illegal subtype
+					static bool Turret_illegal_subtype_warned = false;
+					if (!Turret_illegal_subtype_warned) {
+						Turret_illegal_subtype_warned = true;
+						Warning(LOCATION, "This turret has an illegal subtype!  Trace out and fix!");
+					}
+					sprintf(outstr, "%s", XSTR("Turret", 1487));
+				}
 			}
 		}
 	} else if(swp->num_tertiary_banks) {
@@ -1541,7 +1712,7 @@ void HudGaugeTargetBox::renderTargetShipInfo(object *target_objp)
 		if (n_linebreaks) {
 			p_line = strtok(outstr,linebreak);
 			while (p_line != NULL) {
-				renderPrintf(subsys_name_pos_x, subsys_name_pos_y-h-(10*n_linebreaks), p_line);
+				renderPrintf(subsys_name_pos_x, subsys_name_pos_y-h-((h+1)*n_linebreaks), p_line);
 				p_line = strtok(NULL,linebreak);
 				n_linebreaks--;
 			}
@@ -1771,7 +1942,7 @@ void HudGaugeTargetBox::showTargetData(float frametime)
 	// print out the target distance and speed
 	sprintf(outstr,XSTR( "d: %.0f%s", 350), displayed_target_distance, modifiers[Player_ai->current_target_dist_trend]);
 
-	hud_num_make_mono(outstr);
+	hud_num_make_mono(outstr, font_num);
 	gr_get_string_size(&w,&h,outstr);
 
 	renderString(position[0] + Dist_offsets[0]+hx, position[1] + Dist_offsets[1]+hy, EG_TBOX_DIST, outstr);	
@@ -1800,7 +1971,7 @@ void HudGaugeTargetBox::showTargetData(float frametime)
 	}
 
 	sprintf(outstr, XSTR( "s: %.0f%s", 351), displayed_target_speed, (displayed_target_speed>1)?modifiers[Player_ai->current_target_speed_trend]:"");
-	hud_num_make_mono(outstr);
+	hud_num_make_mono(outstr, font_num);
 
 	renderString(position[0] + Speed_offsets[0]+hx, position[1] + Speed_offsets[1]+hy, EG_TBOX_SPEED, outstr);
 
