@@ -58,6 +58,7 @@
 #include "network/multi_endgame.h"
 #include "autopilot/autopilot.h"
 #include "cmdline/cmdline.h"
+#include "object/objectshield.h"
 
 #define MAX_NUM_SLOTS 6
 
@@ -67,13 +68,11 @@ int count;
 int table[ MAX_NUM_SLOTS ];
 };
 
-#define MAX_SLOT_COUNT 25
-
 class factor_table
 	{
 	public:
 		factor_table();
-		~factor_table(){ delete[] table; };
+		~factor_table(){ delete[] table; }
 		int getNextSlots( int slots_on_ship, int cur_slots );
 
 	private:
@@ -95,9 +94,9 @@ memset( table, 0x00, sizeof( ftable ) * MAX_NUM_SLOTS );
 for( int i = 0 ; i < MAX_NUM_SLOTS; ++i )
 	{
 	table[ i ].count = 0;
-	for( int j = 1; j <= i; ++j )
+	for( int j = 1; j <= i + 1; ++j )
 		{
-		if( isAPrimeFactor( j, i ) )
+		if( isAPrimeFactor( j, i + 1 ) )
 			{
 			table[ i ].table[ table[ i ].count ] = j;
 			table[ i ].count ++;
@@ -108,14 +107,16 @@ for( int i = 0 ; i < MAX_NUM_SLOTS; ++i )
 
 int factor_table::getNextSlots(int slots_on_ship, int cur_slots)
 {
-Assert( slots_on_ship <= MAX_NUM_SLOTS );
-Assert( slots_on_ship >= 0 );
+Assertion( slots_on_ship <= MAX_NUM_SLOTS, "factor_table::getNextSlots() called with %d slots, when MAX_NUM_SLOTS is %d; get a coder!\n", slots_on_ship, MAX_NUM_SLOTS );
+Assertion( slots_on_ship >= 1, "factor_table::getNextSlots() called with %d slots, when only positive integers make sense; get a coder!\n" );
 
-for( int i = 0; i < table[ slots_on_ship ].count; ++i )
+int slots_index = slots_on_ship - 1;
+
+for( int i = 0; i < table[ slots_index ].count; ++i )
 	{
-	if( table[ slots_on_ship ].table[ i ] == cur_slots )
+	if( table[ slots_index ].table[ i ] == cur_slots )
 		{
-		if( table[ slots_on_ship ].count == i + 1 )
+		if( table[ slots_index ].count == i + 1 )
 			{
 			//Overflow back to 1
 			return 1;
@@ -123,13 +124,13 @@ for( int i = 0; i < table[ slots_on_ship ].count; ++i )
 		else
 			{
 			//Next block in the table
-			return table[ slots_on_ship ].table[ i + 1 ];
+			return table[ slots_index ].table[ i + 1 ];
 			}
 		}
 	}
 //Did not find cur_slots, try and get back on track
 
-Assert( 0 );
+Assertion( false, "For some reason, factor_table::getNextSlots() was unable to locate cur_slots. This should never happen; get a coder!\n" );
 return 1;
 }
 
@@ -634,6 +635,11 @@ void process_debug_keys(int k)
 	}
 
 	switch (k) {
+		case KEY_DEBUGGED + KEY_Q:
+		case KEY_DEBUGGED1 + KEY_Q:
+			Snapshot_all_events = true;
+			break;
+
 		case KEY_DEBUGGED + KEY_H:
 			hud_target_toggle_hidden_from_sensors();
 			break;
@@ -1020,12 +1026,21 @@ void process_debug_keys(int k)
 			break;
 		}
 
-		case KEY_DEBUGGED + KEY_R: {
+		case KEY_DEBUGGED + KEY_R:
 		case KEY_DEBUGGED1 + KEY_R:
-			if (Player_ai->target_objnum != -1)
-				ai_issue_rearm_request(&Objects[Player_ai->target_objnum]);
+		{
+			// rearm the target, if we have one
+			object *obj_to_rearm = (Player_ai->target_objnum >= 0) ? &Objects[Player_ai->target_objnum] : Player_obj;
+
+			if (is_support_allowed(obj_to_rearm))
+			{
+				HUD_sourced_printf(HUD_SOURCE_HIDDEN, XSTR("Issuing rearm request for %s", -1), Ships[obj_to_rearm->instance].ship_name);
+				ai_issue_rearm_request(obj_to_rearm);
+			}
 			else
-				ai_issue_rearm_request(Player_obj);
+			{
+				HUD_sourced_printf(HUD_SOURCE_HIDDEN, XSTR("Cannot issue rearm request for %s", -1), Ships[obj_to_rearm->instance].ship_name);
+			}
 
 			break;
 		}
@@ -1286,7 +1301,7 @@ void ppsk_hotkeys(int k)
 		case KEY_F10 + KEY_SHIFTED + KEY_ALTED:
 		case KEY_F11 + KEY_SHIFTED + KEY_ALTED:
 		case KEY_F12 + KEY_SHIFTED + KEY_ALTED:
-			hotkey_set = mission_hotkey_get_set_num(k & ~KEY_SHIFTED+KEY_ALTED);
+			hotkey_set = mission_hotkey_get_set_num(k & (~(KEY_SHIFTED+KEY_ALTED)));
 			hud_target_hotkey_clear( hotkey_set );
 			break;
 
@@ -1722,12 +1737,14 @@ int button_function_critical(int n, net_player *p = NULL)
 				ship * shipp = &Ships[objp->instance];
 				ship_weapon *swp = &shipp->weapons;
 				ship_info *sip = &Ship_info[shipp->ship_info_index];
-				polymodel *pm = model_get( sip->model_num );
-				count = ftables.getNextSlots( pm->gun_banks[ swp->current_primary_bank ].num_slots, swp->primary_bank_slot_count[ swp->current_primary_bank ] );
-				swp->primary_bank_slot_count[ swp->current_primary_bank ] = count;
-				shipp->last_fired_point[ swp->current_primary_bank ] += count - ( shipp->last_fired_point[ swp->current_primary_bank ] % count);
-				shipp->last_fired_point[ swp->current_primary_bank ] -= 1;
-				shipp->last_fired_point[ swp->current_primary_bank ] %= swp->primary_bank_slot_count[ swp->current_primary_bank ];
+				if (sip->flags2 & SIF2_DYN_PRIMARY_LINKING) {
+					polymodel *pm = model_get( sip->model_num );
+					count = ftables.getNextSlots( pm->gun_banks[ swp->current_primary_bank ].num_slots, swp->primary_bank_slot_count[ swp->current_primary_bank ] );
+					swp->primary_bank_slot_count[ swp->current_primary_bank ] = count;
+					shipp->last_fired_point[ swp->current_primary_bank ] += count - ( shipp->last_fired_point[ swp->current_primary_bank ] % count);
+					shipp->last_fired_point[ swp->current_primary_bank ] -= 1;
+					shipp->last_fired_point[ swp->current_primary_bank ] %= swp->primary_bank_slot_count[ swp->current_primary_bank ];
+				}
 			}
 			break;
 
@@ -1941,28 +1958,28 @@ int button_function_critical(int n, net_player *p = NULL)
 			if(at_self){
    			control_used(SHIELD_XFER_TOP);
 			}
-			hud_augment_shield_quadrant(objp, 1);
+			hud_augment_shield_quadrant(objp, FRONT_QUAD);
 			break;
 
 		// transfer shield energy to rear
 		case SHIELD_XFER_BOTTOM:
 			if(at_self)
 				control_used(SHIELD_XFER_BOTTOM);
-			hud_augment_shield_quadrant(objp, 2);
+			hud_augment_shield_quadrant(objp, REAR_QUAD);
 			break;
 
 		// transfer shield energy to left
 		case SHIELD_XFER_LEFT:
 			if(at_self)
 				control_used(SHIELD_XFER_LEFT);
-			hud_augment_shield_quadrant(objp, 3);
+			hud_augment_shield_quadrant(objp, LEFT_QUAD);
 			break;
 			
 		// transfer shield energy to right
 		case SHIELD_XFER_RIGHT:
 			if(at_self)
 				control_used(SHIELD_XFER_RIGHT);
-			hud_augment_shield_quadrant(objp, 0);
+			hud_augment_shield_quadrant(objp, RIGHT_QUAD);
 			break;
 
 		// transfer energy to shield from weapons
@@ -2486,7 +2503,7 @@ int button_function(int n)
 	control_used(n);
 
 	if ( hud_sensors_ok(Player_ship) ) {
-		int keyHasBeenUsed = TRUE;
+		keyHasBeenUsed = TRUE;
 		switch(n) {
 			// target next
 			case TARGET_NEXT:
