@@ -149,12 +149,8 @@ void GUISystem::ParseClassInfo(char* filename)
 		DestroyClassInfo();
 	}
 
-	// open localization
-	lcl_ext_open();
-
 	if ((rval = setjmp(parse_abort)) != 0) {
 		mprintf(("WMCGUI: Unable to parse '%s'!  Error code = %i.\n", filename, rval));
-		lcl_ext_close();
 		return;
 	}
 
@@ -171,9 +167,6 @@ void GUISystem::ParseClassInfo(char* filename)
 		}
 	} while(flag);
 
-	// close localization
-	lcl_ext_close();
-
 	ClassInfoParsed = true;
 }
 
@@ -293,7 +286,7 @@ int ClassInfoEntry::GetCoords(int *x, int *y)
 		*x=Coords[0];
 		rval |= CIE_GC_X_SET;
 	}
-	if (Coords[1]!=INT_MAX && x!=NULL) {
+	if (Coords[1]!=INT_MAX && y!=NULL) {
 		*y=Coords[1];
 		rval |= CIE_GC_Y_SET;
 	}
@@ -439,7 +432,6 @@ GUIObject* GUIScreen::Add(GUIObject* new_gauge)
 		if (tgp->Name == new_gauge->Name) {
 			//Get rid of the new gauge
 			//We don't want it; breaks skinning
-			delete new_gauge;
 
 			if (tgp->Type == new_gauge->Type) {
 				//If the type of the existing object is the same
@@ -1218,7 +1210,7 @@ int Window::DoRefreshSize()
 		
 		int caption_min_size;
 		if (Caption.size() > 0) {
-			gr_get_string_size(&w, &h, (char *)Caption.c_str());
+			gr_get_string_size(&w, &h, Caption.c_str());
 			caption_min_size = w + close_w + hide_w + 5;
 		} else {
 			caption_min_size = close_w + hide_w;
@@ -1269,13 +1261,13 @@ int Window::DoRefreshSize()
 		IMG_INFO(GetCIEImageHandle(WCI_BORDER, CIE_HANDLE_TM), &w, &h);
 		num = (float)((Coords[2]-CornerWidths[3])-(Coords[0]+CornerWidths[2])) / (float)w;
 		BorderRectLists[CIE_HANDLE_TM] = bitmap_rect_list(Coords[0] + CornerWidths[2], Coords[1], fl2i(w*num),h,0,0,num,1.0f);
-		//gr_bitmap_list(&BorderRectLists[CIE_HANDLE_BM], 1, false);
+		//gr_bitmap_list(&BorderRectLists[CIE_HANDLE_BM], 1, GR_RESIZE_NONE);
 	}
 	if (IMG_HANDLE_IS_VALID(GetCIEImageHandle(WCI_BORDER, CIE_HANDLE_BM))) {
 		IMG_INFO(GetCIEImageHandle(WCI_BORDER, CIE_HANDLE_BM), &w, &h);
 		num = (float)((Coords[2]-CornerWidths[3])-(Coords[0]+CornerWidths[2])) / (float)w;
 		BorderRectLists[CIE_HANDLE_BM] = bitmap_rect_list(Coords[0] + CornerWidths[2], Coords[3]-h, fl2i(w*num),h,0,0,num,1.0f);
-		//gr_bitmap_list(&BorderRectLists[CIE_HANDLE_BM], 1, false);
+		//gr_bitmap_list(&BorderRectLists[CIE_HANDLE_BM], 1, GR_RESIZE_NONE);
 	}
 	if (IMG_HANDLE_IS_VALID(GetCIEImageHandle(WCI_BORDER, CIE_HANDLE_ML))) {
 		IMG_INFO(GetCIEImageHandle(WCI_BORDER, CIE_HANDLE_ML), &w, &h);
@@ -1408,15 +1400,15 @@ void Window::DoMove(int dx, int dy)
 	CaptionRectList.screen_rect.y += dy;
 }
 
-void draw_open_rect(int x1, int y1, int x2, int y2, bool resize = false)
+void draw_open_rect(int x1, int y1, int x2, int y2, int resize_mode = GR_RESIZE_NONE)
 {
-	gr_line(x1, y1, x2, y1, resize);
-	gr_line(x2, y1, x2, y2, resize);
-	gr_line(x2, y2, x1, y2, resize);
-	gr_line(x1, y2, x1, y1, resize);
+	gr_line(x1, y1, x2, y1, resize_mode);
+	gr_line(x2, y1, x2, y2, resize_mode);
+	gr_line(x2, y2, x1, y2, resize_mode);
+	gr_line(x1, y2, x1, y1, resize_mode);
 }
 
-extern void gr_opengl_shade(int x, int y, int w, int h, bool resize);
+extern void gr_opengl_shade(int x, int y, int w, int h, int resize_mode);
 
 void Window::DoDraw(float frametime)
 {
@@ -1429,7 +1421,7 @@ void Window::DoDraw(float frametime)
 
 	// shade the background of the window so that it's just slightly transparent
 	gr_set_shader(&WindowShade);
-	gr_opengl_shade(Coords[0], Coords[1], Coords[2], Coords[3], false);
+	gr_opengl_shade(Coords[0], Coords[1], Coords[2], Coords[3], GR_RESIZE_NONE);
 
 	gr_set_color_fast(&Color_text_normal);
 
@@ -1459,31 +1451,31 @@ void Window::DoDraw(float frametime)
 
 	if (IMG_HANDLE_IS_VALID(GetCIEImageHandle(WCI_BORDER, CIE_HANDLE_TM))) {
 		IMG_SET(GetCIEImageHandle(WCI_BORDER, CIE_HANDLE_TM));
-		gr_bitmap_list(&BorderRectLists[CIE_HANDLE_TM], 1, false);
+		gr_bitmap_list(&BorderRectLists[CIE_HANDLE_TM], 1, GR_RESIZE_NONE);
 	} else {
-		gr_line(Coords[0] + BorderSizes[0], Coords[1], Coords[2] - BorderSizes[2], Coords[1], false);
+		gr_line(Coords[0] + BorderSizes[0], Coords[1], Coords[2] - BorderSizes[2], Coords[1], GR_RESIZE_NONE);
 	}
 
 	if (IMG_HANDLE_IS_VALID(GetCIEImageHandle(WCI_BORDER, CIE_HANDLE_BM))) {
 		IMG_SET(GetCIEImageHandle(WCI_BORDER, CIE_HANDLE_BM));
-		gr_bitmap_list(&BorderRectLists[CIE_HANDLE_BM], 1, false);
+		gr_bitmap_list(&BorderRectLists[CIE_HANDLE_BM], 1, GR_RESIZE_NONE);
 	} else {
-		gr_line(Coords[0] + BorderSizes[0], Coords[3], Coords[2] - BorderSizes[2], Coords[3], false);
+		gr_line(Coords[0] + BorderSizes[0], Coords[3], Coords[2] - BorderSizes[2], Coords[3], GR_RESIZE_NONE);
 	}
 
 	if (!(Style & GS_HIDDEN)) {
 		if (IMG_HANDLE_IS_VALID(GetCIEImageHandle(WCI_BORDER, CIE_HANDLE_ML))) {
 			IMG_SET(GetCIEImageHandle(WCI_BORDER, CIE_HANDLE_ML));
-			gr_bitmap_list(&BorderRectLists[CIE_HANDLE_ML], 1, false);
+			gr_bitmap_list(&BorderRectLists[CIE_HANDLE_ML], 1, GR_RESIZE_NONE);
 		} else {
-			gr_line(Coords[0], Coords[1] + BorderSizes[1], Coords[0], Coords[3] - BorderSizes[3], false);
+			gr_line(Coords[0], Coords[1] + BorderSizes[1], Coords[0], Coords[3] - BorderSizes[3], GR_RESIZE_NONE);
 		}
 
 		if (IMG_HANDLE_IS_VALID(GetCIEImageHandle(WCI_BORDER, CIE_HANDLE_MR))) {
 			IMG_SET(GetCIEImageHandle(WCI_BORDER, CIE_HANDLE_MR));
-			gr_bitmap_list(&BorderRectLists[CIE_HANDLE_MR], 1, false);
+			gr_bitmap_list(&BorderRectLists[CIE_HANDLE_MR], 1, GR_RESIZE_NONE);
 		} else {
-			gr_line(Coords[2], Coords[1] + BorderSizes[1], Coords[2], Coords[3] - BorderSizes[3], false);
+			gr_line(Coords[2], Coords[1] + BorderSizes[1], Coords[2], Coords[3] - BorderSizes[3], GR_RESIZE_NONE);
 		}
 	}
 
@@ -1491,7 +1483,7 @@ void Window::DoDraw(float frametime)
 		//Draw the caption background
 		if (IMG_HANDLE_IS_VALID(GetCIEImageHandle(WCI_CAPTION))) {
 			IMG_SET(GetCIEImageHandle(WCI_CAPTION));
-			gr_bitmap_list(&CaptionRectList, 1, false);
+			gr_bitmap_list(&CaptionRectList, 1, GR_RESIZE_NONE);
 		} else {
 			draw_open_rect(Coords[0], Coords[1], Coords[2], CaptionCoords[3]);
 		}
@@ -1510,7 +1502,7 @@ void Window::DoDraw(float frametime)
 			} else {
 				gr_set_color_fast(&Color_text_normal);
 			}
-			gr_string(CloseCoords[0], CloseCoords[1], "X", false);
+			gr_string(CloseCoords[0], CloseCoords[1], "X", GR_RESIZE_NONE);
 		}
 		
 
@@ -1528,13 +1520,13 @@ void Window::DoDraw(float frametime)
 			} else {
 				gr_set_color_fast(&Color_text_normal);
 			}
-			gr_string(HideCoords[0], HideCoords[1], "-", false);
+			gr_string(HideCoords[0], HideCoords[1], "-", GR_RESIZE_NONE);
 		}
 
 		//Caption text
 		gr_set_color_fast(&Color_text_normal);
 
-		gr_string(CaptionCoords[0], CaptionCoords[1], (char *)Caption.c_str(), false);
+		gr_string(CaptionCoords[0], CaptionCoords[1], Caption.c_str(), GR_RESIZE_NONE);
 	}
 }
 
@@ -1596,7 +1588,7 @@ int Button::DoRefreshSize()
 			Coords[3] = Coords[1] + h;
 		}
 	} else {
-		gr_get_string_size(&w, &h, (char *)Caption.c_str());
+		gr_get_string_size(&w, &h, Caption.c_str());
 		if (!(Style & GS_NOAUTORESIZEX)) {
 			Coords[2] = Coords[0] + w;
 		}
@@ -1632,10 +1624,10 @@ void Button::DoDraw(float frametime)
 		draw_open_rect(Coords[0], Coords[1], Coords[2], Coords[3], false);
 
 		int half_x, half_y;
-		gr_get_string_size(&half_x, &half_y, (char *)Caption.c_str());
+		gr_get_string_size(&half_x, &half_y, Caption.c_str());
 		half_x = Coords[0] +(((Coords[2]-Coords[0]) - half_x) / 2);
 		half_y = Coords[1] +(((Coords[3]-Coords[1]) - half_y) / 2);
-		gr_string(half_x, half_y, (char *)Caption.c_str(), false);
+		gr_string(half_x, half_y, Caption.c_str(), GR_RESIZE_NONE);
 	}
 }
 
@@ -1713,7 +1705,7 @@ void Tree::CalcItemsSize(TreeItem *items, int DrawData[4])
 	int temp_largest = DrawData[1];
 	for (TreeItem* tip = (TreeItem*)GET_FIRST(items); tip != END_OF_LIST(items); tip = (TreeItem*)GET_NEXT(tip)) {
 		if (!DrawData[3]) {
-			gr_get_string_size(&w, &h, (char*)tip->Name.c_str());
+			gr_get_string_size(&w, &h, tip->Name.c_str());
 			if ((w + DrawData[0]) > temp_largest) {
 					temp_largest = w + DrawData[0];
 			}
@@ -1806,7 +1798,7 @@ void Tree::DrawItems(TreeItem *items)
 				gr_set_color_fast(&Color_text_normal);
 			}
 
-			gr_string(tip->Coords[0], tip->Coords[1], (char*)tip->Name.c_str(), false);
+			gr_string(tip->Coords[0], tip->Coords[1], tip->Name.c_str(), GR_RESIZE_NONE);
 
 			if (NOT_EMPTY(&tip->Children) && tip->ShowChildren) {
 				DrawItems((TreeItem*)&tip->Children);
@@ -1950,7 +1942,7 @@ int Text::DoRefreshSize()
 		width = gr_screen.clip_right - Coords[0];
 	}
 
-	NumLines = split_str((char*)Content.c_str(), width, LineLengths, LineStartPoints, MAX_TEXT_LINES);
+	NumLines = split_str(Content.c_str(), width, LineLengths, LineStartPoints, MAX_TEXT_LINES);
 
 	//Find the shortest width we need to show all the shortened strings
 	int longest_width = 0;
@@ -2035,7 +2027,7 @@ void Text::DoDraw(float frametime)
 	int font_height = gr_get_font_height();
 
 	for (int i = 0; i < NumLines; i++) {
-		gr_string(ChildCoords[0], ChildCoords[1] + (i*font_height), (char*)Content.substr(LineStartPoints[i] - Content.c_str(), LineLengths[i]).c_str(), false);
+		gr_string(ChildCoords[0], ChildCoords[1] + (i*font_height), Content.substr(LineStartPoints[i] - Content.c_str(), LineLengths[i]).c_str(), GR_RESIZE_NONE);
 	}
 }
 
@@ -2361,7 +2353,7 @@ Checkbox::Checkbox(const SCP_string &in_label, int x_coord, int y_coord, void (*
 int Checkbox::DoRefreshSize()
 {
 	int w, h,tw,th;
-	gr_get_string_size(&w, &h, (char *)Label.c_str());
+	gr_get_string_size(&w, &h, Label.c_str());
 	tw = w;
 	th = h;
 
@@ -2408,11 +2400,11 @@ void Checkbox::DoDraw(float frametime)
 	if ( (IsChecked && ((FlagPtr == NULL) && (BoolFlagPtr == NULL)))
 		|| ((FlagPtr != NULL) && ((*FlagPtr) & Flag))
 		|| ((BoolFlagPtr != NULL) && (*BoolFlagPtr)) ) {
-		gr_string(CheckCoords[0], CheckCoords[1], "X", false);
+		gr_string(CheckCoords[0], CheckCoords[1], "X", GR_RESIZE_NONE);
 	}
 
 	gr_set_color_fast(&Color_text_normal);
-	gr_string(CheckCoords[2] + CB_TEXTCHECKDIST, CheckCoords[1], (char *)Label.c_str(), false);
+	gr_string(CheckCoords[2] + CB_TEXTCHECKDIST, CheckCoords[1], Label.c_str(), GR_RESIZE_NONE);
 }
 
 int Checkbox::DoMouseOver(float frametime)
@@ -2558,10 +2550,10 @@ int ImageAnim::DoRefreshSize()
 void ImageAnim::SetImage(const SCP_string &in_imagename)
 {
 	if (in_imagename.size()) {
-		ImageHandle = IMG_LOAD_ANIM((char*)in_imagename.c_str(), &TotalFrames, &FPS);
+		ImageHandle = IMG_LOAD_ANIM(in_imagename.c_str(), &TotalFrames, &FPS);
 		TotalTime = (float) (TotalFrames/FPS);
 		if (IMG_HANDLE_IS_INVALID(ImageHandle)) {
-			ImageHandle = IMG_LOAD((char*)in_imagename.c_str());
+			ImageHandle = IMG_LOAD(in_imagename.c_str());
 			if (IMG_HANDLE_IS_VALID(ImageHandle)) {
 				TotalFrames = 1;
 			} else {
