@@ -30,7 +30,7 @@
 #define IMG_UNLOAD(a)				bm_unload(a)
 #define IMG_SET(h)					gr_set_bitmap(h)
 #define IMG_SET_FRAME(h, f)			gr_set_bitmap(h + f)
-#define IMG_DRAW(x,y)				gr_bitmap(x,y,false)
+#define IMG_DRAW(x,y)				gr_bitmap(x,y,GR_RESIZE_NONE)
 #define IMG_INFO(ha,w,h)			bm_get_info(ha,w,h)
 #endif
 
@@ -306,8 +306,6 @@ private:
 
 	GUISystem* OwnerSystem;
 
-	bool Active;	//Is this screen active?
-
 	ScreenClassInfoEntry* ScreenClassInfo;
 	GUIObject Guiobjects;
 	SCP_vector<GUIObject*> DeletionCache;
@@ -513,7 +511,6 @@ class Tree : public GUIObject
 	TreeItem Items;
 	void *AssociatedItem;
 
-	int StartLine;
 	TreeItem *SelectedItem;
 	TreeItem *HighlightedItem;
 
@@ -567,7 +564,7 @@ class Text : public GUIObject
 	//Used to display stuff; change only from calculate func
 	int NumLines;
 	int LineLengths[MAX_TEXT_LINES];
-	char *LineStartPoints[MAX_TEXT_LINES];
+	const char *LineStartPoints[MAX_TEXT_LINES];
 
 	//Used for editing
 	int CursorPos;	//Line #, then position in line
