@@ -325,8 +325,7 @@ int fvi_ray_sphere(vec3d *intp,vec3d *p0,vec3d *p1,vec3d *sphere_pos,float spher
  */
 int fvi_ray_boundingbox( vec3d *min, vec3d *max, vec3d * p0, vec3d *pdir, vec3d *hitpt )
 {
-	bool inside = true;
-	bool middle[3] = { true, true, true };
+	int middle = ((1<<0) | (1<<1) | (1<<2));
 	int i;
 	int which_plane;
 	float maxt[3];
@@ -335,48 +334,51 @@ int fvi_ray_boundingbox( vec3d *min, vec3d *max, vec3d * p0, vec3d *pdir, vec3d
 	for (i = 0; i < 3; i++) {
 		if (p0->a1d[i] < min->a1d[i]) {
 			candidate_plane[i] = min->a1d[i];
-			middle[i] = false;
-			inside = false;
+			middle &= ~(1<<i);
 		} else if (p0->a1d[i] > max->a1d[i]) {
 			candidate_plane[i] = max->a1d[i];
-			middle[i] = false;
-			inside = false;
+			middle &= ~(1<<i);
 		}
 	}
 
-	// ray origin inside bounding box			
-	if ( inside ) {
+	// ray origin inside bounding box?
+	// (are all three bits still set?)
+	if (middle == ((1<<0) | (1<<1) | (1<<2))) {
 		*hitpt = *p0;
 		return 1;
 	}
 
 	// calculate T distances to candidate plane
 	for (i = 0; i < 3; i++) {
-		if ( !middle[i] && (pdir->a1d[i] != 0.0f) )
-			maxt[i] = (candidate_plane[i] - p0->a1d[i]) / pdir->a1d[i];
-		else
+		if ( (middle & (1<<i)) || (pdir->a1d[i] == 0.0f) ) {
 			maxt[i] = -1.0f;
+		} else {
+			maxt[i] = (candidate_plane[i] - p0->a1d[i]) / pdir->a1d[i];
+		}
 	}
 
 	// Get largest of the maxt's for final choice of intersection
 	which_plane = 0;
 	for (i = 1; i < 3; i++) {
-		if (maxt[which_plane] < maxt[i])
+		if (maxt[which_plane] < maxt[i]) {
 			which_plane = i;
+		}
 	}
 
 	// check final candidate actually inside box
-	if (maxt[which_plane] < 0.0f)
+	if (maxt[which_plane] < 0.0f) {
 		return 0;
+	}
 
 	for (i = 0; i < 3; i++) {
-		if (which_plane != i) {
-			hitpt->a1d[i] = p0->a1d[i] + maxt[which_plane] * pdir->a1d[i];
-
-			if ( (hitpt->a1d[i] < min->a1d[i]) || (hitpt->a1d[i] > max->a1d[i]) )
-				return 0;
-		} else {
+		if (which_plane == i) {
 			hitpt->a1d[i] = candidate_plane[i];
+		} else {
+			hitpt->a1d[i] = (maxt[which_plane] * pdir->a1d[i]) + p0->a1d[i];
+
+			if ( (hitpt->a1d[i] < min->a1d[i]) || (hitpt->a1d[i] > max->a1d[i]) ) {
+				return 0;
+			}
 		}
 	}
 
@@ -700,11 +702,18 @@ int fvi_polyedge_sphereline(vec3d *hit_point, vec3d *xs0, vec3d *vs, float Rs, i
 	vec3d ve;						// edge velocity
 	float best_sphere_time;		// earliest time sphere hits edge
 	vec3d delta_x;
-	float delta_x_dot_ve, delta_x_dot_vs, ve_dot_vs, ve_sqr, vs_sqr, delta_x_sqr, inv2A;
+	float delta_x_dot_ve, delta_x_dot_vs, ve_dot_vs, ve_sqr, vs_sqr, delta_x_sqr;
 	vec3d temp_edge_hit, temp_sphere_hit;
+	float t_sphere_hit = 0.0f;
+	float A, B, C, temp, discriminant, inv2A;
+	float root, root1, root2;
+	float Rs2 = (Rs * Rs);
+	float Rs_point2 = (0.2f * Rs);
 
 	best_sphere_time = FLT_MAX;
 
+	vs_sqr = vm_vec_mag_squared(vs);
+
 	for (i = 0; i < nv; i++) {
 		// Get vertices of edge to check
 		v0 = *verts[i];
@@ -726,23 +735,18 @@ int fvi_polyedge_sphereline(vec3d *hit_point, vec3d *xs0, vec3d *vs, float Rs, i
 		delta_x_sqr = vm_vec_mag_squared(&delta_x);
 		ve_dot_vs = vm_vec_dotprod(&ve, vs);
 		ve_sqr = vm_vec_mag_squared(&ve);
-		vs_sqr = vm_vec_mag_squared(vs);
-
-		float t_sphere_hit, temp;
 
 		// solve for sphere time
-		float A, B, C, root, discriminant;
-		float root1, root2;
 		A = ve_dot_vs*ve_dot_vs - ve_sqr*vs_sqr;
-		B = 2 * (delta_x_dot_ve*ve_dot_vs - delta_x_dot_vs*ve_sqr);
-		C = delta_x_dot_ve*delta_x_dot_ve + Rs*Rs*ve_sqr - delta_x_sqr*ve_sqr;
+		B = 2.0f * (delta_x_dot_ve*ve_dot_vs - delta_x_dot_vs*ve_sqr);
+		C = delta_x_dot_ve*delta_x_dot_ve + Rs2*ve_sqr - delta_x_sqr*ve_sqr;
 
-		discriminant = B*B - 4*A*C;
-		if (discriminant > 0) {
-			root = fl_sqrt(discriminant);
-			inv2A = 1.0f/(2*A);
-			root1 = (float) ((-B + root)*inv2A);
-			root2 = (float) ((-B - root)*inv2A);
+		discriminant = B*B - 4.0f*A*C;
+		if (discriminant > 0.0f) {
+			inv2A = 1.0f / (2.0f * A);
+			root = sqrt(discriminant);
+			root1 = (-B + root) * inv2A;
+			root2 = (-B - root) * inv2A;
 
 			// sort root1 and root2
 			if (root2 < root1) {
@@ -751,12 +755,11 @@ int fvi_polyedge_sphereline(vec3d *hit_point, vec3d *xs0, vec3d *vs, float Rs, i
 				root2 = temp;
 			}
 
-			if ( (root1 >= -0.05f) && (root1 < 0.0f) ) {
+			if ( (root1 < 0.0f) && (root1 >= -0.05f) )
 				root1 = 0.000001f;
-			}
 
 			// look only at first hit
-			if ( (root1 >= 0) && (root1 <= 1) ) {
+			if ( (root1 <= 1.0f) && (root1 >= 0.0f) ) {
 				t_sphere_hit = root1;
 			} else {
 				goto TryVertex;
@@ -771,40 +774,40 @@ int fvi_polyedge_sphereline(vec3d *hit_point, vec3d *xs0, vec3d *vs, float Rs, i
 			continue;
 
 		vm_vec_scale_add( &temp_sphere_hit, xs0, vs, t_sphere_hit );
-		float q;
+
 		// solve for edge time
-		A = ve_sqr * (ve_dot_vs*ve_dot_vs - ve_sqr*vs_sqr);
-		B = 2*ve_sqr * (delta_x_dot_ve*vs_sqr - delta_x_dot_vs*ve_dot_vs);
-		C = 2*delta_x_dot_ve*delta_x_dot_vs*ve_dot_vs + Rs*Rs*ve_dot_vs*ve_dot_vs 
+		A *= ve_sqr;
+		B = 2.0f*ve_sqr * (delta_x_dot_ve*vs_sqr - delta_x_dot_vs*ve_dot_vs);
+		C = 2.0f*delta_x_dot_ve*delta_x_dot_vs*ve_dot_vs + Rs2*ve_dot_vs*ve_dot_vs 
 			 - delta_x_sqr*ve_dot_vs*ve_dot_vs - delta_x_dot_ve*delta_x_dot_ve*vs_sqr;
 
-		discriminant = B*B - 4*A*C;
+		discriminant = B*B - 4.0f*A*C;
+		inv2A = 1.0f / (2.0f * A);
 
 		// guard against nearly perpendicular sphere edge velocities
-		if ( (discriminant < 0) ) {
-			discriminant = 0;
-		}
+		if (discriminant < 0.0f)
+			root = 0.0f;
+		else
+			root = fl_sqrt(discriminant);
 
-		root = fl_sqrt(discriminant);
-		inv2A = 1.0f/(2*A);
-		root1 = (float) ((-B + root)*inv2A);
-		root2 = (float) ((-B - root)*inv2A);
+		root1 = (-B + root) * inv2A;
+		root2 = (-B - root) * inv2A;
 
 		// given sphere position, find which edge time (position) allows a valid solution
-		if ( (root1 >= 0) && (root1 <= 1) ) {
+		if ( (root1 <= 1.0f) && (root1 >= 0.0f) ) {
 			// try edge root1
 			vm_vec_scale_add( &temp_edge_hit, &v0, &ve, root1 );
-			q = vm_vec_dist_squared(&temp_edge_hit, &temp_sphere_hit);
-			if ( fl_abs(q - Rs*Rs) < 0.2*Rs ) {	// error less than 0.1m absolute (2*delta*Radius)
+			float q = vm_vec_dist_squared(&temp_edge_hit, &temp_sphere_hit);
+			if ( fl_abs(q - Rs2) < Rs_point2 ) {	// error less than 0.1m absolute (2*delta*Radius)
 				goto Hit;
 			}
 		}
 
-		if ( (root2 >= 0) && (root2 <= 1) ) {
+		if ( (root2 <= 1.0f) && (root2 >= 0.0f) ) {
 			// try edge root2
 			vm_vec_scale_add( &temp_edge_hit, &v0, &ve, root2 );
-			q = vm_vec_dist_squared(&temp_edge_hit, &temp_sphere_hit);
-			if ( fl_abs(q - Rs*Rs) < 0.2*Rs ) {	// error less than 0.1m absolute
+			float q = vm_vec_dist_squared(&temp_edge_hit, &temp_sphere_hit);
+			if ( fl_abs(q - Rs2) < Rs_point2 ) {	// error less than 0.1m absolute
 				goto Hit;
 			}
 		} else {
@@ -815,21 +818,21 @@ int fvi_polyedge_sphereline(vec3d *hit_point, vec3d *xs0, vec3d *vs, float Rs, i
 TryVertex:
 		// try V0
 		A = vs_sqr;
-		B = 2*delta_x_dot_vs;
-		C = delta_x_sqr - Rs*Rs;
+		B = 2.0f*delta_x_dot_vs;
+		C = delta_x_sqr - Rs2;
 		int v0_hit;
 		float sphere_v0, sphere_v1;
 
+		v0_hit = 0;
 		sphere_v0 = UNINITIALIZED_VALUE;
 		sphere_v1 = UNINITIALIZED_VALUE;
 
-		v0_hit = 0;
-		discriminant = B*B - 4*A*C;
-		if (discriminant > 0) {
+		inv2A = 1.0f / (2.0f * A);
+		discriminant = B*B - 4.0f*A*C;
+		if (discriminant > 0.0f) {
 			root = fl_sqrt(discriminant);
-			inv2A = 1.0f/(2*A);
-			root1 = (float) ((-B + root)*inv2A);
-			root2 = (float) ((-B - root)*inv2A);
+			root1 = (-B + root) * inv2A;
+			root2 = (-B - root) * inv2A;
 
 			if (root1 > root2) {
 				temp = root1;
@@ -837,11 +840,10 @@ TryVertex:
 				root2 = temp;
 			}
 
-			// look only at the fist hit  (ignore negative first hit)
-			if ( (root1 > 0) && (root1 < 1) ) {
+			// look only at the first hit  (ignore negative first hit)
+			if ( (root1 < 1.0f) && (root1 > 0.0f) ) {
 				v0_hit = 1;
 				sphere_v0 = root1;
-				vm_vec_scale_add( &temp_sphere_hit, xs0, vs, root1 );
 			}
 		}
 
@@ -851,16 +853,15 @@ TryVertex:
 		delta_x_dot_vs = vm_vec_dotprod( &delta_x, vs );
 		int v1_hit;
 
-		B = 2*delta_x_dot_vs;
-		C = delta_x_sqr - Rs*Rs;
+		B = 2.0f*delta_x_dot_vs;
+		C = delta_x_sqr - Rs2;
 
 		v1_hit = 0;
-		discriminant = B*B - 4*A*C;
-		if (discriminant > 0) {
+		discriminant = B*B - 4.0f*A*C;
+		if (discriminant > 0.0f) {
 			root = fl_sqrt(discriminant);
-			inv2A = 1.0f/(2*A);
-			root1 = (float) ((-B + root)*inv2A);
-			root2 = (float) ((-B - root)*inv2A);
+			root1 = (-B + root) * inv2A;
+			root2 = (-B - root) * inv2A;
 
 			if (root1 > root2) {
 				temp = root1;
@@ -869,10 +870,9 @@ TryVertex:
 			}
 
 			// look only at the first hit (ignore negative first hit)
-			if ( (root1 > 0) && (root1 < 1) ) {
+			if ( (root1 < 1.0f) && (root1 > 0.0f) ) {
 				v1_hit = 1;
 				sphere_v1 = root1;
-				vm_vec_scale_add( &temp_sphere_hit, xs0, vs, root1 );
 			}
 		}
 
@@ -897,8 +897,8 @@ TryVertex:
 			continue;
 		}
 
-		vm_vec_scale_add( &temp_sphere_hit, xs0, vs, t_sphere_hit );
-		q = vm_vec_dist_squared(&temp_edge_hit, &temp_sphere_hit);
+		//vm_vec_scale_add( &temp_sphere_hit, xs0, vs, t_sphere_hit );
+		//q = vm_vec_dist_squared(&temp_edge_hit, &temp_sphere_hit);
 
 
 Hit:
