@@ -35,9 +35,9 @@
 char *fs2_open_credit_text = 
 "SOURCE CODE PROJECT STAFF:\n"
 	"\n"
-	"Project Leaders:\n"
+	"Project Leader:\n"
 	"\n"
-	"Cliff \"chief1983\" Gordon"
+	"Fabian \"The E\" Woltermann\n"
 	"\n"
 	"Senior Advisors:\n"
 	"\n"
@@ -45,6 +45,7 @@ char *fs2_open_credit_text =
 	"Michael \"Zacam\" LaFleur\n"
 	"Taylor Richards\n"
 	"Edward \"Inquisitor\" Gardner\n"
+	"Cliff \"chief1983\" Gordon\n"
 	"\n"
 	"Programmers:\n"
 	"\n"
@@ -52,19 +53,13 @@ char *fs2_open_credit_text =
 	"Derek \"Kazan\" Meek\n"
 	"Nick \"phreak\" Iannetta\n"
 	"Mike \"Bobboau\" Abegg\n"
-	"Backslash\n"
-	"Echelon9\n"
-	"Flaming_Sword\n"
-	"FUBAR\n"	
-	"Iss Mneur\n"	
-	"kkmic\n"
-	"Shade\n"
-	"Soulstorm\n"
-	"Sushi\n"
-	"Swifty\n"
-	"Wanderer\n"	
-	"Fabian \"The E\" Woltermann\n"
-	"CommanderDJ\n"
+	"Backslash, Echelon9, Flaming_Sword\n"
+	"FUBAR, Iss Mneur, kkmic\n"
+	"Shade, Soulstorm, Sushi\n"
+	"Swifty, Wanderer, CommanderDJ\n"
+	"Valathil, MageKing17, Yarn\n"
+	"m!m, z64555, zookeeper\n"
+	"jg18, niffiwan\n"
 	"\n"
 	"\n"
 	"Readme Staff:\n"
@@ -113,7 +108,7 @@ char *unmodified_credits = "ORIGINAL VOLITION STAFF:\n\n\n";
 char *mod_check = "Design:";
 
 #define NUM_BUTTONS				5
-#define NUM_IMAGES				46
+#define DEFAULT_NUM_IMAGES		46
 
 #define TECH_DATABASE_BUTTON	0
 #define SIMULATOR_BUTTON		1
@@ -200,8 +195,10 @@ static int	Credits_music_begin_timestamp;
 static int	Credits_frametime;		// frametime of credits_do_frame() loop in ms
 static int	Credits_last_time;		// timestamp used to calc frametime (in ms)
 static float Credits_counter;
+
+static int Credits_num_images;
 static int Credits_artwork_index;
-static int Credits_bmps[NUM_IMAGES];
+static SCP_vector<int> Credits_bmps;
 
 // Positions for credits...
 float Credit_start_pos, Credit_stop_pos, Credit_position = 0.0f;
@@ -223,10 +220,10 @@ enum CreditsPosition
 
 static CreditsPosition SCP_credits_position	= START;
 
-void credits_stop_music()
+void credits_stop_music(bool fade)
 {
 	if ( Credits_music_handle != -1 ) {
-		audiostream_close_file(Credits_music_handle, 1);
+		audiostream_close_file(Credits_music_handle, fade);
 		Credits_music_handle = -1;
 	}
 }
@@ -281,7 +278,7 @@ int credits_screen_button_pressed(int n)
 	return 0;
 }
 
-void credits_parse_table(char* filename)
+void credits_parse_table(const char* filename)
 {
 	int rval;
 
@@ -299,6 +296,13 @@ void credits_parse_table(char* filename)
 	{
 		stuff_string(Credits_music_name, F_NAME, NAME_LENGTH);
 	}
+	if (optional_string("$Number of Images:"))
+	{
+		int temp;
+		stuff_int(&temp);
+		if (temp > 0)
+			Credits_num_images = temp;
+	}
 	if (optional_string("$Start Image Index:"))
 	{
 		stuff_int(&Credits_artwork_index);
@@ -308,9 +312,9 @@ void credits_parse_table(char* filename)
 		{
 			Credits_artwork_index = 0;
 		}
-		else if (Credits_artwork_index >= NUM_IMAGES)
+		else if (Credits_artwork_index >= Credits_num_images)
 		{
-			Credits_artwork_index = NUM_IMAGES - 1;
+			Credits_artwork_index = Credits_num_images - 1;
 		}
 	}
 	if (optional_string("$Text scroll rate:"))
@@ -406,17 +410,11 @@ void credits_parse_table(char* filename)
 
 void credits_parse()
 {
-	// open localization
-	lcl_ext_open();
-
 	// Parse main table
 	credits_parse_table("credits.tbl");
 
 	// Parse modular tables
 	parse_modular_table("*-crd.tbm", credits_parse_table);
-
-	// close localization
-	lcl_ext_close();
 }
 
 void credits_init()
@@ -424,16 +422,23 @@ void credits_init()
 	int i;
 	credits_screen_buttons *b;
 
-	// this is moved up here so we can override it if desired
-	Credits_artwork_index = rand() % NUM_IMAGES;
+	// pre-initialize
+	Credits_num_images = DEFAULT_NUM_IMAGES;
+	Credits_artwork_index = -1;
 
-	// ditto
+	// this is moved up here so we can override it if desired
 	strcpy_s(Credits_music_name, "Cinema");
 
 	// parse credits early so as to set up any overrides (for music and such)
 	Credits_parsed = false;
 	credits_parse();
 
+	// we could conceivably have specified a number of images but not an index,
+	// so if that's the case, set the value here
+	if (Credits_artwork_index < 0) {
+		Credits_artwork_index = rand() % Credits_num_images;
+	}
+
 	int credits_spooled_music_index = event_music_get_spooled_music_index(Credits_music_name);	
 	if(credits_spooled_music_index != -1){
 		char *credits_wavfile_name = Spooled_music[credits_spooled_music_index].filename;		
@@ -628,7 +633,9 @@ void credits_init()
 	Buttons[EXIT_BUTTON][gr_screen.res].button.set_hotkey(KEY_CTRLED | KEY_ENTER);
 
 	Background_bitmap = bm_load(Credits_bitmap_fname[gr_screen.res]);
-	for (i=0; i<NUM_IMAGES; i++){
+
+	Credits_bmps.resize(Credits_num_images);
+	for (i=0; i<Credits_num_images; i++) {
 		Credits_bmps[i] = -1;
 	}
 }
@@ -637,14 +644,14 @@ void credits_close()
 {	
 	int i;
 
-	for (i=0; i<NUM_IMAGES; i++){
-		if (Credits_bmps[i] >= 0){
+	for (i=0; i<Credits_num_images; i++) {
+		if (Credits_bmps[i] >= 0) {
 			bm_release(Credits_bmps[i]);
-			Credits_bmps[i] = -1;
 		}
-	}	
+	}
+	Credits_bmps.clear();
 
-	credits_stop_music();
+	credits_stop_music(true);
 
 	Credit_text_parts.clear();
 
@@ -704,7 +711,7 @@ void credits_do_frame(float frametime)
 	GR_MAYBE_CLEAR_RES(Background_bitmap);
 	if (Background_bitmap >= 0) {
 		gr_set_bitmap(Background_bitmap);
-		gr_bitmap(0, 0);
+		gr_bitmap(0, 0, GR_RESIZE_MENU);
 	} 
 
 	percent = (int) (100.0f - (Credits_artwork_display_time - Credits_counter) * 100.0f / Credits_artwork_fade_time);
@@ -713,7 +720,7 @@ void credits_do_frame(float frametime)
 	}
 
 	next = Credits_artwork_index + 1;
-	if (next >= NUM_IMAGES){
+	if (next >= Credits_num_images){
 		next = 0;
 	}
 
@@ -732,7 +739,7 @@ void credits_do_frame(float frametime)
 		char buf[40];
 
 		if (gr_screen.res == GR_1024) {
-			sprintf(buf, NOX("2_CrIm%.2d"), Credits_artwork_index);
+			sprintf(buf, NOX("2_CrIm%.2d"), next);
 		} else {
 			sprintf(buf, NOX("CrIm%.2d"), next);
 		}
@@ -755,7 +762,7 @@ void credits_do_frame(float frametime)
 		bx2 = Credits_image_coords[gr_screen.res][CREDITS_X_COORD] + ((Credits_image_coords[gr_screen.res][CREDITS_W_COORD] - bw2)/2);
 		by2 = Credits_image_coords[gr_screen.res][CREDITS_Y_COORD] + ((Credits_image_coords[gr_screen.res][CREDITS_H_COORD] - bh2)/2);
 
-		gr_cross_fade(bm1, bm2, bx1, by1, bx2, by2, (float)percent / 100.0f);
+		gr_cross_fade(bm1, bm2, bx1, by1, bx2, by2, (float)percent / 100.0f, GR_RESIZE_MENU);
 	}
 
 	Ui_window.draw();
@@ -770,17 +777,11 @@ void credits_do_frame(float frametime)
 		Buttons[CREDITS_BUTTON][gr_screen.res].button.draw_forced(2);
 	}
 
-	gr_set_clip(Credits_text_coords[gr_screen.res][CREDITS_X_COORD], Credits_text_coords[gr_screen.res][CREDITS_Y_COORD], Credits_text_coords[gr_screen.res][CREDITS_W_COORD], Credits_text_coords[gr_screen.res][CREDITS_H_COORD]);
+	gr_set_clip(Credits_text_coords[gr_screen.res][CREDITS_X_COORD], Credits_text_coords[gr_screen.res][CREDITS_Y_COORD], Credits_text_coords[gr_screen.res][CREDITS_W_COORD], Credits_text_coords[gr_screen.res][CREDITS_H_COORD], GR_RESIZE_MENU);
 	gr_set_font(FONT1);
 	gr_set_color_fast(&Color_normal);
 	
-	int sy; // The current position of the first text part
-	if ( Credit_position > 0 ) {
-		sy = fl2i(Credit_position+0.5f);
-	} else {
-		sy = fl2i(Credit_position-0.5f);
-	}
-
+	int y_offset = 0;
 	for (SCP_vector<SCP_string>::iterator iter = Credit_text_parts.begin(); iter != Credit_text_parts.end(); ++iter)
 	{
 		int height;
@@ -788,12 +789,13 @@ void credits_do_frame(float frametime)
 		gr_get_string_size(NULL, &height, iter->c_str(), iter->length());
 
 		// Check if the text part is actually visible
-		if (sy + height > 0)
+		if (Credit_position + y_offset + height > 0.0f)
 		{
-			gr_string(0x8000, sy, iter->c_str());
+			extern void gr_opengl_string(float sx, float sy, const char *s, int resize_mode);
+			gr_opengl_string((float)0x8000, Credit_position + y_offset, iter->c_str(), GR_RESIZE_MENU);
 		}
 
-		sy = sy + height;
+		y_offset += height;
 	}
 
 	int temp_time;
