@@ -79,7 +79,7 @@ void options_multi_load_bmaps();
 void options_multi_unload_bmaps();
 
 // add a notification message
-void options_multi_add_notify(char *str);
+void options_multi_add_notify(const char *str);
 
 // process and blit any notification messages
 void options_multi_notify_process();
@@ -137,7 +137,7 @@ UI_XSTR Om_pro_text[GR_NUM_RESOLUTIONS][OM_PRO_NUM_TEXT] = {
 		{ "IP Address",	1380,	30,	128,	UI_XSTR_COLOR_GREEN, -1, &Om_pro_bogus },
 		{ "add",				1381,	22,	235,	UI_XSTR_COLOR_GREEN, -1, &Om_pro_buttons[0][OM_PRO_ADD_IP].button },
 		{ "rem.",			1382,	68,	235,	UI_XSTR_COLOR_GREEN, -1, &Om_pro_buttons[0][OM_PRO_DELETE_IP].button },
-		{ "Broadcast Locally",	1397,	42,	260,	UI_XSTR_COLOR_GREEN, -1, &Om_pro_buttons[0][OM_PRO_LOCAL_BROADCAST].button },	
+		{ "Broadcast Locally",	1387,	42,	260,	UI_XSTR_COLOR_GREEN, -1, &Om_pro_buttons[0][OM_PRO_LOCAL_BROADCAST].button },	
 		{ "PXO",				1383,	42,	291,	UI_XSTR_COLOR_GREEN, -1, &Om_pro_buttons[0][OM_PRO_VMT].button },
 		{ "Login",			1384,	14,	309,	UI_XSTR_COLOR_GREEN, -1, &Om_pro_bogus },
 		{ "Password",		1385,	14,	336,	UI_XSTR_COLOR_GREEN, -1, &Om_pro_bogus },
@@ -151,7 +151,7 @@ UI_XSTR Om_pro_text[GR_NUM_RESOLUTIONS][OM_PRO_NUM_TEXT] = {
 		{ "IP Address",	1380,	47,	206,	UI_XSTR_COLOR_GREEN, -1, &Om_pro_bogus },
 		{ "add",				1381,	36,	375,	UI_XSTR_COLOR_GREEN, -1, &Om_pro_buttons[1][OM_PRO_ADD_IP].button },
 		{ "rem.",			1382,	109,	375,	UI_XSTR_COLOR_GREEN, -1, &Om_pro_buttons[1][OM_PRO_DELETE_IP].button },
-		{ "Broadcast Locally",	1397,	68,	417,	UI_XSTR_COLOR_GREEN, -1, &Om_pro_buttons[1][OM_PRO_LOCAL_BROADCAST].button },	
+		{ "Broadcast Locally",	1387,	68,	417,	UI_XSTR_COLOR_GREEN, -1, &Om_pro_buttons[1][OM_PRO_LOCAL_BROADCAST].button },	
 		{ "PXO",				1383,	68,	467,	UI_XSTR_COLOR_GREEN, -1, &Om_pro_buttons[1][OM_PRO_VMT].button },
 		{ "Login",			1384,	23,	495,	UI_XSTR_COLOR_GREEN, -1, &Om_pro_bogus },
 		{ "Password",		1385,	23,	538,	UI_XSTR_COLOR_GREEN, -1, &Om_pro_bogus },
@@ -620,7 +620,7 @@ void options_multi_unload_bmaps()
 }
 
 // add a notification message
-void options_multi_add_notify(char *str)
+void options_multi_add_notify(const char *str)
 {
 	// copy the string
 	memset(Om_notify_string,0,255);
@@ -636,12 +636,13 @@ void options_multi_add_notify(char *str)
 void options_multi_notify_process()
 {
 	int w;
-	char *p_str[3];
+	const char *p_str[3];
 	int n_chars[3];
 	char line[255];
 	int line_count;
 	int y_start;
 	int idx;
+	int line_height;
 	
 	// if there is no timestamp, do nothing
 	if(Om_notify_stamp == -1){
@@ -655,6 +656,7 @@ void options_multi_notify_process()
 	}
 
 	// otherwise display the string
+	line_height = gr_get_font_height() + 1;
 	line_count = split_str(Om_notify_string, 600, n_chars, p_str, 3);	
 	y_start = OM_NOTIFY_Y;
 	gr_set_color_fast(&Color_bright);
@@ -663,9 +665,9 @@ void options_multi_notify_process()
 		strncpy(line, p_str[idx], n_chars[idx]);
 				
 		gr_get_string_size(&w,NULL,line);
-		gr_string((600 - w)/2,y_start,line);
+		gr_string((600 - w)/2,y_start,line,GR_RESIZE_MENU);
 
-		y_start += 10;
+		y_start += line_height;
 	}	
 }
 
@@ -1203,6 +1205,7 @@ void options_multi_protocol_display_ips()
 {
 	int idx;
 	int y_start = Ip_list_coords[gr_screen.res][1];
+	int line_height = gr_get_font_height() + 1;
 		
 	// get the # of items we should be displaying based upon the # of addresses and the starting display point
 	if(Om_ip_start >= Ip_list_max_display[gr_screen.res]){
@@ -1219,8 +1222,8 @@ void options_multi_protocol_display_ips()
 			gr_set_color_fast(&Color_white);
 		}
 
-		gr_printf(Ip_list_coords[gr_screen.res][0], y_start, Om_ip_addrs[idx]);
-		y_start += 10;
+		gr_printf_menu(Ip_list_coords[gr_screen.res][0], y_start, Om_ip_addrs[idx]);
+		y_start += line_height;
 	}
 }
 
@@ -1255,7 +1258,7 @@ void options_multi_protocol_check_ip_list()
 	if(Om_ip_button.pressed()){
 		// determine which item he clicked on	
 		Om_ip_button.get_mouse_pos(NULL, &click_y);
-		item = click_y / 10;
+		item = click_y / (gr_get_font_height() + 1);
 			
 		// determine if there is an item in this location, and select it if so
 		if(item < Om_ip_disp_count){
@@ -1978,7 +1981,7 @@ void options_multi_vox_process_waveform()
 
 	// grey the screen
 	gr_set_shader(&Grey_shader);
-	gr_shade(0,0,gr_screen.clip_width, gr_screen.clip_height, false);
+	gr_shade(0,0,gr_screen.clip_width, gr_screen.clip_height, GR_RESIZE_NONE);
 
 	switch(Om_vox_test_status){
 	case OM_VOX_TEST_RECORDING:
@@ -2009,13 +2012,13 @@ void options_multi_vox_process_waveform()
 			}
 
 			running_avg /= avg_len;
-			gr_line((gr_screen.max_w_unscaled - OM_VOX_WAVE_WIDTH)/2 + idx, OM_VOX_WAVE_Y, (gr_screen.max_w_unscaled - OM_VOX_WAVE_WIDTH)/2 + idx, OM_VOX_WAVE_Y + running_avg);
+			gr_line((gr_screen.max_w_unscaled - OM_VOX_WAVE_WIDTH)/2 + idx, OM_VOX_WAVE_Y, (gr_screen.max_w_unscaled - OM_VOX_WAVE_WIDTH)/2 + idx, OM_VOX_WAVE_Y + running_avg, GR_RESIZE_MENU);
 		}		
 
 		// if this packet would have been dropped, notify the user
 		if(multi_voice_test_packet_tossed()){
 			gr_set_color_fast(&Color_bright);
-			gr_string(OM_VOX_DROP_ICON_X,OM_VOX_DROP_ICON_Y, XSTR( "Packet Overflow", 393));
+			gr_string(OM_VOX_DROP_ICON_X,OM_VOX_DROP_ICON_Y, XSTR( "Packet Overflow", 393), GR_RESIZE_MENU);
 		}		
 		break;
 
@@ -2041,7 +2044,7 @@ void options_multi_vox_process_waveform()
 			}
 
 			running_avg /= avg_len;			
-			gr_line((gr_screen.max_w_unscaled - OM_VOX_WAVE_WIDTH)/2 + idx, OM_VOX_WAVE_Y, (gr_screen.max_w_unscaled - OM_VOX_WAVE_WIDTH)/2 + idx, OM_VOX_WAVE_Y + running_avg);
+			gr_line((gr_screen.max_w_unscaled - OM_VOX_WAVE_WIDTH)/2 + idx, OM_VOX_WAVE_Y, (gr_screen.max_w_unscaled - OM_VOX_WAVE_WIDTH)/2 + idx, OM_VOX_WAVE_Y + running_avg, GR_RESIZE_MENU);
 		}				
 		break;
 	}
@@ -2055,10 +2058,12 @@ void options_multi_vox_process_player_list()
 	int selected_index,click_y;
 	char str[CALLSIGN_LEN+2];
 
+	int line_height = gr_get_font_height() + 1;
+
 	// check for mouse clicks
 	if(Om_vox_plist_button.pressed()){
 		Om_vox_plist_button.get_mouse_pos(NULL,&click_y);
-		selected_index = (click_y / 10) + Om_vox_plist_start;
+		selected_index = (click_y / line_height) + Om_vox_plist_start;
 
 		// if he clicked on a valid player, select him
 		if(Om_vox_players[selected_index] != NULL){
@@ -2085,10 +2090,10 @@ void options_multi_vox_process_player_list()
 			gr_force_fit_string(str, CALLSIGN_LEN+1, Om_vox_plist_coords[gr_screen.res][2]);
 
 			// blit the callsign
-			gr_string(Om_vox_plist_coords[gr_screen.res][0], y_start, str);
+			gr_string(Om_vox_plist_coords[gr_screen.res][0], y_start, str, GR_RESIZE_MENU);
 
 			// increment the y index
-			y_start += 10;
+			y_start += line_height;
 
 			// increment the player count
 			p_count++;
