@@ -121,7 +121,7 @@ static sim_room_buttons Buttons[GR_NUM_RESOLUTIONS][NUM_BUTTONS] = {
 		sim_room_buttons("2_LMB_07",	10,	732,	64,	739,	7),
 		sim_room_buttons("2_LMB_08",	854,	681,	800, 704,	8),
 		sim_room_buttons("2_LMB_09",	914,	681,	915, 660,	9),
-		sim_room_buttons("2_LMB_10",	854,	728,	800, 728,	10),
+		sim_room_buttons("2_LMB_10",	854,	728,	780, 736,	10),
 
 		sim_room_buttons("2_TDB_00",	12,	5,		59,	12,	0),
 		sim_room_buttons("2_TDB_01",	12,	31,	59,	37,	1),
@@ -221,6 +221,9 @@ static int Background_bitmap;
 static UI_WINDOW Ui_window;
 static UI_BUTTON List_buttons[LIST_BUTTONS_MAX];  // buttons for each line of text in list
 
+int Sim_room_overlay_id;
+int Campaign_room_overlay_id;
+
 // globals
 bool Campaign_room_no_campaigns = false;
 
@@ -263,9 +266,9 @@ void sim_room_blit_icons(int line_index, int y_start, fs_builtin_mission *fb = N
 // Finds a hash value for mission filename
 //
 // returns hash value
-int hash_filename(char *filename) {
+int hash_filename(const char *filename) {
 	ulonglong hash_val = 0;
-	char *ptr = filename;
+	const char *ptr = filename;
 	
 	// Don't hash .fsm extension, convert all to upper case
 	for (int i=0; i < ((signed int)(strlen(filename)) - 4); i++) {
@@ -321,7 +324,7 @@ int hash_insert(char *filename) {
 // Checks if a filename already exitst in the hash table
 //
 // returns 1 if found (collision), 0 if no collision
-int campaign_mission_hash_collision(char *filename)
+int campaign_mission_hash_collision(const char *filename)
 {
 	int hash_val = hash_filename(filename);
 	hash_node *cur_node = Campaign_mission_hash_table[hash_val];
@@ -402,7 +405,7 @@ int sim_room_line_add(int type, char *name, char *filename, int x, int y, int fl
 }
 
 // build up a list of all missions in all campaigns.
-int sim_room_campaign_mission_filter(char *filename)
+int sim_room_campaign_mission_filter(const char *filename)
 {
 	int num;
 
@@ -416,7 +419,7 @@ int sim_room_campaign_mission_filter(char *filename)
 }
 
 // filter out all missions already used in existing campaigns
-int sim_room_standalone_mission_filter(char *filename)
+int sim_room_standalone_mission_filter(const char *filename)
 {
 	int type;
 	char mission_name[255];
@@ -852,7 +855,7 @@ int readyroom_continue_campaign()
 			// this isn't the same thing that is done in mission_campaign_next_mission(), but it's
 			// cleaner to do this here than it is to hack that function to do the same thing
 			Campaign.current_mission = Campaign.prev_mission;
-			strncpy( Game_current_mission_filename, Campaign.missions[Campaign.current_mission].name, MAX_FILENAME_LEN );
+			strcpy_s( Game_current_mission_filename, Campaign.missions[Campaign.current_mission].name );
 
 			// set the bit for campaign mode
 			Game_mode |= GM_CAMPAIGN_MODE;
@@ -873,13 +876,13 @@ int readyroom_continue_campaign()
 	if (mc_rval == -1)
 	{  // is campaign and next mission valid?
 		gamesnd_play_iface(SND_GENERAL_FAIL);
-		popup(0, 1, POPUP_OK, XSTR( "The campaign is over.  To replay the campaign, either create a new pilot or restart the campaign in the campaign room.", 112) );
+		popup(PF_USE_AFFIRMATIVE_ICON, 1, POPUP_OK, XSTR( "The campaign is over.  To replay the campaign, either create a new pilot or restart the campaign in the campaign room.", 112) );
 		return -1;
 	}
 	else if(mc_rval == -2)
 	{
 		gamesnd_play_iface(SND_GENERAL_FAIL);
-		popup(0, 1, POPUP_OK, NOX("The current campaign has no missions") );
+		popup(PF_USE_AFFIRMATIVE_ICON, 1, POPUP_OK, NOX("The current campaign has no missions") );
 		return -1;
 	}
 
@@ -897,7 +900,7 @@ void sim_room_commit()
 		return;
 	}
 
-	strncpy(Game_current_mission_filename, sim_room_lines[Selected_line].filename, MAX_FILENAME_LEN);
+	strcpy_s(Game_current_mission_filename, sim_room_lines[Selected_line].filename);
 
 	Game_mode &= ~(GM_CAMPAIGN_MODE);						// be sure this bit is clear
 
@@ -928,7 +931,7 @@ int sim_room_button_pressed(int n)
 
 		case CAMPAIGN_TAB:
 			if ( !strlen(Campaign.filename) ) {
-				popup( PF_NO_NETWORKING, 1, POPUP_OK, XSTR( "The currently active campaign cannot be found, unable to switch to campaign mode!", 1612));
+				popup( PF_USE_AFFIRMATIVE_ICON | PF_NO_NETWORKING, 1, POPUP_OK, XSTR( "The currently active campaign cannot be found, unable to switch to campaign mode!", 1612));
 				break;
 			}
 
@@ -1049,8 +1052,8 @@ void sim_room_init()
 	Background_bitmap = bm_load(Sim_filename[gr_screen.res]);
 
 	// load in help overlay bitmap	
-	help_overlay_load(SIM_ROOM_OVERLAY);
-	help_overlay_set_state(SIM_ROOM_OVERLAY,0);
+	Sim_room_overlay_id = help_overlay_get_index(SIM_ROOM_OVERLAY);
+	help_overlay_set_state(Sim_room_overlay_id,gr_screen.res,0);
 
 	Scroll_offset = Selected_line = 0;
 
@@ -1082,7 +1085,7 @@ void sim_room_init()
 	GR_MAYBE_CLEAR_RES(Background_bitmap);
 	if(Background_bitmap != -1){
 		gr_set_bitmap(Background_bitmap);
-		gr_bitmap(0, 0);
+		gr_bitmap(0, 0, GR_RESIZE_MENU);
 	}
 	Ui_window.draw();
 	gr_flip();		
@@ -1151,9 +1154,6 @@ void sim_room_close()
 	// free global Campaign_* list stuff
 	mission_campaign_free_list();
 
-	// unload the overlay bitmap
-	help_overlay_unload(SIM_ROOM_OVERLAY);
-
 	campaign_mission_hash_table_delete();
 
 	Ui_window.destroy();
@@ -1183,7 +1183,7 @@ void sim_room_do_frame(float frametime)
 			break;
 		}
 
-	if ( help_overlay_active(SIM_ROOM_OVERLAY) ) {
+	if ( help_overlay_active(Sim_room_overlay_id) ) {
 		Buttons[gr_screen.res][HELP_BUTTON].button.reset_status();
 		Ui_window.set_ignore_gadgets(1);
 	}
@@ -1191,14 +1191,14 @@ void sim_room_do_frame(float frametime)
 	k = Ui_window.process() & ~KEY_DEBUGGED;
 
 	if ( (k > 0) || B1_JUST_RELEASED ) {
-		if ( help_overlay_active(SIM_ROOM_OVERLAY) ) {
-			help_overlay_set_state(SIM_ROOM_OVERLAY, 0);
+		if ( help_overlay_active(Sim_room_overlay_id) ) {
+			help_overlay_set_state(Sim_room_overlay_id, gr_screen.res, 0);
 			Ui_window.set_ignore_gadgets(0);
 			k = 0;
 		}
 	}
 
-	if ( !help_overlay_active(SIM_ROOM_OVERLAY) ) {
+	if ( !help_overlay_active(Sim_room_overlay_id) ) {
 		Ui_window.set_ignore_gadgets(0);
 	}
 
@@ -1270,7 +1270,7 @@ void sim_room_do_frame(float frametime)
 	GR_MAYBE_CLEAR_RES(Background_bitmap);
 	if (Background_bitmap >= 0) {
 		gr_set_bitmap(Background_bitmap);
-		gr_bitmap(0, 0);
+		gr_bitmap(0, 0, GR_RESIZE_MENU);
 	}
 
 	Ui_window.draw();
@@ -1298,12 +1298,12 @@ void sim_room_do_frame(float frametime)
 		gr_set_color_fast(&Color_text_heading);
 		strcpy_s(buf, Campaign.name);
 		gr_force_fit_string(buf, 255, list_w1);
-		gr_printf(list_x1, Mission_list_coords[gr_screen.res][1], buf);
+		gr_printf_menu(list_x1, Mission_list_coords[gr_screen.res][1], buf);
 
 		if (Campaign.filename[0] != '\0') {			
 			sprintf(buf, NOX("%s%s"), Campaign.filename, FS_CAMPAIGN_FILE_EXT);
 			gr_force_fit_string(buf, 255, list_w2);
-			gr_printf(list_x2, Mission_list_coords[gr_screen.res][1], buf);		
+			gr_printf_menu(list_x2, Mission_list_coords[gr_screen.res][1], buf);		
 
 			// blit the proper icons if necessary
 			char full_name[256];
@@ -1336,12 +1336,12 @@ void sim_room_do_frame(float frametime)
 
 		strcpy_s(buf, sim_room_lines[line].name);
 		gr_force_fit_string(buf, 255, list_x1 + list_w1 - sim_room_lines[line].x);
-		gr_printf(sim_room_lines[line].x, y, buf);
+		gr_printf_menu(sim_room_lines[line].x, y, buf);
 
 		if (sim_room_lines[line].filename) {
 			strcpy_s(buf, sim_room_lines[line].filename);
 			gr_force_fit_string(buf, 255, list_w2);
-			gr_printf(list_x2, y, buf);
+			gr_printf_menu(list_x2, y, buf);
 		}
 
 		// blit additional icon information
@@ -1355,7 +1355,7 @@ void sim_room_do_frame(float frametime)
 		List_buttons[i++].disable();
 
 	// blit help overlay if active
-	help_overlay_maybe_blit(SIM_ROOM_OVERLAY);
+	help_overlay_maybe_blit(Sim_room_overlay_id, gr_screen.res);
 
 	gr_flip();
 }
@@ -1374,7 +1374,7 @@ void sim_room_blit_icons(int line_index, int y_start, fs_builtin_mission *fb, in
 	// if the line is flagged as a volition file
 	if(is_from_volition && (Mission_icon_bitmaps[MISSION_ICON_VOLITION] >= 0)){		
 		gr_set_bitmap(Mission_icon_bitmaps[MISSION_ICON_VOLITION]);
-		gr_bitmap(Sim_volition_icon_x[gr_screen.res], y_start + MISSION_ICON_VOLITION_Y_OFFSET);
+		gr_bitmap(Sim_volition_icon_x[gr_screen.res], y_start + MISSION_ICON_VOLITION_Y_OFFSET, GR_RESIZE_MENU);
 	}	
 }
 
@@ -1456,7 +1456,7 @@ static int Desc_scroll_offset;
 static int Selected_campaign_index;
 static int Active_campaign_index;
 
-char *Info_text_ptrs[MAX_INFO_LINES];
+const char *Info_text_ptrs[MAX_INFO_LINES];
 int Num_info_lines, Info_text_line_size[MAX_INFO_LINES];
 
 void campaign_room_build_listing()
@@ -1533,7 +1533,7 @@ int campaign_room_reset_campaign(int n)
 		strcat(filename, FS_CAMPAIGN_FILE_EXT);
 
 		mission_campaign_savefile_delete(filename);
-		mission_campaign_load(filename);
+		mission_campaign_load(filename, NULL, 1 , false); // retail doesn't reset stats when resetting the campaign
 		mission_campaign_next_mission();
 
 		vm_free(filename);
@@ -1680,8 +1680,8 @@ void campaign_room_init()
 	Background_bitmap = bm_load(Campaign_filename[gr_screen.res]);
 
 	// load in help overlay bitmap	
-	help_overlay_load(CAMPAIGN_ROOM_OVERLAY);
-	help_overlay_set_state(CAMPAIGN_ROOM_OVERLAY,0);
+	Campaign_room_overlay_id = help_overlay_get_index(CAMPAIGN_ROOM_OVERLAY);
+	help_overlay_set_state(Campaign_room_overlay_id,gr_screen.res,0);
 
 	Num_desc_lines = 0;
 	Desc_scroll_offset = Scroll_offset = 0;
@@ -1723,9 +1723,6 @@ void campaign_room_close()
 	// free the global Campaign_* list stuff
 	mission_campaign_free_list();
 
-	// unload the overlay bitmap
-	help_overlay_unload(CAMPAIGN_ROOM_OVERLAY);
-
 	Ui_window.destroy();
 	common_free_interface_palette();		// restore game palette
 	Pilot.save_player();
@@ -1739,7 +1736,7 @@ void campaign_room_do_frame(float frametime)
 	int font_height = gr_get_font_height();
 	int select_tease_line = -1;  // line mouse is down on, but won't be selected until button released
 
-	if ( help_overlay_active(CAMPAIGN_ROOM_OVERLAY) ) {
+	if ( help_overlay_active(Campaign_room_overlay_id) ) {
 		// Cr_buttons[gr_screen.res][CR_HELP_BUTTON].button.reset_status();
 		Ui_window.set_ignore_gadgets(1);
 	}
@@ -1747,14 +1744,14 @@ void campaign_room_do_frame(float frametime)
 	k = Ui_window.process() & ~KEY_DEBUGGED;
 
 	if ( (k > 0) || B1_JUST_RELEASED ) {
-		if ( help_overlay_active(CAMPAIGN_ROOM_OVERLAY) ) {
-			help_overlay_set_state(CAMPAIGN_ROOM_OVERLAY, 0);
+		if ( help_overlay_active(Campaign_room_overlay_id) ) {
+			help_overlay_set_state(Campaign_room_overlay_id, gr_screen.res, 0);
 			Ui_window.set_ignore_gadgets(0);
 			k = 0;
 		}
 	}
 
-	if ( !help_overlay_active(CAMPAIGN_ROOM_OVERLAY) ) {
+	if ( !help_overlay_active(Campaign_room_overlay_id) ) {
 		Ui_window.set_ignore_gadgets(0);
 	}
 
@@ -1809,7 +1806,7 @@ void campaign_room_do_frame(float frametime)
 	GR_MAYBE_CLEAR_RES(Background_bitmap);
 	if (Background_bitmap >= 0) {
 		gr_set_bitmap(Background_bitmap);
-		gr_bitmap(0, 0);
+		gr_bitmap(0, 0, GR_RESIZE_MENU);
 	}
 
 	Ui_window.draw();
@@ -1825,13 +1822,13 @@ void campaign_room_do_frame(float frametime)
 		if (!stricmp(sim_room_lines[line].filename, Campaign.filename)) {
 			gr_set_color_fast(&Color_white);
 			i = y + font_height / 2 - 1;
-			gr_circle(Cr_list_coords[gr_screen.res][0] - 6, i, 5);
+			gr_circle(Cr_list_coords[gr_screen.res][0] - 6, i, 5, GR_RESIZE_MENU);
 
 			gr_set_color_fast(&Color_bright_white);
-			gr_line(Cr_list_coords[gr_screen.res][0] - 10, i, Cr_list_coords[gr_screen.res][0] - 8, i);
-			gr_line(Cr_list_coords[gr_screen.res][0] - 6, i - 4, Cr_list_coords[gr_screen.res][0] - 6, i - 2);
-			gr_line(Cr_list_coords[gr_screen.res][0] - 4, i, Cr_list_coords[gr_screen.res][0] - 2, i);
-			gr_line(Cr_list_coords[gr_screen.res][0] - 6, i + 2, Cr_list_coords[gr_screen.res][0] - 6, i + 4);
+			gr_line(Cr_list_coords[gr_screen.res][0] - 10, i, Cr_list_coords[gr_screen.res][0] - 8, i, GR_RESIZE_MENU);
+			gr_line(Cr_list_coords[gr_screen.res][0] - 6, i - 4, Cr_list_coords[gr_screen.res][0] - 6, i - 2, GR_RESIZE_MENU);
+			gr_line(Cr_list_coords[gr_screen.res][0] - 4, i, Cr_list_coords[gr_screen.res][0] - 2, i, GR_RESIZE_MENU);
+			gr_line(Cr_list_coords[gr_screen.res][0] - 6, i + 2, Cr_list_coords[gr_screen.res][0] - 6, i + 4, GR_RESIZE_MENU);
 		}
 
 		if (line == Selected_campaign_index)
@@ -1843,7 +1840,7 @@ void campaign_room_do_frame(float frametime)
 
 		strcpy_s(buf, sim_room_lines[line].name);
 		gr_force_fit_string(buf, 255, Cr_list_coords[gr_screen.res][0] + Cr_list_coords[gr_screen.res][2] - sim_room_lines[line].x);
-		gr_printf(sim_room_lines[line].x, y, buf);
+		gr_printf_menu(sim_room_lines[line].x, y, buf);
 		line++;
 	}
 
@@ -1863,7 +1860,7 @@ void campaign_room_do_frame(float frametime)
 		strncpy(line_text, Info_text_ptrs[i], Info_text_line_size[i]);
 		line_text[Info_text_line_size[i]] = 0;
 		drop_white_space(line_text);
-		gr_string(Cr_info_coords[gr_screen.res][0], Cr_info_coords[gr_screen.res][1] + y, line_text);
+		gr_string(Cr_info_coords[gr_screen.res][0], Cr_info_coords[gr_screen.res][1] + y, line_text, GR_RESIZE_MENU);
 		y += font_height;
 		i++;
 	}
@@ -1875,7 +1872,7 @@ void campaign_room_do_frame(float frametime)
 	}
 
 	// blit help overlay if active
-	help_overlay_maybe_blit(CAMPAIGN_ROOM_OVERLAY);
+	help_overlay_maybe_blit(Campaign_room_overlay_id, gr_screen.res);
 
 	gr_flip();
 }
