@@ -45,6 +45,7 @@
 #include "missionui/redalert.h"
 #include "pilotfile/pilotfile.h"
 #include "popup/popup.h"
+#include "menuui/mainhallmenu.h"
 
 
 // campaign wasn't ended
@@ -90,14 +91,14 @@ LOCAL UI_BUTTON Campaign_okb, Campaign_cancelb;
 campaign Campaign;
 
 
-bool campaign_is_ignored(char *filename, bool add_extension = false);
+bool campaign_is_ignored(const char *filename);
 
 /**
  * Returns a string (which is malloced in this routine) of the name of the given freespace campaign file.  
  * In the type field, we return if the campaign is a single player or multiplayer campaign.  
  * The type field will only be valid if the name returned is non-NULL
  */
-int mission_campaign_get_info(char *filename, char *name, int *type, int *max_players, char **desc)
+int mission_campaign_get_info(const char *filename, char *name, int *type, int *max_players, char **desc)
 {
 	int rval, i, success = 0;
 	char campaign_type[NAME_LENGTH], fname[MAX_FILENAME_LEN];
@@ -113,9 +114,6 @@ int mission_campaign_get_info(char *filename, char *name, int *type, int *max_pl
 	}
 	Assert(fname_len < MAX_FILENAME_LEN);
 
-	// open localization
-	lcl_ext_open();
-
 	*type = -1;
 	do {
 		if ((rval = setjmp(parse_abort)) != 0) {
@@ -167,9 +165,6 @@ int mission_campaign_get_info(char *filename, char *name, int *type, int *max_pl
 		}
 	} while (0);
 
-	// close localization
-	lcl_ext_close();
-
 	Assert(success);
 	return success;
 }
@@ -179,7 +174,7 @@ int mission_campaign_get_info(char *filename, char *name, int *type, int *max_pl
  * @return Number of missions added to the 'list', and up to 'max' missions may be added to 'list'.  
  * @return Negative on error.
  */
-int mission_campaign_get_mission_list(char *filename, char **list, int max)
+int mission_campaign_get_mission_list(const char *filename, char **list, int max)
 {
 	int rval, i, num = 0;
 	char name[MAX_FILENAME_LEN];
@@ -240,18 +235,18 @@ void mission_campaign_free_list()
 	Campaign_names_inited = 0;
 }
 
-int mission_campaign_maybe_add(char *filename)
+int mission_campaign_maybe_add(const char *filename)
 {
 	char name[NAME_LENGTH];
 	char *desc = NULL;
 	int type, max_players;
 
-	if ( mission_campaign_get_info( filename, name, &type, &max_players, &desc) ) {
-		// don't add ignored campaigns
-		if (campaign_is_ignored(filename)) {
-			return 0;
-		}
+	// don't add ignored campaigns
+	if (campaign_is_ignored(filename)) {
+		return 0;
+	}
 
+	if ( mission_campaign_get_info( filename, name, &type, &max_players, &desc) ) {
 		if ( !MC_multiplayer && (type == CAMPAIGN_TYPE_SINGLE) ) {
 			Campaign_names[Num_campaigns] = vm_strdup(name);
 
@@ -413,20 +408,17 @@ void mission_campaign_get_sw_info()
  * this file.  If you change the format of the campaign file, you should be sure these related
  * functions work properly and update them if it breaks them.
  */
-int mission_campaign_load( char *filename, player *pl, int load_savefile )
+int mission_campaign_load( char *filename, player *pl, int load_savefile, bool reset_stats )
 {
 	int len, rval, i;
 	char name[NAME_LENGTH], type[NAME_LENGTH], temp[NAME_LENGTH];
 
-	filename = cf_add_ext(filename, FS_CAMPAIGN_FILE_EXT);
-
 	if (campaign_is_ignored(filename)) {
 		Campaign_file_missing = 1;
 		return CAMPAIGN_ERROR_IGNORED;
 	}
 
-	// open localization
-	lcl_ext_open();	
+	filename = cf_add_ext(filename, FS_CAMPAIGN_FILE_EXT);
 
 	if ( pl == NULL )
 		pl = Player;
@@ -440,9 +432,6 @@ int mission_campaign_load( char *filename, player *pl, int load_savefile )
 	if ((rval = setjmp(parse_abort)) != 0) {
 		mprintf(("Error parsing '%s'\r\nError code = %i.\r\n", filename, rval));
 
-		// close localization
-		lcl_ext_close();
-
 		Campaign.filename[0] = 0;
 		Campaign.num_missions = 0;
 
@@ -528,6 +517,8 @@ int mission_campaign_load( char *filename, player *pl, int load_savefile )
 			cm = &Campaign.missions[Campaign.num_missions];
 			cm->name = vm_strdup(name);
 
+			cm->notes = NULL;
+
 			cm->briefing_cutscene[0] = 0;
 			if ( optional_string("+Briefing Cutscene:") )
 				stuff_string( cm->briefing_cutscene, F_NAME, NAME_LENGTH );
@@ -549,6 +540,19 @@ int mission_campaign_load( char *filename, player *pl, int load_savefile )
 				cm->main_hall = temp;
 			}
 
+			// Goober5000 - substitute main hall (like substitute music)
+			if (optional_string("+Substitute Main Hall:")) {
+				stuff_string(temp, F_RAW, 32);
+
+				// see if this main hall exists
+				main_hall_defines *mhd = main_hall_get_pointer(temp);
+				if (mhd != NULL) {
+					cm->main_hall = temp;
+				} else {
+					mprintf(("Substitute main hall '%s' not found\n", temp));
+				}
+			}
+
 			// Goober5000 - new debriefing persona stuff!
 			cm->debrief_persona_index = 0;
 			if (optional_string("+Debriefing Persona Index:"))
@@ -563,9 +567,6 @@ int mission_campaign_load( char *filename, player *pl, int load_savefile )
 
 				} else {
 					if ( cm->formula == -1 ){
-						// close localization
-						lcl_ext_close();
-
 						Campaign_load_failure = CAMPAIGN_ERROR_SEXP_EXHAUSTED;
 						return CAMPAIGN_ERROR_SEXP_EXHAUSTED;
 					}
@@ -603,15 +604,13 @@ int mission_campaign_load( char *filename, player *pl, int load_savefile )
 
 				} else {
 					if ( cm->mission_loop_formula == -1 ){
-						// close localization
-						lcl_ext_close();
-
 						Campaign_load_failure = CAMPAIGN_ERROR_SEXP_EXHAUSTED;
 						return CAMPAIGN_ERROR_SEXP_EXHAUSTED;
 					}
 				}
 			}
 
+			cm->level = 0;
 			if (optional_string("+Level:")) {
 				stuff_int( &cm->level );
 				if ( cm->level == 0 )  // check if the top (root) of the whole tree
@@ -620,6 +619,7 @@ int mission_campaign_load( char *filename, player *pl, int load_savefile )
 			} else
 				Campaign.realign_required = 1;
 
+			cm->pos = 0;
 			if (optional_string("+Position:"))
 				stuff_int( &cm->pos );
 			else
@@ -629,24 +629,21 @@ int mission_campaign_load( char *filename, player *pl, int load_savefile )
 				cm->num_goals = -1;
 				cm->num_events = -1;
 				cm->num_variables = -1;
-				cm->notes = NULL;
 
 			} else {
 				cm->num_goals = 0;
 				cm->num_events = 0;
-				cm->num_variables = -1;
+				cm->num_variables = 0;
 			}
 
 			cm->goals = NULL;
 			cm->events = NULL;
 			cm->variables = NULL;
+
 			Campaign.num_missions++;
 		}
 	}
 
-	// close localization
-	lcl_ext_close();
-
 	// set up the other variables for the campaign stuff.  After initializing, we must try and load
 	// the campaign save file for this player.  Since all campaign loads go through this routine, I
 	// think this place should be the only necessary place to load the campaign save stuff.  The campaign
@@ -670,6 +667,12 @@ int mission_campaign_load( char *filename, player *pl, int load_savefile )
 				Campaign_load_failure = CAMPAIGN_ERROR_SAVEFILE;
 				return CAMPAIGN_ERROR_SAVEFILE;
 			} else {
+				// make sure we initialize red alert data for the new CSG
+				red_alert_clear();
+				// and reset stats when requested
+				if (reset_stats) {
+					pl->stats.init();
+				}
 				Pilot.save_savefile();
 			}
 		}
@@ -846,7 +849,7 @@ void mission_campaign_delete_all_savefiles( char *pilot_name )
 	int dir_type, num_files, i;
 	char file_spec[MAX_FILENAME_LEN + 2], *ext;
 	char filename[1024];
-	int (*filter_save)(char *filename);
+	int (*filter_save)(const char *filename);
 	SCP_vector<SCP_string> names;
 
 	ext = NOX(".csg");
@@ -899,7 +902,7 @@ int mission_campaign_next_mission()
 		return -2;
 
 	Campaign.current_mission = Campaign.next_mission;	
-	strncpy( Game_current_mission_filename, Campaign.missions[Campaign.current_mission].name, MAX_FILENAME_LEN );
+	strcpy_s( Game_current_mission_filename, Campaign.missions[Campaign.current_mission].name );
 
 	// check for end of loop.
 	if (Campaign.current_mission == Campaign.loop_reentry) {
@@ -924,16 +927,31 @@ int mission_campaign_previous_mission()
 		return 0;
 
 	Campaign.current_mission = Campaign.prev_mission;
+	Campaign.prev_mission = -1;
 	Campaign.next_mission = Campaign.current_mission;
 	Campaign.num_missions_completed--;
 	Campaign.missions[Campaign.next_mission].completed = 0;
 
+	if (Campaign.num_variables > 0) {
+		vm_free( Campaign.variables );
+	}
+
+	Campaign.num_variables = Campaign.redalert_num_variables;
+
+	// copy backed up variables over
+	if (Campaign.redalert_num_variables > 0) {
+		Assert( Campaign.redalert_variables );
+
+		Campaign.variables = (sexp_variable *) vm_malloc(Campaign.num_variables * sizeof(sexp_variable));
+		memcpy(Campaign.variables, Campaign.redalert_variables, Campaign.redalert_num_variables * sizeof(sexp_variable));
+	}
+
 	Pilot.save_savefile();
 
 	// reset the player stats to be the stats from this level
-	memcpy( &Player->stats, &Campaign.missions[Campaign.current_mission].stats, sizeof(Player->stats) );
+	Player->stats.assign( Campaign.missions[Campaign.current_mission].stats );
 
-	strncpy( Game_current_mission_filename, Campaign.missions[Campaign.current_mission].name, MAX_FILENAME_LEN );
+	strcpy_s( Game_current_mission_filename, Campaign.missions[Campaign.current_mission].name );
 	Num_granted_ships = 0;
 	Num_granted_weapons = 0;
 
@@ -979,10 +997,10 @@ void mission_campaign_eval_next_mission()
 /**
  * Store mission's goals and events in Campaign struct
  */
-void mission_campaign_store_goals_and_events_and_variables()
+void mission_campaign_store_goals_and_events()
 {
 	char *name;
-	int cur, i, j;
+	int cur, i;
 	cmission *mission;
 
 	cur = Campaign.current_mission;
@@ -1054,6 +1072,15 @@ void mission_campaign_store_goals_and_events_and_variables()
 		} else
 			Int3();
 	}
+}
+
+void mission_campaign_store_variables()
+{
+	int cur, i, j;
+	cmission *mission;
+
+	cur = Campaign.current_mission;
+	mission = &Campaign.missions[cur];
 
 	// Goober5000 - handle campaign-persistent variables -------------------------------------
 	if (mission->variables != NULL) {
@@ -1109,9 +1136,18 @@ void mission_campaign_store_goals_and_events_and_variables()
 		sexp_variable *n_variables = (sexp_variable *) vm_malloc(total_variables * sizeof(sexp_variable));
 		Assert( n_variables );
 
+		if (Campaign.redalert_num_variables > 0) {
+			vm_free( Campaign.redalert_variables );
+		}
+
+		Campaign.redalert_num_variables = Campaign.num_variables;
+
 		// copy existing variables over
 		if (Campaign.num_variables > 0) {
 			Assert( Campaign.variables );
+
+			Campaign.redalert_variables = (sexp_variable *) vm_malloc(Campaign.num_variables * sizeof(sexp_variable));
+			memcpy(Campaign.redalert_variables, Campaign.variables, Campaign.num_variables * sizeof(sexp_variable));
 			memcpy(n_variables, Campaign.variables, Campaign.num_variables * sizeof(sexp_variable));
 
 			variable_count = Campaign.num_variables;
@@ -1157,6 +1193,12 @@ void mission_campaign_store_goals_and_events_and_variables()
 	// --------------------------------------------------------------------------
 }
 
+void mission_campaign_store_goals_and_events_and_variables()
+{
+	mission_campaign_store_goals_and_events();
+	mission_campaign_store_variables();
+}
+
 /**
  * Called when the player's mission is over.  It updates the internal store of goals
  * and their status then saves the state of the campaign in the campaign file.  
@@ -1193,7 +1235,7 @@ void mission_campaign_mission_over(bool do_next_mission)
 	// update campaign.mission stats (used to allow backout inRedAlert)
 	// .. but we don't do this if we are inside of the prev/current loop hack
 	if ( Campaign.prev_mission != Campaign.current_mission ) {
-		memcpy( &mission->stats, &Player->stats, sizeof(Player->stats) );
+		mission->stats.assign( Player->stats );
 		if(!(Game_mode & GM_MULTIPLAYER)){
 			scoring_backout_accept( &mission->stats );
 		}
@@ -1321,7 +1363,8 @@ void mission_campaign_clear()
 		Campaign.missions[i].flags = 0;
 		Campaign.missions[i].main_hall = "";
 		Campaign.missions[i].debrief_persona_index = 0;
-		init_scoring_element(&Campaign.missions[i].stats);
+
+		Campaign.missions[i].stats.init();
 	}
 
 	memset(Campaign.name, 0, NAME_LENGTH);
@@ -1345,6 +1388,11 @@ void mission_campaign_clear()
 		vm_free(Campaign.variables);
 		Campaign.variables = NULL;
 	}
+	Campaign.redalert_num_variables = 0;
+	if (Campaign.redalert_variables != NULL) {
+		vm_free(Campaign.redalert_variables);
+		Campaign.redalert_variables = NULL;
+	}
 }
 
 /**
@@ -1398,13 +1446,9 @@ void read_mission_goal_list(int num)
 	int i, z, rval, event_count, count = 0;
 
 	filename = Campaign.missions[num].name;
-
-	// open localization
-	lcl_ext_open();	
 	
 	if ((rval = setjmp(parse_abort)) != 0) {
 		mprintf(("MISSIONCAMPAIGN: Unable to parse '%s'!  Error code = %i.\n", filename, rval));
-		lcl_ext_close();
 		return;
 	}
 
@@ -1503,9 +1547,6 @@ void read_mission_goal_list(int num)
 	}
 
 	// Goober5000 - variables do not need to be read here
-
-	// close localization
-	lcl_ext_close();
 }
 
 /**
@@ -1630,26 +1671,18 @@ void mission_campaign_save_persistent( int type, int sindex )
 		Int3();
 }
 
-bool campaign_is_ignored(char *filename, bool add_extension)
+bool campaign_is_ignored(const char *filename)
 {
-	bool current_campaign_ignored = false;
-	int i;
-	char campaign_name[NAME_LENGTH] = {""};
+	SCP_string filename_no_ext = filename;
+	drop_extension(filename_no_ext);
 
-	strcpy_s(campaign_name, filename);
-
-	if (add_extension) {
-		strcat_s(campaign_name, FS_CAMPAIGN_FILE_EXT);
-	}
-
-	for (i = 0; i < (int)Ignored_campaigns.size(); i++) {
-		if (!stricmp (campaign_name, Ignored_campaigns[i].c_str())) {
-			current_campaign_ignored = true;
-			break;
+	for (SCP_vector<SCP_string>::iterator ii = Ignored_campaigns.begin(); ii != Ignored_campaigns.end(); ++ii) {
+		if (ii->compare(filename_no_ext) == 0) {
+			return true;
 		}
 	}
 	
-	return current_campaign_ignored;
+	return false;
 }
 
 // returns 0: loaded, !0: error
@@ -1667,7 +1700,7 @@ int mission_load_up_campaign( player *pl )
 
 	// last used...
 	if ( strlen(pl->current_campaign) ) {
-		if (!campaign_is_ignored(pl->current_campaign, true)) {
+		if (!campaign_is_ignored(pl->current_campaign)) {
 			return mission_campaign_load(pl->current_campaign, pl);
 		}
 		else {
@@ -1791,8 +1824,8 @@ void mission_campaign_skip_to_next(int start_game)
 			// closes out mission stuff, sets up next one
 			mission_campaign_mission_over();
 
-			if ( Campaign.next_mission == -1 ) {
-				// go to main hall, tha campaign is over!
+			if ( Campaign.next_mission == -1 || (The_mission.flags & MISSION_FLAG_END_TO_MAINHALL) ) {
+				// go to main hall, either the campaign is over or the FREDer requested it.
 				gameseq_post_event(GS_EVENT_MAIN_MENU);
 			} else {
 				// go to next mission
