@@ -13,6 +13,7 @@
 #include "mission/missiongoals.h"
 #include "mission/missionlog.h"
 #include "missionui/missionscreencommon.h"
+#include "debugconsole/console.h"
 #include "freespace2/freespace.h"
 #include "gamesequence/gamesequence.h"
 #include "hud/hud.h"
@@ -161,10 +162,10 @@ struct goal_buttons {
 struct goal_text {
 	int m_num_lines;
 	int m_line_sizes[MAX_GOAL_LINES];
-	char *m_lines[MAX_GOAL_LINES];
+	const char *m_lines[MAX_GOAL_LINES];
 
 	void init();
-	int add(char *text = NULL);
+	int add(const char *text = NULL);
 	void display(int n, int y);
 };
 
@@ -172,6 +173,7 @@ int Num_mission_events;
 int Num_goals = 0;								// number of goals for this mission
 int Event_index = -1;  // used by sexp code to tell what event it came from
 bool Log_event = false;
+bool Snapshot_all_events = false;
 int Mission_goal_timestamp;
 
 mission_event Mission_events[MAX_MISSION_EVENTS];
@@ -184,7 +186,7 @@ static goal_text Goal_text;
 static int Mission_directive_sound_timestamp;	// timestamp to control when directive succcess sound gets played
 static int Mission_directive_special_timestamp;	// used to specially mark a directive as true even though it's not
 
-char *Goal_type_text(int n)
+const char *Goal_type_text(int n)
 {
 	switch (n) {
 		case 0:	
@@ -284,7 +286,7 @@ void goal_list::icons_display(int yoff)
 
 			if ((y >= Goal_screen_text_y - ys / 2) && (y + ys <= Goal_screen_text_y + Goal_screen_text_h + ys / 2)) {
 				gr_set_bitmap(bmp);
-				gr_bitmap(Goal_screen_icon_x, y);
+				gr_bitmap(Goal_screen_icon_x, y, GR_RESIZE_MENU);
 			}
 		}
 	}
@@ -303,7 +305,7 @@ void goal_text::init()
 // Adds lines of goal text.  If passed NULL (or nothing passed) a blank line is added.  If
 // the text is too long, it is automatically split into more than one line.
 // Returns the number of lines added.
-int goal_text::add(char *text)
+int goal_text::add(const char *text)
 {
 	int max, count;
 
@@ -346,8 +348,8 @@ void goal_text::display(int n, int y)
 		y1 = y + h / 2 - 1;
 
 		// custom_size me
-		gr_line(Goal_screen_icon_x, y1, Goal_screen_text_x - 2, y1);
-		gr_line(Goal_screen_text_x + w + 1, y1, Goal_screen_icon_x + Goal_screen_text_w, y1);
+		gr_line(Goal_screen_icon_x, y1, Goal_screen_text_x - 2, y1, GR_RESIZE_MENU);
+		gr_line(Goal_screen_text_x + w + 1, y1, Goal_screen_icon_x + Goal_screen_text_w, y1, GR_RESIZE_MENU);
 
 	} else {
 		gr_set_color_fast(&Color_text_normal);
@@ -355,7 +357,7 @@ void goal_text::display(int n, int y)
 		buf[m_line_sizes[n]] = 0;
 	}
 
-	gr_printf(Goal_screen_text_x, y, buf);
+	gr_printf_menu(Goal_screen_text_x, y, buf);
 }
 
 // mission_init_goals: initializes info for goals.  Called as part of mission initialization.
@@ -556,7 +558,7 @@ void mission_show_goals_do_frame(float frametime)
 	GR_MAYBE_CLEAR_RES(Goals_screen_bg_bitmap);
 	if (Goals_screen_bg_bitmap >= 0) {
 		gr_set_bitmap(Goals_screen_bg_bitmap);
-		gr_bitmap(0, 0);
+		gr_bitmap(0, 0, GR_RESIZE_MENU);
 	}
 	Goals_screen_ui_window.draw();
 
@@ -698,26 +700,26 @@ void ML_render_objectives_key()
 	// display icon key at the bottom
 	if (Lcl_gr) {
 		gr_set_bitmap(Goal_complete_bitmap);
-		gr_bitmap(Objective_key_icon_coords_gr[gr_screen.res][0][0], Objective_key_icon_coords_gr[gr_screen.res][0][1]);
+		gr_bitmap(Objective_key_icon_coords_gr[gr_screen.res][0][0], Objective_key_icon_coords_gr[gr_screen.res][0][1], GR_RESIZE_MENU);
 		gr_set_bitmap(Goal_incomplete_bitmap);
-		gr_bitmap(Objective_key_icon_coords_gr[gr_screen.res][1][0], Objective_key_icon_coords_gr[gr_screen.res][1][1]);
+		gr_bitmap(Objective_key_icon_coords_gr[gr_screen.res][1][0], Objective_key_icon_coords_gr[gr_screen.res][1][1], GR_RESIZE_MENU);
 		gr_set_bitmap(Goal_failed_bitmap);
-		gr_bitmap(Objective_key_icon_coords_gr[gr_screen.res][2][0], Objective_key_icon_coords_gr[gr_screen.res][2][1]);
+		gr_bitmap(Objective_key_icon_coords_gr[gr_screen.res][2][0], Objective_key_icon_coords_gr[gr_screen.res][2][1], GR_RESIZE_MENU);
 
-		gr_string(Objective_key_text_coords_gr[gr_screen.res][0][0], Objective_key_text_coords_gr[gr_screen.res][0][1] , XSTR("Complete",	1437));
-		gr_string(Objective_key_text_coords_gr[gr_screen.res][1][0], Objective_key_text_coords_gr[gr_screen.res][1][1] , XSTR("Incomplete", 1438));
-		gr_string(Objective_key_text_coords_gr[gr_screen.res][2][0], Objective_key_text_coords_gr[gr_screen.res][2][1] , XSTR("Failed",		1439));
+		gr_string(Objective_key_text_coords_gr[gr_screen.res][0][0], Objective_key_text_coords_gr[gr_screen.res][0][1] , XSTR("Complete",	1437), GR_RESIZE_MENU);
+		gr_string(Objective_key_text_coords_gr[gr_screen.res][1][0], Objective_key_text_coords_gr[gr_screen.res][1][1] , XSTR("Incomplete", 1438), GR_RESIZE_MENU);
+		gr_string(Objective_key_text_coords_gr[gr_screen.res][2][0], Objective_key_text_coords_gr[gr_screen.res][2][1] , XSTR("Failed",		1439), GR_RESIZE_MENU);
 	} else {
 		gr_set_bitmap(Goal_complete_bitmap);
-		gr_bitmap(Objective_key_icon_coords[gr_screen.res][0][0], Objective_key_icon_coords[gr_screen.res][0][1]);
+		gr_bitmap(Objective_key_icon_coords[gr_screen.res][0][0], Objective_key_icon_coords[gr_screen.res][0][1], GR_RESIZE_MENU);
 		gr_set_bitmap(Goal_incomplete_bitmap);
-		gr_bitmap(Objective_key_icon_coords[gr_screen.res][1][0], Objective_key_icon_coords[gr_screen.res][1][1]);
+		gr_bitmap(Objective_key_icon_coords[gr_screen.res][1][0], Objective_key_icon_coords[gr_screen.res][1][1], GR_RESIZE_MENU);
 		gr_set_bitmap(Goal_failed_bitmap);
-		gr_bitmap(Objective_key_icon_coords[gr_screen.res][2][0], Objective_key_icon_coords[gr_screen.res][2][1]);
+		gr_bitmap(Objective_key_icon_coords[gr_screen.res][2][0], Objective_key_icon_coords[gr_screen.res][2][1], GR_RESIZE_MENU);
 
-		gr_string(Objective_key_text_coords[gr_screen.res][0][0], Objective_key_text_coords[gr_screen.res][0][1] , XSTR("Complete",	1437));
-		gr_string(Objective_key_text_coords[gr_screen.res][1][0], Objective_key_text_coords[gr_screen.res][1][1] , XSTR("Incomplete", 1438));
-		gr_string(Objective_key_text_coords[gr_screen.res][2][0], Objective_key_text_coords[gr_screen.res][2][1] , XSTR("Failed",		1439));
+		gr_string(Objective_key_text_coords[gr_screen.res][0][0], Objective_key_text_coords[gr_screen.res][0][1] , XSTR("Complete",	1437), GR_RESIZE_MENU);
+		gr_string(Objective_key_text_coords[gr_screen.res][1][0], Objective_key_text_coords[gr_screen.res][1][1] , XSTR("Incomplete", 1438), GR_RESIZE_MENU);
+		gr_string(Objective_key_text_coords[gr_screen.res][2][0], Objective_key_text_coords[gr_screen.res][2][1] , XSTR("Failed",		1439), GR_RESIZE_MENU);
 	}
 }
 
@@ -919,8 +921,12 @@ void mission_process_event( int event )
 
 	if (sindex >= 0) {
 		Sexp_useful_number = 1;
-		if (Mission_events[event].mission_log_flags != 0) {
+		if (Snapshot_all_events || Mission_events[event].mission_log_flags != 0) {
 			Log_event = true;
+			
+			Current_event_log_buffer = &Mission_events[event].event_log_buffer;
+			Current_event_log_variable_buffer = &Mission_events[event].event_log_variable_buffer;
+			Current_event_log_argument_buffer = &Mission_events[event].event_log_argument_buffer;
 		}
 		result = eval_sexp(sindex);
 
@@ -943,11 +949,13 @@ void mission_process_event( int event )
 		if (Sexp_useful_number){
 			Mission_events[event].flags |= MEF_CURRENT;
 		}
+
+		if ((Mission_events[event].mission_log_flags != 0) || Snapshot_all_events){
+			maybe_write_to_event_log(result);
+		}
 	}
 
-	if (Mission_events[event].mission_log_flags != 0) {
-		maybe_write_to_event_log(result);
-	}
+	Log_event = false;
 
 	Event_index = -1;
 	Mission_events[event].result = result;
@@ -1094,7 +1102,7 @@ void mission_eval_goals()
 		std_multi_update_goals();
 	}
 	
-	Log_event = false;
+	Snapshot_all_events = false;
 }
 
 //	evaluate_primary_goals() will determine if the primary goals for a mission are complete
@@ -1257,77 +1265,101 @@ void mission_goal_mark_events_complete()
 }
 
 // some debug console functions to help list and change the status of mission goals
-DCF(show_mission_goals,"List and change the status of mission goals")
+DCF(show_mission_goals,"Lists the status of mission goals")
 {
 	int i, type;
 
-	if (Dc_command)
-		Dc_status = 1;
-
-	if (Dc_help) {
+	if (dc_optional_string_either("help", "--help")) {
 		dc_printf("Usage: show_mission_goals\n\nList all mission goals and their current status.\n");
-		Dc_status = 0;
+		return;
 	}
 
-	if (Dc_status) {
-		for (i=0; i<Num_goals; i++) {
-			type = Mission_goals[i].type & GOAL_TYPE_MASK;
-			dc_printf("%2d. %32s(%10s) -- ", i, Mission_goals[i].name, Goal_type_text(type));
-			if ( Mission_goals[i].satisfied == GOAL_COMPLETE )
-				dc_printf("satisfied.\n");
-			else if ( Mission_goals[i].satisfied == GOAL_INCOMPLETE )
-				dc_printf("not satisfied\n");
-			else if ( Mission_goals[i].satisfied == GOAL_FAILED )
-				dc_printf("failed\n");
-			else
-				dc_printf("\t[unknown goal status].\n");
-		}
+	if (dc_optional_string_either("status", "--status") || dc_optional_string_either("?", "--?")) {
+		// Don't do anything, but advance the parser past the flag
 	}
+
+	for (i=0; i<Num_goals; i++) {
+		type = Mission_goals[i].type & GOAL_TYPE_MASK;
+		dc_printf("%2d. %32s(%10s) -- ", i, Mission_goals[i].name, Goal_type_text(type));
+		if ( Mission_goals[i].satisfied == GOAL_COMPLETE )
+			dc_printf("satisfied.\n");
+		else if ( Mission_goals[i].satisfied == GOAL_INCOMPLETE )
+			dc_printf("unsatisfied\n");
+		else if ( Mission_goals[i].satisfied == GOAL_FAILED )
+			dc_printf("failed\n");
+		else
+			dc_printf("Warning! Mission goal %i is in an invalid state! (value: %i)", i, Mission_goals[i].satisfied);
+	}
+	
 }
 
 //XSTR:OFF
-DCF(change_mission_goal, "Change the mission goal")
+DCF(change_mission_goal, "Changes the mission goal status")
 {
 	int num;
+	bool val_b;
+	char *string;
 
-	if ( Dc_command ) {
-		dc_get_arg(ARG_INT);
-		if ( Dc_arg_int >= Num_goals ) {
-			dc_printf ("First parameter must be a valid goal number.\n");
-			return;
-		}
-
-		num = Dc_arg_int;
-		dc_get_arg(ARG_TRUE|ARG_FALSE|ARG_STRING);
-		if ( Dc_arg_type & ARG_TRUE )
-			Mission_goals[num].satisfied = GOAL_COMPLETE;
-		else if ( Dc_arg_type & ARG_FALSE )
-			Mission_goals[num].satisfied = GOAL_FAILED;
-		else if ( Dc_arg_type & ARG_NONE )
-			Mission_goals[num].satisfied = GOAL_INCOMPLETE;
-		else if ( Dc_arg_type & ARG_STRING) {
-			if ( !stricmp(Dc_arg, "satisfied") )
-				Mission_goals[num].satisfied = GOAL_COMPLETE;
-			else if ( !stricmp( Dc_arg, "failed") )
-				Mission_goals[num].satisfied = GOAL_FAILED;
-			else if ( !stricmp( Dc_arg, "unknown") )
-				Mission_goals[num].satisfied = GOAL_INCOMPLETE;
-			else
-				dc_printf("Unknown status %s.  Use 'satisfied', 'failed', or 'unknown'\n", Dc_arg);
-		}
-	}
-
-	if ( Dc_help ) {
-		dc_printf("Usage: change_mission_goal <goal_num> <status>\n");
+	if (dc_optional_string_either("help", "--help")) {
+		dc_printf("Usage: change_mission_goal <goal_num> [status]\n");
 		dc_printf("<goal_num> --  Integer number of goal to change.  See show_mission_goals\n");
-		dc_printf("<status>   --  [bool] where a true value makes the goal satisfied,\n");
-		dc_printf("               a false value makes the goal failed.\n");
-		dc_printf("The <status> field may also be one of 'satisfied', 'failed', or 'unknown'\n");
-		dc_printf("\nExamples:\n\n'change_mission_goal 1 true' makes goal 1 successful.\n");
-		dc_printf("'change_mission_goal 2' marks goal 2 not complete\n");
-		dc_printf("'change_mission_goal 0 satisfied' marks goal 0 as satisfied\n");
-		Dc_status = 0;
+		dc_printf("[status]   --  Goal status to change to.\n\n");
+
+		dc_printf("The optional [status] field may be either a bool type or a string.\n");
+		dc_printf("\ttrue  -- Goal status set to 'complete'\n");
+		dc_printf("\tfalse -- Goal status set to 'failed'\n\n");
+
+		dc_printf("A string value of 'satisfied', 'failed', or 'unknown' will set the goal status to the respective state.\n");
+		dc_printf("If [status] is not given, then the goal status will be set to 'unknown'");
+
+		dc_printf("Examples:\n");
+		dc_printf("\t'change_mission_goal 1 true'  makes goal 1 as successful.\n");
+		dc_printf("\t'change_mission_goal 2'       marks goal 2 as unknown/incomplete\n");
+		dc_printf("\t'change_mission_goal 0 satisfied'    marks goal 0 as satisfied\n");
+		return;
 	}
+
+	dc_stuff_int(&num);
+	if ( num >= Num_goals ) {
+		dc_printf (" Error: Invalid value for <goal_num>. Valid values: 0 - %i\n", Num_goals);
+		return;
+	}
+
+	if (dc_optional_string("satisfied")) {
+		Mission_goals[num].satisfied = GOAL_COMPLETE;
+
+	} else if (dc_optional_string("failed")) {
+		Mission_goals[num].satisfied = GOAL_FAILED;
+
+	} else if (dc_optional_string("unsatisfied")) {
+		Mission_goals[num].satisfied = GOAL_INCOMPLETE;
+
+	} else if (dc_maybe_stuff_boolean(&val_b)) {
+		val_b ? Mission_goals[num].satisfied = GOAL_COMPLETE : Mission_goals[num].satisfied = GOAL_FAILED;
+
+	} else {
+		// No argument given
+		Mission_goals[num].satisfied = GOAL_INCOMPLETE;
+	}
+
+	switch(Mission_goals[num].satisfied) {
+	case GOAL_COMPLETE:
+		string = "satisfied";
+		break;
+
+	case GOAL_FAILED:
+		string = "failed";
+		break;
+
+	case GOAL_INCOMPLETE:
+		string = "unsatisfied";
+		break;
+
+	default:
+		dc_printf("Warning! Mission goal %i is in an invalid state! (value: %i)", num, Mission_goals[num].satisfied);
+		return;
+	}
+	dc_printf("Mission goal %i set to '%s'\n", num, string);
 }
 //XSTR:ON
 
