@@ -260,7 +260,7 @@ static UI_XSTR Hotkey_text[GR_NUM_RESOLUTIONS][HOTKEY_NUM_TEXT] = {
 
 
 static struct {
-	char *label;
+	const char *label;
 	int type;
 	int index;
 	int y;  // Y coordinate of line
@@ -276,6 +276,8 @@ static UI_WINDOW Ui_window;
 static UI_BUTTON List_buttons[LIST_BUTTONS_MAX];  // buttons for each line of text in list
 //static UI_BUTTON List_region;
 
+int Hotkey_overlay_id;
+
 //////////////////////
 
 
@@ -509,7 +511,7 @@ int get_wing_hotkeys(int n)
 }
 
 // add a line of hotkey smuck to end of list
-int hotkey_line_add(char *text, int type, int index, int y)
+int hotkey_line_add(const char *text, int type, int index, int y)
 {
 	if (Num_lines >= MAX_LINES)
 		return 0;
@@ -564,7 +566,7 @@ int hotkey_line_add_sorted(char *text, int type, int index, int start)
 int hotkey_build_team_listing(int enemy_team_mask, int y, bool list_enemies)
 {
 	ship_obj *so;
-	char *str = NULL;
+	const char *str = NULL;
 	int i, j, s, z, start, team_mask;
 	int font_height = gr_get_font_height();
 
@@ -954,7 +956,8 @@ void mission_hotkey_init()
 	Buttons[gr_screen.res][SCROLL_DOWN_BUTTON].button.set_hotkey(KEY_PAGEDOWN);
 
 	// ensure help overlay is off
-	help_overlay_set_state(HOTKEY_OVERLAY,0);
+	Hotkey_overlay_id = help_overlay_get_index(HOTKEY_OVERLAY);
+	help_overlay_set_state(Hotkey_overlay_id,gr_screen.res,0);
 
 	// load in relevant bitmaps
 	Background_bitmap = bm_load(Hotkey_background_fname[gr_screen.res]);
@@ -986,9 +989,6 @@ void mission_hotkey_close()
 	if (Wing_bmp >= 0)
 		bm_release(Wing_bmp);
 
-	// unload the overlay bitmap
-//	help_overlay_unload(HOTKEY_OVERLAY);
-
 	// unpause all weapon sounds
 	weapon_unpause_sounds();
 
@@ -1013,7 +1013,7 @@ void mission_hotkey_do_frame(float frametime)
 	int select_tease_line = -1;  // line mouse is down on, but won't be selected until button released
 	color circle_color;
 
-	if ( help_overlay_active(HOTKEY_OVERLAY) ) {
+	if ( help_overlay_active(Hotkey_overlay_id) ) {
 		Buttons[gr_screen.res][HELP_BUTTON].button.reset_status();
 		Ui_window.set_ignore_gadgets(1);
 	}
@@ -1021,14 +1021,14 @@ void mission_hotkey_do_frame(float frametime)
 	k = Ui_window.process() & ~KEY_DEBUGGED;
 
 	if ( (k > 0) || B1_JUST_RELEASED ) {
-		if ( help_overlay_active(HOTKEY_OVERLAY) ) {
-			help_overlay_set_state(HOTKEY_OVERLAY, 0);
+		if ( help_overlay_active(Hotkey_overlay_id) ) {
+			help_overlay_set_state(Hotkey_overlay_id, gr_screen.res, 0);
 			Ui_window.set_ignore_gadgets(0);
 			k = 0;
 		}
 	}
 
-	if ( !help_overlay_active(HOTKEY_OVERLAY) ) {
+	if ( !help_overlay_active(Hotkey_overlay_id) ) {
 		Ui_window.set_ignore_gadgets(0);
 	}
 
@@ -1142,9 +1142,10 @@ void mission_hotkey_do_frame(float frametime)
 		}
 	}
 
+	GR_MAYBE_CLEAR_RES(Background_bitmap);
 	if (Background_bitmap >= 0) {
 		gr_set_bitmap(Background_bitmap);
-		gr_bitmap(0, 0);
+		gr_bitmap(0, 0, GR_RESIZE_MENU);
 
 	} else
 		gr_clear();
@@ -1157,7 +1158,7 @@ void mission_hotkey_do_frame(float frametime)
 	gr_set_color_fast(&Color_text_normal);
 	strcpy_s(buf, Scan_code_text[Key_sets[Cur_hotkey]]);
 	gr_get_string_size(&w, &h, buf);
-	gr_printf(Hotkey_function_name_coords[gr_screen.res][0] + (Hotkey_function_name_coords[gr_screen.res][2] - w) / 2, Hotkey_function_name_coords[gr_screen.res][1], buf);
+	gr_printf_menu(Hotkey_function_name_coords[gr_screen.res][0] + (Hotkey_function_name_coords[gr_screen.res][2] - w) / 2, Hotkey_function_name_coords[gr_screen.res][1], buf);
 
 	gr_set_font(FONT1);
 	line = Scroll_offset;
@@ -1171,25 +1172,25 @@ void mission_hotkey_do_frame(float frametime)
 
 				gr_get_string_size(&w, &h, Hotkey_lines[line].label);
 				i = y + h / 2 - 1;
-				gr_line(Hotkey_list_coords[gr_screen.res][0], i, Hotkey_ship_x[gr_screen.res] - 2, i);
-				gr_line(Hotkey_ship_x[gr_screen.res] + w + 1, i, Hotkey_list_coords[gr_screen.res][0] + Hotkey_list_coords[gr_screen.res][2], i);
+				gr_line(Hotkey_list_coords[gr_screen.res][0], i, Hotkey_ship_x[gr_screen.res] - 2, i, GR_RESIZE_MENU);
+				gr_line(Hotkey_ship_x[gr_screen.res] + w + 1, i, Hotkey_list_coords[gr_screen.res][0] + Hotkey_list_coords[gr_screen.res][2], i, GR_RESIZE_MENU);
 				break;
 
 			case HOTKEY_LINE_WING:
 				gr_set_bitmap(Wing_bmp);
 				bm_get_info(Wing_bmp, NULL, &h, NULL);
 				i = y + font_height / 2 - h / 2 - 1;
-				gr_bitmap(Hotkey_wing_icon_x[gr_screen.res], i);
+				gr_bitmap(Hotkey_wing_icon_x[gr_screen.res], i, GR_RESIZE_MENU);
 
 //				i = y + font_height / 2 - 1;
 //				gr_set_color_fast(&circle_color);
-//				gr_circle(ICON_LIST_X + 4, i, 5);
+//				gr_circle(ICON_LIST_X + 4, i, 5, GR_RESIZE_MENU);
 
 //				gr_set_color_fast(&Color_bright);
-//				gr_line(ICON_LIST_X, i, ICON_LIST_X + 2, i);
-//				gr_line(ICON_LIST_X + 4, i - 4, ICON_LIST_X + 4, i - 2);
-//				gr_line(ICON_LIST_X + 6, i, ICON_LIST_X + 8, i);
-//				gr_line(ICON_LIST_X + 4, i + 2, ICON_LIST_X + 4, i + 4);
+//				gr_line(ICON_LIST_X, i, ICON_LIST_X + 2, i, GR_RESIZE_MENU);
+//				gr_line(ICON_LIST_X + 4, i - 4, ICON_LIST_X + 4, i - 2, GR_RESIZE_MENU);
+//				gr_line(ICON_LIST_X + 6, i, ICON_LIST_X + 8, i, GR_RESIZE_MENU);
+//				gr_line(ICON_LIST_X + 4, i + 2, ICON_LIST_X + 4, i + 4, GR_RESIZE_MENU);
 
 				hotkeys = get_wing_hotkeys(Hotkey_lines[line].index);
 				break;
@@ -1228,7 +1229,7 @@ void mission_hotkey_do_frame(float frametime)
 		if (hotkeys) {
 			for (i=0; i<MAX_KEYED_TARGETS; i++) {
 				if (hotkeys & (1 << i)) {
-					gr_printf(Hotkey_list_coords[gr_screen.res][0] + Hotkey_function_field_width[gr_screen.res]*i, y, Scan_code_text[Key_sets[i]]);
+					gr_printf_menu(Hotkey_list_coords[gr_screen.res][0] + Hotkey_function_field_width[gr_screen.res]*i, y, Scan_code_text[Key_sets[i]]);
 				}
 			}
 /*
@@ -1244,7 +1245,7 @@ void mission_hotkey_do_frame(float frametime)
 			buf[strlen(buf) - 2] = 0;  // lose the ", " on the end
 
 			gr_force_fit_string(buf, 255, GROUP_LIST_W);
-			gr_printf(GROUP_LIST_X, y, buf);*/
+			gr_printf_menu(GROUP_LIST_X, y, buf);*/
 		}
 	
 		// draw ship/wing name
@@ -1253,10 +1254,10 @@ void mission_hotkey_do_frame(float frametime)
 		if (Hotkey_lines[line].type == HOTKEY_LINE_SUBSHIP) {
 			// indent
 			gr_force_fit_string(buf, 255, Hotkey_list_coords[gr_screen.res][0] + Hotkey_list_coords[gr_screen.res][2] - (Hotkey_ship_x[gr_screen.res]+20));
-			gr_printf(Hotkey_ship_x[gr_screen.res]+20, y, buf);
+			gr_printf_menu(Hotkey_ship_x[gr_screen.res]+20, y, buf);
 		} else {
 			gr_force_fit_string(buf, 255, Hotkey_list_coords[gr_screen.res][0] + Hotkey_list_coords[gr_screen.res][2] - Hotkey_ship_x[gr_screen.res]);
-			gr_printf(Hotkey_ship_x[gr_screen.res], y, buf);
+			gr_printf_menu(Hotkey_ship_x[gr_screen.res], y, buf);
 		}
 
 		line++;
@@ -1267,7 +1268,7 @@ void mission_hotkey_do_frame(float frametime)
 		List_buttons[i++].disable();
 
 	// blit help overlay if active
-	help_overlay_maybe_blit(HOTKEY_OVERLAY);
+	help_overlay_maybe_blit(Hotkey_overlay_id, gr_screen.res);
 
 	gr_flip();
 }
