@@ -87,6 +87,8 @@ struct p_dock_instance;
 #define MISSION_FLAG_ALL_ATTACK					(1<<20)	// all teams at war - Goober5000
 #define MISSION_FLAG_USE_AP_CINEMATICS			(1<<21) // Kazan - use autopilot cinematics
 #define MISSION_FLAG_DEACTIVATE_AP         	    (1<<22) // KeldorKatarn - deactivate autopilot (patch approved by Kazan)
+#define MISSION_FLAG_ALWAYS_SHOW_GOALS     	    (1<<23) // Karajorma - Show the mission goals, even for training missions
+#define MISSION_FLAG_END_TO_MAINHALL			(1<<24) // niffiwan - Return to the mainhall after debrief
 
 // some mice macros for mission type
 #define IS_MISSION_MULTI_COOP			(The_mission.game_type & MISSION_TYPE_MULTI_COOP)
@@ -120,14 +122,8 @@ typedef struct support_ship_info {
 // defines a mission cutscene.
 typedef struct mission_cutscene {
 	int type; 
-	char cutscene_name[NAME_LENGTH];	
-	int formula; 
-
-	mission_cutscene( ) 
-		: type( 0 ), formula( -1 )
-	{ 
-		cutscene_name[ 0 ] = 0;
-	}
+	char filename[MAX_FILENAME_LEN];
+	int formula;
 } mission_cutscene;
 
 typedef struct mission {
@@ -301,7 +297,6 @@ extern int	Num_goal_type_names;
 extern int	Num_reinforcement_type_names;
 extern int	Player_starts;
 extern fix	Entry_delay_time;
-extern int	Fred_num_texture_replacements;	// Goober5000
 extern int	Loading_screen_bm_index;
 
 extern int Num_unknown_ship_classes;
@@ -329,22 +324,9 @@ typedef struct texture_replace {
 	char old_texture[MAX_FILENAME_LEN];
 	char new_texture[MAX_FILENAME_LEN];
 	int new_texture_id;
-
-	texture_replace( )
-	{
-		Reset( );
-	}
-
-	void Reset( )
-	{
-		new_texture_id = -1;
-		ship_name[ 0 ] = 0;
-		old_texture[ 0 ] = 0;
-		new_texture[ 0 ] = 0;
-	}
 } texture_replace;
 
-extern texture_replace *Fred_texture_replacements;
+extern SCP_vector<texture_replace> Fred_texture_replacements;
 
 typedef struct alt_class {
 	int ship_class;				
@@ -357,7 +339,9 @@ typedef struct alt_class {
 //	a parse object
 //	information from a $OBJECT: definition is read into this struct to
 // be copied into the real object, ship, etc. structs
-typedef struct p_object {
+class p_object
+{
+public:
 	char	name[NAME_LENGTH];
 	p_object *next, *prev;
 
@@ -441,110 +425,15 @@ typedef struct p_object {
 	float ship_max_shield_strength;
 
 	// Goober5000
-	int num_texture_replacements;
-	texture_replace replacement_textures[MAX_REPLACEMENT_TEXTURES];	// replacement textures - Goober5000
+	SCP_vector<texture_replace> replacement_textures;
 
 	SCP_vector<alt_class> alt_classes;	
 
 	int alt_iff_color[MAX_IFFS][MAX_IFFS];
 
-	void Reset( )
-	{
-		int i = 0;
-
-		name[ 0 ] = '\0';
-
-		next = NULL;
-		prev = NULL;
-
-		memset( &pos, 0, sizeof( pos ) );
-		memset( &orient, 0, sizeof( orient ) );
-		ship_class = 0;
-		team = 0;
-		behavior = 0;
-		ai_goals = 0;
-		cargo1 = 0;
-
-		status_count = 0;
-		for ( i = 0; i < MAX_OBJECT_STATUS; i++ )
-		{
-			status_type[ i ] = 0;
-			status[ i ] = 0;
-			target[ i ] = 0;
-		}
-
-		subsys_index = 0;
-		subsys_count = 0;
-		initial_velocity = 0;
-		initial_hull = 0;
-		initial_shields = 0;
-		arrival_location = 0;
-		arrival_distance = 0;
-		arrival_anchor = 0;
-		arrival_path_mask = 0;
-		arrival_cue = 0;
-		arrival_delay = 0;
-		departure_location = 0;
-		departure_anchor = 0;
-		departure_path_mask = 0;
-		departure_cue = 0;
-		departure_delay = 0;
-
-		misc[ 0 ] = '\0';
-
-		wingnum = 0;
-		pos_in_wing = 0;
-
-		flags = 0;
-		flags2 = 0;
-		escort_priority = 0;
-		ai_class = 0;
-		hotkey = 0;
-		score = 0;
-		assist_score_pct = 0.;
-		orders_accepted = 0;
-		dock_list = NULL;
-		created_object = NULL;
-		group = 0;
-		persona_index = 0;
-		kamikaze_damage = 0;
-		use_special_explosion = false;
-		special_exp_damage = -1;
-		special_exp_blast = -1;
-		special_exp_inner = -1;
-		special_exp_outer = -1;
-		use_shockwave = false;
-		special_exp_shockwave_speed = -1;
-		special_exp_deathroll_time = 0;
-
-		special_hitpoints = 0;
-		special_shield = -1;
-
-		net_signature = 0;
-		destroy_before_mission_time = 0;
-
-		wing_status_wing_index = 0;
-		wing_status_wing_pos = 0;
-
-		respawn_count = 0;
-		respawn_priority = 0;
-
-		alt_type_index = 0;
-		callsign_index = 0;
-
-		ship_max_hull_strength = 0.0f;
-		ship_max_shield_strength = 0.0f;
-
-		num_texture_replacements = 0;
-		
-		for ( i = 0; i < MAX_REPLACEMENT_TEXTURES; i++ )
-			replacement_textures[ i ].Reset( );
-
-		alt_classes.clear( );	
-
-		memset( alt_iff_color, 0, sizeof( alt_iff_color ) );
-	}
-} p_object;
+	p_object();
+	~p_object();
+};
 
 // defines for flags used for p_objects when they are created.  Used to help create special
 // circumstances for those ships.  This list of bitfield indicators MUST correspond EXACTLY
@@ -591,7 +480,7 @@ typedef struct p_object {
 // same caveat: This list of bitfield indicators MUST correspond EXACTLY
 // (i.e., order and position must be the same) to its counterpart in MissionParse.cpp!!!!
 
-#define MAX_PARSE_OBJECT_FLAGS_2	22
+#define MAX_PARSE_OBJECT_FLAGS_2	24
 
 #define P2_SF2_PRIMITIVE_SENSORS			(1<<0)
 #define P2_SF2_NO_SUBSPACE_DRIVE			(1<<1)
@@ -615,10 +504,13 @@ typedef struct p_object {
 #define P2_SF2_CLOAKED						(1<<19)
 #define P2_SF2_SHIP_LOCKED					(1<<20)
 #define P2_SF2_WEAPONS_LOCKED				(1<<21)
+#define P2_SF2_SCRAMBLE_MESSAGES			(1<<22)
+#define P2_OF_NO_COLLIDE					(1<<23) // This actually changes the OF_COLLIDES object flag
 
 // and again: these flags do not appear in the array
+//#define blah							(1<<28)
 //#define blah							(1<<29)
-//#define blah							(1<<30)
+#define P2_RED_ALERT_DELETED			(1<<30)	// Goober5000 - used analogously to SEF_PLAYER_DELETED
 #define P2_ALREADY_HANDLED				(1<<31)	// Goober5000 - used for docking currently, but could be used generically
 
 
@@ -668,7 +560,7 @@ extern int Num_teams;
 
 extern char			Player_start_shipname[NAME_LENGTH];
 extern int			Player_start_shipnum;
-extern p_object	Player_start_pobject;
+extern p_object	*Player_start_pobject;
 
 extern int Mission_palette;  // index of palette file to use for mission
 extern int Nebula_index;  // index into Nebula_filenames[] of nebula to use in mission.
@@ -679,11 +571,11 @@ extern p_object *Arriving_support_ship;
 extern char Neb2_texture_name[MAX_FILENAME_LEN];
 
 
-int parse_main(char *mission_name, int flags = 0);
+int parse_main(const char *mission_name, int flags = 0);
 p_object *mission_parse_get_arrival_ship(ushort net_signature);
-p_object *mission_parse_get_arrival_ship(char *name);
+p_object *mission_parse_get_arrival_ship(const char *name);
 p_object *mission_parse_get_parse_object(ushort net_signature);
-p_object *mission_parse_get_parse_object(char *name);
+p_object *mission_parse_get_parse_object(const char *name);
 int parse_create_object(p_object *objp);
 void resolve_parse_flags(object *objp, int parse_flags, int parse_flags2);
 
@@ -693,8 +585,8 @@ void mission_parse_close();
 int parse_wing_create_ships(wing *wingp, int num_to_create, int force = 0, int specific_instance = -1 );
 
 // function for getting basic mission data without loading whole mission
-int mission_parse_is_multi(char *filename, char *mission_name );
-int mission_parse_get_multi_mission_info(char *filename);
+int mission_parse_is_multi(const char *filename, char *mission_name );
+int mission_parse_get_multi_mission_info(const char *filename);
 
 // called externally from multiplayer code
 int mission_do_departure(object *objp, bool goal_is_to_warp = false);
@@ -703,7 +595,7 @@ int mission_do_departure(object *objp, bool goal_is_to_warp = false);
 void mission_parse_fixup_players(void);
 
 // get a index to a perminently kept around name of a ship or wing
-int get_parse_name_index(char *name);
+int get_parse_name_index(const char *name);
 
 // called from freespace game level loop
 void mission_parse_eval_stuff();
@@ -737,7 +629,7 @@ void mission_parse_reset_callsign();
 int is_training_mission();
 
 // code to save/restore mission parse stuff
-int get_mission_info(char *filename, mission *missionp = NULL, bool basic = true);
+int get_mission_info(const char *filename, mission *missionp = NULL, bool basic = true);
 
 // Goober5000
 void parse_dock_one_docked_object(p_object *pobjp, p_object *parent_pobjp);
