@@ -687,14 +687,14 @@ void chatbox_render()
 	}
 
 	// clear the multiplayer chat window
-	// gr_set_clip(Chatbox_x1, Chatbox_y1, Chatbox_w, Chatbox_h);
+	// gr_set_clip(Chatbox_x1, Chatbox_y1, Chatbox_w, Chatbox_h, GR_RESIZE_MENU);
 	// gr_clear();
 	// gr_reset_clip();
 
 	// draw the background bitmap if we're supposed to
 	if ( (Chatbox_bitmap != -1) && (Chatbox_mode_flags & CHATBOX_FLAG_DRAW_BOX)) {
 		gr_set_bitmap( Chatbox_bitmap );
-		gr_bitmap(Chatbox_x1, Chatbox_y1);		
+		gr_bitmap(Chatbox_x1, Chatbox_y1, GR_RESIZE_MENU);		
 	}
 
 	// render the chat lines
@@ -776,12 +776,12 @@ void chatbox_chat_init()
 }
 
 // int Test_color = 0;
-void chatbox_add_line(char *msg, int pid, int add_id)
+void chatbox_add_line(const char *msg, int pid, int add_id)
 {
 	int backup;
 	int	n_lines,idx;
 	int	n_chars[3];		
-	char	*p_str[3];			// for the initial line (unindented)
+	const char	*p_str[3];			// for the initial line (unindented)
 	char msg_extra[CHATBOX_STRING_LEN];
 
 	if(!Chatbox_created){
@@ -887,11 +887,12 @@ void chatbox_add_line(char *msg, int pid, int add_id)
 
 void chatbox_render_chat_lines()
 {
-   int started_at,player_num,count,ly;	
+   int started_at,player_num,count,ly,line_height;	
 	
 	started_at = Brief_start_display_index;
 	count = 0;	
 	ly = Chatbox_begin_y;
+	line_height = gr_get_font_height() + 1;
 	while((count < Chatbox_max_lines) && (count < Num_brief_chat_lines)){	
 		// determine what player this chat line came from, and set the appropriate text color
 		player_num = Brief_chat_lines[started_at][0];	   		
@@ -900,7 +901,7 @@ void chatbox_render_chat_lines()
 		gr_set_color_fast(Color_netplayer[player_num]);
 
 		// draw the first line (no indent)						
-		gr_string(Chatbox_begin_x + Brief_chat_indents[started_at],ly,&Brief_chat_lines[started_at][1]);		
+		gr_string(Chatbox_begin_x + Brief_chat_indents[started_at],ly,&Brief_chat_lines[started_at][1],GR_RESIZE_MENU);		
 
 		// if we're in a team vs. team game, blit the player color icon
 		if(Netgame.type_flags & NG_TYPE_TEAM){
@@ -910,14 +911,14 @@ void chatbox_render_chat_lines()
 				if(Net_players[player_num].flags & NETINFO_FLAG_TEAM_CAPTAIN){
 					if(Multi_common_icons[MICON_TEAM0_SELECT] != -1){
 						gr_set_bitmap(Multi_common_icons[MICON_TEAM0_SELECT]);
-						gr_bitmap(Chatbox_icon_x,ly-2);
+						gr_bitmap(Chatbox_icon_x,ly-2,GR_RESIZE_MENU);
 					} 
 				}
 				// just you're average peon
 				else {
 					if(Multi_common_icons[MICON_TEAM0] != -1){
 						gr_set_bitmap(Multi_common_icons[MICON_TEAM0]);
-						gr_bitmap(Chatbox_icon_x,ly-2);
+						gr_bitmap(Chatbox_icon_x,ly-2,GR_RESIZE_MENU);
 					}
 				}
 				break;
@@ -926,14 +927,14 @@ void chatbox_render_chat_lines()
 				if(Net_players[player_num].flags & NETINFO_FLAG_TEAM_CAPTAIN){
 					if(Multi_common_icons[MICON_TEAM1_SELECT] != -1){
 						gr_set_bitmap(Multi_common_icons[MICON_TEAM1_SELECT]);
-						gr_bitmap(Chatbox_icon_x,ly-2);
+						gr_bitmap(Chatbox_icon_x,ly-2,GR_RESIZE_MENU);
 					}
 				}
 				// just your average peon
 				else {
 					if(Multi_common_icons[MICON_TEAM1] != -1){
 						gr_set_bitmap(Multi_common_icons[MICON_TEAM1]);
-						gr_bitmap(Chatbox_icon_x,ly-2);
+						gr_bitmap(Chatbox_icon_x,ly-2,GR_RESIZE_MENU);
 					}
 				}
 				break;
@@ -942,7 +943,7 @@ void chatbox_render_chat_lines()
 
 		// increment the count and line position
 		count++;		
-		ly += 10;
+		ly += line_height;
 		
 		// increment the started at index
 		started_at = Brief_chat_next_index[started_at];
