@@ -16,9 +16,13 @@ void fiction_viewer_do_frame(float frametime);
 
 // fiction stuff
 int mission_has_fiction();
-char *fiction_file();
-char *fiction_font();
+const char *fiction_file();
+const char *fiction_font();
+const char *fiction_voice();
 void fiction_viewer_reset();
-void fiction_viewer_load(char *filename, char *font_filename);
+void fiction_viewer_load(const char *filename, const char *font_filename, const char* voice_filename);
+
+void fiction_viewer_pause();
+void fiction_viewer_unpause();
 
 #endif
