@@ -47,6 +47,7 @@
 #include "network/multi_campaign.h"
 #include "network/multi_endgame.h"
 #include "missionui/chatbox.h"
+#include "pilotfile/pilotfile.h"
 
 
 #define MAX_TOTAL_DEBRIEF_LINES	200
@@ -60,6 +61,10 @@
 #define DEBRIEF_ALLTIME_STATS		2
 #define DEBRIEF_ALLTIME_KILLS		3
 
+#define DEBRIEFING_FONT	FONT1
+
+extern float Brief_text_wipe_time_elapsed;
+
 // 3rd coord is max width in pixels
 int Debrief_title_coords[GR_NUM_RESOLUTIONS][3] = {
 	{ // GR_640
@@ -313,11 +318,7 @@ static int Debrief_multi_loading_bitmap;
 static int Rank_bitmap;
 static int Medal_bitmap;
 static int Badge_bitmap;
-static int Wings_bitmap;
-static int Crest_bitmap;
-//static int Rank_text_bitmap;
-//static int Medal_text_bitmap;
-//static int Badge_text_bitmap;
+
 static int Promoted;
 static int Debrief_accepted;
 static int Turned_traitor;
@@ -329,10 +330,6 @@ static int Recommend_active;
 static int Award_active;
 static int Text_offset;
 static int Num_text_lines = 0;
-static int Num_debrief_lines = 0;
-//static int Num_normal_debrief_lines = 0;
-static int Text_type[MAX_TOTAL_DEBRIEF_LINES];
-static char *Text[MAX_TOTAL_DEBRIEF_LINES];
 
 static int Debrief_inited = 0;
 static int New_stage;
@@ -345,6 +342,8 @@ static int Debrief_music_timeout = 0;
 static int Multi_list_size;
 static int Multi_list_offset;
 
+int Debrief_overlay_id;
+
 int Debrief_multi_stages_loaded = 0;
 int Debrief_multi_voice_loaded = 0;
 
@@ -446,29 +445,9 @@ voice_map Debrief_promotion_voice_mapping[NUM_VOLITION_CAMPAIGNS][MAX_CAMPAIGN_M
 	}
 };
 
-#define DB_AWARD_WINGS		0
-#define DB_AWARD_MEDAL		1
-#define DB_AWARD_SOC			2
-#define DB_AWARD_RANK		3
-#define DB_AWARD_BADGE		4
-#define DB_AWARD_BG			5
-static char* Debrief_award_filename[GR_NUM_RESOLUTIONS][6] = {
-	{
-		"DebriefWings",
-		"DebriefMedal",
-		"DebriefCrest",
-		"DebriefRank",
-		"DebriefBadge",
-		"DebriefAward"
-	},
-	{
-		"2_DebriefWings",
-		"2_DebriefMedal",
-		"2_DebriefCrest",
-		"2_DebriefRank",
-		"2_DebriefBadge",
-		"2_DebriefAward"
-	}
+static char* Debrief_award_background[GR_NUM_RESOLUTIONS] = {
+	"DebriefAward",
+	"2_DebriefAward"
 };
 
 #define AWARD_TEXT_MAX_LINES				5
@@ -485,7 +464,7 @@ void debrief_award_text_clear();
 
 
 // functions
-char *debrief_tooltip_handler(char *str)
+const char *debrief_tooltip_handler(const char *str)
 {
 	if (!stricmp(str, NOX("@.Medal"))) {
 		if (Award_active){
@@ -514,7 +493,7 @@ char *debrief_tooltip_handler(char *str)
 
 	} else if (!stricmp(str, NOX("@Badge"))) {
 		if (Badge_bitmap >= 0){
-			return Medals[Player->stats.m_badge_earned].name;
+			return Medals[Player->stats.m_badge_earned.back()].name;
 		}
 	}
 
@@ -878,8 +857,8 @@ void debrief_ui_init()
 	debrief_buttons_init();
 
 	// load in help overlay bitmap	
-	help_overlay_load(DEBRIEFING_OVERLAY);
-	help_overlay_set_state(DEBRIEFING_OVERLAY,0);
+	Debrief_overlay_id = help_overlay_get_index(DEBRIEFING_OVERLAY);
+	help_overlay_set_state(Debrief_overlay_id,gr_screen.res,0);
 
 	if ( Game_mode & GM_MULTIPLAYER ) {
 		// close down any old instances of the chatbox
@@ -899,7 +878,7 @@ void debrief_ui_init()
 		Warning(LOCATION, "Could not load the background bitmap for debrief screen");
 	}
 
-	Award_bg_bitmap = bm_load(Debrief_award_filename[gr_screen.res][DB_AWARD_BG]);
+	Award_bg_bitmap = bm_load(Debrief_award_background[gr_screen.res]);
 	Debrief_multi_loading_bitmap = bm_load(Debrief_loading_bitmap_fname[gr_screen.res]);
 }
 
@@ -940,10 +919,8 @@ int debrief_find_persona_index()
 // Goober5000
 // V sez: "defaults to number 9 (Petrarch) for non-volition missions
 // this is an ugly, nasty, hateful way of doing this, but it saves us changing the missions at this point"
-void debrief_choose_voice(char *voice_dest, char *voice_base, int default_to_base = 0)
+void debrief_choose_voice(char *voice_dest, char *voice_base, int persona_index, int default_to_base = 0)
 {
-	// see if we have a persona
-	int persona_index = debrief_find_persona_index();
 	if (persona_index >= 0)
 	{
 		// get voice file
@@ -969,39 +946,52 @@ void debrief_choose_voice(char *voice_dest, char *voice_base, int default_to_bas
 	}
 }
 
+void debrief_choose_medal_variant(char *buf, int medal_earned, int zero_based_version_index)
+{
+	Assert(buf != NULL && medal_earned >= 0 && zero_based_version_index >= 0);
+
+	// start with the regular file name, adapted for resolution
+	sprintf(buf, NOX("%s%s"), Resolution_prefixes[gr_screen.res], Medals[medal_earned].debrief_bitmap);
+
+	// if the medal has multiple versions, we may want to choose a specific bitmap
+	char *p = strstr(buf, "##");
+	if (p != NULL)
+	{
+		int version;
+		char number[8];
+
+		// possible to earn more than we have variants for
+		if (zero_based_version_index >= Medals[medal_earned].num_versions)
+			version = Medals[medal_earned].num_versions - 1;
+		else
+			version = zero_based_version_index;
+
+		// also, this is dumb
+		if (Medals[medal_earned].version_starts_at_1)
+			version++;
+
+		sprintf(number, NOX("%.2d"), version);
+		Assert(strlen(number) == 2);
+		strncpy(p, number, 2);
+	}
+}
+
 void debrief_award_init()
 {
 	char buf[80];
-	int i;
 
 	Rank_bitmap = -1; 
 	Medal_bitmap = -1;
 	Badge_bitmap = -1;
-	Wings_bitmap = -1;
-	Crest_bitmap = -1;
 	Promoted = -1;
 
 	// be sure there are no old award texts floating around
 	debrief_award_text_clear();
 
 	// handle medal earned
-	if (Player->stats.m_medal_earned != -1) {
-		if (Player->stats.m_medal_earned == 13) {  // special hack for the wings..
-			int ver;
-			if ( Player->stats.medals[13] > 1 ) {
-				ver = 1;
-			} else {
-				ver = 0;
-			}
-			sprintf(buf, NOX("%s%.2d"), Debrief_award_filename[gr_screen.res][DB_AWARD_WINGS], ver);	
-			Wings_bitmap = bm_load(buf);
-
-		} else if (Player->stats.m_medal_earned == 17) {  // special hack for the soc crest
-			Crest_bitmap = bm_load(Debrief_award_filename[gr_screen.res][DB_AWARD_SOC]);
-		} else {
-			sprintf(buf, NOX("%s%.2d"), Debrief_award_filename[gr_screen.res][DB_AWARD_MEDAL], Player->stats.m_medal_earned);
-			Medal_bitmap = bm_load(buf);
-		}
+	if ( Player->stats.m_medal_earned != -1 ) {
+		debrief_choose_medal_variant(buf, Player->stats.m_medal_earned, Player->stats.medal_counts[Player->stats.m_medal_earned] - 1);
+		Medal_bitmap = bm_load(buf);
 
 		debrief_add_award_text(Medals[Player->stats.m_medal_earned].name);
 	}
@@ -1009,35 +999,50 @@ void debrief_award_init()
 	// handle promotions
 	if ( Player->stats.m_promotion_earned != -1 ) {
 		Promoted = Player->stats.m_promotion_earned;
-		sprintf(buf, NOX("%s%.2d"), Debrief_award_filename[gr_screen.res][DB_AWARD_RANK], Promoted + 1);
+		debrief_choose_medal_variant(buf, Rank_medal_index, Promoted);
 		Rank_bitmap = bm_load(buf);
 
-		Promotion_stage.text = Ranks[Promoted].promotion_text;
+		// see if we have a persona
+		int persona_index = debrief_find_persona_index();
+
+		// use persona-specific promotion text if it exists; otherwise, use default
+		if (Ranks[Promoted].promotion_text.find(persona_index) != Ranks[Promoted].promotion_text.end()) {
+			Promotion_stage.text = Ranks[Promoted].promotion_text[persona_index];
+		} else {
+			Promotion_stage.text = Ranks[Promoted].promotion_text[-1];
+		}
 		Promotion_stage.recommendation_text = "";
 
 		// choose appropriate promotion voice for this mission
-		debrief_choose_voice(Promotion_stage.voice, Ranks[Promoted].promotion_voice_base);
+		debrief_choose_voice(Promotion_stage.voice, Ranks[Promoted].promotion_voice_base, persona_index);
 
 		debrief_add_award_text(Ranks[Promoted].name);
 	}
 
 	// handle badge earned
 	// only grant badge if earned and allowed.  (no_promotion really means no promotion and no badges)
-	if ( Player->stats.m_badge_earned != -1 ) {
-		i = Player->stats.m_badge_earned;
-		sprintf(buf, NOX("%s%.2d"), Debrief_award_filename[gr_screen.res][DB_AWARD_BADGE], Medals[i].badge_num + 1);
+	if ( Player->stats.m_badge_earned.size() ) {
+		debrief_choose_medal_variant(buf, Player->stats.m_badge_earned.back(), Player->stats.medal_counts[Player->stats.m_badge_earned.back()] - 1);
 		Badge_bitmap = bm_load(buf);
 
-		Badge_stage.text = Medals[i].promotion_text;
+		// see if we have a persona
+		int persona_index = debrief_find_persona_index();
+
+		// use persona-specific badge text if it exists; otherwise, use default
+		if (Medals[Player->stats.m_badge_earned.back()].promotion_text.find(persona_index) != Medals[Player->stats.m_badge_earned.back()].promotion_text.end()) {
+			Badge_stage.text = Medals[Player->stats.m_badge_earned.back()].promotion_text[persona_index];
+		} else {
+			Badge_stage.text = Medals[Player->stats.m_badge_earned.back()].promotion_text[-1];
+		}
 		Badge_stage.recommendation_text = "";
 
-		// choose appropriate voice
-		debrief_choose_voice(Badge_stage.voice, Medals[Player->stats.m_badge_earned].voice_base);
+		// choose appropriate badge voice for this mission
+		debrief_choose_voice(Badge_stage.voice, Medals[Player->stats.m_badge_earned.back()].voice_base, persona_index);
 
-		debrief_add_award_text(Medals[i].name);
+		debrief_add_award_text(Medals[Player->stats.m_badge_earned.back()].name);
 	}
 
-	if ((Rank_bitmap >= 0) || (Medal_bitmap >= 0) || (Badge_bitmap >= 0) || (Wings_bitmap >= 0) || (Crest_bitmap >= 0)) {
+	if ((Rank_bitmap >= 0) || (Medal_bitmap >= 0) || (Badge_bitmap >= 0)) {
 		Award_active = 1;
 	} else {
 		Award_active = 0;
@@ -1056,12 +1061,8 @@ void debrief_traitor_init()
 		int rval;
 		int stage_num;
 
-		// open localization
-		lcl_ext_open();
-
 		if ((rval = setjmp(parse_abort)) != 0) {
 			mprintf(("TABLES: Unable to parse '%s'!  Error code = %i.\n", "traitor.tbl", rval));
-			lcl_ext_close();
 			return;
 		}
 
@@ -1094,15 +1095,12 @@ void debrief_traitor_init()
 //		}
 
 		// Goober5000
-		debrief_choose_voice(stagep->voice, traitor_voice_file, 1);
+		debrief_choose_voice(stagep->voice, traitor_voice_file, debrief_find_persona_index(), 1);
 
 		required_string("$Recommendation text:");
 		stuff_string( stagep->recommendation_text, F_MULTITEXT, NULL);
 
 		inited = 1;
-
-		// close localization
-		lcl_ext_close();
 	}
 
 	// disable the accept button if in single player and I am a traitor
@@ -1234,30 +1232,30 @@ void debrief_multi_list_draw()
 					// draw his "selected" icon
 					if(((np->state == NETPLAYER_STATE_DEBRIEF_ACCEPT) || (np->state == NETPLAYER_STATE_DEBRIEF_REPLAY)) && (Multi_common_icons[MICON_TEAM0_SELECT] != -1)){
 						gr_set_bitmap(Multi_common_icons[MICON_TEAM0_SELECT]);
-						gr_bitmap(Debrief_list_coords[gr_screen.res][0], Debrief_list_coords[gr_screen.res][1] + y - 2);
+						gr_bitmap(Debrief_list_coords[gr_screen.res][0], Debrief_list_coords[gr_screen.res][1] + y - 2, GR_RESIZE_MENU);
 					} 
 					// draw his "normal" icon
 					else if(Multi_common_icons[MICON_TEAM0] != -1){
 						gr_set_bitmap(Multi_common_icons[MICON_TEAM0]);
-						gr_bitmap(Debrief_list_coords[gr_screen.res][0], Debrief_list_coords[gr_screen.res][1] + y - 2);
+						gr_bitmap(Debrief_list_coords[gr_screen.res][0], Debrief_list_coords[gr_screen.res][1] + y - 2, GR_RESIZE_MENU);
 					}					
 				} else if(np->p_info.team == 1){
 					// draw his "selected" icon
 					if(((np->state == NETPLAYER_STATE_DEBRIEF_ACCEPT) || (np->state == NETPLAYER_STATE_DEBRIEF_REPLAY)) && (Multi_common_icons[MICON_TEAM1_SELECT] != -1)){						
 						gr_set_bitmap(Multi_common_icons[MICON_TEAM1_SELECT]);
-						gr_bitmap(Debrief_list_coords[gr_screen.res][0], Debrief_list_coords[gr_screen.res][1] + y - 2);
+						gr_bitmap(Debrief_list_coords[gr_screen.res][0], Debrief_list_coords[gr_screen.res][1] + y - 2, GR_RESIZE_MENU);
 					} 
 					// draw his "normal" icon
 					else if(Multi_common_icons[MICON_TEAM1] != -1){
 						gr_set_bitmap(Multi_common_icons[MICON_TEAM1]);
-						gr_bitmap(Debrief_list_coords[gr_screen.res][0], Debrief_list_coords[gr_screen.res][1] + y - 2);
+						gr_bitmap(Debrief_list_coords[gr_screen.res][0], Debrief_list_coords[gr_screen.res][1] + y - 2, GR_RESIZE_MENU);
 					}					
 				}
 			} else {
 				// draw the team 0 selected icon
 				if(((np->state == NETPLAYER_STATE_DEBRIEF_ACCEPT) || (np->state == NETPLAYER_STATE_DEBRIEF_REPLAY)) && (Multi_common_icons[MICON_TEAM0_SELECT] != -1)){
 					gr_set_bitmap(Multi_common_icons[MICON_TEAM0_SELECT]);
-					gr_bitmap(Debrief_list_coords[gr_screen.res][0], Debrief_list_coords[gr_screen.res][1] + y - 2);
+					gr_bitmap(Debrief_list_coords[gr_screen.res][0], Debrief_list_coords[gr_screen.res][1] + y - 2, GR_RESIZE_MENU);
 				}
 			}
 		}
@@ -1268,7 +1266,7 @@ void debrief_multi_list_draw()
 		}		
 
 		// bli
-		gr_string(Debrief_list_coords[gr_screen.res][0] + MULTI_LIST_TEAM_OFFSET, Debrief_list_coords[gr_screen.res][1] + y, str);
+		gr_string(Debrief_list_coords[gr_screen.res][0] + MULTI_LIST_TEAM_OFFSET, Debrief_list_coords[gr_screen.res][1] + y, str, GR_RESIZE_MENU);
 
 		y += font_height;
 		z++;
@@ -1308,7 +1306,7 @@ void debrief_accept(int ok_to_post_start_game_event)
 	Weapon_energy_cheat=0;
 
 	if ( (/*Cheats_enabled ||*/ Turned_traitor || Must_replay_mission) && (Game_mode & GM_CAMPAIGN_MODE) ) {
-		char *str;
+		const char *str;
 		int z;
 
 		if (Game_mode & GM_MULTIPLAYER) {
@@ -1348,6 +1346,8 @@ void debrief_accept(int ok_to_post_start_game_event)
 		// mission that isn't in a campaign.
 		if ( Game_mode & GM_CAMPAIGN_MODE ) {
 
+			mission_campaign_store_variables();
+
 			// check for possible mission loop
 			// check for (1) mission loop available, (2) don't have to repeat last mission
 			if(!(Game_mode & GM_MULTIPLAYER)){
@@ -1387,8 +1387,8 @@ void debrief_accept(int ok_to_post_start_game_event)
 					Campaign.loop_enabled = 0;
 				}
 
-				// check if campaign is over
-				if ( Campaign.next_mission == -1 ) {
+				// check if campaign is over, or if FREDer wants the mainhall
+				if ( Campaign.next_mission == -1 || (The_mission.flags & MISSION_FLAG_END_TO_MAINHALL) ) {
 					gameseq_post_event(GS_EVENT_MAIN_MENU);
 				} else {
 					if ( ok_to_post_start_game_event ) {
@@ -1487,15 +1487,15 @@ void debrief_render_stagenum()
 	sprintf(buf, XSTR( "%d of %d", 445), Current_stage + 1, Num_stages);
 	gr_get_string_size(&w, NULL, buf);
 	gr_set_color_fast(&Color_bright_blue);
-	gr_string(Debrief_stage_info_coords[gr_screen.res][0] - w, Debrief_stage_info_coords[gr_screen.res][1], buf);
+	gr_string(Debrief_stage_info_coords[gr_screen.res][0] - w, Debrief_stage_info_coords[gr_screen.res][1], buf, GR_RESIZE_MENU);
 	gr_set_color_fast(&Color_white);
 }
 
 // render the mission difficulty at the specified y location
 void debrief_render_mission_difficulty(int y_loc)
 {	
-	gr_string(0, y_loc, XSTR( "Skill Level", 1509));
-	gr_string(Debrief_text_x2[gr_screen.res], y_loc, Skill_level_names(Game_skill_level));	
+	gr_string(0, y_loc, XSTR( "Skill Level", 1509), GR_RESIZE_MENU);
+	gr_string(Debrief_text_x2[gr_screen.res], y_loc, Skill_level_names(Game_skill_level), GR_RESIZE_MENU);	
 }
 
 // render the mission time at the specified y location
@@ -1504,40 +1504,8 @@ void debrief_render_mission_time(int y_loc)
 	char time_str[30];
 	
 	game_format_time(Missiontime, time_str);
-	gr_string(0, y_loc, XSTR( "Mission Time", 446));
-	gr_string(Debrief_text_x2[gr_screen.res], y_loc, time_str);	
-}
-
-// render out the debriefing text to the scroll window
-void debrief_render()
-{
-	int y, z, font_height;
-
-	if ( Num_stages <= 0 )
-		return;
-
-	font_height = gr_get_font_height();
-
-	gr_set_clip(Debrief_text_wnd_coords[gr_screen.res][0], Debrief_text_wnd_coords[gr_screen.res][1], Debrief_text_wnd_coords[gr_screen.res][2], Debrief_text_wnd_coords[gr_screen.res][3]);
-	y = 0;
-	z = Text_offset;
-	while (y + font_height <= Debrief_text_wnd_coords[gr_screen.res][3]) {
-		if (z >= Num_text_lines)
-			break;
-
-		if (Text_type[z] == TEXT_TYPE_NORMAL)
-			gr_set_color_fast(&Color_white);
-		else
-			gr_set_color_fast(&Color_bright_red);
-
-		if (Text[z])
-			gr_string(0, y, Text[z]);
-
-		y += font_height;
-		z++;
-	}
-
-	gr_reset_clip();
+	gr_string(0, y_loc, XSTR( "Mission Time", 446), GR_RESIZE_MENU);
+	gr_string(Debrief_text_x2[gr_screen.res], y_loc, time_str, GR_RESIZE_MENU);	
 }
 
 // render out the stats info to the scroll window
@@ -1547,8 +1515,8 @@ void debrief_stats_render()
 	int i, y, font_height;	
 
 	gr_set_color_fast(&Color_blue);
-	gr_set_clip(Debrief_text_wnd_coords[gr_screen.res][0], Debrief_text_wnd_coords[gr_screen.res][1], Debrief_text_wnd_coords[gr_screen.res][2], Debrief_text_wnd_coords[gr_screen.res][3]);
-	gr_string(0, 0, Debrief_current_callsign);
+	gr_set_clip(Debrief_text_wnd_coords[gr_screen.res][0], Debrief_text_wnd_coords[gr_screen.res][1], Debrief_text_wnd_coords[gr_screen.res][2], Debrief_text_wnd_coords[gr_screen.res][3], GR_RESIZE_MENU);
+	gr_string(0, 0, Debrief_current_callsign, GR_RESIZE_MENU);
 	font_height = gr_get_font_height();
 	y = 30;
 	
@@ -1588,16 +1556,16 @@ void debrief_stats_render()
 
 				if (!i) {
 					if ( Current_stage == DEBRIEF_MISSION_KILLS )
-						gr_printf(0, y, XSTR( "Mission Kills by Ship Type", 447));
+						gr_printf_menu(0, y, XSTR( "Mission Kills by Ship Type", 447));
 					else
-						gr_printf(0, y, XSTR( "All-time Kills by Ship Type", 448));
+						gr_printf_menu(0, y, XSTR( "All-time Kills by Ship Type", 448));
 
 				} else if (i > 1) {
 					//Assert: Was debrief_setup_ship_kill_stats called?
 					Assert(Debrief_stats_kills != NULL);
 
-					gr_printf(0, y, "%s", Debrief_stats_kills[i - 2].text);
-					gr_printf(Debrief_text_x2[gr_screen.res], y, "%d", Debrief_stats_kills[i - 2].num);
+					gr_printf_menu(0, y, "%s", Debrief_stats_kills[i - 2].text);
+					gr_printf_menu(Debrief_text_x2[gr_screen.res], y, "%d", Debrief_stats_kills[i - 2].num);
 				}
 
 				y += font_height;
@@ -1606,9 +1574,9 @@ void debrief_stats_render()
 
 			if (Num_text_lines == 2) {
 				if ( Current_stage == DEBRIEF_MISSION_KILLS )
-					gr_printf(0, y, XSTR( "(No ship kills this mission)", 449));
+					gr_printf_menu(0, y, XSTR( "(No ship kills this mission)", 449));
 				else
-					gr_printf(0, y, XSTR( "(No ship kills)", 450));
+					gr_printf_menu(0, y, XSTR( "(No ship kills)", 450));
 			}
 
 			break;
@@ -1690,7 +1658,7 @@ void debrief_button_pressed(int num)
 			break;
 
 		case TEXT_SCROLL_DOWN:
-			if (Max_debrief_Lines < Num_text_lines) {
+			if (Max_debrief_Lines < (Num_text_lines - Text_offset)) {
 				Text_offset++;
 				gamesnd_play_iface(SND_SCROLL);
 			} else {
@@ -1849,44 +1817,6 @@ void debrief_check_buttons()
 	*/
 }
 
-void debrief_text_stage_init(const char *src, int type)
-{
-	int i, n_lines, n_chars[MAX_DEBRIEF_LINES];
-	char line[MAX_DEBRIEF_LINE_LEN];
-	const char *p_str[MAX_DEBRIEF_LINES];
-
-	n_lines = split_str(src, Debrief_text_wnd_coords[gr_screen.res][2], n_chars, p_str, MAX_DEBRIEF_LINES);
-	Assert(n_lines >= 0);
-
-	// if you hit this, you proba	
-	if(n_lines >= MAX_DEBRIEF_LINES){
-		Warning(LOCATION, "You have come close to the limit of debriefing lines, try adding more stages");	
-	}
-
-	for ( i=0; i<n_lines; i++ ) {
-		Assert(n_chars[i] < MAX_DEBRIEF_LINE_LEN);
-		Assert(Num_text_lines < MAX_TOTAL_DEBRIEF_LINES);
-		strncpy(line, p_str[i], n_chars[i]);
-		line[n_chars[i]] = 0;
-		drop_white_space(line);
-		Text_type[Num_text_lines] = type;
-		Text[Num_text_lines++] = vm_strdup(line);
-	}
-
-	return;
-}
-
-void debrief_free_text()
-{
-	int i;
-
-	for (i=0; i<Num_debrief_lines; i++)
-		if (Text[i])
-			vm_free(Text[i]);
-
-	Num_debrief_lines = 0;
-}
-
 // setup the debriefing text lines for rendering
 void debrief_text_init()
 {
@@ -1903,21 +1833,20 @@ void debrief_text_init()
 		}
 	}
 
-	// release old text lines first
-	debrief_free_text();
-	Num_text_lines = Text_offset = 0;
+	Num_text_lines = Text_offset = brief_color_text_init("", Debrief_text_wnd_coords[gr_screen.res][2], default_debriefing_color, 0, 0);	// Initialize color stuff -MageKing17
 
 	fsspeech_start_buffer();
 
 	if (Current_mode == DEBRIEF_TAB) {
 		for (i=0; i<Num_debrief_stages; i++) {
 			if (i)
-				Text[Num_text_lines++] = NULL;  // add a blank line between stages
+				// add a blank line between stages
+				Num_text_lines += brief_color_text_init("\n", Debrief_text_wnd_coords[gr_screen.res][2], default_debriefing_color, 0, MAX_DEBRIEF_LINES, true);
 
 			src = Debrief_stages[i]->text.c_str();
 
 			if (*src) {
-				debrief_text_stage_init(src, TEXT_TYPE_NORMAL);
+				Num_text_lines += brief_color_text_init(src, Debrief_text_wnd_coords[gr_screen.res][2], default_debriefing_color, 0, MAX_DEBRIEF_LINES, true);
 
 				if (use_sim_speech && !Recommend_active) {
 					fsspeech_stuff_buffer(src);
@@ -1932,8 +1861,9 @@ void debrief_text_init()
 					src = XSTR( "We have no recommendations for you.", 1054);
 
 				if (*src) {
-					Text[Num_text_lines++] = NULL;
-					debrief_text_stage_init(src, TEXT_TYPE_RECOMMENDATION);
+					Num_text_lines += brief_color_text_init("\n", Debrief_text_wnd_coords[gr_screen.res][2], default_recommendation_color, 0, MAX_DEBRIEF_LINES, true);
+
+					Num_text_lines += brief_color_text_init(src, Debrief_text_wnd_coords[gr_screen.res][2], default_recommendation_color, 0, MAX_DEBRIEF_LINES, true);
 					r_count++;
 
 					if (use_sim_speech) {
@@ -1943,8 +1873,8 @@ void debrief_text_init()
 				}
 			}
 		}
+		Brief_text_wipe_time_elapsed = BRIEF_TEXT_WIPE_TIME;	// Skip the wipe effect
 
-		Num_debrief_lines = Num_text_lines;
 		if(use_sim_speech) {
 			fsspeech_play_buffer(FSSPEECH_FROM_BRIEFING);
 		}
@@ -1996,6 +1926,9 @@ void debrief_init()
 //	Campaign.loop_enabled = 0;
 	Campaign.loop_mission = CAMPAIGN_LOOP_MISSION_UNINITIALIZED;
 
+	// MageKing17 - Set the font so that wordwrapping in brief_color_text_init() calculates based on the same font as the debriefing itself.
+	gr_set_font(DEBRIEFING_FONT);
+
 	// set up the right briefing for this guy
 	if(MULTI_TEAM){
 		Debriefing = &Debriefings[Net_player->p_info.team];
@@ -2020,7 +1953,7 @@ void debrief_init()
 	Current_stage = -1;
 	New_stage = 0;
 	Debrief_cue_voice = 0;
-	Num_text_lines = Num_debrief_lines = 0;
+	Num_text_lines = 0;
 	Debrief_first_voice_flag = 1;
 
 	Debrief_multi_voice_loaded = 0;
@@ -2028,7 +1961,7 @@ void debrief_init()
 	if ( (Game_mode & GM_CAMPAIGN_MODE) && ( !MULTIPLAYER_CLIENT )	) {
 		// MUST store goals and events first - may be used to evaluate next mission
 		// store goals and events
-		mission_campaign_store_goals_and_events_and_variables();
+		mission_campaign_store_goals_and_events();
 
 		// evaluate next mission
 		mission_campaign_eval_next_mission();
@@ -2085,6 +2018,9 @@ void debrief_init()
 	}
 	*/
 
+	// Just calculate this once instead of every frame. -MageKing17
+	Max_debrief_Lines = Debrief_text_wnd_coords[gr_screen.res][3]/gr_get_font_height(); //Make the max number of lines dependent on the font height.
+
 	// start up the appropriate music
 	debrief_init_music();
 
@@ -2121,6 +2057,7 @@ void debrief_init()
 void debrief_close()
 {
 	int i;
+	scoring_struct *sc;
 
 	Assert(Debrief_inited);
 
@@ -2133,11 +2070,17 @@ void debrief_close()
 			if(MULTIPLAYER_MASTER){
 				for(i=0; i<MAX_PLAYERS; i++){
 					if(MULTI_CONNECTED(Net_players[i]) && !MULTI_STANDALONE(Net_players[i]) && !MULTI_PERM_OBSERVER(Net_players[i]) && (Net_players[i].m_player != NULL)){
-						scoring_backout_accept(&Net_players[i].m_player->stats);
+						sc = &Net_players[i].m_player->stats;
+						scoring_backout_accept(sc);
+
+						if (Net_player == &Net_players[i]) {
+							Pilot.update_stats_backout( sc );
+						}
 					}
 				}
 			} else {
 				scoring_backout_accept( &Player->stats );
+				Pilot.update_stats_backout( &Player->stats );
 			}
 		}
 	} else {
@@ -2148,6 +2091,7 @@ void debrief_close()
 				Campaign.next_mission = Campaign.current_mission;
 			}
 			scoring_backout_accept( &Player->stats );
+			Pilot.update_stats_backout( &Player->stats );
 		}
 	}
 
@@ -2156,20 +2100,9 @@ void debrief_close()
 		Player->show_skip_popup = 1;
 	}
 
-	if (Num_debrief_lines) {
-		for (i=0; i<Num_debrief_lines; i++){
-			if (Text[i]){
-				vm_free(Text[i]);
-			}
-		}
-	}
-
 	// clear out debrief info parsed from mission file - taylor
 	mission_debrief_common_reset();
 
-	// unload the overlay bitmap
-//	help_overlay_unload(DEBRIEFING_OVERLAY);
-
 	// clear out award text 
 	Debrief_award_text_num_lines = 0;
 
@@ -2177,8 +2110,6 @@ void debrief_close()
 	common_music_close();
 	chatbox_close();
 
-//	rank_bitmaps_release();
-
 	// unload bitmaps
 	if (Background_bitmap >= 0){
 		bm_release(Background_bitmap);
@@ -2200,14 +2131,6 @@ void debrief_close()
 		bm_release(Badge_bitmap);
 	}
 
-	if (Wings_bitmap >= 0) {
-		bm_release(Wings_bitmap);
-	}
-	
-	if (Crest_bitmap >= 0) {
-		bm_release(Crest_bitmap);
-	}
-
 	Debrief_ui_window.destroy();
 	common_free_interface_palette();		// restore game palette
 	show_stats_close();
@@ -2297,7 +2220,7 @@ void debrief_draw_award_text()
 		x = (Medal_bitmap < 0) ? (Debrief_award_text_coords[gr_screen.res][0] + (field_width - sw) / 2) : Debrief_award_text_coords[gr_screen.res][0];
 		if (i==AWARD_TEXT_MAX_LINES-1) x += 7;				// hack because of the shape of the box
 		gr_set_color_fast(&Color_white);
-		gr_string(x, curr_y, Debrief_award_text[i]);
+		gr_string(x, curr_y, Debrief_award_text[i], GR_RESIZE_MENU);
 
 		// adjust y pos, including a little extra between the "pairs"
 		curr_y += fh;
@@ -2321,8 +2244,8 @@ void debrief_award_text_clear() {
 // this is the nastiest code I have ever written.  if you are modifying this, i feel bad for you.
 void debrief_add_award_text(char *str)
 {
-	Assert(Debrief_award_text_num_lines <= AWARD_TEXT_MAX_LINES);
-	if (Debrief_award_text_num_lines > AWARD_TEXT_MAX_LINES) {
+	Assert(Debrief_award_text_num_lines < AWARD_TEXT_MAX_LINES);
+	if (Debrief_award_text_num_lines >= AWARD_TEXT_MAX_LINES) {
 		return;
 	}
 
@@ -2334,7 +2257,9 @@ void debrief_add_award_text(char *str)
 
 	// maybe translate for displaying
 	if (Lcl_gr) {
-		medals_translate_name(Debrief_award_text[Debrief_award_text_num_lines], AWARD_TEXT_MAX_LINE_LENGTH);
+		lcl_translate_medal_name_gr(Debrief_award_text[Debrief_award_text_num_lines]);
+	} else if (Lcl_pl) {
+		lcl_translate_medal_name_pl(Debrief_award_text[Debrief_award_text_num_lines]);
 	}
 
 	Debrief_award_text_num_lines++;
@@ -2354,7 +2279,7 @@ void debrief_add_award_text(char *str)
 void debrief_do_frame(float frametime)
 {
 	int k=0, new_k=0;
-	char *please_wait_str = XSTR("Please Wait", 1242);
+	const char *please_wait_str = XSTR("Please Wait", 1242);
 	char buf[256];
 
 	Assert(Debrief_inited);	
@@ -2378,21 +2303,21 @@ void debrief_do_frame(float frametime)
 		GR_MAYBE_CLEAR_RES(Background_bitmap);
 		if (Background_bitmap >= 0) {
 			gr_set_bitmap(Background_bitmap);
-			gr_bitmap(0, 0);
+			gr_bitmap(0, 0, GR_RESIZE_MENU);
 		}
 
 		Debrief_ui_window.draw();
 		chatbox_render();
 		if ( Debrief_multi_loading_bitmap > -1 ){
 			gr_set_bitmap(Debrief_multi_loading_bitmap);		
-			gr_bitmap( Please_wait_coords[gr_screen.res][0], Please_wait_coords[gr_screen.res][1] );
+			gr_bitmap( Please_wait_coords[gr_screen.res][0], Please_wait_coords[gr_screen.res][1], GR_RESIZE_MENU );
 		}
 
 		// draw "Please Wait"		
 		gr_set_color_fast(&Color_normal);
 		gr_set_font(FONT2);
 		gr_get_string_size(&str_w, &str_h, please_wait_str);
-		gr_string((gr_screen.max_w - str_w) / 2, (gr_screen.max_h - str_h) / 2, please_wait_str);
+		gr_string((gr_screen.max_w - str_w) / 2, (gr_screen.max_h - str_h) / 2, please_wait_str, GR_RESIZE_MENU);
 		gr_set_font(FONT1);
 
 		gr_flip();
@@ -2420,7 +2345,7 @@ void debrief_do_frame(float frametime)
 		Debrief_multi_voice_loaded = 1;
 	}
 
-	if ( help_overlay_active(DEBRIEFING_OVERLAY) ) {
+	if ( help_overlay_active(Debrief_overlay_id) ) {
 		Buttons[gr_screen.res][HELP_BUTTON].button.reset_status();
 		Debrief_ui_window.set_ignore_gadgets(1);
 	}
@@ -2435,15 +2360,15 @@ void debrief_do_frame(float frametime)
 	}
 
 	if ( (k > 0) || (new_k > 0) || B1_JUST_RELEASED ) {
-		if ( help_overlay_active(DEBRIEFING_OVERLAY) ) {
-			help_overlay_set_state(DEBRIEFING_OVERLAY, 0);
+		if ( help_overlay_active(Debrief_overlay_id) ) {
+			help_overlay_set_state(Debrief_overlay_id, gr_screen.res, 0);
 			Debrief_ui_window.set_ignore_gadgets(0);
 			k = 0;
 			new_k = 0;
 		}
 	}
 
-	if ( !help_overlay_active(DEBRIEFING_OVERLAY) ) {
+	if ( !help_overlay_active(Debrief_overlay_id) ) {
 		Debrief_ui_window.set_ignore_gadgets(0);
 	}
 
@@ -2501,57 +2426,30 @@ void debrief_do_frame(float frametime)
 	GR_MAYBE_CLEAR_RES(Background_bitmap);
 	if (Background_bitmap >= 0) {
 		gr_set_bitmap(Background_bitmap);
-		gr_bitmap(0, 0);
+		gr_bitmap(0, 0, GR_RESIZE_MENU);
 	} 
 
 	// draw the awarded stuff, G
 	if ( Award_active && (Award_bg_bitmap >= 0) ) {
 		gr_set_bitmap(Award_bg_bitmap);
-		gr_bitmap(Debrief_award_wnd_coords[gr_screen.res][0], Debrief_award_wnd_coords[gr_screen.res][1]);
+		gr_bitmap(Debrief_award_wnd_coords[gr_screen.res][0], Debrief_award_wnd_coords[gr_screen.res][1], GR_RESIZE_MENU);
 		if (Rank_bitmap >= 0) {
 			gr_set_bitmap(Rank_bitmap);
-			gr_bitmap(Debrief_award_coords[gr_screen.res][0], Debrief_award_coords[gr_screen.res][1]);
+			gr_bitmap(Debrief_award_coords[gr_screen.res][0], Debrief_award_coords[gr_screen.res][1], GR_RESIZE_MENU);
 		}
 
 		if (Medal_bitmap >= 0) {
 			gr_set_bitmap(Medal_bitmap);
-			gr_bitmap(Debrief_award_coords[gr_screen.res][0], Debrief_award_coords[gr_screen.res][1]);
+			gr_bitmap(Debrief_award_coords[gr_screen.res][0], Debrief_award_coords[gr_screen.res][1], GR_RESIZE_MENU);
 		}
 
 		if (Badge_bitmap >= 0) {
 			gr_set_bitmap(Badge_bitmap);
-			gr_bitmap(Debrief_award_coords[gr_screen.res][0], Debrief_award_coords[gr_screen.res][1]);
-		}
-
-		if (Wings_bitmap >= 0) {
-			gr_set_bitmap(Wings_bitmap);
-			gr_bitmap(Debrief_award_coords[gr_screen.res][0], Debrief_award_coords[gr_screen.res][1]);
-		}
-
-		if (Crest_bitmap >= 0) {
-			gr_set_bitmap(Crest_bitmap);
-			gr_bitmap(Debrief_award_coords[gr_screen.res][0], Debrief_award_coords[gr_screen.res][1]);
+			gr_bitmap(Debrief_award_coords[gr_screen.res][0], Debrief_award_coords[gr_screen.res][1], GR_RESIZE_MENU);
 		}
 
 		//  draw medal/badge/rank labels
 		debrief_draw_award_text();
-
-/*		if (Rank_text_bitmap >= 0) {
-			gr_set_bitmap(Rank_text_bitmap);
-			gr_bitmap(Debrief_award_coords[gr_screen.res][0], Debrief_award_coords[gr_screen.res][1]);
-		}
-
-	
-		if (Medal_text_bitmap >= 0) {
-			gr_set_bitmap(Medal_text_bitmap);
-			gr_bitmap(Debrief_award_text_coords[gr_screen.res][0], Debrief_award_text_coords[gr_screen.res][1]);
-		}
-
-		if (Badge_text_bitmap >= 0) {
-			gr_set_bitmap(Badge_text_bitmap);
-			gr_bitmap(Debrief_award_text_coords[gr_screen.res][0], Debrief_award_text_coords[gr_screen.res][1]);
-		}
-*/
 	}
 	
 	Debrief_ui_window.draw();
@@ -2565,23 +2463,26 @@ void debrief_do_frame(float frametime)
 	gr_set_color_fast(&Color_bright_white);
 	strcpy_s(buf, The_mission.name);
 	gr_force_fit_string(buf, 255, Debrief_title_coords[gr_screen.res][2]);
-	gr_string(Debrief_title_coords[gr_screen.res][0], Debrief_title_coords[gr_screen.res][1], buf);	
+	gr_string(Debrief_title_coords[gr_screen.res][0], Debrief_title_coords[gr_screen.res][1], buf, GR_RESIZE_MENU);	
 
 #if !defined(NDEBUG)
 	gr_set_color_fast(&Color_normal);
-	gr_printf(Debrief_title_coords[gr_screen.res][0], Debrief_title_coords[gr_screen.res][1] - 10, NOX("[name: %s, mod: %s]"), Mission_filename, The_mission.modified);
+	gr_printf_menu(Debrief_title_coords[gr_screen.res][0], Debrief_title_coords[gr_screen.res][1] - 10, NOX("[name: %s, mod: %s]"), Mission_filename, The_mission.modified);
 #endif
 
+	// Set the font for the debriefing instead of relying on the implicit font-setting of Debrief_ui_window.draw() -MageKing17
+	gr_set_font(DEBRIEFING_FONT);
+
 	// draw the screen-specific text
 	switch (Current_mode) {
 		case DEBRIEF_TAB:
 			if ( Num_debrief_stages <= 0 ) {
 				gr_set_color_fast(&Color_white);
 				Assert( Game_current_mission_filename != NULL );
-				gr_printf(Debrief_text_wnd_coords[gr_screen.res][0], Debrief_text_wnd_coords[gr_screen.res][1], XSTR( "No Debriefing for mission: %s", 458), Game_current_mission_filename);
+				gr_printf_menu(Debrief_text_wnd_coords[gr_screen.res][0], Debrief_text_wnd_coords[gr_screen.res][1], XSTR( "No Debriefing for mission: %s", 458), Game_current_mission_filename);
 
 			} else {
-				debrief_render();
+				brief_render_text(Text_offset, Debrief_text_wnd_coords[gr_screen.res][0], Debrief_text_wnd_coords[gr_screen.res][1], Debrief_text_wnd_coords[gr_screen.res][3], frametime);
 			}
 
 			break;
@@ -2591,18 +2492,12 @@ void debrief_do_frame(float frametime)
 			break;
 	} // end switch
 
-	if (gr_screen.res == 1) {
-		Max_debrief_Lines = 450/gr_get_font_height(); //Make the max number of lines dependent on the font height. 225 and 85 are magic numbers, based on the window size in retail. 
-	} else {
-		Max_debrief_Lines = 340/gr_get_font_height();
-	}
-
-	if (Max_debrief_Lines < Num_text_lines) {
+	if ( (Max_debrief_Lines + Text_offset) < Num_text_lines ) {
 		int w;
 
-		gr_set_color_fast(&Color_red);
+		gr_set_color_fast(&Color_more_indicator);
 		gr_get_string_size(&w, NULL, XSTR( "More", 459));
-		gr_printf(Debrief_text_wnd_coords[gr_screen.res][0] + Debrief_text_wnd_coords[gr_screen.res][2] / 2 - w / 2, Debrief_text_wnd_coords[gr_screen.res][1] + Debrief_text_wnd_coords[gr_screen.res][3], XSTR( "More", 459));
+		gr_printf_menu(Debrief_text_wnd_coords[gr_screen.res][0] + Debrief_text_wnd_coords[gr_screen.res][2] / 2 - w / 2, Debrief_text_wnd_coords[gr_screen.res][1] + Debrief_text_wnd_coords[gr_screen.res][3], XSTR( "More", 459));
 	}
 
 	debrief_render_stagenum();
@@ -2623,13 +2518,13 @@ void debrief_do_frame(float frametime)
 	// AL 3-6-98: Needed to move key reading here, since popups are launched from this code, and we don't
 	//				  want to include the mouse pointer which is drawn in the flip
 
-	if ( !help_overlay_active(DEBRIEFING_OVERLAY) ) {
+	if ( !help_overlay_active(Debrief_overlay_id) ) {
 		debrief_check_buttons();
 		debrief_do_keys(new_k);	
 	}
 
 	// blit help overlay if active
-	help_overlay_maybe_blit(DEBRIEFING_OVERLAY);
+	help_overlay_maybe_blit(Debrief_overlay_id, gr_screen.res);
 
 	gr_flip();
 
