@@ -13,6 +13,7 @@
 #include "parse/parselo.h"
 #include "globalincs/linklist.h"
 #include "io/timer.h"
+#include "debugconsole/console.h"
 #include "freespace2/freespace.h"
 #include "gamesnd/gamesnd.h"
 #include "render/3d.h"
@@ -81,52 +82,43 @@ vec3d Nebl_bolt_strike;			// strike point of the bolt being generated
 storm_type *Storm = NULL;
 
 // vars
-DCF(b_scale, "")
+DCF(b_scale, "Sets the scale factor for debug nebula bolts")
 {
-	dc_get_arg(ARG_FLOAT);
-	Bolt_types[DEBUG_BOLT].b_scale = Dc_arg_float;
+	dc_stuff_float(&Bolt_types[DEBUG_BOLT].b_scale);
 }
-DCF(b_rand, "")
+DCF(b_rand, "Sets the randomness factor for debug nebula bolts")
 {
-	dc_get_arg(ARG_FLOAT);
-	Bolt_types[DEBUG_BOLT].b_rand = Dc_arg_float;
+	dc_stuff_float(&Bolt_types[DEBUG_BOLT].b_rand);
 }
-DCF(b_shrink, "")
+DCF(b_shrink, "Sets the shrink factor for debug nebula bolts")
 {
-	dc_get_arg(ARG_FLOAT);
-	Bolt_types[DEBUG_BOLT].b_shrink = Dc_arg_float;
+	dc_stuff_float(&Bolt_types[DEBUG_BOLT].b_shrink);
 }
-DCF(b_poly_pct, "")
+DCF(b_poly_pct, "Sets b_poly_pct")
 {
-	dc_get_arg(ARG_FLOAT);
-	Bolt_types[DEBUG_BOLT].b_poly_pct = Dc_arg_float;
+	dc_stuff_float(&Bolt_types[DEBUG_BOLT].b_poly_pct);
 }
-DCF(b_add, "")
+DCF(b_add, "Sets b_add")
 {
-	dc_get_arg(ARG_FLOAT);
-	Bolt_types[DEBUG_BOLT].b_add = Dc_arg_float;
+	dc_stuff_float(&Bolt_types[DEBUG_BOLT].b_add);
 }
-DCF(b_strikes, "")
+DCF(b_strikes, "Sets num_strikes")
 {
-	dc_get_arg(ARG_INT);
-	Bolt_types[DEBUG_BOLT].num_strikes = Dc_arg_int;
+	dc_stuff_int(&Bolt_types[DEBUG_BOLT].num_strikes);
 }
-DCF(b_noise, "")
+DCF(b_noise, "Sets noise factor")
 {
-	dc_get_arg(ARG_FLOAT);
-	Bolt_types[DEBUG_BOLT].noise = Dc_arg_float;
+	dc_stuff_float(&Bolt_types[DEBUG_BOLT].noise);
 }
-DCF(b_bright, "")
+DCF(b_bright, "Sets brightness factor")
 {
-	dc_get_arg(ARG_FLOAT);
-	Bolt_types[DEBUG_BOLT].b_bright = Dc_arg_float;
+	dc_stuff_float(&Bolt_types[DEBUG_BOLT].b_bright);
 }
-DCF(b_lifetime, "")
+DCF(b_lifetime, "Sets lifetime duration")
 {
-	dc_get_arg(ARG_INT);
-	Bolt_types[DEBUG_BOLT].lifetime = Dc_arg_int;
+	dc_stuff_int(&Bolt_types[DEBUG_BOLT].lifetime);
 }
-DCF(b_list, "")
+DCF(b_list, "Displays status of debug lightning commands")
 {
 	dc_printf("Debug lightning bolt settings :\n");
 
@@ -144,12 +136,12 @@ DCF(b_list, "")
 // nebula lightning intensity (0.0 to 1.0)
 float Nebl_intensity = 0.6667f;
 
-DCF(lightning_intensity, "")
+DCF(lightning_intensity, "Sets lightning intensity between 0.0 and 1.0 (Default is 0.6667)")
 {
-	dc_get_arg(ARG_FLOAT);
-	float val = Dc_arg_float;
-    
-    CLAMP(val, 0.0f, 1.0f);
+	float val;
+	dc_stuff_float(&val);
+
+	CLAMP(val, 0.0f, 1.0f);
 
 	Nebl_intensity = 1.0f - val;
 }
@@ -939,6 +931,7 @@ void nebl_generate_section(bolt_type *bi, float width, l_node *a, l_node *b, l_s
 	size_t idx;	
 	vec3d temp, pt;
 	vec3d glow_a, glow_b;
+	vertex tempv;
 
 	// direction matrix
 	vm_vec_sub(&dir, &a->pos, &b->pos);
@@ -953,6 +946,8 @@ void nebl_generate_section(bolt_type *bi, float width, l_node *a, l_node *b, l_s
 
 	// rotate the basic section into world	
 	for(idx=0; idx<3; idx++){
+		memset(&tempv, 0, sizeof(tempv));
+        
 		// rotate to world		
 		if(pinch_a){			
 			vm_vec_rotate(&pt, &Nebl_ring_pinched[idx], &m);
@@ -962,39 +957,30 @@ void nebl_generate_section(bolt_type *bi, float width, l_node *a, l_node *b, l_s
 		}
 		vm_vec_add2(&pt, &a->pos);
 			
-		// transform
-		if (Cmdline_nohtl) {
-			g3_rotate_vertex(&c->vex[idx], &pt);
-		} else {
-			g3_transfer_vertex(&c->vex[idx], &pt);
-		}
-		g3_project_vertex(&c->vex[idx]);		
+		g3_transfer_vertex(&c->vex[idx], &pt);
+		g3_rotate_vertex(&tempv, &pt);
+		g3_project_vertex(&tempv);
 
 		// if first frame, keep track of the average screen pos
-		if((c->vex[idx].screen.xyw.x >= 0)
-			&& (c->vex[idx].screen.xyw.x < gr_screen.max_w)
-			&& (c->vex[idx].screen.xyw.y >= 0)
-			&& (c->vex[idx].screen.xyw.y < gr_screen.max_h))
-		{
-			Nebl_flash_x += c->vex[idx].screen.xyw.x;
-			Nebl_flash_y += c->vex[idx].screen.xyw.y;
+		if (tempv.codes == 0) {
+			Nebl_flash_x += tempv.screen.xyw.x;
+			Nebl_flash_y += tempv.screen.xyw.y;
 			Nebl_flash_count++;
 		}
+
+		if (Cmdline_nohtl) {
+			memcpy(&c->vex[idx], &tempv, sizeof(vertex));
+		}
 	}
 	// calculate the glow points		
 	nebl_calc_facing_pts_smart(&glow_a, &glow_b, &dir_normal, &a->pos, pinch_a ? 0.5f : width * 6.0f, Nebl_type->b_add);
 	if (Cmdline_nohtl) {
 		g3_rotate_vertex(&c->glow_vex[0], &glow_a);
-	} else {
-		g3_transfer_vertex(&c->glow_vex[0], &glow_a);
-	}
-	g3_project_vertex(&c->glow_vex[0]);
-	if (Cmdline_nohtl) {
 		g3_rotate_vertex(&c->glow_vex[1], &glow_b);
 	} else {
+		g3_transfer_vertex(&c->glow_vex[0], &glow_a);
 		g3_transfer_vertex(&c->glow_vex[1], &glow_b);
 	}
-	g3_project_vertex(&c->glow_vex[1]);	
 
 	// maybe do a cap
 	if(cap != NULL){		
@@ -1009,40 +995,31 @@ void nebl_generate_section(bolt_type *bi, float width, l_node *a, l_node *b, l_s
 			}
 			vm_vec_add2(&pt, &b->pos);
 			
-			// transform
-			if (Cmdline_nohtl) {
-				g3_rotate_vertex(&cap->vex[idx], &pt);
-			} else {
-				g3_transfer_vertex(&cap->vex[idx], &pt);
-			}
-			g3_project_vertex(&cap->vex[idx]);			
+			g3_transfer_vertex(&cap->vex[idx], &pt);
+			g3_rotate_vertex(&tempv, &pt);
+			g3_project_vertex(&tempv);
 
-			// if first frame, keep track of the average screen pos			
-			if( (c->vex[idx].screen.xyw.x >= 0)
-				&& (c->vex[idx].screen.xyw.x < gr_screen.max_w)
-				&& (c->vex[idx].screen.xyw.y >= 0)
-				&& (c->vex[idx].screen.xyw.y < gr_screen.max_h))
-			{
-				Nebl_flash_x += c->vex[idx].screen.xyw.x;
-				Nebl_flash_y += c->vex[idx].screen.xyw.y;
+			// if first frame, keep track of the average screen pos
+			if (tempv.codes == 0) {
+				Nebl_flash_x += tempv.screen.xyw.x;
+				Nebl_flash_y += tempv.screen.xyw.y;
 				Nebl_flash_count++;
 			}
+
+			if (Cmdline_nohtl) {
+				memcpy(&cap->vex[idx], &tempv, sizeof(vertex));
+			}
 		}
 		
 		// calculate the glow points		
 		nebl_calc_facing_pts_smart(&glow_a, &glow_b, &dir_normal, &b->pos, pinch_b ? 0.5f : width * 6.0f, bi->b_add);
 		if (Cmdline_nohtl) {
 			g3_rotate_vertex(&cap->glow_vex[0], &glow_a);
-		} else {
-			g3_transfer_vertex(&cap->glow_vex[0], &glow_a);
-		}
-		g3_project_vertex(&cap->glow_vex[0]);
-		if (Cmdline_nohtl) {
 			g3_rotate_vertex(&cap->glow_vex[1], &glow_b);
 		} else {
+			g3_transfer_vertex(&cap->glow_vex[0], &glow_a);
 			g3_transfer_vertex(&cap->glow_vex[1], &glow_b);
 		}
-		g3_project_vertex(&cap->glow_vex[1]);
 	}
 }
 
