@@ -609,16 +609,16 @@ void multi_ingame_select_init()
 	// create a ship, then find a ship to copy crucial information from.  Save and restore the wing
 	// number to be safe.
 	/*
-	wingnum_save = Player_start_pobject.wingnum;
-	net_signature = Player_start_pobject.net_signature;
-	goals_save = Player_start_pobject.ai_goals;
-	Player_start_pobject.wingnum = -1;
-	Player_start_pobject.net_signature = 0;
-	Player_start_pobject.ai_goals = -1;
-	objnum = parse_create_object( &Player_start_pobject );
-	Player_start_pobject.wingnum = wingnum_save;
-	Player_start_pobject.net_signature = net_signature;
-	Player_start_pobject.ai_goals = goals_save;
+	wingnum_save = Player_start_pobject->wingnum;
+	net_signature = Player_start_pobject->net_signature;
+	goals_save = Player_start_pobject->ai_goals;
+	Player_start_pobject->wingnum = -1;
+	Player_start_pobject->net_signature = 0;
+	Player_start_pobject->ai_goals = -1;
+	objnum = parse_create_object( Player_start_pobject );
+	Player_start_pobject->wingnum = wingnum_save;
+	Player_start_pobject->net_signature = net_signature;
+	Player_start_pobject->ai_goals = goals_save;
 
 	if ( objnum == -1 ) {
 		nprintf(("Network", "Bailing ingame join because unable to create parse object player ship\n"));
@@ -807,7 +807,7 @@ void multi_ingame_select_do()
 	GR_MAYBE_CLEAR_RES(Multi_ingame_bitmap);
 	if(Multi_ingame_bitmap != -1){
 		gr_set_bitmap(Multi_ingame_bitmap);
-		gr_bitmap(0,0);
+		gr_bitmap(0,0,GR_RESIZE_MENU);
 	}
 	Multi_ingame_window.draw();
 
@@ -836,7 +836,7 @@ void multi_ingame_select_close()
 	Multi_ingame_window.destroy();	
 
 	// stop main hall music
-	main_hall_stop_music();	
+	main_hall_stop_music(true);
 }
 
 // display an individual ships information, starting at the indicated y pixel value
@@ -845,31 +845,33 @@ void multi_ingame_join_display_ship(object *objp,int y_start)
 	int icon_num,idx;
 	int y_spacing;
 	ship_weapon *wp;
+
+	int line_height = gr_get_font_height() + 1;
 	
 	// blit the ship name itself
 	gr_set_color_fast(&Color_normal);
-	gr_string(Mi_name_field[gr_screen.res][MI_FIELD_X],y_start+10, Ships[objp->instance].ship_name);
+	gr_string(Mi_name_field[gr_screen.res][MI_FIELD_X],y_start+10, Ships[objp->instance].ship_name, GR_RESIZE_MENU);
 	
 	// blit the ship class icon
 	icon_num = multi_ingame_get_ship_class_icon(Ships[objp->instance].ship_info_index);
 	if(icon_num != -1){
 		gr_set_bitmap(Multi_ingame_ship_icon[icon_num].bmaps[0]);
-		gr_bitmap(Mi_class_field[gr_screen.res][MI_FIELD_X] + 15, y_start);
+		gr_bitmap(Mi_class_field[gr_screen.res][MI_FIELD_X] + 15, y_start, GR_RESIZE_MENU);
 	}
 	
 	gr_set_color_fast(&Color_bright);
 	wp = &Ships[objp->instance].weapons;
 	
 	// blit the ship's primary weapons	
-	y_spacing = (Mi_spacing[gr_screen.res] - (wp->num_primary_banks * 10)) / 2;
+	y_spacing = (Mi_spacing[gr_screen.res] - (wp->num_primary_banks * line_height)) / 2;
 	for(idx=0;idx<wp->num_primary_banks;idx++){
-		gr_string(Mi_primary_field[gr_screen.res][MI_FIELD_X], y_start + y_spacing + (idx * 10), Weapon_info[wp->primary_bank_weapons[idx]].name);
+		gr_string(Mi_primary_field[gr_screen.res][MI_FIELD_X], y_start + y_spacing + (idx * line_height), Weapon_info[wp->primary_bank_weapons[idx]].name, GR_RESIZE_MENU);
 	}
 
 	// blit the ship's secondary weapons	
-	y_spacing = (Mi_spacing[gr_screen.res] - (wp->num_secondary_banks * 10)) / 2;
+	y_spacing = (Mi_spacing[gr_screen.res] - (wp->num_secondary_banks * line_height)) / 2;
 	for(idx=0;idx<wp->num_secondary_banks;idx++){
-		gr_string(Mi_secondary_field[gr_screen.res][MI_FIELD_X], y_start + y_spacing + (idx * 10), Weapon_info[wp->secondary_bank_weapons[idx]].name);
+		gr_string(Mi_secondary_field[gr_screen.res][MI_FIELD_X], y_start + y_spacing + (idx * line_height), Weapon_info[wp->secondary_bank_weapons[idx]].name, GR_RESIZE_MENU);
 	}	
 
 	// blit the shield/hull integrity
@@ -890,10 +892,10 @@ void multi_ingame_join_display_avail()
 
 		// draw the border
 		gr_set_color_fast(&Color_bright_blue);
-		gr_line(Mi_name_field[gr_screen.res][MI_FIELD_X]-1,y_start-1, (Mi_name_field[gr_screen.res][MI_FIELD_X]-1) + (Mi_width[gr_screen.res]+2),y_start-1);
-		gr_line(Mi_name_field[gr_screen.res][MI_FIELD_X]-1,y_start + Mi_spacing[gr_screen.res] - 2, (Mi_name_field[gr_screen.res][MI_FIELD_X]-1) + (Mi_width[gr_screen.res]+2),y_start + Mi_spacing[gr_screen.res] - 2);
-		gr_line(Mi_name_field[gr_screen.res][MI_FIELD_X]-1,y_start, Mi_name_field[gr_screen.res][MI_FIELD_X]-1, y_start + Mi_spacing[gr_screen.res] - 2);
-		gr_line((Mi_name_field[gr_screen.res][MI_FIELD_X]-1) + (Mi_width[gr_screen.res]+2), y_start,(Mi_name_field[gr_screen.res][MI_FIELD_X]-1) + (Mi_width[gr_screen.res]+2),y_start + Mi_spacing[gr_screen.res] - 2);
+		gr_line(Mi_name_field[gr_screen.res][MI_FIELD_X]-1,y_start-1, (Mi_name_field[gr_screen.res][MI_FIELD_X]-1) + (Mi_width[gr_screen.res]+2),y_start-1, GR_RESIZE_MENU);
+		gr_line(Mi_name_field[gr_screen.res][MI_FIELD_X]-1,y_start + Mi_spacing[gr_screen.res] - 2, (Mi_name_field[gr_screen.res][MI_FIELD_X]-1) + (Mi_width[gr_screen.res]+2),y_start + Mi_spacing[gr_screen.res] - 2, GR_RESIZE_MENU);
+		gr_line(Mi_name_field[gr_screen.res][MI_FIELD_X]-1,y_start, Mi_name_field[gr_screen.res][MI_FIELD_X]-1, y_start + Mi_spacing[gr_screen.res] - 2, GR_RESIZE_MENU);
+		gr_line((Mi_name_field[gr_screen.res][MI_FIELD_X]-1) + (Mi_width[gr_screen.res]+2), y_start,(Mi_name_field[gr_screen.res][MI_FIELD_X]-1) + (Mi_width[gr_screen.res]+2),y_start + Mi_spacing[gr_screen.res] - 2, GR_RESIZE_MENU);
 	}
 
 	moveup = GET_FIRST(&Ship_obj_list);	
@@ -940,7 +942,7 @@ void multi_ingame_handle_timeout()
 	/*
 	// uncomment this block to disable the timer
 	gr_set_color_fast(&Color_bright_red);
-	gr_string(Multi_ingame_timer_coords[gr_screen.res][0], Multi_ingame_timer_coords[gr_screen.res][1], "Timer disabled!!");
+	gr_string(Multi_ingame_timer_coords[gr_screen.res][0], Multi_ingame_timer_coords[gr_screen.res][1], "Timer disabled!!", GR_RESIZE_MENU);
 	return;
 	*/
 
@@ -956,7 +958,7 @@ void multi_ingame_handle_timeout()
 	gr_set_color_fast(&Color_bright);
 	memset(tl_string,0,100);
 	sprintf(tl_string,XSTR("Time remaining : %d s\n",682),time_left);	
-	gr_string(Multi_ingame_timer_coords[gr_screen.res][0], Multi_ingame_timer_coords[gr_screen.res][1], tl_string);
+	gr_string(Multi_ingame_timer_coords[gr_screen.res][0], Multi_ingame_timer_coords[gr_screen.res][1], tl_string, GR_RESIZE_MENU);
 }
 
 
@@ -1259,7 +1261,7 @@ void process_ingame_wings_packet( ubyte *data, header *hinfo )
 				// kind of stupid, but bash the name since it won't get recreated properly from
 				// the parse_wing_create_ships call.
 				shipp = &Ships[shipnum];
-				sprintf(shipp->ship_name, NOX("%s %d"), wingp->name, which_one + 1);
+				wing_bash_ship_name(shipp->ship_name, wingp->name, which_one + 1);
 				nprintf(("Network", "Created %s\n", shipp->ship_name));
 
 				objp = &Objects[shipp->objnum];
@@ -1818,10 +1820,11 @@ void send_ingame_ship_update_packet(net_player *p,ship *sp)
 	objp = &Objects[sp->objnum];
 	ADD_USHORT(objp->net_signature);
 	ADD_UINT(objp->flags);
+	ADD_INT(objp->n_quadrants);
 	ADD_FLOAT(objp->hull_strength);
 	
 	// shield percentages
-	for(idx=0; idx<MAX_SHIELD_SECTIONS; idx++){
+	for(idx=0; idx<objp->n_quadrants; idx++){
 		f_tmp = objp->shield_quadrant[idx];
 		ADD_FLOAT(f_tmp);
 	}
@@ -1835,6 +1838,7 @@ void process_ingame_ship_update_packet(ubyte *data, header *hinfo)
 	float garbage;
 	int flags;
 	int idx;
+	int n_quadrants;
 	ushort net_sig;
 	object *lookup;
 	float f_tmp;
@@ -1843,14 +1847,15 @@ void process_ingame_ship_update_packet(ubyte *data, header *hinfo)
 	// get the net sig for the ship and do a lookup
 	GET_USHORT(net_sig);
 	GET_INT(flags);
-   
+	GET_INT(n_quadrants);
+
 	// get the object
 	lookup = multi_get_network_object(net_sig);
 	if(lookup == NULL){
 		// read in garbage values if we can't find the ship
 		nprintf(("Network","Got ingame ship update for unknown object\n"));
 		GET_FLOAT(garbage);
-		for(idx=0;idx<MAX_SHIELD_SECTIONS;idx++){
+		for(idx=0;idx<n_quadrants;idx++){
 			GET_FLOAT(garbage);
 		}
 
@@ -1859,8 +1864,9 @@ void process_ingame_ship_update_packet(ubyte *data, header *hinfo)
 	}
 	// otherwise read in the ship values
 	lookup->flags = flags;
+	lookup->n_quadrants = n_quadrants;
  	GET_FLOAT(lookup->hull_strength);
-	for(idx=0;idx<MAX_SHIELD_SECTIONS;idx++){
+	for(idx=0;idx<n_quadrants;idx++){
 		GET_FLOAT(f_tmp);
 		lookup->shield_quadrant[idx] = f_tmp;
 	}
