@@ -27,6 +27,7 @@
 #include "object/objectdock.h"
 #include "object/objectshield.h"
 #include "parse/scripting.h"
+#include "debugconsole/console.h"
 
 
 #define COLLIDE_DEBUG
@@ -171,17 +172,9 @@ int ship_ship_check_collision(collision_info_struct *ship_ship_hit_info, vec3d *
 		}
 	}
 
-	//	If either of these objects doesn't get collision checks, abort.
-	if (heavy_sip->flags & SIF_NO_COLLIDE) {
-		return 0;
-	}
-
-	if (light_sip->flags & SIF_NO_COLLIDE) {
-		return 0;
-	}
-
 	// Set up model_collide info
 	mc_info mc;
+	mc_info_init(&mc);
 
 	// Do in heavy object RF
 	mc.model_num = heavy_sip->model_num;	// Fill in the model to check
@@ -206,9 +199,16 @@ int ship_ship_check_collision(collision_info_struct *ship_ship_hit_info, vec3d *
 		mc.p0 = &p0_rotated;				// Point 1 of ray to check
 		vm_vec_sub(&ship_ship_hit_info->light_rel_vel, &p1, &p0_rotated);
 		vm_vec_scale(&ship_ship_hit_info->light_rel_vel, 1/flFrametime);
-	} else {
-		// should be no ships that can rotate this fast
-		Int3();
+	}
+	// should be no ships that can rotate this fast
+	else {
+#ifndef NDEBUG
+		static bool Warned_about_fast_rotational_collisions = false;
+		if (!Warned_about_fast_rotational_collisions) {
+			Warning(LOCATION, "Ship '%s' rotates too quickly!  Rotational collision detection has been skipped.", heavy_sip->name);
+			Warned_about_fast_rotational_collisions = true;
+		}
+#endif
 		ship_ship_hit_info->collide_rotate = 0;
 		mc.p0 = &p0;							// Point 1 of ray to check
 		vm_vec_sub(&ship_ship_hit_info->light_rel_vel, &light_obj->phys_info.vel, &heavy_obj->phys_info.vel);
@@ -350,7 +350,7 @@ int ship_ship_check_collision(collision_info_struct *ship_ship_hit_info, vec3d *
 	if (valid_hit_occured) {
 
 		// Collision debug stuff
-#ifdef DEBUG
+#ifndef NDEBUG
 		object *collide_obj = NULL;
 		if (heavy_obj == Player_obj) {
 			collide_obj = light_obj;
@@ -936,7 +936,7 @@ int get_ship_quadrant_from_global(vec3d *global_pos, object *objp)
 
 	vm_vec_sub(&tpos, global_pos, &objp->pos);
 	vm_vec_rotate(&rotpos, &tpos, &objp->orient);
-	return get_quadrant(&rotpos);
+	return get_quadrant(&rotpos, objp);
 }
 
 #define	MIN_REL_SPEED_FOR_LOUD_COLLISION		50		// relative speed of two colliding objects at which we play the "loud" collide sound
