@@ -42,6 +42,24 @@ obj_pair pair_used_list;
 obj_pair pair_free_list;
 
 SCP_vector<int> Collision_sort_list;
+
+class collider_pair
+{
+public:
+	object *a;
+	object *b;
+	int signature_a;
+	int signature_b;
+	int next_check_time;
+	bool initialized;
+
+	// we need to define a constructor because the hash map can
+	// implicitly insert an object when we use the [] operator
+	collider_pair()
+		: a(NULL), b(NULL), signature_a(-1), signature_b(-1), next_check_time(-1), initialized(false)
+	{}
+};
+
 SCP_hash_map<uint, collider_pair> Collision_cached_pairs;
 
 struct checkobject;
@@ -650,15 +668,16 @@ int collide_subdivide(vec3d *p0, vec3d *p1, float prad, vec3d *q0, vec3d *q1, fl
 //			is useful if a moving object wants to prevent a collision.
 int objects_will_collide(object *A, object *B, float duration, float radius_scale)
 {
-	object	A_future;
+	vec3d	prev_pos;
 	vec3d	hitpos;
+	int ret;
 
 
-	A_future = *A;
-	vm_vec_scale_add2(&A_future.pos, &A->phys_info.vel, duration);
+	prev_pos = A->pos;
+	vm_vec_scale_add2(&A->pos, &A->phys_info.vel, duration);
 
 	if (radius_scale == 0.0f) {
-		return ship_check_collision_fast(B, &A_future, &hitpos );
+		ret = ship_check_collision_fast(B, A, &hitpos);
 	} else {
 		float		size_A, size_B, dist, r;
 		vec3d	nearest_point;
@@ -668,18 +687,23 @@ int objects_will_collide(object *A, object *B, float duration, float radius_scal
 
 		//	If A is moving, check along vector.
 		if (A->phys_info.speed != 0.0f) {
-			r = find_nearest_point_on_line(&nearest_point, &A->pos, &A_future.pos, &B->pos);
+			r = find_nearest_point_on_line(&nearest_point, &prev_pos, &A->pos, &B->pos);
 			if (r < 0) {
-				nearest_point = A->pos;
+				nearest_point = prev_pos;
 			} else if (r > 1) {
-				nearest_point = A_future.pos;
+				nearest_point = A->pos;
 			}
 			dist = vm_vec_dist_quick(&B->pos, &nearest_point);
-			return (dist < size_A + size_B);
+			ret = (dist < size_A + size_B);
 		} else {
-			return vm_vec_dist_quick(&B->pos, &A->pos) < size_A + size_B;
+			ret = vm_vec_dist_quick(&B->pos, &prev_pos) < size_A + size_B;
 		}
 	}
+
+	// Reset the position to the previous value
+	A->pos = prev_pos;
+
+	return ret;
 }
 
 //	Return true if the vector from *start_pos to *end_pos is within objp->radius*radius_scale of *objp
@@ -899,6 +923,7 @@ int weapon_will_never_hit( object *obj_weapon, object *other, obj_pair * current
 int pp_collide(vec3d *curpos, vec3d *goalpos, object *goalobjp, float radius)
 {
 	mc_info mc;
+	mc_info_init(&mc);
 
 	Assert(goalobjp->type == OBJ_SHIP);
 
