@@ -26,6 +26,7 @@
 #include "object/objcollide.h"
 #include "object/object.h"
 #include "object/objectdock.h"
+#include "object/deadobjectdock.h"
 #include "object/objectshield.h"
 #include "object/objectsnd.h"
 #include "observer/observer.h"
@@ -40,6 +41,7 @@
 #include "weapon/shockwave.h"
 #include "weapon/swarm.h"
 #include "weapon/weapon.h"
+#include "debugconsole/console.h"
 
 
 
@@ -102,8 +104,45 @@ obj_flag_name Object_flag_names[] = {
 	{OF_LASER_PROTECTED,		"laser-protect-ship",		1,	},
 	{OF_MISSILE_PROTECTED,		"missile-protect-ship",		1,	},
 	{OF_IMMOBILE,				"immobile",					1,	},
+	{OF_COLLIDES,				"collides",					1,  },
 };
 
+// all we need to set are the pointers, but type, parent, and instance are useful to set as well
+object::object()
+	: next(NULL), prev(NULL), type(OBJ_NONE), parent(-1), instance(-1), n_quadrants(0), hull_strength(0.0),
+	  sim_hull_strength(0.0), net_signature(0), num_pairs(0), dock_list(NULL), dead_dock_list(NULL), collision_group_id(0)
+{
+	memset(&(this->phys_info), 0, sizeof(physics_info));
+}
+
+object::~object()
+{
+	objsnd_num.clear();
+
+	dock_free_dock_list(this);
+	dock_free_dead_dock_list(this);
+}
+
+// DO NOT set next and prev to NULL because they keep the object on the free and used lists
+void object::clear()
+{
+	signature = num_pairs = collision_group_id = 0;
+	parent = parent_sig = instance = -1;
+	type = parent_type = OBJ_NONE;
+	flags = 0;
+	pos = last_pos = vmd_zero_vector;
+	orient = last_orient = vmd_identity_matrix;
+	radius = hull_strength = sim_hull_strength = 0.0f;
+	physics_init( &phys_info );
+	shield_quadrant.clear();
+	objsnd_num.clear();
+	net_signature = 0;
+
+	// just in case nobody called obj_delete last mission
+	dock_free_dock_list(this);
+	dock_free_dead_dock_list(this);
+}
+
 /**
  * Scan the object list, freeing down to num_used objects
  *
@@ -225,7 +264,7 @@ float get_max_shield_quad(object *objp)
 		return 0.0f;
 	}
 
-	return Ships[objp->instance].ship_max_shield_strength / MAX_SHIELD_SECTIONS;
+	return Ships[objp->instance].ship_max_shield_strength / objp->n_quadrants;
 }
 
 // Goober5000
@@ -291,7 +330,8 @@ void obj_init()
 	object *objp;
 	
 	Object_inited = 1;
-	memset( Objects, 0, sizeof(object)*MAX_OBJECTS );
+	for (i = 0; i < MAX_OBJECTS; ++i)
+		Objects[i].clear();
 	Viewer_obj = NULL;
 
 	list_init( &obj_free_list );
@@ -301,16 +341,12 @@ void obj_init()
 	// Link all object slots into the free list
 	objp = Objects;
 	for (i=0; i<MAX_OBJECTS; i++)	{
-		objp->type = OBJ_NONE;
-		objp->signature = i + 100;
-		objp->collision_group_id = 0;
-		
 		list_append(&obj_free_list, objp);
 		objp++;
 	}
 
 	Object_next_signature = 1;	//0 is invalid, others start at 1
-	Num_objects = 0;			
+	Num_objects = 0;
 	Highest_object_index = 0;
 
 	if ( Cmdline_old_collision_sys ) {
@@ -336,7 +372,10 @@ int obj_allocate(void)
 	int objnum;
 	object *objp;
 
-	if (!Object_inited) obj_init();
+	if (!Object_inited) {
+		mprintf(("Why hasn't obj_init() been called yet?\n"));
+		obj_init();
+	}
 
 	if ( Num_objects >= MAX_OBJECTS-10 ) {
 		int	num_freed;
@@ -391,7 +430,10 @@ void obj_free(int objnum)
 {
 	object *objp;
 
-	if (!Object_inited) obj_init();
+	if (!Object_inited) {
+		mprintf(("Why hasn't obj_init() been called yet?\n"));
+		obj_init();
+	}
 
 	Assert( objnum >= 0 );	// Trying to free bogus object!!!
 
@@ -399,7 +441,7 @@ void obj_free(int objnum)
 	objp = &Objects[objnum];
 
 	// remove objp from the used list
-	list_remove( &obj_used_list, objp);
+	list_remove( &obj_used_list, objp );
 
 	// add objp to the end of the free
 	list_append( &obj_free_list, objp );
@@ -437,6 +479,9 @@ int obj_create(ubyte type,int parent_obj,int instance, matrix * orient,
 	obj = &Objects[objnum];
 	Assert(obj->type == OBJ_NONE);		//make sure unused 
 
+	// clear object in preparation for setting of custom values
+	obj->clear();
+
 	Assert(Object_next_signature > 0);	// 0 is bogus!
 	obj->signature = Object_next_signature++;
 
@@ -457,22 +502,14 @@ int obj_create(ubyte type,int parent_obj,int instance, matrix * orient,
 		obj->last_pos			= *pos;
 	}
 
-	obj->orient 				= orient?*orient:vmd_identity_matrix;
-	obj->last_orient			= obj->orient;
+	if (orient)	{
+		obj->orient 			= *orient;
+		obj->last_orient		= *orient;
+	}
 	obj->radius 				= radius;
 
-	obj->flags &= ~OF_INVULNERABLE;		//	Make vulnerable.
-	physics_init( &obj->phys_info );
-
-	obj->num_pairs = 0;
-	obj->net_signature = 0;			// be sure to reset this value so new objects don't take on old signatures.	
-
-	obj->collision_group_id = 0;
-
-	// Goober5000
-	obj->dock_list = NULL;
-	obj->dead_dock_list = NULL;
-
+	obj->n_quadrants = DEFAULT_SHIELD_SECTIONS; // Might be changed by the ship creation code
+	obj->shield_quadrant.resize(obj->n_quadrants);
 	return objnum;
 }
 
@@ -569,11 +606,15 @@ void obj_delete(int objnum)
 		Error( LOCATION, "Unhandled object type %d in obj_delete_all_that_should_be_dead", objp->type );
 	}
 
+	// delete any dock information we still have
+	dock_free_dock_list(objp);
+	dock_free_dead_dock_list(objp);
+
 	// if a persistant sound has been created, delete it
 	obj_snd_delete_type(OBJ_INDEX(objp));		
 
 	objp->type = OBJ_NONE;		//unused!
-	objp->signature = 0;		
+	objp->signature = 0;
 
 	obj_free(objnum);
 }
@@ -584,7 +625,10 @@ void obj_delete_all_that_should_be_dead()
 {
 	object *objp, *temp;
 
-	if (!Object_inited) obj_init();
+	if (!Object_inited) {
+		mprintf(("Why hasn't obj_init() been called yet?\n"));
+		obj_init();
+	}
 
 	// Move all objects
 	objp = GET_FIRST(&obj_used_list);
@@ -683,6 +727,8 @@ void obj_player_fire_stuff( object *objp, control_info ci )
 	shipp = NULL;
 	if((objp->type == OBJ_SHIP) && (objp->instance >= 0) && (objp->instance < MAX_SHIPS)){
 		shipp = &Ships[objp->instance];
+	} else {
+		return;
 	}
 
 	// single player pilots, and all players in multiplayer take care of firing their own primaries
@@ -1199,12 +1245,12 @@ void obj_move_all_post(object *objp, float frametime)
 
 			//Check for changing team colors
 			ship* shipp = &Ships[objp->instance];
-			if (Ship_info[shipp->ship_info_index].uses_team_colors && shipp->secondary_team_name != "<none>") {
+			if (Ship_info[shipp->ship_info_index].uses_team_colors && stricmp(shipp->secondary_team_name.c_str(), "none")) {
 				if (f2fl(Missiontime) * 1000 > f2fl(shipp->team_change_timestamp) * 1000 + shipp->team_change_time) {
 					shipp->team_name = shipp->secondary_team_name;
 					shipp->team_change_timestamp = 0;
 					shipp->team_change_time = 0;
-					shipp->secondary_team_name = "<none>";
+					shipp->secondary_team_name = "none";
 				}
 			}
 
@@ -1369,7 +1415,7 @@ void obj_move_all(float frametime)
 #endif
 
 		// pre-move
-		obj_move_all_pre(objp, frametime);
+		PROFILE("Pre Move", obj_move_all_pre(objp, frametime));
 
 		// store last pos and orient
 		objp->last_pos = cur_pos;
@@ -1382,12 +1428,12 @@ void obj_move_all(float frametime)
 				multi_oo_interp(objp);
 			} else {
 				// physics
-				obj_move_call_physics(objp, frametime);
+				PROFILE("Physics", obj_move_call_physics(objp, frametime));
 			}
 		}
 
 		// move post
-		obj_move_all_post(objp, frametime);
+		PROFILE("Post Move", obj_move_all_post(objp, frametime));
 
 		// Equipment script processing
 		if (objp->type == OBJ_SHIP) {
@@ -1453,6 +1499,7 @@ void obj_move_all(float frametime)
 	// do pre-collision stuff for beam weapons
 	beam_move_all_pre();
 
+	profile_begin("Collision Detection");
 	if ( Collisions_enabled ) {
 		if ( Cmdline_old_collision_sys ) {
 			obj_check_all_collisions();
@@ -1460,6 +1507,7 @@ void obj_move_all(float frametime)
 			obj_sort_and_collide();
 		}
 	}
+	profile_end("Collision Detection");
 
 	turret_swarm_check_validity();
 
@@ -1872,7 +1920,7 @@ void obj_reset_all_collisions()
 
 		// next
 		moveup = GET_NEXT(moveup);
-	}		
+	}
 }
 
 // Goober5000
@@ -1933,6 +1981,8 @@ bool object_glide_forced(object *objp)
  */
 int obj_get_by_signature(int sig)
 {
+	Assert(sig > 0);
+
 	object *objp = GET_FIRST(&obj_used_list);
 	while( objp !=END_OF_LIST(&obj_used_list) )
 	{
