@@ -16,13 +16,16 @@
 #include "nebula/neb.h"
 #include "ship/ship.h"
 
-#include <list>
+#include <vector>
+#include <algorithm>
 #include "weapon/weapon.h"
 #include "debris/debris.h"
 #include "asteroid/asteroid.h"
 
 
-typedef struct sorted_obj {
+class sorted_obj
+{
+public:
 	object			*obj;					// a pointer to the original object
 	float			z, min_z, max_z;		// The object's z values relative to viewer
 
@@ -32,7 +35,7 @@ typedef struct sorted_obj {
 	}
 
 	bool operator < (const sorted_obj &other) const;
-} sorted_obj;
+};
 
 inline bool sorted_obj::operator < (const sorted_obj &other) const
 {
@@ -99,7 +102,7 @@ inline bool sorted_obj::operator < (const sorted_obj &other) const
 }
 
 
-SCP_list<sorted_obj> Sorted_objects;
+SCP_vector<sorted_obj> Sorted_objects;
 SCP_vector<object*> effect_ships; 
 
 // Used to (fairly) quicky find the 8 extreme
@@ -220,7 +223,7 @@ void obj_render_all(void (*render_function)(object *objp), bool *draw_viewer_las
 	if ( Sorted_objects.empty() )
 		return;
 
-	Sorted_objects.sort();
+	std::sort(Sorted_objects.begin(), Sorted_objects.end());
 
 #ifdef DYN_CLIP_DIST
 	if (!Cmdline_nohtl)
@@ -239,9 +242,12 @@ void obj_render_all(void (*render_function)(object *objp), bool *draw_viewer_las
 
 	Interp_no_flush = 1;
 
+	bool full_neb = ((The_mission.flags & MISSION_FLAG_FULLNEB) && (Neb2_render_mode != NEB2_RENDER_NONE) && !Fred_running);
+	bool c_viewer = (!Viewer_mode || (Viewer_mode & VM_PADLOCK_ANY) || (Viewer_mode & VM_OTHER_SHIP) || (Viewer_mode & VM_TRACK));
+
 	// now draw them
 	// only render models in this loop in order to minimize state changes
-	SCP_list<sorted_obj>::iterator os;
+	SCP_vector<sorted_obj>::iterator os;
 	for (os = Sorted_objects.begin(); os != Sorted_objects.end(); ++os) {
 		object *obj = os->obj;
 
@@ -250,15 +256,15 @@ void obj_render_all(void (*render_function)(object *objp), bool *draw_viewer_las
 		//This is for ship cockpits. Bobb, feel free to optimize this any way you see fit
 		if ( (obj == Viewer_obj)
 			&& (obj->type == OBJ_SHIP)
-			&& (Ship_info[Ships[obj->instance].ship_info_index].flags2 & SIF2_SHOW_SHIP_MODEL)
-			&& (!Viewer_mode || (Viewer_mode & VM_PADLOCK_ANY) || (Viewer_mode & VM_OTHER_SHIP) || (Viewer_mode & VM_TRACK)) )
+			&& c_viewer
+			&& (Ship_info[Ships[obj->instance].ship_info_index].flags2 & SIF2_SHOW_SHIP_MODEL) )
 		{
 			(*draw_viewer_last) = true;
 			continue;
 		}
 
 		// if we're fullneb, fire up the fog - this also generates a fog table
-		if((The_mission.flags & MISSION_FLAG_FULLNEB) && (Neb2_render_mode != NEB2_RENDER_NONE) && !Fred_running){
+		if (full_neb) {
 			// get the fog values
 			neb2_get_adjusted_fog_values(&fog_near, &fog_far, obj);
 
@@ -304,7 +310,7 @@ void obj_render_all(void (*render_function)(object *objp), bool *draw_viewer_las
 
 			// only reset fog if the fog mode has changed - since regenerating a fog table takes
 			// a bit of time
-			if((fog_near != gr_screen.fog_near) || (fog_far != gr_screen.fog_far)){
+			if((GR_FOGMODE_FOG != gr_screen.current_fog_mode) || (fog_near != gr_screen.fog_near) || (fog_far != gr_screen.fog_far)) {
 				gr_fog_set(GR_FOGMODE_FOG, gr_screen.current_fog_color.red, gr_screen.current_fog_color.green, gr_screen.current_fog_color.blue, fog_near, fog_far);
 			}
 
