@@ -12,6 +12,7 @@
 #include "stats/medals.h"
 #include "cfile/cfilesystem.h"
 #include "menuui/techmenu.h"
+#include "cutscene/cutscenes.h"
 
 #include <iostream>
 #include <sstream>
@@ -189,7 +190,7 @@ void pilotfile_convert::csg_import_ships_weapons()
 
 	// get last ship flown index
 	for (idx = 0; idx < ship_count; idx++) {
-		if ( csg->ship_list[idx].name.compare(plr->last_ship_flown) ) {
+		if ( csg->ship_list[idx].name.compare(plr->last_ship_flown) == 0 ) {
 			csg->last_ship_flown_index = idx;
 			break;
 		}
@@ -202,7 +203,7 @@ void pilotfile_convert::csg_import_ships_weapons()
 	}
 
 	// create list of medals (since it's missing from the old files)
-	list_size = (int)Medals.size();
+	list_size = Num_medals;
 
 	for (idx = 0; idx < list_size; idx++) {
 		ilist.name = Medals[idx].name;
@@ -439,11 +440,14 @@ void pilotfile_convert::csg_import_red_alert()
 		for (j = 0; j < 3; j++) {
 			i = cfread_int(cfp);
 
-			if (i >= weapon_list_size || i < 0) {
+			if (i >= weapon_list_size || i < -1) {
 				throw std::runtime_error("Data check failure (RedAlert-weapon)!");
+			} else if (i >= 0) {
+				weapons.index = csg->weapon_list[i].index;
+			} else {
+				weapons.index = -1;
 			}
 
-			weapons.index = csg->weapon_list[i].index;
 			weapons.count = cfread_int(cfp);
 
 			if (weapons.index >= 0) {
@@ -456,11 +460,14 @@ void pilotfile_convert::csg_import_red_alert()
 		for (j = 0; j < 4; j++) {
 			i = cfread_int(cfp);
 
-			if (i >= weapon_list_size) {
+			if (i >= weapon_list_size || i < -1) {
 				throw std::runtime_error("Data check failure (RedAlert-weapon)!");
+			} else if (i >= 0) {
+				weapons.index = csg->weapon_list[i].index;
+			} else {
+				weapons.index = -1;
 			}
 
-			weapons.index = csg->weapon_list[i].index;
 			weapons.count = cfread_int(cfp);
 
 			if (weapons.index >= 0) {
@@ -686,6 +693,9 @@ void pilotfile_convert::csg_export_flags()
 	// tips
 	cfwrite_ubyte((ubyte)plr->tips, cfp);
 
+	// special rank
+	cfwrite_int(csg->stats.rank, cfp);
+
 	endSection();
 }
 
@@ -1093,6 +1103,34 @@ void pilotfile_convert::csg_export_variables()
 	endSection();
 }
 
+void pilotfile_convert::csg_export_cutscenes() {
+	SCP_vector<cutscene_info>::iterator cut;
+
+	startSection(Section::Cutscenes);
+
+	// convert the old int bitfield to the new vector
+	// the 32 is the size-in-bits of the old int on all platforms
+	// supported by FSO prior to 3.7.0
+	size_t size = Cutscenes.size();
+	size_t viewableScenes = 0;
+	for (size_t j=0; j<size && j<32; ++j) {
+		if ( csg->cutscenes & (1<<j) ) {
+			Cutscenes.at(j).viewable = true;
+			viewableScenes++;
+		}
+	}
+
+	// output cutscene data in new format
+	cfwrite_uint(viewableScenes, cfp);
+
+	for(cut = Cutscenes.begin(); cut != Cutscenes.end(); ++cut) {
+		if(cut->viewable)
+			cfwrite_string_len(cut->filename, cfp);
+	}
+
+	endSection();
+}
+
 void pilotfile_convert::csg_export()
 {
 	Assert( cfp != NULL );
@@ -1113,7 +1151,7 @@ void pilotfile_convert::csg_export()
 	csg_export_redalert();
 	csg_export_hud();
 	csg_export_variables();
-
+	csg_export_cutscenes();
 
 	// and... we're done! :)
 }
