@@ -33,7 +33,7 @@
 
 #include <string>
 
-struct object;
+class object;
 class WarpEffect;
 
 //	Part of the player died system.
@@ -305,8 +305,10 @@ typedef struct cockpit_display_info {
 // structure definition for a linked list of subsystems for a ship.  Each subsystem has a pointer
 // to the static data for the subsystem.  The obj_subsystem data is defined and read in the model
 // code.  Other dynamic data (such as current_hits) should remain in this structure.
-typedef	struct ship_subsys {
-	struct ship_subsys *next, *prev;				//	Index of next and previous objects in list.
+class ship_subsys
+{
+public:
+	class ship_subsys *next, *prev;				//	Index of next and previous objects in list.
 	model_subsystem *system_info;					// pointer to static data for this subsystem -- see model.h for definition
 
 	int			parent_objnum;						// objnum of the parent ship
@@ -369,7 +371,7 @@ typedef	struct ship_subsys {
 	int subsys_cargo_name;			// cap ship cargo on subsys
 	fix time_subsys_cargo_revealed;	// added by Goober5000
 
-	triggered_rotation trigger;		//the actual currently running animation and assosiated states
+	int triggered_rotation_index;		//the actual currently running animation and assosiated states
 
 	float points_to_target;
 	float base_rotation_rate_pct;
@@ -397,8 +399,14 @@ typedef	struct ship_subsys {
 
 	//Per-turret ownage settings - SUSHI
 	int turret_max_bomb_ownage; 
-	int turret_max_target_ownage; 
-} ship_subsys;
+	int turret_max_target_ownage;
+
+	ship_subsys()
+		: next(NULL), prev(NULL)
+	{}
+
+	void clear();
+};
 
 // structure for subsystems which tells us the total count of a particular type of subsystem (i.e.
 // we might have 3 engines), and the relative strength of the subsystem.  The #defines in model.h
@@ -416,7 +424,7 @@ typedef struct ship_flag_name {
 	int flag_list;						// is this flag in the 1st or 2nd ship flags list?
 } ship_flag_name;
 
-#define MAX_SHIP_FLAG_NAMES					9
+#define MAX_SHIP_FLAG_NAMES					17
 extern ship_flag_name Ship_flag_names[];
 
 // states for the flags variable within the ship structure
@@ -496,6 +504,7 @@ extern ship_flag_name Ship_flag_names[];
 #define SF2_SHIP_LOCKED						(1<<24)		// Karajorma - Prevents the player from changing the ship class on loadout screen
 #define SF2_WEAPONS_LOCKED					(1<<25)		// Karajorma - Prevents the player from changing the weapons on the ship on the loadout screen
 #define SF2_SHIP_SELECTIVE_LINKING			(1<<26)		// RSAXVC - Allow pilot to pick firing configuration
+#define SF2_SCRAMBLE_MESSAGES				(1<<27)		// Goober5000 - all messages sent from this ship appear scrambled
 
 // If any of these bits in the ship->flags are set, ignore this ship when targeting
 extern int TARGET_SHIP_IGNORE_FLAGS;
@@ -517,7 +526,10 @@ typedef struct ship_spark {
 #define AWACS_WARN_25		(1 << 1)
 #define AWACS_WARN_75		(1 << 2)
 
-typedef struct ship {
+// NOTE: Can't be treated as a struct anymore, since it has STL data structures in its object tree!
+class ship
+{
+public:
 	int	objnum;
 	int	ai_index;			// Index in Ai_info of ai_info associated with this ship.
 	int	ship_info_index;	// Index in ship_info for this ship
@@ -542,15 +554,13 @@ typedef struct ship {
 
 	// targeting laser info
 	char targeting_laser_bank;						// -1 if not firing, index into polymodel gun points if it _is_ firing
+	int targeting_laser_objnum;					// -1 if invalid, beam object # otherwise
+
 	// corkscrew missile stuff
 	ubyte num_corkscrew_to_fire;						// # of corkscrew missiles lef to fire
 	int corkscrew_missile_bank;
-	// END PACK
-
-	// targeting laser info
-	int targeting_laser_objnum;					// -1 if invalid, beam object # otherwise
-	// corkscrew missile stuff
 	int next_corkscrew_fire;						// next time to fire a corkscrew missile
+	// END PACK
 
 	int	final_death_time;				// Time until big fireball starts
 	int	death_time;				// Time until big fireball starts
@@ -643,6 +653,8 @@ typedef struct ship {
 
 	int	shield_hits;						//	Number of hits on shield this frame.
 
+	SCP_vector<vec3d>	shield_points;
+
 	float		wash_intensity;
 	vec3d	wash_rot_axis;
 	int		wash_timestamp;
@@ -796,7 +808,11 @@ typedef struct ship {
 	fix team_change_timestamp;
 	int team_change_time;
 
-} ship;
+	float autoaim_fov;
+
+	// reset to a completely blank ship
+	void clear();
+};
 
 struct ai_target_priority {
 	char name[NAME_LENGTH];
@@ -840,17 +856,6 @@ typedef struct exited_ship {
 	char	cargo1;
 	float damage_ship[MAX_DAMAGE_SLOTS];		// A copy of the arrays from the ship so that we can figure out what damaged it
 	int   damage_ship_id[MAX_DAMAGE_SLOTS];
-
-	exited_ship()
-		: team( 0 ), flags( 0 ), time( 0 ), hull_strength( 0 ),
-		  time_cargo_revealed( 0 ), cargo1( 0 )
-	{ 
-		ship_name[ 0 ] = '\0';
-		obj_signature = -1;
-		ship_class = -1; 
-		memset( damage_ship, 0, sizeof( damage_ship ) );
-		memset( damage_ship_id, 0, sizeof( damage_ship_id ) );
-	}
 } exited_ship;
 
 extern SCP_vector<exited_ship> Ships_exited;
@@ -902,7 +907,8 @@ extern int ship_find_exited_ship_by_signature( int signature);
 
 #define	SIF_NO_FRED					(1 << 31)	// not available in fred
 
-// flags2 list. If this is updated MAX_SHIP_FLAGS must also be updated!
+
+// flags2 list.
 #define SIF2_DEFAULT_IN_TECH_DATABASE		(1 << 0)	// default in tech database - Goober5000
 #define SIF2_DEFAULT_IN_TECH_DATABASE_M		(1 << 1)	// ditto - Goober5000
 #define SIF2_FLASH							(1 << 2)	// makes a flash when it explodes
@@ -919,8 +925,10 @@ extern int ship_find_exited_ship_by_signature( int signature);
 #define SIF2_NO_ETS							(1 << 13)	// The E - No ETS on this ship class
 #define SIF2_NO_LIGHTING					(1 << 14)	// Valathil - No lighting for this ship
 #define SIF2_DYN_PRIMARY_LINKING			(1 << 15)	// RSAXVC - Dynamically generate weapon linking options
-// !!! IF YOU ADD A FLAG HERE BUMP MAX_SHIP_FLAGS !!!
-#define	MAX_SHIP_FLAGS	16		//	Number of distinct flags for flags field in ship_info struct
+#define SIF2_AUTO_SPREAD_SHIELDS			(1 << 16)	// zookeeper - auto spread shields
+#define SIF2_DRAW_WEAPON_MODELS				(1 << 17)	// the ship draws weapon models of any sort (used to be a boolean)
+#define SIF2_MODEL_POINT_SHIELDS			(1 << 18)	// zookeeper - uses model-defined shield points instead of quadrants
+
 #define	SIF_DEFAULT_VALUE		0
 #define SIF2_DEFAULT_VALUE		0
 
@@ -938,7 +946,7 @@ extern int ship_find_exited_ship_by_signature( int signature);
 
 // masks for preventing only non flag entry SIF flags from being cleared
 #define SIF_MASK				SIF_AFTERBURNER
-#define SIF2_MASK				0
+#define SIF2_MASK				SIF2_DRAW_WEAPON_MODELS
 
 #define REGULAR_WEAPON	(1<<0)
 #define DOGFIGHT_WEAPON (1<<1)
@@ -1085,16 +1093,6 @@ typedef struct man_thruster {
 	float radius;
 
 	vec3d pos, norm;
-	man_thruster()
-		: use_flags( 0 ), tex_nframes( 0 ), tex_fps( 0 ), length( 0. ), radius( 0. )
-	{
-		tex_id=-1;
-		start_snd=-1;
-		loop_snd=-1;
-		stop_snd=-1;
-		memset( &pos, 0, sizeof( vec3d ) );
-		memset( &norm, 0, sizeof( vec3d ) );
-	}
 } man_thruster;
 
 //Warp type defines
@@ -1149,7 +1147,10 @@ typedef struct path_metadata {
 } path_metadata;
 
 // The real FreeSpace ship_info struct.
-typedef struct ship_info {
+// NOTE: Can't be treated as a struct anymore, since it has STL data structures in its object tree!
+class ship_info
+{
+public:
 	char		name[NAME_LENGTH];				// name for the ship
 	char		alt_name[NAME_LENGTH];			// display another name for the ship
 	char		short_name[NAME_LENGTH];		// short name, for use in the editor?
@@ -1183,13 +1184,11 @@ typedef struct ship_info {
 	float		rotdamp;								// rotational drag
 	float		delta_bank_const;
 	vec3d	max_vel;								//	max velocity of the ship in the linear directions -- read from ships.tbl
-	vec3d	afterburner_max_vel;				//	max velocity of the ship in the linear directions when afterburners are engaged -- read from ships.tbl
 	vec3d	max_rotvel;							// maximum rotational velocity
 	vec3d	rotation_time;						// time to rotate in x/y/z dimension traveling at max rotvel
 	float		srotation_time;					//	scalar, computed at runtime as (rotation_time.x + rotation_time.y)/2
 	float		max_rear_vel;						// max speed ship can go backwards.
 	float		forward_accel;
-	float		afterburner_forward_accel;		// forward acceleration with afterburner engaged
 	float		forward_decel;
 	float		slide_accel;
 	float		slide_decel;
@@ -1269,6 +1268,8 @@ typedef struct ship_info {
 	float		max_weapon_regen_per_second;	// Goober5000 - max percent/100 of weapon energy regenerated per second
 
 	// Afterburner fields
+	vec3d		afterburner_max_vel;				//	max velocity of the ship in the linear directions when afterburners are engaged -- read from ships.tbl
+	float		afterburner_forward_accel;		// forward acceleration with afterburner engaged
 	float		afterburner_fuel_capacity;		// maximum afterburner fuel that can be stored
 	float		afterburner_burn_rate;			// rate in fuel/second that afterburner consumes fuel
 	float		afterburner_recover_rate;		//	rate in fuel/second that afterburner recovers fuel
@@ -1280,17 +1281,25 @@ typedef struct ship_info {
 	int		cmeasure_max;						//	Number of charges of countermeasures this ship can hold.
 
 	int num_primary_banks;										// Actual number of primary banks (property of model)
-	int num_secondary_banks;									//	Actual number of secondary banks (property of model)
-	int primary_bank_weapons[MAX_SHIP_PRIMARY_BANKS];			// Weapon_info[] index for the weapon in the bank
-	
+	int primary_bank_weapons[MAX_SHIP_PRIMARY_BANKS];			// Weapon_info[] index for the weapon in the bank	
 	// Goober5000's ballistic conversion
 	int primary_bank_ammo_capacity[MAX_SHIP_PRIMARY_BANKS];	// Capacity of primary ballistic bank
-	
+
+	int num_secondary_banks;									//	Actual number of secondary banks (property of model)
 	int secondary_bank_weapons[MAX_SHIP_SECONDARY_BANKS];	// Weapon_info[] index for the weapon in the bank
 	int secondary_bank_ammo_capacity[MAX_SHIP_SECONDARY_BANKS];	// Capacity of bank (not number of missiles)
 
+	bool draw_primary_models[MAX_SHIP_PRIMARY_BANKS];
+	bool draw_secondary_models[MAX_SHIP_SECONDARY_BANKS];
+	float weapon_model_draw_distance;
+
 	float	max_hull_strength;				// Max hull strength of this class of ship.
 	float	max_shield_strength;
+	float	auto_shield_spread;
+	bool	auto_shield_spread_bypass;
+	int		auto_shield_spread_from_lod;
+
+	int		shield_point_augment_ctrls[4];	// Re-mapping of shield augmentation controls for model point shields
 
 	float	hull_repair_rate;				//How much of the hull is repaired every second
 	float	subsys_repair_rate;		//How fast 
@@ -1315,6 +1324,11 @@ typedef struct ship_info {
 	char	overhead_filename[MAX_FILENAME_LEN];	// filename for animation that plays weapons loadout
 	int 	selection_effect;
 
+	int bii_index_ship;						// if this ship has a briefing icon that overrides the normal icon set
+	int bii_index_ship_with_cargo;
+	int bii_index_wing;
+	int bii_index_wing_with_cargo;
+
 	int	score;								// default score for this ship
 
 	int	scan_time;							// time to scan this ship (in ms)
@@ -1355,8 +1369,8 @@ typedef struct ship_info {
 	float		thruster01_glow_rad_factor;
 	float		thruster02_glow_rad_factor;
 	float		thruster03_glow_rad_factor;
-	float		thruster_dist_rad_factor;
 	float		thruster02_glow_len_factor;
+	float		thruster_dist_rad_factor;
 	float		thruster_dist_len_factor;
 
 	bool		draw_distortion;
@@ -1364,10 +1378,9 @@ typedef struct ship_info {
 	int splodeing_texture;
 	char splodeing_texture_name[MAX_FILENAME_LEN];
 
-	bool draw_primary_models[MAX_SHIP_PRIMARY_BANKS];
-	bool draw_secondary_models[MAX_SHIP_SECONDARY_BANKS];
-	bool draw_models; //any weapon mode will be drawn
-	float weapon_model_draw_distance;
+	// Goober5000
+	SCP_vector<texture_replace> replacement_textures;
+
 	
 	int armor_type_idx;
 	int shield_armor_type_idx;
@@ -1418,7 +1431,7 @@ typedef struct ship_info {
 	SCP_vector<cockpit_display_info> displays;
 
 	SCP_map<SCP_string, path_metadata> pathMetadata;
-} ship_info;
+};
 
 extern int Num_wings;
 extern ship Ships[MAX_SHIPS];
@@ -1440,7 +1453,6 @@ typedef struct engine_wash_info
 	float		length;			// length of engine wash, measured from thruster
 	float		intensity;		// intensity of engine wash
 	
-	engine_wash_info();
 } engine_wash_info;
 
 extern SCP_vector<engine_wash_info> Engine_wash_info;
@@ -1476,6 +1488,7 @@ typedef struct wing {
 
 	int	wave_count;								// max ships per wave (as defined by the number of ships in the ships list)
 	int	total_arrived_count;					// count of number of ships that we have created, regardless of wave
+	int red_alert_skipped_ships;				// Goober5000 - if we skipped over any indexes while creating red-alert ships
 	int	current_count;							// count of number of ships actually in this wing -- used for limit in next array
 	int	ship_index[MAX_SHIPS_PER_WING];	// index into ships array of all ships currently in the wing
 
@@ -1607,16 +1620,14 @@ extern void physics_ship_init(object *objp);
 //	Stuff vector *pos with absolute position.
 extern int get_subsystem_pos(vec3d *pos, object *objp, ship_subsys *subsysp);
 
-//Template stuff, here's as good a place as any.
-int parse_ship_values(ship_info* sip, bool isTemplate, bool first_time, bool replace);
-extern int ship_template_lookup(char *name = NULL);
+int parse_ship_values(ship_info* sip, bool first_time, bool replace);
 void parse_ship_particle_effect(ship_info* sip, particle_effect* pe, char *id_string);
 
-extern int ship_info_lookup(char *name = NULL);
-extern int ship_name_lookup(char *name, int inc_players = 0);	// returns the index into Ship array of name
-extern int ship_type_name_lookup(char *name);
+extern int ship_info_lookup(const char *name = NULL);
+extern int ship_name_lookup(const char *name, int inc_players = 0);	// returns the index into Ship array of name
+extern int ship_type_name_lookup(const char *name);
 
-extern int wing_lookup(char *name);
+extern int wing_lookup(const char *name);
 
 // returns 0 if no conflict, 1 if conflict, -1 on some kind of error with wing struct
 extern int wing_has_conflicting_teams(int wing_index);
@@ -1624,7 +1635,10 @@ extern int wing_has_conflicting_teams(int wing_index);
 // next function takes optional second parameter which says to ignore the current count of ships
 // in the wing -- used to tell is the wing exists or not, not whether it exists and has ships currently
 // present.
-extern int wing_name_lookup(char *name, int ignore_count = 0);
+extern int wing_name_lookup(const char *name, int ignore_count = 0);
+
+// for generating a ship name for arbitrary waves/indexes of that wing... correctly handles the # character
+extern void wing_bash_ship_name(char *ship_name, const char *wing_name, int index);
 
 extern int Player_ship_class;
 
@@ -1695,7 +1709,7 @@ extern int	ship_do_rearm_frame( object *objp, float frametime );
 extern float ship_calculate_rearm_duration( object *objp );
 extern void	ship_wing_cleanup( int shipnum, wing *wingp );
 
-extern object *ship_find_repair_ship( object *requester );
+extern int ship_find_repair_ship( object *requester_obj, object **ship_we_found = NULL );
 extern void ship_close();	// called in game_shutdown() to free malloced memory
 
 
@@ -1712,7 +1726,7 @@ extern int ship_query_general_type(int ship);
 extern int ship_class_query_general_type(int ship_class);
 extern int ship_query_general_type(ship *shipp);
 extern int ship_docking_valid(int docker, int dockee);
-extern int get_quadrant(vec3d *hit_pnt);						//	Return quadrant num of last hit ponit.
+extern int get_quadrant(vec3d *hit_pnt, object *shipobjp = NULL);	//	Return quadrant num of given hit point.
 
 extern void ship_obj_list_rebuild();	// only called by save/restore code
 extern int ship_query_state(char *name);
@@ -1795,7 +1809,7 @@ int get_max_ammo_count_for_primary_bank(int ship_class, int bank, int ammo_type)
 
 int get_max_ammo_count_for_bank(int ship_class, int bank, int ammo_type);
 
-int is_support_allowed(object *objp);
+int is_support_allowed(object *objp, bool do_simple_check = false);
 
 // Given an object and a turret on that object, return the actual firing point of the gun
 // and its normal.   This uses the current turret angles.  We are keeping track of which
@@ -1881,9 +1895,6 @@ float ship_get_warpout_speed(object *objp);
 // returns true if ship is beginning to speed up in warpout
 int ship_is_beginning_warpout_speedup(object *objp);
 
-// given a ship info type, return a species
-int ship_get_species_by_type(int ship_info_index);
-
 // return the length of the ship class
 float ship_class_get_length(ship_info *sip);
 
@@ -1895,19 +1906,18 @@ extern void ship_subsystem_set_new_ai_class(int ship_num, char *subsystem, int n
 extern void wing_load_squad_bitmap(wing *w);
 
 // Goober5000 - needed by new hangar depart code
-extern int ship_has_dock_bay(int shipnum);
-
-// Goober5000 - needed by new hangar depart code
-extern int ship_get_ship_with_dock_bay(int team);
+extern bool ship_has_dock_bay(int shipnum);
+extern bool ship_useful_for_departure(int shipnum, int path_mask = 0);
+extern int ship_get_ship_for_departure(int team);
 
 // Goober5000 - moved here from hudbrackets.cpp
-extern int ship_subsys_is_fighterbay(ship_subsys *ss);
+extern bool ship_subsys_is_fighterbay(ship_subsys *ss);
 
 // Goober5000
-extern int ship_fighterbays_all_destroyed(ship *shipp);
+extern bool ship_fighterbays_all_destroyed(ship *shipp);
 
 // Goober5000
-extern int ship_subsys_takes_damage(ship_subsys *ss);
+extern bool ship_subsys_takes_damage(ship_subsys *ss);
 
 //phreak
 extern int ship_fire_tertiary(object *objp);
@@ -1931,9 +1941,9 @@ void ship_end_render_cockpit_display(int cockpit_display_num);
 int warptype_match(char *p);
 
 // Goober5000
-int ship_starting_wing_lookup(char *wing_name);
-int ship_squadron_wing_lookup(char *wing_name);
-int ship_tvt_wing_lookup(char *wing_name);
+int ship_starting_wing_lookup(const char *wing_name);
+int ship_squadron_wing_lookup(const char *wing_name);
+int ship_tvt_wing_lookup(const char *wing_name);
 
 // Goober5000
 int ship_class_compare(int ship_class_1, int ship_class_2);
