@@ -15,51 +15,40 @@
 #include "globalincs/globals.h"
 #include "globalincs/pstypes.h"
 
-struct scoring_struct;
-struct player;
+class scoring_struct;
+class player;
 
 #define MAX_BADGES				3
-#define MAX_ASSIGNABLE_MEDALS	12		// index into Medals array of the first medal which cannot be assigned
+extern int Rank_medal_index;
 
 extern scoring_struct *Player_score;
 
 // NUM_MEDALS stored in scoring.h since needed for player scoring structure
 
-typedef struct medal_stuff {
+class medal_stuff
+{
+public:
 	char	name[NAME_LENGTH];
 	char	bitmap[MAX_FILENAME_LEN];
+	char	debrief_bitmap[MAX_FILENAME_LEN];
 	int	num_versions;
+	bool version_starts_at_1;
+	bool available_from_start;
 	int	kills_needed;
-	int badge_num;
 
-	//If this is a badge (kills_needed > 1)
+	//If this is a badge (kills_needed > 0)
 	char voice_base[MAX_FILENAME_LEN];
-	char *promotion_text;
+	SCP_map<int, char*> promotion_text;
 
-	medal_stuff() {
-		name[0] = '\0';
-		bitmap[0] = '\0';
-		num_versions = 1;
-		kills_needed = 0;
-		badge_num = 0;
-		voice_base[0] = '\0';
-		promotion_text = NULL;
-	}
+	medal_stuff();
+	~medal_stuff();
 
-	~medal_stuff() {
-		if (promotion_text) {
-			vm_free(promotion_text);
-			promotion_text = NULL;
-		}
-	}
-
-	medal_stuff(const medal_stuff &m) { clone(m); }
+	medal_stuff(const medal_stuff &m);
 	const medal_stuff &operator=(const medal_stuff &m);
 
 private:
 	void clone(const medal_stuff &m);
-
-} medal_stuff;
+};
 
 extern SCP_vector<medal_stuff> Medals;
 
@@ -70,21 +59,12 @@ extern void parse_medal_tbl();
 #define MM_POPUP				1		// called from within some other tight loop (don't use gameseq_ functions)
 
 // main medals screen
-void medal_main_init(player *pl,int mode = MM_NORMAL);
+void medal_main_init(player *pl, int mode = MM_NORMAL);
 
 // return 0 if the screen should close (used for MM_POPUP mode)
 int medal_main_do();
 void medal_main_close();
 
-void init_medal_bitmaps();
-void init_snazzy_regions();
-void blit_medals();
-void blit_label(char *label,int *coords);
-void blit_callsign();
-
-// individual medals
-void medals_translate_name(char *name, int max_len);
-
 int medals_info_lookup(const char *name);
 
 #endif
