@@ -31,6 +31,7 @@
 #include "network/multi_pmsg.h"
 #include "ai/ai_profiles.h"
 #include "pilotfile/pilotfile.h"
+#include "debugconsole/console.h"
 
 /*
 // uncomment to get extra debug messages when a player scores
@@ -58,14 +59,10 @@ void parse_rank_tbl()
 {
 	atexit(scoreing_close);
 	char buf[MULTITEXT_LENGTH];
-	int rval, idx;
-
-	// open localization
-	lcl_ext_open();
+	int rval, idx, persona;
 
 	if ((rval = setjmp(parse_abort)) != 0) {
 		mprintf(("TABLES: Unable to parse '%s'!  Error code = %i.\n", "rank.tbl", rval));
-		lcl_ext_close();
 		return;
 	} 
 
@@ -86,11 +83,25 @@ void parse_rank_tbl()
 		stuff_string( Ranks[idx].bitmap, F_NAME, MAX_FILENAME_LEN );
 		required_string("$Promotion Voice Base:");
 		stuff_string( Ranks[idx].promotion_voice_base, F_NAME, MAX_FILENAME_LEN );
-		required_string("$Promotion Text:");
-		stuff_string(buf, F_MULTITEXT, sizeof(buf));
-		drop_white_space(buf);
-		compact_multitext_string(buf);
-		Ranks[idx].promotion_text = vm_strdup(buf);
+		while (check_for_string("$Promotion Text:")) {
+			required_string("$Promotion Text:");
+			stuff_string(buf, F_MULTITEXT, sizeof(buf));
+			drop_white_space(buf);
+			compact_multitext_string(buf);
+			persona = -1;
+			if (optional_string("+Persona:")) {
+				stuff_int(&persona);
+				if (persona < 0) {
+					Warning(LOCATION, "Debriefing text for %s rank is assigned to an invalid persona: %i (must be 0 or greater).\n", Ranks[idx].name, persona);
+					continue;
+				}
+			}
+			Ranks[idx].promotion_text[persona] = vm_strdup(buf);
+		}
+		if (Ranks[idx].promotion_text.find(-1) == Ranks[idx].promotion_text.end()) {
+			Warning(LOCATION, "%s rank is missing default debriefing text.\n", Ranks[idx].name);
+			Ranks[idx].promotion_text[-1] = "";
+		}
 		idx++;
 	}
 
@@ -103,64 +114,117 @@ void parse_rank_tbl()
 			Int3();
 	}
 #endif
-
-	// close localization
-	lcl_ext_close();
 }
 
 // initialize a nice blank scoring element
-void init_scoring_element(scoring_struct *s)
+void scoring_struct::init()
 {
-	if (s == NULL) {
-		Int3();	//	DaveB -- Fix this!
-		// read_pilot_file(char* callsign);
-		return;
-	}
+	flags = 0;
+	score = 0;
+	rank = RANK_ENSIGN;
 
-	memset(s, 0, sizeof(scoring_struct));
+	medal_counts.assign(Num_medals, 0);
 
-	s->rank = RANK_ENSIGN;
+	memset(kills, 0, MAX_SHIP_CLASSES * sizeof(int));
+	assists = 0;
+	kill_count = 0;
+	kill_count_ok = 0;
+	p_shots_fired = 0;
+	s_shots_fired = 0;
 
-	s->m_medal_earned = -1;		// hasn't earned a medal yet
-	s->m_promotion_earned = -1;
-	s->m_badge_earned = -1;
+	p_shots_hit = 0;
+	s_shots_hit = 0;
 
-	s->flags = 0;
+	p_bonehead_hits = 0;
+	s_bonehead_hits = 0;
+	bonehead_kills = 0;
+
+	missions_flown = 0;
+	flight_time = 0;
+	last_flown = 0;
+	last_backup = 0;
+
+	m_medal_earned = -1;		// hasn't earned a medal yet
+	m_promotion_earned = -1;
+	m_badge_earned.clear();
+
+	m_score = 0;
+	memset(m_kills, 0, MAX_SHIP_CLASSES * sizeof(int));
+	memset(m_okKills, 0, MAX_SHIP_CLASSES * sizeof(int));
+	m_kill_count = 0;
+	m_kill_count_ok = 0;
+	m_assists = 0;
+	mp_shots_fired = 0;
+	ms_shots_fired = 0;
+	mp_shots_hit = 0;
+	ms_shots_hit = 0;
+	mp_bonehead_hits = 0;
+	ms_bonehead_hits = 0;
+	m_bonehead_kills = 0;
+	m_player_deaths = 0;
+
+	memset(m_dogfight_kills, 0, MAX_PLAYERS * sizeof(int));
 }
 
-#ifndef NDEBUG
-//XSTR:OFF
-void scoring_eval_harbison( ship *shipp )
+// clone someone else's scoring element
+void scoring_struct::assign(const scoring_struct &s)
 {
-	FILE *fp;
+	flags = s.flags;
+	score = s.score;
+	rank = s.rank;
 
-	if ( !stricmp(shipp->ship_name, "alpha 2") && (!stricmp(Game_current_mission_filename, "demo01") || !stricmp(Game_current_mission_filename, "sm1-01")) ) {
-		int death_count;
+	medal_counts.assign(s.medal_counts.begin(), s.medal_counts.end());
 
-		fp = fopen("i:\\volition\\cww\\harbison.txt", "r+t");
-		if ( !fp )
-			return;
-		fscanf(fp, "%d", &death_count );
-		death_count++;
-		fseek(fp, 0, SEEK_SET);
-		fprintf(fp, "%d\n", death_count);
-		fclose(fp);
-	}
+	memcpy(kills, s.kills, MAX_SHIP_CLASSES * sizeof(int));
+	assists = s.assists;
+	kill_count = s.kill_count;
+	kill_count_ok = s.kill_count_ok;
+	p_shots_fired = s.p_shots_fired;
+	s_shots_fired = s.s_shots_fired;
+
+	p_shots_hit = s.p_shots_hit;
+	s_shots_hit = s.s_shots_hit;
+
+	p_bonehead_hits = s.p_bonehead_hits;
+	s_bonehead_hits = s.s_bonehead_hits;
+	bonehead_kills = s.bonehead_kills;
+
+	missions_flown = s.missions_flown;
+	flight_time = s.flight_time;
+	last_flown = s.last_flown;
+	last_backup = s.last_backup;
+
+	m_medal_earned = s.m_medal_earned;
+	m_promotion_earned = s.m_promotion_earned;
+	m_badge_earned = s.m_badge_earned;
+
+	m_score = s.m_score;
+	memcpy(m_kills, s.m_kills, MAX_SHIP_CLASSES * sizeof(int));
+	memcpy(m_okKills, s.m_okKills, MAX_SHIP_CLASSES * sizeof(int));
+	m_kill_count = s.m_kill_count;
+	m_kill_count_ok = s.m_kill_count_ok;
+	m_assists = s.m_assists;
+	mp_shots_fired = s.mp_shots_fired;
+	ms_shots_fired = s.ms_shots_fired;
+	mp_shots_hit = s.mp_shots_hit;
+	ms_shots_hit = s.ms_shots_hit;
+	mp_bonehead_hits = s.mp_bonehead_hits;
+	ms_bonehead_hits = s.ms_bonehead_hits;
+	m_bonehead_kills = s.m_bonehead_kills;
+	m_player_deaths = s.m_player_deaths;
+
+	memcpy(m_dogfight_kills, s.m_dogfight_kills, MAX_PLAYERS * sizeof(int));
 }
-//XSTR:ON
-#endif
 
 // initialize the Player's mission-based stats before he goes into a mission
 void scoring_level_init( scoring_struct *scp )
 {
-	int i;
-
 	scp->m_medal_earned = -1;		// hasn't earned a medal yet
 	scp->m_promotion_earned = -1;
-	scp->m_badge_earned = -1;
+	scp->m_badge_earned.clear();
 	scp->m_score = 0;
 	scp->m_assists = 0;
-	scp->mp_shots_fired=0;
+	scp->mp_shots_fired = 0;
 	scp->mp_shots_hit = 0;
 	scp->ms_shots_fired = 0;
 	scp->ms_shots_hit = 0;
@@ -169,19 +233,15 @@ void scoring_level_init( scoring_struct *scp )
 	scp->ms_bonehead_hits=0;
 	scp->m_bonehead_kills=0;
 
-	for (i=0; i<MAX_SHIP_CLASSES; i++){
-		scp->m_kills[i] = 0;
-		scp->m_okKills[i]=0;
-	}
+	memset(scp->m_kills, 0, MAX_SHIP_CLASSES * sizeof(int));
+	memset(scp->m_okKills, 0, MAX_SHIP_CLASSES * sizeof(int));
 
 	scp->m_kill_count = 0;
 	scp->m_kill_count_ok = 0;
 	
-	scp->m_player_deaths =0;
+	scp->m_player_deaths = 0;
 
-	for(i=0; i<MAX_PLAYERS; i++){
-		scp->m_dogfight_kills[i] = 0;
-	}
+	memset(scp->m_dogfight_kills, 0, MAX_PLAYERS * sizeof(int));
 
 	if (The_mission.ai_profile != NULL) {
 		Kill_percentage = The_mission.ai_profile->kill_percentage_scale[Game_skill_level];
@@ -245,8 +305,7 @@ void scoring_eval_badges(scoring_struct *sc)
 	}
 
 	// total_kills should now reflect the number of kills on hostile fighters/bombers.  Check this number
-	// against badge kill numbers, and return the badge index if we would get a new one.
-	int badge = -1;
+	// against badge kill numbers, and award the appropriate badges as neccessary.
 	int last_badge_kills = 0;
 	for (i = 0; i < Num_medals; i++ ) {
 		if ( total_kills >= Medals[i].kills_needed
@@ -254,16 +313,12 @@ void scoring_eval_badges(scoring_struct *sc)
 			&& Medals[i].kills_needed > 0 )
 		{
 			last_badge_kills = Medals[i].kills_needed;
-			badge = i;
+			if (sc->medal_counts[i] < 1) {
+				sc->medal_counts[i] = 1;
+				sc->m_badge_earned.push_back(i);
+			}
 		}
 	}
-
-	// if player could have a badge based on kills, and doesn't currently have this badge, then
-	// return the badge id.
-	if ( (badge != -1 ) && (sc->medals[badge] < 1) ) {
-		sc->medals[badge] = 1;
-		sc->m_badge_earned = badge;
-	}
 }
 
 // central point for dealing with accepting the score for a misison.
@@ -276,7 +331,7 @@ void scoring_do_accept(scoring_struct *score)
 
 	// do medal stuff
 	if ( score->m_medal_earned != -1 ){
-		score->medals[score->m_medal_earned]++;
+		score->medal_counts[score->m_medal_earned]++;
 	}
 
 	// return when in training mission.  We can grant a medal in training, but don't
@@ -321,8 +376,10 @@ void scoring_backout_accept( scoring_struct *score )
 	int idx;
 
 	// if a badge was earned, take it back
-	if ( score->m_badge_earned != -1){
-		score->medals[score->m_badge_earned] = 0;
+	if ( score->m_badge_earned.size() ){
+		for (size_t medal = 0; medal < score->m_badge_earned.size(); medal++) {
+			score->medal_counts[score->m_badge_earned[medal]] = 0;
+		}
 	}
 
 	// return when in training mission.  We can grant a medal in training, but don't
@@ -352,8 +409,8 @@ void scoring_backout_accept( scoring_struct *score )
 
 	// if the player was given a medal, take it back
 	if ( score->m_medal_earned != -1 ) {
-		score->medals[score->m_medal_earned]--;
-		Assert( score->medals[score->m_medal_earned] >= 0 );
+		score->medal_counts[score->m_medal_earned]--;
+		Assert( score->medal_counts[score->m_medal_earned] >= 0 );
 	}
 
 	// if the player was promoted, take it back
@@ -413,9 +470,11 @@ void scoring_level_close(int accepted)
 			}
 
 			// if a badge was earned, take it back
-			if ( Player->stats.m_badge_earned != -1){
-				Player->stats.medals[Player->stats.m_badge_earned] = -1;
-				Player->stats.m_badge_earned = -1;
+			if ( Player->stats.m_badge_earned.size() ){
+				for (size_t medal = 0; medal < Player->stats.m_badge_earned.size(); medal++) {
+					Player->stats.medal_counts[Player->stats.m_badge_earned[medal]] = 0;
+				}
+				Player->stats.m_badge_earned.clear();
 			}
 		}
 
@@ -426,7 +485,7 @@ void scoring_level_close(int accepted)
 }
 
 // STATS damage, assists recording stuff
-void scoring_add_damage(object *ship_obj,object *other_obj,float damage)
+void scoring_add_damage(object *ship_objp,object *other_obj,float damage)
 {
 	int found_slot, signature;
 	int lowest_index,idx;
@@ -460,16 +519,16 @@ void scoring_add_damage(object *ship_obj,object *other_obj,float damage)
 	}
 	
 	// don't count damage done to a ship by himself
-	if(use_obj == ship_obj){
+	if(use_obj == ship_objp){
 		return;
 	}
 
 	// get a pointer to the ship and add the actual amount of damage done to it
 	// get the ship object, and determine the _actual_ amount of damage done
-	sp = &Ships[ship_obj->instance];
+	sp = &Ships[ship_objp->instance];
 	// see comments at beginning of function
-	if(ship_obj->hull_strength < 0.0f){
-		actual_damage = damage + ship_obj->hull_strength;
+	if(ship_objp->hull_strength < 0.0f){
+		actual_damage = damage + ship_objp->hull_strength;
 	} else {
 		actual_damage = damage;
 	}
@@ -488,7 +547,7 @@ void scoring_add_damage(object *ship_obj,object *other_obj,float damage)
 
 	// only evaluate possible kill/assist numbers if the hitting object (use_obj) is a piloted ship (ie, ignore asteroids, etc)
 	// don't store damage a ship may do to himself
-	if((ship_obj->type == OBJ_SHIP) && (use_obj->type == OBJ_SHIP)){
+	if((ship_objp->type == OBJ_SHIP) && (use_obj->type == OBJ_SHIP)){
 		found_slot = 0;
 		// try and find an open slot
 		for(idx=0;idx<MAX_DAMAGE_SLOTS;idx++){
@@ -524,7 +583,7 @@ void scoring_add_damage(object *ship_obj,object *other_obj,float damage)
 char Scoring_debug_text[4096];
 
 // evaluate a kill on a ship
-int scoring_eval_kill(object *ship_obj)
+int scoring_eval_kill(object *ship_objp)
 {		
 	float max_damage_pct;		// the pct% of total damage the max damage object did
 	int max_damage_index;		// the index into the dying ship's damage_ship[] array corresponding the greatest amount of damage
@@ -545,19 +604,19 @@ int scoring_eval_kill(object *ship_obj)
 	}
 
 	// we don't evaluate kills on anything except ships
-	if(ship_obj->type != OBJ_SHIP){
+	if(ship_objp->type != OBJ_SHIP){
 		return -1;	
 	}
-	if((ship_obj->instance < 0) || (ship_obj->instance >= MAX_SHIPS)){
+	if((ship_objp->instance < 0) || (ship_objp->instance >= MAX_SHIPS)){
 		return -1;
 	}
 
 	// assign the dead ship
-	dead_ship = &Ships[ship_obj->instance];
+	dead_ship = &Ships[ship_objp->instance];
 
 	// evaluate player deaths
 	if(Game_mode & GM_MULTIPLAYER){
-		net_player_num = multi_find_player_by_object(ship_obj);
+		net_player_num = multi_find_player_by_object(ship_objp);
 		if(net_player_num != -1){
 			Net_players[net_player_num].m_player->stats.m_player_deaths++;
 			nprintf(("Network","Setting player %s deaths to %d\n",Net_players[net_player_num].m_player->callsign,Net_players[net_player_num].m_player->stats.m_player_deaths));
@@ -565,16 +624,11 @@ int scoring_eval_kill(object *ship_obj)
 			is_enemy_player = true;
 		}
 	} else {
-		if(ship_obj == Player_obj){
+		if(ship_objp == Player_obj){
 			Player->stats.m_player_deaths++;
 		}
 	}
 
-
-#ifndef NDEBUG
-	scoring_eval_harbison( dead_ship );
-#endif
-
 	net_player_num = -1;
 
 	// clear out invalid damager ships
@@ -675,7 +729,7 @@ int scoring_eval_kill(object *ship_obj)
 			// otherwise increment his valid kill count and score
 			else {
 				// dogfight mode
-				if(MULTI_DOGFIGHT && (multi_find_player_by_object(ship_obj) < 0)){
+				if(MULTI_DOGFIGHT && (multi_find_player_by_object(ship_objp) < 0)){
 					// don't add a kill for dogfight kills on non-players
 				} else {
 					plr->stats.m_okKills[si_index]++;		
@@ -749,7 +803,7 @@ int scoring_eval_kill(object *ship_obj)
 				// send appropriate stats
 				if(Netgame.type_flags & NG_TYPE_DOGFIGHT){
 					// evaluate dogfight kills
-					multi_df_eval_kill(&Net_players[net_player_num], ship_obj);
+					multi_df_eval_kill(&Net_players[net_player_num], ship_objp);
 
 					// update stats
 					send_player_stats_block_packet(&Net_players[net_player_num], STATS_DOGFIGHT_KILLS);
@@ -810,6 +864,11 @@ int scoring_eval_kill_on_weapon(object *weapon_obj, object *other_obj) {
 
 	weapon *dead_wp;						// the weapon that was killed
 	weapon_info *dead_wip;				// info on the weapon that was killed
+
+	if((weapon_obj->instance < 0) || (weapon_obj->instance >= MAX_WEAPONS)){
+		return -1;
+	}
+    
 	dead_wp = &Weapons[weapon_obj->instance]; //assign the dead weapon
 	dead_wip = &Weapon_info[dead_wp->weapon_info_index];
 
@@ -828,10 +887,6 @@ int scoring_eval_kill_on_weapon(object *weapon_obj, object *other_obj) {
 		return -1;
 	}
 
-	if((weapon_obj->instance < 0) || (weapon_obj->instance >= MAX_WEAPONS)){
-		return -1;
-	}
-
 	// clear out invalid damager ships
 	for(idx=0; idx<MAX_DAMAGE_SLOTS; idx++){
 		if((dead_wp->damage_ship_id[idx] >= 0) && (ship_get_by_signature(dead_wp->damage_ship_id[idx]) < 0)){
@@ -1421,26 +1476,53 @@ void scoring_bash_rank(player *pl,int rank)
 	pl->stats.rank = rank;
 }
 
-DCF(rank, "changes scoring vars")
+DCF(rank, "changes player rank")
 {
-	if(Dc_command){		
-		dc_get_arg(ARG_INT);		
-		
-		// parse the argument and change things around accordingly		
-		if((Dc_arg_type & ARG_INT) && (Player != NULL)){							
-			scoring_bash_rank(Player,Dc_arg_int);
-		}		
+	int rank;
+
+	if (dc_optional_string_either("help", "--help")) {
+		dc_printf("Usage: rank <index>\n");
+		dc_printf(" <index> The rank index you wish to have. For retail ranks, these correspond to:\n");
+		dc_printf("\t0 : Ensign\n");
+		dc_printf("\t1 : Lieutenant Junior Grade\n");
+		dc_printf("\t2 : Lietenant\n");
+		dc_printf("\t3 : Lieutenant Commander\n");
+		dc_printf("\t4 : Commander\n");
+		dc_printf("\t5 : Captain\n");
+		dc_printf("\t6 : Commodore\n");
+		dc_printf("\t7 : Rear Admiral\n");
+		dc_printf("\t8 : Vice Admiral\n");
+		dc_printf("\t9 : Admiral\n\n");
+		return;
+	}
+
+	if (dc_optional_string_either("status", "--status") || dc_optional_string_either("?", "--?")) {
+		if (Player != NULL) {
+			dc_printf("Current rank is %i\n", Player->stats.rank);
+		} else {
+			dc_printf("Error! Current Player not active or loaded\n");
+		}
+	}
+
+	dc_stuff_int(&rank);
+	
+	// parse the argument and change things around accordingly
+	if (Player != NULL) {
+			scoring_bash_rank(Player, rank);
+	} else {
+		dc_printf("Error! Current Player not active or loaded\n");
 	}
-	dc_printf("Usage\n0 : Ensign\n1 : Lieutenant Junior Grade\n");
-	dc_printf("2 : Lietenant\n3 : Lieutenant Commander\n");
-	dc_printf("4 : Commander\n5 : Captain\n6 : Commodore\n");
-	dc_printf("7 : Rear Admiral\n8 : Vice Admiral\n9 : Admiral");
 }
 
 void scoreing_close()
 {
+	SCP_map<int, char*>::iterator it;
 	for(int i = 0; i<NUM_RANKS; i++) {
-		if(Ranks[i].promotion_text)
-			vm_free(Ranks[i].promotion_text);
+		for (it = Ranks[i].promotion_text.begin(); it != Ranks[i].promotion_text.end(); ++it) {
+			if (it->second) {
+				vm_free(it->second);
+			}
+		}
+		Ranks[i].promotion_text.clear();
 	}
 }
