@@ -12,7 +12,7 @@
 #ifndef __FREESPACE_CORKSCREW_H__
 #define __FREESPACE_CORKSCREW_H__
 
-struct object;
+class object;
 
 extern int Corkscrew_num_missiles_fired;
 
