@@ -19,7 +19,7 @@
 // 
 
 // prototypes
-struct object;
+class object;
 struct vec3d;
 
 // ---------------------------------------------------------------------------------------------------------------------
