@@ -294,8 +294,13 @@ void shockwave_move(object *shockwave_objp, float frametime)
 		if ( objp->type == OBJ_WEAPON ) {
 			// only apply to missiles with hitpoints
 			weapon_info* wip = &Weapon_info[Weapons[objp->instance].weapon_info_index];
-			if (wip->weapon_hitpoints <= 0 || !(wip->wi_flags2 & WIF2_TAKES_SHOCKWAVE_DAMAGE) || (Weapon_info[sw->weapon_info_index].wi_flags2 & WIF2_CIWS))
+			if (wip->weapon_hitpoints <= 0 || !(wip->wi_flags2 & WIF2_TAKES_SHOCKWAVE_DAMAGE))
 				continue;
+			if (sw->weapon_info_index >= 0) {
+				if (Weapon_info[sw->weapon_info_index].wi_flags2 & WIF2_CIWS) {
+					continue;
+				}
+			}
 		}
 
 	
@@ -332,6 +337,10 @@ void shockwave_move(object *shockwave_objp, float frametime)
 		switch(objp->type) {
 		case OBJ_SHIP:
 			sw->obj_sig_hitlist[sw->num_objs_hit++] = objp->signature;
+			// If we're doing an AoE Electronics shockwave, do the electronics stuff. -MageKing17
+			if ( (sw->weapon_info_index >= 0) && (Weapon_info[sw->weapon_info_index].wi_flags3 & WIF3_AOE_ELECTRONICS) && !(objp->flags & OF_INVULNERABLE) ) {
+				weapon_do_electronics_effect(objp, &sw->pos, sw->weapon_info_index);
+			}
 			ship_apply_global_damage(objp, shockwave_objp, &sw->pos, damage );
 			weapon_area_apply_blast(NULL, objp, &sw->pos, blast, 1);
 			break;
@@ -386,6 +395,7 @@ void shockwave_render(object *objp)
 	Assert(objp->type == OBJ_SHOCKWAVE);
 	Assert(objp->instance >= 0 && objp->instance < MAX_SHOCKWAVES);
 
+    memset(&p, 0, sizeof(p));
 	sw = &Shockwaves[objp->instance];
 
 	if( (sw->delay_stamp != -1) && !timestamp_elapsed(sw->delay_stamp)){
@@ -727,10 +737,21 @@ void shockwave_page_in()
 	}
 }
 
+void shockwave_create_info_init(shockwave_create_info *sci)
+{ 
+	sci->name[ 0 ] = '\0';
+	sci->pof_name[ 0 ] = '\0';
+
+	sci->inner_rad = sci->outer_rad = sci->damage = sci->blast = sci->speed = 0.0f;
+
+	sci->rot_angles.p = sci->rot_angles.b = sci->rot_angles.h = 0.0f;
+	sci->damage_type_idx = sci->damage_type_idx_sav = -1;
+}
+
 /**
  * Loads a shockwave in preparation for a mission
  */
-void shockwave_create_info::load()
+void shockwave_create_info_load(shockwave_create_info *sci)
 {
 	int i = -1;
 
@@ -738,9 +759,9 @@ void shockwave_create_info::load()
 	// checking for that case lets us handle a situation where a 2D shockwave
 	// of "none" was specified and a valid 3D shockwave was specified
 
-	if ( strlen(name) )
-		i = shockwave_load(name, false);
+	if ( strlen(sci->name) )
+		i = shockwave_load(sci->name, false);
 
-	if ( (i < 0) && strlen(pof_name) )
-		shockwave_load(pof_name, true);
+	if ( (i < 0) && strlen(sci->pof_name) )
+		shockwave_load(sci->pof_name, true);
 }
