@@ -20,8 +20,8 @@
 #include "graphics/generic.h"
 #include "model/model.h"
 
-struct object;
-struct ship_subsys;
+class object;
+class ship_subsys;
 
 #define	WP_UNUSED			-1
 #define	WP_LASER			0		// PLEASE NOTE that this flag specifies ballistic primaries as well - Goober5000
@@ -53,7 +53,7 @@ extern int Num_weapon_subtypes;
 //#define	WIF_SHOCKWAVE		(1 << 8)				//	Explosion has a shockwave
 //WMC - These are no longer needed so these spots are free
 #define	WIF_HOMING_JAVELIN	(1 << 8)				// WC Saga Javelin HS style heatseeker, locks only on target's engines
-#define  WIF_TURNS			(1 << 9)				// Set this if the weapon ever changes heading.  If you
+#define	WIF_TURNS			(1 << 9)				// Set this if the weapon ever changes heading.  If you
 															// don't set this and the weapon turns, collision detection
 															// won't work, I promise!
 #define	WIF_SWARM			(1 << 10)			// Missile "swarms".. ie changes heading and twists on way to target
@@ -63,7 +63,7 @@ extern int Num_weapon_subtypes;
 #define	WIF_BOMB				(1 << 14)			// Bomb-type missile, can be targeted
 #define	WIF_HUGE				(1 << 15)			//	Huge damage (generally 500+), probably only fired at huge ships.
 #define	WIF_NO_DUMBFIRE	(1	<<	16)			// Missile cannot be fired dumbfire (ie requires aspect lock)
-#define  WIF_THRUSTER		(1 << 17)			// Has thruster cone and/or glow
+#define	WIF_THRUSTER		(1 << 17)			// Has thruster cone and/or glow
 #define	WIF_IN_TECH_DATABASE		(1 << 18)
 #define	WIF_PLAYER_ALLOWED		(1 << 19)   // allowed to be on starting wing ships/in weaponry pool
 #define	WIF_BOMBER_PLUS	(1 << 20)			//	Fire this missile only at a bomber or big ship.  But not a fighter.
@@ -71,14 +71,15 @@ extern int Num_weapon_subtypes;
 #define	WIF_CORKSCREW		(1 << 21)			// corkscrew style missile
 #define	WIF_PARTICLE_SPEW	(1 << 22)			// spews particles as it travels
 #define	WIF_EMP				(1 << 23)			// weapon explodes with a serious EMP effect
-#define  WIF_ENERGY_SUCK	(1 << 24)			// energy suck primary (impact effect)
+#define	WIF_ENERGY_SUCK	(1 << 24)			// energy suck primary (impact effect)
 #define	WIF_FLAK				(1 << 25)			// use for big-ship turrets - flak gun
 #define	WIF_BEAM				(1 << 26)			// if this is a beam weapon : NOTE - VERY SPECIAL CASE
 #define	WIF_TAG				(1 << 27)			// this weapon has a tag effect when it hits
 #define	WIF_SHUDDER			(1 << 28)			// causes the weapon to shudder. shudder is proportional to the mass and damage of the weapon
-#define	WIF_MFLASH			(1 << 29)			// has muzzle flash
-#define	WIF_LOCKARM			(1 << 30)			// if the missile was fired without a lock, it does significanlty less damage on impact
-#define  WIF_STREAM			(1 << 31)			// handled by "trigger down/trigger up" instead of "fire - wait - fire - wait"
+#define	WIF_LOCKARM			(1 << 29)			// if the missile was fired without a lock, it does significanlty less damage on impact
+#define	WIF_STREAM			(1 << 30)			// handled by "trigger down/trigger up" instead of "fire - wait - fire - wait"
+// NOTE: the remaining WIF is reserved for a flag that truly needs to be in the WIF_ flag field,
+// as opposed to any typical new flag that can be added to WIF3
 
 #define WIF2_BALLISTIC					(1 << 0)	// ballistic primaries - Goober5000
 #define WIF2_PIERCE_SHIELDS				(1 << 1)	// shield pierceing -Bobboau
@@ -115,6 +116,12 @@ extern int Num_weapon_subtypes;
 
 #define WIF3_NOLINK						(1 << 0)	// This weapon can not be linked with others
 #define WIF3_USE_EMP_TIME_FOR_CAPSHIP_TURRETS	(1 << 1)	// override MAX_TURRET_DISRUPT_TIME in emp.cpp - Goober5000
+#define WIF3_NO_LINKED_PENALTY			(1 << 2)	// This weapon does not count into linked firing penalty
+#define WIF3_NO_HOMING_SPEED_RAMP 		(1 << 3)	// Disables the 1s long speed ramping when firing locked-on secondaries
+#define WIF3_CMEASURE_ASPECT_HOME_ON	(1 << 4)	// This countermeasure flag makes aspect seekers home on the countermeasure instead of going into dumbfire mode
+#define WIF3_TURRET_INTERCEPTABLE		(1 << 5)	// These two flags mark a weapon as being interceptable by the AI
+#define WIF3_FIGHTER_INTERCEPTABLE		(1 << 6)	// (like WIF_BOMB), without forcing it to be tagetable -MageKing17
+#define WIF3_AOE_ELECTRONICS			(1 << 7)	// Apply electronics effect across the weapon's entire area of effect instead of just on the impacted ship -MageKing17
 
 
 #define	WIF_HOMING					(WIF_HOMING_HEAT | WIF_HOMING_ASPECT | WIF_HOMING_JAVELIN)
@@ -214,6 +221,8 @@ typedef struct weapon {
 	float alpha_current;		// the current alpha value
 
 	float weapon_max_vel;		// might just as well store the data here
+	float launch_speed;			// the initial forward speed (can vary due to additive velocity or acceleration)
+								// currently only gets set when weapon_info->acceleration_time is used
 
 	bool collisionOccured;
 	mc_info collisionInfo; // The last collision of this weapon or NULL if it had none
@@ -302,6 +311,8 @@ enum InFlightSoundType
 	ALWAYS
 };
 
+#define MAX_SUBSTITUTION_PATTERNS	10
+
 typedef struct weapon_info {
 	char	name[NAME_LENGTH];				// name of this weapon
 	char	alt_name[NAME_LENGTH];			// alt name of this weapon
@@ -335,7 +346,9 @@ typedef struct weapon_info {
 	color	laser_color_2;						// for cycling between glow colors
 	float	laser_head_radius, laser_tail_radius;
 
-	float	max_speed;							// initial speed of the weapon
+	float	max_speed;							// max speed of the weapon
+	float	acceleration_time;					// how many seconds to reach max speed (secondaries only)
+	float	vel_inherit_amount;					// how much of the parent ship's velocity is inherited (0.0..1.0)
 	float	free_flight_time;
 	float mass;									// mass of the weapon
 	float fire_wait;							// fire rate -- amount of time before you can refire the weapon
@@ -499,10 +512,6 @@ typedef struct weapon_info {
 	float alpha_min;			// minimum alpha value to use
 	float alpha_cycle;			// cycle between max and min by this much each frame
 
-	//WMC - scripting stuff
-	script_hook sc_collide_ship;
-	script_hook sc_collide_weapon;
-
 	int weapon_hitpoints;
 
 	int	burst_shots;
@@ -519,8 +528,9 @@ typedef struct weapon_info {
 	int				num_targeting_priorities;
 
 	// the optional pattern of weapons that this weapon will fire
-	SCP_vector<int> weapon_substitution_pattern; //weapon_indexs
-	SCP_vector<SCP_string> weapon_substitution_pattern_names; // weapon names so that we can generate the indexs after sort
+	size_t			num_substitution_patterns;
+	int				weapon_substitution_pattern[MAX_SUBSTITUTION_PATTERNS]; //weapon_indexes
+	char			weapon_substitution_pattern_names[MAX_SUBSTITUTION_PATTERNS][NAME_LENGTH]; // weapon names so that we can generate the indexs after sort
 
 	int			score; //Optional score for destroying the weapon
 
@@ -649,6 +659,8 @@ void weapon_get_laser_color(color *c, object *objp);
 
 void weapon_hit_do_sound(object *hit_obj, weapon_info *wip, vec3d *hitpos, bool is_armed);
 
+void weapon_do_electronics_effect(object *ship_objp, vec3d *blast_pos, int wi_index);
+
 // return a scale factor for damage which should be applied for 2 collisions
 float weapon_get_damage_scale(weapon_info *wip, object *wep, object *target);
 
